/**  Standardowa posta macierzowa problemu optymalizacji liniowej.
 *
 * Klasa przechowujca problem optymalizacji liniowej w postaci macierzy
 * A, b i c formy standardowej (augmented form):
 * min { c_T*x : A*x=b, x>=0 }, gdzie:<br>
 * A - Macierz wsp. funkcji ogranicze [m x n],<br>
 * b - Wektor ogranicze [m],<br>
 * c - Wektor kosztw [n].
 *
 * Ograniczenia przeksztacane s do rwnociowych poprzez
 * dodawanie/odejmowanie zmiennych dopeniajcych (slackw).
 * @see LinearProblem
 */

#ifndef _AugmentedForm_h
#define _AugmentedForm_h

// SYSTEM INCLUDES
//
//#include <iostream>

// PROJECT INCLUDES
//
//#include <newmat/newmat.h>
//#include <wx/txtstrm.h>


// LOCAL INCLUDES
//
#include "LinearProblem.h"
#include "util.h"

// FORWARD REFERENCES
//


class AugmentedForm
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    AugmentedForm(void);

    /** Konstruktor.
     * @param A Macierz wspczynnikw funkcji ogranicze.
     * @param b Wektor ogranicze.
     * @param c Wektor kosztw.
     * @param actualVariables Liczba znaczcych zmiennych.
     *
     */
    AugmentedForm(const Matrix&          A,
                  const ColumnVector&    b,
                  const ColumnVector&    c,
                  const int              actualVariables);


    /** Konstruktor.
     *
     *  @param problem Problem optymalizacji liniowej.
     */
    AugmentedForm(const LinearProblem& problem);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    AugmentedForm(const AugmentedForm& from);


    /** Destruktor.
     */
    ~AugmentedForm(void);


// OPERATORS


    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    AugmentedForm&              operator=(const AugmentedForm& from);

    /** Operator porwnania.
     */
    bool                        operator==(const AugmentedForm& right) const;

    /** Operator wysania do strumienia.
     */
    friend wxTextOutputStream&  operator<<(wxTextOutputStream&      os,
                                           const AugmentedForm&  augmentedForm);


// OPERATIONS
// ACCESS
    /** Zwraca macierz wsp. funkcji ogranicze
     */
    const Matrix&           A() const;

    /** Zwraca wektor ogranicze.
     */
    const ColumnVector&     b() const;

    /** Zwraca wektor kosztw.
     */
    const ColumnVector&     c() const;

    /** Zwraca liczb zmiennych znaczcych.
     */
    int                     ActualVariables() const;

    /** Zwraca liczb ogranicze.
     */
    int                     m() const;

    /** Zwraca liczb zmiennych.
     */
    int                     n() const;
// INQUIRY

protected:
private:
    Matrix          mA; //< Wsp. funkcji ogranicze [m x n].
    ColumnVector    mb; //< Ograniczenia [m].
    ColumnVector    mc; //< Koszty [n].
    int             mActualVariables; //< Liczba znaczcych zmiennych.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _AugmentedForm_h
