#include <precomp.h>

#include "Logger.h"
#include "CauchyMutation.h"  // class implemented
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

CauchyMutation::~CauchyMutation()
{
}// ~CauchyMutation


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<MutationMethod>
CauchyMutation::Clone() const
{
    return auto_ptr<MutationMethod>(new CauchyMutation(*this));
}// Clone

/* virtual */ void
CauchyMutation::Mutate(Population&                  population,
                       double                       radioactivity,
                       double                       mutationRate,
                       EvolutionaryMethodIteration& iteration) const
{
    if (population.Size() == 0) return;

    LOG(DEBUG, wxString::Format(wxT("Radioactivity:     %g"), radioactivity));

    unsigned dimensions = population[0].mPoint.nrows();

    for (unsigned index = 0; index < population.Size(); ++index)
    {
        double random = RandomDouble(0.0, 1.0);

        //TODO: Moe jednak da moliwo mutacji najlepszego chromosomu...
        if (index != population.BestIndex() && random < radioactivity)
        {
            for (unsigned j = 1; j <= dimensions; ++j)
            {
                population[index].mPoint(j) +=
                                    RandomCauchy(0.0, 1.0) * mutationRate;
            }

            population.ReevaluateChromosome(index);

            iteration.AddMutant(population[index].mPoint);
        }
        else
            iteration.AddSurvivor(population[index].mPoint);
    }

    population.GatherInfo(false);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
CauchyMutation::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
CauchyMutation::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

CauchyMutation::CauchyMutation()
    : MutationMethod()
{
}// CauchyMutation

CauchyMutation::CauchyMutation(const CauchyMutation& from)
    : MutationMethod(from)
{
}// CauchyMutation

/* static */ CauchyMutation
                    CauchyMutation::mCauchyMutation;
const /* static */ wxString
                    CauchyMutation::mName(_("Cauchy distribution mutation"));
