/** Klasa reprezentujca mutacj z rozkadem Cauchy'ego.
 *
 * Mutacja w algortymie optymalizacji ewolucyjnej polega na wybraniu z pewnym
 * prawdopodobiestwem kandydatw do przeprowadzenia mutacji z puli chromosomw,
 * ktre przetrway etap selekcji. Do wsprzenych wybranych chromosomw w tej
 * konkretnej metodzie mutacji dodawana jest warto zmiennej losowej o
 * rozkadzie Cauchy'ego. Krzywa rozkadu Cauchy'ego przypomina krzyw dzwonow
 * rozkadu normalnego, jednak nie ma tak spaszczonych "ogonw". Cakiem
 * prawdomodobne jest wylsoowanie bardzo duej wartoci, wic uycie tej metody
 * jest szczeglnie przydatne w problemach z minimami lokalnymi.
 *
 * @see EvolutionaryMethod
 */

#ifndef _CauchyMutation_h
#define _CauchyMutation_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "MutationMethod.h"

// FORWARD REFERENCES
//


class CauchyMutation
    : public MutationMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~CauchyMutation(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<MutationMethod>  Clone() const;

    /** Funkcja dokonujca mutacji wybranych chromosomw populacji bazowej.
     *
     * Funkcja z prawdopodobiestwem przekazanym w parametrze
     * <code>mutation</code> przydziela chromosom do grupy przeznaczonych do
     * mutacji, ktra polega na dodaniu do wsprzdnych chromosomy wartoci
     * zmiennej losowej o rozkadzie Cauchy'ego.
     *
     * @param population Populacja bazowa, ktrej osobniki bd mutowane.
     * @param radioactivity natenie zjawiska mutacji - prawdopodobiestwo
     * przeznaczenia chromosomu do mutacji.
     * @param mutationRate wspczynnik mnoony przez wylosowan warto
     * zmiennej losowej,
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void            Mutate
                            (Population&                  population,
                             double                       radioactivity,
                             double                       mutationRate,
                             EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Zwr opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    CauchyMutation(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    CauchyMutation(const CauchyMutation& from);

    /** Egzemplarz metody
     */
    static CauchyMutation       mCauchyMutation;
    /** Nazwa metody
     */
    static const wxString       mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _CauchyMutation_h
