#include <precomp.h>

#include "Configuration.h"  // class implemented
#include "Logger.h"

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Configuration::~Configuration()
{
    Save();
}// ~Configuration


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ Configuration&
Configuration::GetInstance()
{
    static Configuration instance;

    return instance;
}// GetInstance

void
Configuration::SetDefaults()
{
    mStepLimit              = 20000;
    mDifferentiationDelta   = 0.001;
    mInitialSectionLength   = 20;
}// SetDefaults

void
Configuration::Save()
{
    auto_ptr<wxConfigBase> ap_config(new wxConfig(mApplicationName, mVendorName));

    wxString old_path = ap_config->GetPath();
    ap_config->SetPath(mConfigPath);

    ap_config->Write(mStepLimitPath, mStepLimit);
    ap_config->Write(mDifferentiationDeltaPath, mDifferentiationDelta);
    ap_config->Write(mInitialSectionLengthPath, mInitialSectionLength);

    ap_config->SetPath(old_path);
}// Save

//============================= ACCESS     ===================================

long
Configuration::StepLimit() const
{
    return mStepLimit;
}// StepLimit

double
Configuration::DifferentiationDelta() const
{
    return mDifferentiationDelta;
}// DifferentiationDelta

double
Configuration::InitialSectionLength() const
{
    return mInitialSectionLength;
}// InitialSectionLength

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

Configuration::Configuration()
    // Te domylne wartoci nie maj za bardzo znaczenia. Istotne s tylko
    // granice przedziaw DoubleParameter'w.
    // Dolny limit dla mStepLimit jest niestety okrelony w interfejsie...
    : mStepLimit(20000),
      mDifferentiationDelta(0.001, false, 0, true, 1.0),    // TODO: do 1 ?
      mInitialSectionLength(20, true, 0),
      mApplicationName(wxTheApp->GetAppName()),
      mVendorName(wxTheApp->GetVendorName())
{
    SetDefaults();
    Load();
}// Configuration

void
Configuration::Load()
{
    double double_value;

    auto_ptr<wxConfigBase> ap_config(new wxConfig(mApplicationName, mVendorName));

    wxString old_path = ap_config->GetPath();
    ap_config->SetPath(mConfigPath);

    ap_config->Read(mStepLimitPath, &mStepLimit);

    if (ap_config->Read(mDifferentiationDeltaPath, &double_value))
    {
        // TODO: wyjtek?
        mDifferentiationDelta = double_value;
    }

    if (ap_config->Read(mInitialSectionLengthPath, &double_value))
    {
        mInitialSectionLength = double_value;
    }

    ap_config->SetPath(old_path);
}// Load

const /* static */ wxString Configuration::mConfigPath = wxT("/Configuration");
const /* static */ wxString Configuration::mStepLimitPath = wxT("Step_Limit");
const /* static */ wxString Configuration::mDifferentiationDeltaPath =
        wxT("Differentiation_Delta");
const /* static */ wxString Configuration::mInitialSectionLengthPath =
        wxT("Initial_Step_Length");
