/** Klasa bazowa dla klas danych pojedynczej iteracji optymalizacji
 * z ograniczeniami.
 */

#ifndef _ConstrainedOptimizationIteration_h
#define _ConstrainedOptimizationIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "IterationData.h"

// FORWARD REFERENCES
//

class ConstrainedOptimizationIteration
    : public IterationData
{
public:

// LIFECYCLE

    ConstrainedOptimizationIteration(bool       previousApproximationValid,
                                     bool       newApproximationValid,
                                     unsigned   evaluationCount,
                                     unsigned   iterationCount,
                                     double     penaltyParam);

    /** Destruktor.
     */
    ~ConstrainedOptimizationIteration(void);

// OPERATORS
// OPERATIONS
// ACCESS

    virtual ColumnVector                EndValuePoint() const = 0;

    virtual vector<IterationDataIdType> ClassIds() const;

    static  IterationDataIdType         ClassId();


    unsigned                            EvaluationCount() const;
    unsigned                            IterationCount() const;

    double                              PenaltyParam() const;

// INQUIRY

    bool                                IsPreviousApproximationValid() const;
    bool                                IsNewApproximationValid() const;

protected:
private:

    bool            mPreviousApproximationValid;
    bool            mNewApproximationValid;

    unsigned        mEvaluationCount;
    unsigned        mIterationCount;

    double          mPenaltyParam;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ConstrainedOptimizationIteration_h
