#include <precomp.h>

#include "Constraint.h"  // class implemented
#include "Logger.h"
#include "Symbols.h"

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Constraint::Constraint()
    : mcpFunction(), mType(Constraint::LESS_EQUAL_THAN)
{
}// Constraint

Constraint::Constraint(const Constraint& from)
    : mcpFunction(from.mcpFunction), mType(from.mType)
{
}// Constraint

Constraint::Constraint
        (std::auto_ptr<const FunctionBase> apFunction, ConstraintType type)
    : mcpFunction(apFunction), mType(type)
{
}// Constraint

Constraint::~Constraint()
{
}// ~Constraint


//============================= OPERATORS ====================================

Constraint&
Constraint::operator=(const Constraint& from)
{
    mcpFunction = from.mcpFunction;
    mType       = from.mType;

    return *this;
}// =

//============================= OPERATIONS ===================================

double
Constraint::Value(const ColumnVector& point) const
{
    assert(mcpFunction.Get() != NULL);

    double value = mcpFunction->Evaluate(point);

    switch(mType)
    {
        case EQUAL              : return fabs(value); break;
        case LESS_EQUAL_THAN    : return value; break;
        case GREATER_EQUAL_THAN : return -value; break;
        default                 : LOG(ERROR, wxT("Impossible!"));
                                  abort();
                                  break;
    }
}

bool
Constraint::IsValid(const   ColumnVector&   point,
                            bool            includeBoundary,
                            double          tolerance) const
{
    if (includeBoundary)    return (Value(point) - tolerance) <= 0.0;
    else                    return (Value(point) - tolerance) < 0.0;
}

Constraint
Constraint::DeepCopy() const
{
    Constraint copy(*this);
    copy.mcpFunction = auto_ptr<const FunctionBase>(copy.mcpFunction->Clone());

    return copy;
}

//============================= ACCESS     ===================================

const FunctionBase&
Constraint::Function() const
{
    assert(mcpFunction.Get() != NULL);

    return *mcpFunction;
}

const Constraint::ConstraintType
Constraint::Type() const
{
    return mType;
}

wxString
Constraint::ToString(bool full) const
{
    const Symbols& symbols = Symbols::GetInstance();
    const static wxString constraint_types[] =
        {
//            wxString(wxT("\t= 0")),
//            wxString(wxT("\t<= 0")),
//            wxString(wxT("\t>= 0"))
            // Windowsowe listy nie lubi tabw
            wxString(wxT(" = 0")),
            wxString(wxString(wxT(' ')) + symbols[Symbols::LESS_EQUAL]
                    + wxString(wxT(" 0"))),
            wxString(wxString(wxT(' ')) + symbols[Symbols::GREATER_EQUAL]
                    + wxString(wxT(" 0"))),
        };

    assert(mcpFunction.Get() != NULL);


    return mcpFunction->ToString(full) + constraint_types[mType];
}// ToString

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
