#ifndef _CountedPtr_h
#define _CountedPtr_h

///** Wskanik z licznikiem referencji.
//*/
//template<class T>
//struct Counter {
//    Counter(T* pointer)
//        : mPointer(pointer), mCount(1)
//        { }
//
//    T*  mPointer;
//    int mCount;
//};


/**  Prosty, zliczajcy referencje wskanik.
 *
 * Napisane w oparciu o implementacje Yorata Sharona:
 * http://ootips.org/yonat/4dev/counted_ptr.h
 */
template <class T>
class CountedPtr
{

    // TODO: doda friend dla CountedPtr na inne typy wskanikw i ukry to, co
    // nie powinno by na wierzchu.

public:
// LIFECYCLE

    /** Konstruktor.
     */
    explicit CountedPtr(T* pointer = NULL)
        : mpCount(NULL), mpObject(NULL)
    {
        if (pointer != NULL)
        {
            mpObject = pointer;
            mpCount  = new unsigned;
            *mpCount = 1;
        }
    }

    /** Konstruktor z <code>auto_ptr</code>.
     *
     * <code>aPointer</code> oczywicie oddaje piecz nad wskanikiem.
     */
    CountedPtr(std::auto_ptr<T> aPointer)
        : mpCount(new unsigned)
    {
        // Poniewa new moe wyrzuci wyjtek, to z aPointer.release czekamy, a
        // alokacja si powiedzie.
        *mpCount = 1;
        mpObject = aPointer.release();
    }

    /** Konstruktor kopiujcy.
     */
    CountedPtr(const CountedPtr& from)
        : mpCount(NULL), mpObject(NULL)
    {
        Acquire(from.mpObject, from.mpCount);
    }

    /** Konstruktor z <code>auto_ptr</code> na wskanik innego typu.
     *
     * <code>aPointer</code> oczywicie oddaje piecz nad wskanikiem.
     */
    template<class T2>
    CountedPtr(std::auto_ptr<T2> aPointer)
        : mpCount(new unsigned)
    {
        // Poniewa new moe wyrzuci wyjtek, to z aPointer.release czekamy, a
        // alokacja si powiedzie.
        *mpCount = 1;
        mpObject = aPointer.release();
    }

    /** Konstruktor z CountedPtr na wskanik innego typu.
     */
    template<class T2>
    CountedPtr(const CountedPtr<T2>& cPointer)
        : mpCount(NULL), mpObject(NULL)
    {
        Acquire(cPointer.mpObject, cPointer.mpCount);
    }


    /** Destruktor.
     */
    ~CountedPtr()
    {
        Drop();
    }

// OPERATORS

    /** Operator przypisania.
     */
    CountedPtr& operator=(const CountedPtr& from) throw()
    {
        if (this != &from)
        {
            Drop();
            Acquire(from.mpObject, from.mpCount);
        }

        return *this;
    }

    /** Operator przypisania (z <code>auto_ptr</code>).
     *
     * <code>aPointer</code> oczywicie przestaje nadzorowa wskanik.
     */
    CountedPtr& operator=(std::auto_ptr<T> aPointer) throw()
    {
        Reset(aPointer.release());

        return *this;
    }

    /** Operator przypisania z innego typu <code>CountedPtr</code>a.
     */
    template <class T2>
    CountedPtr& operator=(const CountedPtr<T2>& from) throw()
    {
        if (this != &from)
        {
            Drop();
            Acquire(from.mpCounter);
        }

        return *this;
    }

    /** Operator dereferencji.
     */
    T& operator*() const throw()
    {
        assert(mpCount  != NULL);
        assert(mpObject != NULL);

        return *mpObject;
    }

    /** Operator <code>-\></code>.
     */
    T* operator->() const throw()
    {
        assert(mpCount != NULL);

        return mpObject;
    }

// OPERATIONS

    /** Zmiana wskanika.
     *
     * CountedPtr zwalnia wskanik, ktrym si "opiekowa" i bierze piecz nad
     * nowym.
     */
    void Reset(T* pNew)
    {
        Drop();
        if (pNew != NULL)
        {
            mpObject = pNew;
            mpCount  = new unsigned;
            *mpCount = 1;
        }
    }

// ACCESS

    /** Zwraca wskanik, ktrym si opiekuje (ale go nie zwalnia).
     */
    T* Get() const
    {
        return mpObject;
    }

    /** Dodaje referencj do zliczanego wskanika.
     */
    void Acquire(T* pObject, unsigned* pCount) throw()
    {
        assert (mpObject == NULL);
        assert (mpCount == NULL);

        if (pCount != NULL)
        {
            mpObject = pObject;
            mpCount = pCount;
            ++(*mpCount);
        }
    }

    /** Dodaje referencj do zliczanego wskanika.
     *
     * Wersja dla innego typu wskanika.
     */
    template<class T2>
    void Acquire(T2* pObject, unsigned* pCount) throw()
    {
        assert (mpObject == NULL);
        assert (mpCount == NULL);

        if (pCount != NULL)
        {
            mpObject = pObject;
            mpCount = pCount;
            ++(*mpCount);
        }
    }


    /** Zwalania wskanik.
     */
    void Drop()
    {
        if (mpCount != NULL)
        {
            --(*mpCount);
            if (*mpCount == 0)
            {
                delete mpObject;
                delete mpCount;
            }
            mpCount = NULL;
            mpObject = NULL;
        }
    }

    unsigned*   mpCount;
    T*          mpObject;
protected:
private:
};

#endif
