/** Klasa bazowa dla metod krzyowania.
 *
 * Krzyowanie osobnikw populacji w optymalizacji ewolucyjnej polega na
 * tworzeniu nowego chromosomu na podstawie wsprzdnych dwch lub wicej
 * wybranych istniejcych chromosomw.
 *
 * @see InBetweenCrossover, ThreeParticipantsCrossover, EvolutionaryMethod
 */

#ifndef _CrossoverMethod_h
#define _CrossoverMethod_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "methods/EvolutionaryMethodIteration.h"
#include "Population.h"

// FORWARD REFERENCES
//


class CrossoverMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    virtual ~CrossoverMethod(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach krzyowania.
     *
     * @return Kopia prototypu metody
     */
    virtual std::auto_ptr<CrossoverMethod>      Clone() const = 0;

    /** Przeprowad krzyowanie.
     *
     * Funkcja abstrakcyjna implementowana w konkretnych metodach krzyowania
     * dziedziczcych z klasy CrossoverMethod.
     *
     * @param population Populacja, ktrej osobniki maj by poddane procesowi
     * krzyowania.
     * @param crossoverRate Wspczynnik krzyowania. Ma rne znaczenie w
     * rnych metodach.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void        Crossover
                          (Population&                  population,
                           double                       crossoverRate,
                           EvolutionaryMethodIteration& iteration) const = 0;

// ACCESS

    /** Zwr reefrencj na kontener metod krzyowania.
     *
     * @return referencja na konterer metod krzyowania.
     */
    static const std::vector<CrossoverMethod*>& CrossoverMethods();

    /** Przedstaw opis metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach krzyowania.
     *
     * @return Opis metody.
     */
    virtual wxString                            ToString() const = 0;

    /** Zwr nazw metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach krzyowania.
     *
     * @return Nazwa metody.
     */
    virtual wxString                            Name() const = 0;

    /** Zwr nazw klasy metod manipulacji populacj.
     *
     * Zwraca nazw klasy metod manipulacji populacj. W tym przypadku "Metoda
     * krzyowania" lub odpowiednik w innym jezyku.
     *
     * @return Nazwa klasy manipulacji populacj.
     */
    static wxString                             BaseClassName();

// INQUIRY

protected:

    /** Konstruktor domylny.
     */
    CrossoverMethod(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    CrossoverMethod(const CrossoverMethod& from);

private:

    /** Zwraca referencj na kontener metod krzyowania.
     *
     * @return referencja na kontener metod krzyowania.
     */
    static std::vector<CrossoverMethod*>&       rCrossoverMethods();

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _CrossoverMethod_h
