#include <precomp.h>

#include "Logger.h"

#include "CustomValueCondition.h"  // class implemented

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================
CustomValueCondition::CustomValueCondition()
    : StandardStopCondition(),
      mMetWhenGreater(false)
{
    mName = wxT("custom value");
}// CustomValueCondition

CustomValueCondition::CustomValueCondition(
                const DoubleParameter&  epsilon,
                bool                    metWhenGreater)
    : StandardStopCondition(epsilon),
      mMetWhenGreater(metWhenGreater)
{
}// CustomValueCondition

CustomValueCondition::CustomValueCondition(const wxString&  name,
                                           bool             metWhenGreater)
    : StandardStopCondition(),
      mMetWhenGreater(metWhenGreater)
{
    mName = name;
}// CustomValueCondition

CustomValueCondition::CustomValueCondition(const CustomValueCondition& from)
    : StandardStopCondition(from),
      mMetWhenGreater(from.mMetWhenGreater)
{
}// CustomValueCondition

CustomValueCondition::~CustomValueCondition()
{
}// ~CustomValueCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
CustomValueCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new CustomValueCondition(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString
CustomValueCondition::ToString() const
{
    wxString description;

    description << Name();
    if (!mMetWhenGreater)   description << wxT(" < ");
    else                    description << wxT(" > ");

    description << wxString::Format(wxT("%g"), static_cast<double>(Epsilon()));

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
CustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                            const ColumnVector&    lastPoint,
                            double                 previousValue,
                            double                 lastValue,
                            double                 customValue) const
{
    if (!mMetWhenGreater)   return (customValue < mEpsilon);
    else                    return (customValue > mEpsilon);
}// IsMet

/* virtual */ bool
CustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                            const ColumnVector&    lastPoint,
                            double                 previousValue,
                            const FunctionBase&    function,
                            double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0, customValue);
}// IsMet

/* virtual */ bool
CustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                            const ColumnVector&    lastPoint,
                            const FunctionBase&    function,
                            double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0, customValue);
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
