/** Warunek stopu oparty na podanej liczbie.
 *
 * Sprawdza, czy podana liczba (moe to by np. norma gradientu) jest
 * mniejsza lub wiksza od wartoci granicznej (<em>epsilon</em>).
 */

#ifndef _CustomValueCondition_h
#define _CustomValueCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class CustomValueCondition
    : public StandardStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    CustomValueCondition(void);

    /** Konstruktor.
     *
     * @param name Nazwa sprawdzanego parametru.
     * @param metWhenGreater Informacja, czy warunek jest speniony,
     * gdy warto jest wiksza ni epsilon (false - speniony gdy mniejsza).
     */
    CustomValueCondition(const wxString& name,
                         bool  metWhenGreater = false);

    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     * @param metWhenGreater Informacja, czy warunek jest speniony,
     * gdy warto jest wiksza ni epsilon (false - speniony gdy mniejsza).
     */
    CustomValueCondition(const DoubleParameter& epsilon,
                         bool  metWhenGreater = false);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    CustomValueCondition(const CustomValueCondition& from);


    /** Destruktor.
     */
    ~CustomValueCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

    /** Zwraca tekstowy opis warunku stopu.
     */
    virtual wxString        ToString() const;

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              double                lastValue,
                              double                customValue) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              const FunctionBase&   function,
                              double                customValue) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue) const;

protected:
private:
    bool mMetWhenGreater;   /// Informacja, czy warunek jest speniony,
                            /// gdy warto jest wiksza ni epsilon
                            /// (false - speniony gdy mniejsza).
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _CustomValueCondition_h
