#include <precomp.h>

#include "DirectionalUnivariateAdapter.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DirectionalUnivariateAdapter::DirectionalUnivariateAdapter(const FunctionBase&   function)
    : FunctionBase(function.IsDifferentiable()),
      UnivariateFunction(function.IsDifferentiable()),
      mFunction(function),
      mBase(0),
      mDirection(0)
{
}// DirectionalUnivariateAdapter

DirectionalUnivariateAdapter::DirectionalUnivariateAdapter(const FunctionBase&   function,
                                     const ColumnVector&   x,
                                     const ColumnVector&   d)
    : FunctionBase(function.IsDifferentiable()),
      UnivariateFunction(function.IsDifferentiable()),
      mFunction(function),
      mBase(x),
      mDirection(d)
{
}// DirectionalUnivariateAdapter

DirectionalUnivariateAdapter::~DirectionalUnivariateAdapter()
{
}// ~DirectionalUnivariateAdapter


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
DirectionalUnivariateAdapter::Clone() const
{
    // Ta funkcja nigdy nie powinna by klonowana.
    assert(false);

    return auto_ptr<FunctionBase>(new DirectionalUnivariateAdapter(*this));
}// Clone

//============================= ACCESS     ===================================

void
DirectionalUnivariateAdapter::SetDirection(const ColumnVector& x, const ColumnVector& d)
{
    mBase       = x;
    // TODO: usun normalizacj
    mDirection  = d/d.norm_Frobenius(); // Normalizacja kierunku
}// SetDirection

unsigned
DirectionalUnivariateAdapter::EvaluationCount() const
{
    return mFunction.EvaluationCount();
}// EvaluationCount

/* virtual */ wxString
DirectionalUnivariateAdapter::ToString(bool full) const
{
    wxString result;

    if (full)
    {
        result = _("Univariate adapter");

        // TODO: zrobi co z wrzucaniem wektorw do stringw

        result << wxT("\n");
    }
    else    result = wxT("Unimplemented");

    result << mFunction.ToString(full);

    return result;
}// ToString

/* virtual */ VariableNames
DirectionalUnivariateAdapter::PresentVariables() const
{
    return VariableNames();
}// PresentVariables

//============================= INQUIRY    ===================================

/* virtual */ bool
DirectionalUnivariateAdapter::ContainsUnknown(const VariableNames& variables) const
{
    return false;
}// ContainsUnknown

/////////////////////////////// PROTECTED  ///////////////////////////////////

DirectionalUnivariateAdapter::DirectionalUnivariateAdapter(
        const DirectionalUnivariateAdapter& from)
    : FunctionBase(from),
      UnivariateFunction(from),
      mFunction(from.mFunction),
      mBase(from.mBase),
      mDirection(from.mDirection)
{
}

/* virtual */ double
DirectionalUnivariateAdapter::DoEvaluate(const ColumnVector& point) const
    throw(EvaluationError)
{
    assert(point.nrows() == 1);

    return mFunction.Evaluate(mBase + (point(1) * mDirection));
}

/* virtual */ double
DirectionalUnivariateAdapter::DoEvaluateNoThrow(const ColumnVector& point) const
    throw()
{
    assert(point.nrows() == 1);

    return mFunction.EvaluateNoThrow(mBase + (point(1) * mDirection));
}

/////////////////////////////// PRIVATE    ///////////////////////////////////
