/** Przeksztaca funkcj wielu zmiennych w funkcj jednej zmiennej.
 *
 * Otrzymana funkcja jest nastpujca:
 *
 * f(t) = x + dt
 *
 * gdzie <code>d</code> to kierunek, a <code>x</code> to punkt, przez ktry
 * przechodzi prosta wyznaczona przez <code>d</code> (okrela take punkt 0).
 */

#ifndef _DirectionalUnivariateAdapter_h
#define _DirectionalUnivariateAdapter_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "UnivariateFunction.h"

// FORWARD REFERENCES
//


class DirectionalUnivariateAdapter
    : public UnivariateFunction
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param function Funkcja podstawowa. Nie staje si ona
     * wasnoci obiektu klasy <code>DirectionalUnivariateAdapter</code>, odpowiedzialno
     * za ni spoczywa na kodzie uytkownika (oraz za to, aby funkcja nie
     * zostaa usunita przed usuniciem obiektu tej klasy).
     */
    DirectionalUnivariateAdapter(const FunctionBase&   function);

    // Nie moe by protected, bo si nie skompiluje przez ten wyej
    DirectionalUnivariateAdapter(const DirectionalUnivariateAdapter& from);

    /** Konstruktor.
     *
     * @param function Funkcja podstawowa. Nie staje si ona
     * wasnoci obiektu klasy <code>DirectionalUnivariateAdapter</code>, odpowiedzialno
     * za ni spoczywa na kodzie uytkownika (oraz za to, aby funkcja nie
     * zostaa usunita przed usuniciem obiektu tej klasy).
     * @param x Punkt, przez ktry przechodzi prosta wyznaczona przez kierunek.
     * W tym punkcie jest 0.
     * @param d Kierunek, jest automatycznie normalizowany.
     */
    DirectionalUnivariateAdapter(const FunctionBase&   function,
                                 const ColumnVector&   x,
                                 const ColumnVector&   d);


    /** Destruktor.
     */
    virtual ~DirectionalUnivariateAdapter(void);


// OPERATORS

// OPERATIONS

    /// Nie powinno by uywane.
    virtual std::auto_ptr<FunctionBase> Clone() const;

// ACCESS

    /** Ustawia kierunek.
     *
     * @param x Punkt, przez ktry przechodzi prosta wyznaczona przez kierunek.
     * W tym punkcie jest 0.
     * @param d Kierunek, jest automatycznie normalizowany.
     */
    void                SetDirection(const ColumnVector& x,
                                     const ColumnVector& d);

    /** Zwraca liczb wylicze funkcji.
     */
    unsigned            EvaluationCount() const;

    // TODO: poprawi opis, jak ToString zostanie skoczone
    /** Zwraca tekstowy opis funkcji.
     *
     * Funkcja nie w peni zaimplementowana.
     */
    virtual wxString    ToString(bool full = true) const;

    /** Zwraca pust list zmiennych.
     */
    virtual VariableNames   PresentVariables() const;
// INQUIRY

//    /** Sprawdza, czy funkcja jest rniczkowalna.
//     */
//    virtual bool        IsDifferentiable() const;

    /** Zwraca <code>false</code>
     */
    virtual bool        ContainsUnknown(const VariableNames& variables) const;

protected:

    /** Oblicza warto funkcji w punkcie.
     *
     * f(t) = x + dt
     *
     * @param point Punkt <code>t</code>. Wektor musi by jednowymiarowy.
     *
     * @return x + d * point(1)
     */
    virtual double      DoEvaluateNoThrow(const ColumnVector& point) const
        throw();

    virtual double      DoEvaluate(const ColumnVector& point) const
        throw(EvaluationError);

    const FunctionBase& mFunction;    ///< Funkcja podstawowa
    ColumnVector        mBase;        ///< Punkt "0" -- x
    ColumnVector        mDirection;   ///< Kierunek
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DirectionalUnivariateAdapter_h
