#include <precomp.h>

#include "util.h"

#include "DistanceCondition.h"  // class implemented
#include "Logger.h"

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DistanceCondition::DistanceCondition()
    : StandardStopCondition()
{
    mName = wxT("|| x k+1 - x k ||");
}// DistanceCondition

DistanceCondition::DistanceCondition(const DoubleParameter& epsilon)
    : StandardStopCondition(epsilon)
{
}// DistanceCondition

DistanceCondition::DistanceCondition(const DistanceCondition& from)
    : StandardStopCondition(from)
{
}// DistanceCondition

DistanceCondition::~DistanceCondition()
{
}// ~DistanceCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
DistanceCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new DistanceCondition(*this));
}// Clone

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

/* virtual */ bool
DistanceCondition::IsMet(const ColumnVector&    previousPoint,
                         const ColumnVector&    lastPoint,
                         double                 previousValue,
                         double                 lastValue,
                         double                 customValue) const
{
    // Warunek nie jest speniony w pierwszej iteracji.
    // W metodach sprawdzajcych warunek stopu przed wykonaniem
    // kadej iteracji (np. w metodach gradientowych) naley inicjowa
    // poprzedni punkt sta NullPoint.

    LOG(DEBUG, wxT("prev: ") << endl << previousPoint);
    LOG(DEBUG, wxT("last: ") << endl << lastPoint);

    if (previousPoint == NullPoint)     { return false; }

    ColumnVector step(lastPoint - previousPoint);

    return (step.norm_Frobenius() < mEpsilon);
}// IsMet

/* virtual */ bool
DistanceCondition::IsMet(const ColumnVector&    previousPoint,
                         const ColumnVector&    lastPoint,
                         double                 previousValue,
                         const FunctionBase&    function,
                         double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0);
}// IsMet

/* virtual */ bool
DistanceCondition::IsMet(const ColumnVector&    previousPoint,
                         const ColumnVector&    lastPoint,
                         const FunctionBase&    function,
                         double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0);
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
