/** Warunek stopu oparty na odlegoci dwch punktw.
 *
 * Sprawdza, czy odlego midzy podanymi dwoma punktami jest mniejsza
 * od wartoci granicznej (<em>epsilon</em>):
 * \f[ \|x^2 - x^1\| < \varepsilon . \f]
 */

#ifndef _DistanceCondition_h
#define _DistanceCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class DistanceCondition
    : public StandardStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    DistanceCondition(void);


    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     */
    DistanceCondition(const DoubleParameter& epsilon);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    DistanceCondition(const DistanceCondition& from);


    /** Destruktor.
     */
    ~DistanceCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS
// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              double                lastValue,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DistanceCondition_h
