#include <precomp.h>

#include "DoubleParameter.h"  // class implemented
#include "Symbols.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DoubleParameter::DoubleParameter(double value,
                                 double min,
                                 bool   leftOpen,
                                 double max,
                                 bool   rightOpen,
                                 bool   integer) throw(Error)
    : mValue(value),
      mMin(min),
      mMax(max),
      mLeftOpen(leftOpen),
      mRightOpen(rightOpen),
      mInteger(integer)
{
    if (!IsValid(value)) throw Error(_("Constraint error"));
}// DoubleParameter

DoubleParameter::DoubleParameter(const DoubleParameter& from) throw()
    : mValue(from.mValue),
      mMin(from.mMin), mMax(from.mMax),
      mInteger(from.mInteger)
{
}// DoubleParameter

DoubleParameter::~DoubleParameter()
{
}// ~DoubleParameter


//============================= OPERATORS ====================================

DoubleParameter&
DoubleParameter::operator=(const DoubleParameter& from) throw()
{
    mValue      = from.mValue;
    mMin        = from.mMin;
    mMax        = from.mMax;
    mLeftOpen   = from.mLeftOpen;
    mRightOpen  = from.mRightOpen;
    mInteger    = from.mInteger;

    return *this;
}// =

DoubleParameter&
DoubleParameter::operator=(double from) throw(Error)
{
    if (IsValid(from))  mValue = from;
    else                throw Error(_("Constraint error"));

    return *this;
}// =

DoubleParameter::operator double() const throw()
{
    return mValue;
}// double

//============================= OPERATIONS ===================================

wxString
DoubleParameter::BoundaryToString() const
{
    wxString text;
    if (mMin == -DBL_MAX)
    {
        text << wxT("( -") << Symbols::GetInstance()[Symbols::INFINITY]
             << wxT("; ");
    }
    else
    {
        text << wxString::Format(wxT("%c %.6g; "),
                    mLeftOpen   ? '(' : '[',
                    mMin);
    }

    if (mMax == DBL_MAX)
    {
        text << Symbols::GetInstance()[Symbols::INFINITY] << wxT(" )");
    }
    else
    {
        text << wxString::Format(wxT("%.6g %c"),
                    mMax,
                    mRightOpen   ? ')' : ']');
    }

    return text;
}// BoundaryToString

//============================= ACCESS     ===================================
void
DoubleParameter::SetIntegerType(bool integer)
{
    mInteger = integer;
}// SetIntegerType
//============================= INQUIRY    ===================================

bool
DoubleParameter::IsValid(double value) const
{
    if      (mInteger       && (value != floor(value))) return false;
    if      (mLeftOpen      && (value <= mMin))         return false;
    else if (!mLeftOpen     && (value <  mMin))         return false;
    else if (mRightOpen     && (value >= mMax))         return false;
    else if (!mRightOpen    && (value >  mMax))         return false;
    else                                                return true;
}// IsValid

bool
DoubleParameter::HasIntegerType() const
{
    return mInteger;
}// HasIntegerType
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
