/** Parametr o wartoci zmiennoprzecinkowej i ograniczonym przedziale wartoci.
 *
 * Klasa przewidziana przede wszystkim dla parametrw metod optymalizacji.
 * Dopuszczalne wartoci s ograniczone do zadanego przedziau, ktry moe by
 * lewo- bd prawostronnie otwarty albo zamknity.
 */

#ifndef _DoubleParameter_h
#define _DoubleParameter_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Error.h"

// LOCAL INCLUDES
//

// FORWARD REFERENCES
//


class DoubleParameter
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param value Warto pocztkowa.
     * @param min       Minimalna dopuszczalna warto.
     * @param leftOpen  Czy przedzia otwarty z lewej strony.
     * @param max       Maksymalna dopuszczalna warto.
     * @param rightOpen Czy przedzia otwarty z prawej strony.
     * @param integer   Czy warto musi by liczb cakowit.
     */
    DoubleParameter(double  value       = 0.0,
                    double  min         = -DBL_MAX,
                    bool    leftOpen    = false,
                    double  max         = DBL_MAX,
                    bool    rightOpen   = false,
                    bool    integer     = false) throw(Error);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    DoubleParameter(const DoubleParameter& from) throw();


    /** Destruktor.
     */
    ~DoubleParameter(void);


// OPERATORS

    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    DoubleParameter&    operator=(const DoubleParameter& from) throw();

    /** Operator przypisania.
     */
    DoubleParameter&    operator=(double from) throw(Error);

    /** Operator rzutowania.
     */
                        operator double() const throw();

// OPERATIONS

    /** Tworzy napis przedstawiajcy dopuszczalny przedzia.
     *
     * Np. <code>(0, 1]</code>
     */
    wxString            BoundaryToString() const;

// ACCESS

    /** Ustawia doln granic przedziau dopuszczalnych wartoci.
     *
     * @param min Warto.
     * @param open Czy przedzia jest lewostronnie otwarty.
     */
    void                SetLowerBoundary(double min, bool open = false);

    /** Ustawia grn granic przedziau dopuszczalnych wartoci.
     *
     * @param max Warto.
     * @param open Czy przedzia jest prawostronnie otwarty.
     */
    void                SetUpperBoundary(double max, bool open = false);

    /** Ustawia typ wartoci na liczby cakowite lub dowolne.
     *
     * @param integer Czy warto musi by liczb cakowit.
     */
    void                SetIntegerType(bool integer);
// INQUIRY

    /** Sprawdza, czy warto mieci si w dopuszczalnym przedziale.
     */
    bool                IsValid(double value) const;

    /** Podaje informacj, czy warto musi by liczb cakowit.
     */
    bool                HasIntegerType() const;

protected:
private:
    double  mValue;     ///< Warto.
    double  mMin;       ///< Minimalna dopuszczalna warto.
    double  mMax;       ///< Maksymalna dopuszczalna warto.
    bool    mLeftOpen;  ///< Czy przedzia otwarty z lewej strony.
    bool    mRightOpen; ///< Czy przedzia otwarty z prawej strony.
    bool    mInteger;   ///< Czy warto musi by liczb cakowit.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DoubleParameter_h
