#include <precomp.h>

#include "CustomValueCondition.h"
#include "util.h"

#include "EMIterationsCondition.h"  // class implemented
#include "Logger.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

EMIterationsCondition::EMIterationsCondition()
    : EMStopCondition(CountedPtr<StandardStopCondition>(
                        new CustomValueCondition(
                            DoubleParameter(25, 1, false, DBL_MAX, false, true),                            true)))
{
    mName = _("Stagnant iterations");
}// EMIterationsCondition

EMIterationsCondition::EMIterationsCondition
    (const EMIterationsCondition& from)
    : EMStopCondition(from)
{
}// EMIterationsCondition

EMIterationsCondition::~EMIterationsCondition()
{
}// ~EMIterationsCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
EMIterationsCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new EMIterationsCondition(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString
EMIterationsCondition::ToString() const
{
    wxString description;

    description << Name()
                << wxT(" > ")
                << wxString::Format(wxT("%g"), static_cast<double>(
                            mcpComparison->Epsilon()));

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
EMIterationsCondition::IsMet(
                const ColumnVector&         previousBestPoint,
                const ColumnVector&         bestPoint,
                const FunctionBase&         function,
                double                      pointStandardDeviation,
                double                      valueStandardDeviation,
                unsigned                    stagnantIterations) const
{
    assert(mcpComparison.Get() != NULL);

    return mcpComparison->IsMet(NullPoint, NullPoint, 0.0, 0.0,
            stagnantIterations);
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
