#include <precomp.h>

#include "CustomValueCondition.h"
#include "util.h"

#include "EMPointDeviationCondition.h"  // class implemented
#include "Logger.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

EMPointDeviationCondition::EMPointDeviationCondition()
    : EMStopCondition(CountedPtr<StandardStopCondition>(
                        new CustomValueCondition(
                            DoubleParameter(1, 0, true))))
{
    mName = _("Chromosome standard deviation");
}// EMPointDeviationCondition

EMPointDeviationCondition::EMPointDeviationCondition
    (const EMPointDeviationCondition& from)
    : EMStopCondition(from)
{
}// EMPointDeviationCondition

EMPointDeviationCondition::~EMPointDeviationCondition()
{
}// ~EMPointDeviationCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
EMPointDeviationCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new EMPointDeviationCondition(*this));
}// Clone

//============================= ACCESS     ===================================

//============================= INQUIRY    ===================================

/* virtual */ bool
EMPointDeviationCondition::IsMet(
                const ColumnVector&         previousBestPoint,
                const ColumnVector&         bestPoint,
                const FunctionBase&         function,
                double                      pointStandardDeviation,
                double                      valueStandardDeviation,
                unsigned                    stagnantIterations) const
{
    assert(mcpComparison.Get() != NULL);

    return mcpComparison->IsMet(NullPoint, NullPoint, 0.0, 0.0,
            pointStandardDeviation);
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
