/** Warunek stopu dla metody ewolucyjnej.
 */

#ifndef _EMStopCondition_h
#define _EMStopCondition_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "FunctionBase.h"
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class EMStopCondition
    : public StopConditionBase
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    EMStopCondition(void);

    /** Konstruktor.
     *
     * @param cpComparison Obiekt dokonujcy porwnania podawanych
     * w podklasach wartoci z epsilon.
     */
    EMStopCondition(CountedPtr<StandardStopCondition> cpComparison);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    EMStopCondition(const EMStopCondition& from);


    /** Destruktor.
     */
    virtual ~EMStopCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

    /** Zwraca warto minimaln <em>epsilon</em>.
     */
    virtual DoubleParameter         Epsilon() const;

    /** Zwraca referencj na <em>epsilon</em>.
     */
    virtual DoubleParameter&        rEpsilon();

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @param previousBestPoint Najlepszy punkt w poprzedniej iteracji.
     * @param bestPoint Najlepszy punkt w obecnej iteracji.
     * @param function Funkcja celu.
     * @param standardDeviation
     * @param pointStandardDeviation Odchylenie standardowe liczone
     * dla wszystkich punktw obecnej populacji.
     * @param valueStandardDeviation Odchylenie standardowe liczone dla
     * wartoci funkcji we wszystkich punktach obecnej populacji.
     * @param stagnantIterations Liczba iteracji, w ktrych nie zmieni
     * si najlepszy punkt.
     *
     * @return Czy warunek stopu jest speniony.
     */
    virtual bool        IsMet(
                const ColumnVector&         previousBestPoint,
                const ColumnVector&         bestPoint,
                const FunctionBase&         function,
                double                      pointStandardDeviation,
                double                      valueStandardDeviation,
                unsigned                    stagnantIterations)
                                                                    const;

// PUBLIC MEMBERS

protected:
    /** Obiekt dokonujcy porwnania podanych punktw lub wartoci
     * z epsilon.
     */
    CountedPtr<StandardStopCondition>    mcpComparison;
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _EMStopCondition_h
