#include <precomp.h>

#include <wx/fs_zip.h>
#include <wx/stdpaths.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>

#include "Eduoptim2.h"  // class implemented

#include "Configuration.h"
#include "Logger.h"

#include <config.h>

IMPLEMENT_APP(Eduoptim2)

LOGGER(DEBUG);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ bool
Eduoptim2::OnInit()
{
    LOGGER_ON;

    SetAppName(wxT("eduoptim2"));
    SetVendorName(wxT("PWR"));

#ifdef __UNIX_LIKE__
    // Ustaw prefix (powinien by zapodany przez configure)
    wxStandardPaths &r_paths = dynamic_cast<wxStandardPaths&>
        (wxStandardPaths::Get());
    r_paths.SetInstallPrefix(wxT(PREFIX));
    LOG(DEBUG, wxT("Set prefix to: ") << r_paths.GetInstallPrefix());
#endif

    // Inicjuj obsug rnych formatw plikw
    wxFileSystem::AddHandler(new wxZipFSHandler);
    wxImage::AddHandler(new wxPNGHandler);
    wxImage::AddHandler(new wxCURHandler);
    wxImage::AddHandler(new wxICOHandler);

    // Inicjuj locale
    mLocale.Init(wxLocale::GetSystemLanguage());

    bool result = mLocale.AddCatalog(wxT("eduoptim2"));

    // Inicjuj zasoby
    wxXmlResource::Get()->AddHandler(new wxBitmapXmlHandler);
    wxXmlResource::Get()->AddHandler(new wxImageXmlHandler);
    wxXmlResource::Get()->AddHandler(new wxIconXmlHandler);
    wxXmlResource::Get()->Load(ResourcesPath() + wxT("resources.xrc"));


    // Inicjuj metody
    Method::Init();

    mpMainFrame = new MainFrame(NULL, -1, wxT(""), wxDefaultPosition,
            wxDefaultSize, 0);

    mpMainFrame->Show(true);

    // wxWidgets chyba samo zniszczy to okno.
    mpProgressDialog = new ProgressDialog(mpMainFrame, -1, wxT(""));

    SetTopWindow(mpMainFrame);

    return true;
}// Init

//============================= ACCESS     ===================================

ProgressDialog&
Eduoptim2::rProgressDialog()
{
    assert(mpProgressDialog != NULL);

    return *mpProgressDialog;
}// rProgressDialog

wxString
Eduoptim2::ResourcesPath() const
{
    wxFileName path;
    path.AssignDir(wxStandardPaths::Get().GetDataDir());

    // dodawanie tego do cieki nie dziaa (pewnie ze wzgldu na '#' i ':')
    return path.GetFullPath() + wxT("resources.xrs#zip:");
}// ResourcesPath

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

