#include <precomp.h>

#include "GradientMethodIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

GradientMethodIteration::GradientMethodIteration()
    : mStartingPoint(),
      mGradient(),
      mDirection(),
      mTau(0.0),
      mEndPoint(NullPoint)
{
}// GradientMethodIteration

GradientMethodIteration::GradientMethodIteration(
        const ColumnVector& startingPoint,
        const ColumnVector& gradient)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mDirection(),
      mTau(0.0),
      mEndPoint(NullPoint)
{
}// GradientMethodIteration

GradientMethodIteration::GradientMethodIteration(
        const ColumnVector& startingPoint,
        const ColumnVector& gradient,
        const ColumnVector& direction)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mDirection(direction),
      mTau(0.0),
      mEndPoint(NullPoint)
{
}// GradientMethodIteration

GradientMethodIteration::GradientMethodIteration(
        const ColumnVector& startingPoint,
        const ColumnVector& gradient,
        const ColumnVector& direction,
        const ColumnVector& endPoint)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mDirection(direction),
      mTau(0.0),
      mEndPoint(endPoint)
{
}// GradientMethodIteration

GradientMethodIteration::GradientMethodIteration(
        const ColumnVector& startingPoint,
        const ColumnVector& gradient,
        const ColumnVector& direction,
              double        tau,
        const ColumnVector& endPoint)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mDirection(direction),
      mTau(tau),
      mEndPoint(endPoint)
{
}// GradientMethodIteration

GradientMethodIteration::~GradientMethodIteration()
{
}// ~GradientMethodIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
GradientMethodIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), GradientMethodIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
GradientMethodIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
GradientMethodIteration::EndValuePoint() const
{
    if (StepMade())     return mEndPoint;
    else                return mStartingPoint;
}// EndValue

const ColumnVector&
GradientMethodIteration::StartingPoint() const
{
    return mStartingPoint;
}// StartingPoint

ColumnVector&
GradientMethodIteration::rStartingPoint()
{
    return mStartingPoint;
}// rStrartingPoint

const ColumnVector&
GradientMethodIteration::Gradient() const
{
    return mGradient;
}// Gradient

ColumnVector&
GradientMethodIteration::rGradient()
{
    return mGradient;
}// rGradient

const ColumnVector&
GradientMethodIteration::Direction() const
{
    return mDirection;
}// Direction

ColumnVector&
GradientMethodIteration::rDirection()
{
    return mDirection;
}// rDirection

double
GradientMethodIteration::Tau() const
{
    return mTau;
}// Tau

double&
GradientMethodIteration::rTau()
{
    return mTau;
}// rTau

const ColumnVector&
GradientMethodIteration::EndPoint() const
{
    return mEndPoint;
}// EndPoint

ColumnVector&
GradientMethodIteration::rEndPoint()
{
    return mEndPoint;
}// rEndPoint
//============================= INQUIRY    ===================================
bool
GradientMethodIteration::StepMade() const
{
    return (mEndPoint != NullPoint);
}// BaseChanged

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
