/** Klasa bazowa dla klas danych iteracji metod gradientowych.
 *
 * @see Result
 */

#ifndef _GradientMethodIteration_h
#define _GradientMethodIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "IterationData.h"
#include "util.h"

// FORWARD REFERENCES
//


class GradientMethodIteration
    : public IterationData
{
public:

// LIFECYCLE

    /** Konstruktor domylny.
     */
    GradientMethodIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param gradient Gradient funkcji w punkcie wyjcia.
     */
    GradientMethodIteration(const ColumnVector& staringPoint,
                            const ColumnVector& gradient);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param gradient Gradient funkcji w punkcie wyjcia.
     * @param direction Kierunek, w ktrym zostanie wykonany krok.
     */
    GradientMethodIteration(const ColumnVector& staringPoint,
                            const ColumnVector& gradient,
                            const ColumnVector& direction);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param gradient Gradient funkcji w punkcie wyjcia.
     * @param direction Kierunek, w ktrym zostanie wykonany krok.
     * @param endPoint Punkt wyliczony w toku iteracji.
     */
    GradientMethodIteration(const ColumnVector& staringPoint,
                            const ColumnVector& gradient,
                            const ColumnVector& direction,
                            const ColumnVector& endPoint);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param gradient Gradient funkcji w punkcie wyjcia.
     * @param direction Kierunek, w ktrym zostanie wykonany krok.
     * @param endPoint Punkt wyliczony w toku iteracji.
     */
    GradientMethodIteration(const ColumnVector& staringPoint,
                            const ColumnVector& gradient,
                            const ColumnVector& direction,
                                  double        tau,
                            const ColumnVector& endPoint);

    /** Destruktor.
     */
    ~GradientMethodIteration(void);

// OPERATORS

// OPERATIONS

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const ColumnVector&         StartingPoint() const;
    ColumnVector&               rStartingPoint();

    const ColumnVector&         Gradient() const;
    ColumnVector&               rGradient();

    const ColumnVector&         Direction() const;
    ColumnVector&               rDirection();

    double                      Tau() const;
    double&                     rTau();

    const ColumnVector&         EndPoint() const;
    ColumnVector&               rEndPoint();

// INQUIRY
    bool                        StepMade() const;

protected:
private:
    ColumnVector    mStartingPoint; ///< Punkt pocztkowy iteracji.

    ColumnVector    mGradient;      ///< Gradient funkcji w punkcie bazowym.

    ColumnVector    mDirection;     ///< Kierunek kroku.

    double          mTau;           ///< Dugo kroku.

    ColumnVector    mEndPoint;      ///< Punkt po wykonaniu kroku.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _GradientMethodIteration_h
