#include <precomp.h>

#include "History.h"  // class implemented
#include "Logger.h"

LOGGER(INFO)


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

History::History()
{
}// History

History::~History()
{
}// ~History


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
History::InsertUserString(const wxString& string)
{
    HistoryItems::iterator position;

    position = find_if(mHistoryItems.begin(), mHistoryItems.end(),
            bind2nd(ptr_fun(History::CompareString), string));

    if (position == mHistoryItems.end())
    {   // Dodajemy i by moe usuwamy ostatni uytkownika
        LOG(DEBUG, wxT("Not found, adding"));
        mHistoryItems.insert(mHistoryItems.begin(), HistoryItem(string, false));

        if (mMaxUserEntries < CountAndFindLast(position))
        {
            LOG(DEBUG, wxT("Full, erasing the last one"));
            mHistoryItems.erase(position);
        }
    }
    else
    {   // Przesuwamy na pocztek
        HistoryItem itemCopy = *position;
        mHistoryItems.erase(position);
        mHistoryItems.insert(mHistoryItems.begin(), itemCopy);
    }
}// InsertUserString

void
History::InsertPermanentString(const wxString& string)
{
    mHistoryItems.insert(mHistoryItems.end(), HistoryItem(string, true));
}// InsertPermanentString

void
History::Save(wxConfigBase* pConfig) const
{
    HistoryItems::const_iterator i;
    unsigned number = 0;

    for (i = mHistoryItems.begin(); i != mHistoryItems.end(); ++i)
    {
        if (i->second == false)
        {
            pConfig->Write(wxString::Format(wxT("%03u"), number), i->first);
            ++number;
        }
    }
}// Save

void
History::Load(wxConfigBase* pConfig)
{
    long dummy_index;
    bool success;
    wxString entry_name;
    unsigned long read_count = 0;

    LOG(DEBUG, wxT("Entering Load()"));

    mHistoryItems.erase(mHistoryItems.begin(), mHistoryItems.end());

    success = pConfig->GetFirstEntry(entry_name, dummy_index);
    while (success && (read_count <= mMaxUserEntries))
    {
        unsigned long number;
        // Upewnij si, e czytasz to, co trzeba
        if (entry_name.ToULong(&number) && (number == read_count))
        {
            wxString entry_value;
            bool result = pConfig->Read(entry_name, &entry_value);
            assert (result);

            LOG(DEBUG, wxT("Read from ") << entry_name << wxT(": ") << entry_value);
            // Nie uywam InsertUserString, eby zachowa kolejno z
            // konfiguracji
            mHistoryItems.insert(mHistoryItems.end(),
                    HistoryItem(entry_value, false));
            ++read_count;
        }
        else
        {
            LOG(WARNING, wxT("Unrecognized history item in configuration! ")
                    << entry_name);
            LOG(WARNING, wxT("Entry ignored."));
        }
        success = pConfig->GetNextEntry(entry_name, dummy_index);
    }
}// Load


//============================= ACCESS     ===================================

wxArrayString
History::Strings() const
{
    wxArrayString array;
    HistoryItems::const_iterator i;

    for (i = mHistoryItems.begin(); i != mHistoryItems.end(); ++i)
    {
        array.Add(i->first);
    }

    return array;
}// Strings

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

unsigned
History::CountAndFindLast(HistoryItems::iterator& rLast)
{
    HistoryItems::iterator position;
    unsigned count = 0;

    for (position = mHistoryItems.begin(); position != mHistoryItems.end();
            ++position)
    {
        if (position->second == false)
        {
            rLast = position;
            ++count;
        }
    }

    LOG(DEBUG, wxT("User item count: ") << count);

    return count;
}// CountAndFindLast


/* static */ bool
History::CompareString(const HistoryItem   historyItem,
                       const wxString      string)
{
    return (historyItem.first == string);
}// CompareString

const /* static */ unsigned History::mMaxUserEntries = 10;
