/** Klasa przechowujca histori wprowadzonych przez uytkownika cigw znakw.
 *
 * Oprcz napisw uytkownika mona take wprowadzi na stae predefiniowane
 * napisy.
 *
 * Historia ma ograniczon liczb "slotw" na napisy wprowadzone przez
 * uytkownika, najstarsze s usuwane po ich zapenieniu. Predefiniowane cigi
 * znakw nie s nigdy usuwane (ani wliczane w ogln liczb slotw).
 *
 * @see HistoryManager
 */

#ifndef _History_h
#define _History_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//

// FORWARD REFERENCES
//


class History
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    History(void);


    /** Destruktor.
     */
    ~History(void);


// OPERATORS

// OPERATIONS

    /** Dodaj tekst uytkownika.
     *
     * Dodaje do historii cig znakw wprowadzony przez uytkownika. Jeli cig
     * ten znajduje si ju w historii, to zostanie on przesunity na jej
     * pocztek. Jeli nie, to zostanie dodany na pocztek.
     *
     * W przypadku dodania cigu, gdy lista historii jest ju pena, usuwany
     * zostaje ostatni w kolejnoci cig znakw dodany przez uytkownika.
     */
    void            InsertUserString(const wxString& string);

    /** Dodaj tekst permanentny.
     *
     * Dodaje na koniec historii cig znakw, ktry nie bdzie usuwany nawet, gdy
     * caa lista historii bdzie zapeniona.
     *
     * W przeciwiestwie do InsertUserString nie jest sprawdzane, czy cig
     * znakw ju istnieje.
     */
    void            InsertPermanentString(const wxString& string);

    void            Save(wxConfigBase*  pConfig) const;

    void            Load(wxConfigBase*  pConfig);

// ACCESS

    /** Zwraca tablic wszystkich napisw znajdujcych si w historii.
     */
    wxArrayString   Strings() const;

// INQUIRY

protected:
private:
    /** Element listy historii, zawiera waciwy cig znakw oraz flag, czy
     * jest to element "stay" (tzn. nie wprowadzony przez uytkownika, ktrego
     * nie mona usun z listy).
     */
    typedef std::pair<wxString, bool>   HistoryItem;
    typedef std::vector<HistoryItem>    HistoryItems;

    HistoryItems            mHistoryItems;  ///< Wektor elementw historii.

    /// Maksymalna ilo pamitanych wpisw uytkownika.
    static const unsigned   mMaxUserEntries;


    /** Zlicza wszystkie napisy wprowadzone przez uytkownika oraz wskazuje na
     * ostatni z nich.
     *
     * @param rLast Referencja na iterator, ktry jest ustawiany na ostatni
     * element historii wprowadzony przez uytkownika.
     *
     * @return Liczba napisw wprowadzonych przez uytkownika.
     */
    unsigned                CountAndFindLast(HistoryItems::iterator& rLast);


    /** Porwnuje tekst elementu historii z podanym cigiem znakw.
     */
    // TODO: wooolneee, nie dziaaj parametry przez referencj...
    static bool CompareString(const HistoryItem    historyItem,
                              const wxString       string);
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _History_h
