#include <precomp.h>

#include "HistoryManager.h"  // class implemented
#include "Logger.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HistoryManager::~HistoryManager()
{
    Save();
}// ~HistoryManager


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* static */ History&
HistoryManager::GetHistory(HistoryType type)
{
    static HistoryManager historyManager;

    assert (type < ELEMENT_COUNT);

    return historyManager.mHistories[type];
}// GetHistory

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

HistoryManager::HistoryManager()
    // W momencie niszczenia obiektu i zapisu obiekt aplikacji moe ju by
    // czciowo (bd cakiem?) zniszczony, dlatego zapamitujemy nazw.
    : mApplicationName(wxTheApp->GetAppName()),
      mVendorName(wxTheApp->GetVendorName())
{
    Load();
    Init();
}// HistoryManager

void
HistoryManager::Init()
{
    mHistories[NONLINEAR_FUNCTION].InsertPermanentString(
            wxT("x^2 + 6*sin(x)"));
    mHistories[NONLINEAR_FUNCTION].InsertPermanentString(
            wxT("(x - 2)^4 + (x - 2*y)^2"));
    mHistories[NONLINEAR_FUNCTION].InsertPermanentString(
            wxT("100*(y - x^2)^2 + (1 - x)^2"));

    mHistories[NONLINEAR_CONSTRAINT].InsertPermanentString(
            wxT("x - 10"));
    mHistories[NONLINEAR_CONSTRAINT].InsertPermanentString(
            wxT("x^2 + y^2 - 16"));

    mHistories[LINEAR_FUNCTION].InsertPermanentString(
            wxT("-x + 2*y"));
    mHistories[LINEAR_CONSTRAINT].InsertPermanentString(
            wxT("x + y - 10"));
}// Init

void
HistoryManager::Load()
{
    auto_ptr<wxConfigBase> ap_config(new wxConfig(mApplicationName, mVendorName));

    wxString old_path = ap_config->GetPath();

    ap_config->SetPath(HistoryManager::mMainPath);

    // Poor man's foreach...
    for (int i = 0; i < ELEMENT_COUNT; ++i)
    {
        ap_config->SetPath(HistoryManager::mHistoryPaths[i]);
        mHistories[i].Load(ap_config.get());
        ap_config->SetPath(wxT("../"));
    }

    ap_config->SetPath(old_path);
}// Load

void
HistoryManager::Save() const
{
    auto_ptr<wxConfigBase> ap_config(new wxConfig(mApplicationName, mVendorName));

    wxString old_path = ap_config->GetPath();

    ap_config->SetPath(HistoryManager::mMainPath);

    for (int i = 0; i < ELEMENT_COUNT; ++i)
    {
        ap_config->SetPath(HistoryManager::mHistoryPaths[i]);
        mHistories[i].Save(ap_config.get());
        ap_config->SetPath(wxT("../"));
    }

    ap_config->SetPath(old_path);
}// Save

const /* static */ wxString HistoryManager::mMainPath = wxT("/History");
const /* static */ wxString HistoryManager::mHistoryPaths[] =
    { wxT("NonlinearFunctions"),
      wxT("NonlinearConstraints"),
      wxT("LinearFunctions"),
      wxT("LinearConstraints") };
