/** Klasa zarzdzajca zbiorem historii funkcji.
 *
 * Zawiera osobne obiekty History dla rnych typw funkcji i ogranicze.
 * Historie s zapisywane w konfiguracji i odczytywane przy pierwszym uyciu.
 * Poza tym w funkcji Init s wprowadzane predefiniowane wartoci.
 *
 * @see History
 */

#ifndef _HistoryManager_h
#define _HistoryManager_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "History.h"

// FORWARD REFERENCES
//


class HistoryManager
{
public:
    /// Typ historii.
    enum HistoryType
    {
        NONLINEAR_FUNCTION = 0, ///< Funkcje nieliniowe.
        NONLINEAR_CONSTRAINT,   ///< Ograniczenia nieliniowe.
        LINEAR_FUNCTION,        ///< Funkcje liniowe.
        LINEAR_CONSTRAINT,      ///< Ograniczenia liniowe.

        // Liczba elementw, musi by na kocu!
        ELEMENT_COUNT
    };

// LIFECYCLE

    /** Destruktor.
     */
    ~HistoryManager(void);


// OPERATORS

// OPERATIONS
// ACCESS

    /** Zwraca referencj na wskazany typ historii.
     */
    static History&         GetHistory(HistoryType  type);

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    HistoryManager(void);

    /** Inicjuje historie predefiniowanymi wartociami.
     *
     * @see History::InsertPermanentString
     */
    void    Init();

    /** aduje zapisan w konfiguracji histori.
     */
    void    Load();

    /** Zapisuje wszystkie historie do konfiguracji.
     */
    void    Save() const;

    /// Tablica zawierajca wszystkie historie.
    History     mHistories[ELEMENT_COUNT];

    /// Nazwa aplikacji (uywana przy zapisie konfiguracji).
    const wxString mApplicationName;
    const wxString mVendorName;

    /// Gwna cieka do historii w konfiguracji.
    const static wxString   mMainPath;
    /// (pod)cieki do konfiguracji poszczeglnych historii.
    const static wxString   mHistoryPaths[ELEMENT_COUNT];
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _HistoryManager_h
