#include <precomp.h>

#include "Logger.h"
#include "HyperbolicInnerPenalty.h"  // class implemented

LOGGER(INFO);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HyperbolicInnerPenalty::HyperbolicInnerPenalty()
    : InnerPenalty()
{
}// HyperbolicInnerPenalty

HyperbolicInnerPenalty::HyperbolicInnerPenalty(CountedPtr<const FunctionBase> cpFunction)
    : InnerPenalty(cpFunction)
{
}// HyperbolicInnerPenalty

HyperbolicInnerPenalty::~HyperbolicInnerPenalty()
{
}// ~HyperbolicInnerPenalty


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
HyperbolicInnerPenalty::Clone() const
{
    return auto_ptr<FunctionBase>(new HyperbolicInnerPenalty(*this));
}

/* virtual */ double
HyperbolicInnerPenalty::PenaltyValue(double arg) const
{
    return -1.0 / arg;
}

//============================= ACCESS     ===================================

/* virtual */ wxString
HyperbolicInnerPenalty::ToString(bool full) const
{
    wxString result(InnerFunction().ToString());

    if (full)
    {
        result << wxT("\n")
          << _("Penalty function: inner, hyberbolic,")
          << wxT("\n")
          << _("based on following constraints:")
          << wxT("\n");

        result << ConstraintsToString();

        result << InnerPenalty::ToString(full);

        result << wxT("\n");
    }

    return result;
}// ToString

/* virtual *//* FunctionIdType
HyperbolicInnerPenalty::Id() const
{
    return HyperbolicInnerPenalty::ClassId();
}// Id
*/
/* static *//* FunctionIdType
HyperbolicInnerPenalty::ClassId()
{
    const static FunctionIdType HyperbolicInnerPenaltyId = GenerateId();

    return HyperbolicInnerPenaltyId;
}// ClassId
*/
//============================= INQUIRY    ===================================

/* virtual */ int
HyperbolicInnerPenalty::FunctionId() const
{
    return 1;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

HyperbolicInnerPenalty::HyperbolicInnerPenalty(
        const HyperbolicInnerPenalty&   from)
    : InnerPenalty(from)
{
}// HyperbolicInnerPenalty

/////////////////////////////// PRIVATE    ///////////////////////////////////
