/** Hiperboliczna funkcja bariery.
 *
 * @see InnerPenalty, LogarithmicInnerPenalty
 */

#ifndef _HyperbolicInnerPenalty_h
#define _HyperbolicInnerPenalty_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Error.h"
#include "InnerPenalty.h"

// FORWARD REFERENCES
//


class HyperbolicInnerPenalty
    : public InnerPenalty
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    HyperbolicInnerPenalty();

    /** Konstruktor.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj, do ktrej dodana
     * zostanie warto kary.
     */
    HyperbolicInnerPenalty(CountedPtr<const FunctionBase> cpFunction);

    /** Destruktor.
     */
    ~HyperbolicInnerPenalty(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase> Clone() const;


    /** Warto kary.
     *
     * Funkcja zwraca warto kary w zalenoci od odlegoci od ograniczenia wg
     * wzoru: \f$ 1 / -g(x) \f$.
     *
     * @param arg przekroczenie ograniczenia.
     */
    virtual double                  PenaltyValue(double arg) const;


// ACCESS

    virtual wxString                ToString(bool full = true) const;
/*
    static FunctionIdType           ClassId();

    virtual FunctionIdType          Id() const;
*/

// INQUIRY

    virtual int                     FunctionId() const;

protected:
    HyperbolicInnerPenalty(const HyperbolicInnerPenalty& from);

private:

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _HyperbolicInnerPenalty_h
