#include <precomp.h>

#include "InBetweenCrossover.h"  // class implemented
#include "Logger.h"
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

InBetweenCrossover::~InBetweenCrossover()
{
}// ~InBetweenCrossover

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<CrossoverMethod>
InBetweenCrossover::Clone() const
{
    return auto_ptr<CrossoverMethod>(new InBetweenCrossover(*this));
}// Clone

/* virtual */ void
InBetweenCrossover::Crossover(Population&                   population,
                              double                        crossoverRate,
                              EvolutionaryMethodIteration&  iteration) const
{
    while (population.Size() < population.DesiredSize())
    {
        size_t  parent_index_1 = RandomIndex(0, population.Size());
        size_t  parent_index_2 = RandomIndex(1, population.Size());

        if (parent_index_1 == parent_index_2) --parent_index_1;

        LOG(DEBUG, wxString::Format(wxT("First parent: ch%d: "),
                                    parent_index_1)
                << population[parent_index_1].mPoint.as_row()
                << wxString::Format(wxT("value = %g"),
                                    population[parent_index_1].mValue));
        LOG(DEBUG, wxString::Format(wxT("Second parent: ch%d: "),
                                    parent_index_2)
                << population[parent_index_2].mPoint.as_row()
                << wxString::Format(wxT("value = %g"),
                                    population[parent_index_2].mValue));

        ColumnVector    child = (population[parent_index_1].mPoint +
                                 population[parent_index_2].mPoint) / 2.0;

        LOG(DEBUG, wxT("Child: ") << child.as_row());

        iteration.AddChild(child);

        population.AddChromosome(child);
    }

    population.GatherInfo(false);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
InBetweenCrossover::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
InBetweenCrossover::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

InBetweenCrossover::InBetweenCrossover()
    : CrossoverMethod()
{
}// InBetweenCrossover

InBetweenCrossover::InBetweenCrossover(const InBetweenCrossover& from)
    : CrossoverMethod(from)
{
}// InBetweenCrossover

/* static */ InBetweenCrossover
                    InBetweenCrossover::mInBetweenCrossover;
/* static */ const wxString
                    InBetweenCrossover::mName(_("In-between crossover"));
