/** Klasa reprezentujca metod prostego krzyowania dwch
 * chromosomw.
 *
 * Klasa dokonuje krzyowania chromosomw poprzez tworzenie nowego
 * chromosomu-dziecka w poowie drogi pomidzy dwoma chromosomami-rodzicami.
 * Krzyowanie takie powoduje bardzo szybkie kurczenie si obszaru zajmowanego
 * przez populacj, wic jej przesuwanie si w kierunku minimum trzeba zapewni
 * innymi sposobami np. poprzez uycie duego wspczynnika natenia
 * mutacji.
 *
 * @see EvolutionaryMethod
 */

#ifndef _InBetweenCrossover_h
#define _InBetweenCrossover_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CrossoverMethod.h"

// FORWARD REFERENCES
//


class InBetweenCrossover
    : public CrossoverMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~InBetweenCrossover(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<CrossoverMethod>  Clone() const;

    /* Przeprowad krzyowanie.
     *
     * Funkcja doprowadza populacj do pocztkowego stanu liczebnoci poprzez
     * tworzenie nowych nowych chromosomw w poowie odcink czcego dwa rne
     * wylosowane chromosomy.
     *
     * @param population Populacja, ktrej osobniki maj by poddane procesowi
     * krzyowania.
     * @param crossoverRate Wspczynnik krzyowania. nie uywany w tej
     * metodzie.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void        Crossover
                              (Population&                  population,
                               double                       crossoverRate,
                               EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Przedstaw opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    InBetweenCrossover(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    InBetweenCrossover(const InBetweenCrossover& from);

    /** Egzemplarz metody
     */
    static InBetweenCrossover       mInBetweenCrossover;
    /** Nazwa metody
     */
    static const wxString           mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _InBetweenCrossover_h
