#include <precomp.h>

#include "Logger.h"
#include "InnerPenalty.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

InnerPenalty::InnerPenalty()
    : PenaltyFunction()
{
}// InnerPenalty

InnerPenalty::InnerPenalty(CountedPtr<const FunctionBase> cpFunction)
    : PenaltyFunction(cpFunction)
{
      mInitialPenaltyParam      = DoubleParameter(1.0, 0, true);
      mPenaltyParamMultiplier   = DoubleParameter(0.5, 0, true, 1, true);
      mTolerance                = DoubleParameter(0.0, 0.0, false, 0.0, false);
}// InnerPenalty

InnerPenalty::~InnerPenalty()
{
}// ~InnerPenalty


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


//============================= ACCESS     ===================================

/* virtual */ wxString
InnerPenalty::ToString(bool full) const
{
    wxString result(/*InnerFunction().ToString()*/wxT(""));

    if (full)
    {
        result << _("Reaction to point out of bounds:") << wxT("\n");

        switch (mOutOfBoundsReaction)
        {
            case EXCEPTION: result << _("Throw an exception");
                            break;
            case MAX_VALUE: result << _("Return maximum value");
                            break;
            default:        assert(false);
                            break;
        }

        result << wxT("\n");
    }

    return result;
}// ToString

/* virtual */ FunctionIdType
InnerPenalty::Id() const
{
    return InnerPenalty::ClassId();
}// Id

/* static */ FunctionIdType
InnerPenalty::ClassId()
{
    const static FunctionIdType InnerPenaltyId = GenerateId();

    return InnerPenaltyId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
InnerPenalty::IsValid(const ColumnVector& point) const
{
    Constraint::Container::const_iterator i;

    // Sprawd IsValid dla wszystkich ogranicze
    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
    {
        if (i->IsValid(point, false, 0) == false)   return false;
    }

    return true;
}// IsValid

/* virtual */ bool
InnerPenalty::IsValidStartingPoint(const ColumnVector& point) const
{
    // TODO: Czy nie da sprawdzania Isinf?. Funkcja kary szybko ronie, mona
    // by to sprawdzi.
    return  IsValid(point);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

// TODO: zrefaktorowa t i ponisz funkcj.
/* virtual */ double
InnerPenalty::DoEvaluateNoThrow(const ColumnVector& point) const throw()
{
    Constraint::Container::const_iterator i;
    // Musi by lokalna referencja, inaczej nie zatrzymuje si na
    // Constraints().end(), nie wnikaem czemu.
    const Constraint::Container& constraints = Constraints();
    double value;

    value = InnerFunction().EvaluateNoThrow(point);

    LOG(DEBUG,wxT("penalty param = ") << PenaltyParam());
    LOG(DEBUG, wxT("Point:") << point.as_row());
    LOG(DEBUG, wxT("Function value: ") << value);

    for (i = constraints.begin(); i != constraints.end(); ++i)
    {
        double constraint_value = i->Value(point);

        //LOG(DEBUG,wxT("Constraint value: ") << constraint_value);

        if (constraint_value < 0.0)
        {
            value += PenaltyParam() * PenaltyValue(constraint_value);
            //LOG(DEBUG,wxT("value: ") << value);
        }
        else
        {
            if (mOutOfBoundsReaction == MAX_VALUE)  return DBL_MAX;
            else                                    return mNan;
        }

    }
    LOG(DEBUG, wxT("Value after adding constraints: ") << value);

    return value;
}// DoEvaluateNoThrow

/* virtual */ double
InnerPenalty::DoEvaluate(const ColumnVector& point) const throw(EvaluationError)
{
    Constraint::Container::const_iterator i;
    // Musi by lokalna referencja, inaczej nie zatrzymuje si na
    // Constraints().end(), nie wnikaem czemu.
    const Constraint::Container& constraints = Constraints();
    double value;

    value = InnerFunction().Evaluate(point);

    LOG(DEBUG,wxT("penalty param = ") << PenaltyParam());
    LOG(DEBUG, wxT("Point:") << point.as_row());
    LOG(DEBUG, wxT("Function value: ") << value);

    for (i = constraints.begin(); i != constraints.end(); ++i)
    {
        double constraint_value = i->Value(point);

        //LOG(DEBUG,wxT("Constraint value: ") << constraint_value);

        if (constraint_value < 0.0)
        {
            value += PenaltyParam() * PenaltyValue(constraint_value);
            //LOG(DEBUG,wxT("value: ") << value);
        }
        else
        {
            switch (mOutOfBoundsReaction)
            {
                case MAX_VALUE: LOG(INFO,wxT("Returning DBL_MAX"));
                                return DBL_MAX;
                                break;
                //  TODO: Dopisa punkt do komunikatu!
                case EXCEPTION: throw EvaluationError
                                            (wxT("Point out of bounds"), point);
                default:        break;
            }
        }

    }
    LOG(DEBUG, wxT("Value after adding constraints: ") << value);

    return value;
}// DoEvaluate
/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ double InnerPenalty::mNan = sqrtf(-1.f);
