/** Wewntrzna funkcja kary (funkcja bariery).
 *
 * Wewntrzna funkcja kary dolicza do wartoci funkcji celu dodatkow wartoc
 *
 * @see OuterPenalty, PenaltyFunction
 */

#ifndef _InnerPenalty_h
#define _InnerPenalty_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Error.h"
#include "PenaltyFunction.h"

// FORWARD REFERENCES
//


class InnerPenalty
    : public PenaltyFunction
{
public:
// LIFECYCLE

    /** Konstruktor domyslny.
     */
    InnerPenalty();

    /** Konstruktor.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj, do ktrej dodana
     * zostanie warto kary.
     */
    InnerPenalty(CountedPtr<const FunctionBase> cpFunction);


    /** Destruktor.
     */
    ~InnerPenalty(void);


// OPERATORS

// OPERATIONS

    /** Modyfikacja wspczynnika kary.
     */
    //void        ModifyPenalty();

    /** Resetuje wspczynnik kary do pocztkowej wartoci.
     */
    //void        ResetPenalty();


// ACCESS

    virtual wxString ToString(bool full = true) const;

    static FunctionIdType           ClassId();

    /** Zwraca ID funkcji kary.
     */
    virtual FunctionIdType          Id() const;

// INQUIRY

    virtual bool    IsValid(const ColumnVector& point) const;

    virtual bool    IsValidStartingPoint(const ColumnVector& point) const;

protected:
    /** Zwraca warto funkcji celu powikszon o warto funkcji kary.
     *
     * @exception Error Gdy wynik lub wartoci porednie w trakcie oblicze
     * przestan by skoczone.
     */
    virtual double   DoEvaluate(const ColumnVector& point) const
                                                        throw(EvaluationError);

    virtual double   DoEvaluateNoThrow(const ColumnVector& point) const
                                                        throw();

    virtual double   PenaltyValue(double arg) const = 0;

private:

    /// NaN, przyda si do zwracania z DoEvaluateNoThrow
    static const double mNan;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _InnerPenalty_h
