#include <precomp.h>


#include "IterationValueInterpolation.h"  // class implemented
#include "Logger.h"
#include "ProblemBase.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

IterationValueInterpolation::IterationValueInterpolation(const Result& result)
    : FunctionBase(false),
      UnivariateFunction(false),
      mResult(result)
{
}// IterationValueInterpolation

IterationValueInterpolation::~IterationValueInterpolation()
{
}// ~IterationValueInterpolation


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
IterationValueInterpolation::Clone() const
{
    // Ta klasa nie powinna by klonowana
    assert(false);

    return auto_ptr<FunctionBase>(new IterationValueInterpolation(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString
IterationValueInterpolation::ToString(bool full) const
{
    return _("Iteration-Value function");
}// ToString

/* virtual */ VariableNames
IterationValueInterpolation::PresentVariables() const
{
    assert(false);  // Nie powinno by uywane
    return VariableNames();
}// PresentVariables

/* virtual */ bool
IterationValueInterpolation::ContainsUnknown(const VariableNames& variables)
    const
{
    assert(false);
    return false;
}// ContainsUnknown

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

IterationValueInterpolation::IterationValueInterpolation(
        const IterationValueInterpolation& from)
    : FunctionBase(from),
      UnivariateFunction(from),
      mResult(from.mResult) // referencja!
{
}

/* virtual */ double
IterationValueInterpolation::DoEvaluateNoThrow(const ColumnVector& point)
    const throw()
{
    const static double NAN_value = sqrt(-1.);

    assert (point.nrows() == 1);

    double x = point(1);
    // Rozwizanie tymczasowe
    if ((x >= mResult.FirstIterationNumber())
            && (x <= mResult.LastIterationNumber()))
    {
        if (x == floor(x))
        {// bierz warto
            unsigned iteration_number = static_cast<unsigned>(x);

            return mResult.Problem().BaseFunction().EvaluateNoThrow(
                    mResult.Iteration(iteration_number).EndValuePoint());
        }
        else
        {// interpoluj
            unsigned lower  = static_cast<unsigned>(floor(x));
            unsigned higher = static_cast<unsigned>(ceil(x));
            double alpha    = x - lower;

            const FunctionBase& f = mResult.Problem().BaseFunction();
            double lower_value =
                f.EvaluateNoThrow(mResult.Iteration(lower).EndValuePoint());
            double higher_value =
                f.EvaluateNoThrow(mResult.Iteration(higher).EndValuePoint());

//            LOG(DEBUG, wxT("lower index value:  ") << lower_value);

            return lower_value + alpha*(higher_value - lower_value);
        }
    }
    else    return NAN_value;
}// DoEvaluateNoThrow

/////////////////////////////// PRIVATE    ///////////////////////////////////
