/** Funkcja "iteracja -> warto funkcji celu", z interpolacj liniow.
 *
 * Przydaje si do wykrelenia stosownego wykresu.
 *
 * @see IterationData, Result
 */

#ifndef _IterationValueInterpolation_h
#define _IterationValueInterpolation_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "UnivariateFunction.h"
#include "Result.h"

// FORWARD REFERENCES
//


class IterationValueInterpolation
    : public UnivariateFunction
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    IterationValueInterpolation(const Result& result);

    /** Destruktor.
     */
    ~IterationValueInterpolation(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase> Clone() const;

// ACCESS

    virtual wxString        ToString(bool full = true) const;

    /** Zwraca puste zmienne.
     *
     * Nie dziaa w trybie debug, ta funkcja nie powinna by w ogle uywana dla
     * tej klasy.
     */
    virtual VariableNames   PresentVariables() const;

    /** Zwraca false.
     *
     * Nie dziaa w trybie debug, ta funkcja nie powinna by w ogle uywana dla
     * tej klasy.
     */
    virtual bool            ContainsUnknown(const VariableNames& variables)
                                const;

// INQUIRY

protected:

    IterationValueInterpolation(const IterationValueInterpolation& from);

    /** Oblicza warto funkcji.
     *
     * Warto jest obliczana na podstawie wartoci w danej iteracji (metoda
     * IterationData::EndValuePoint()) za pomoc interpolacji liniowej.
     *
     * @param point Jednowymiarowy wektor stanowicy argument funkcji (numer
     * iteracji).
     */
    virtual double          DoEvaluateNoThrow(const ColumnVector& point)
                                const throw();


private:
    const Result&   mResult;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _IterationValueInterpolation_h
