#include <precomp.h>

#include "Error.h"
#include "Logger.h"
#include "LinearFunction.h"  // class implemented
#include "ProblemBase.h"

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

LinearFunction::LinearFunction()
    : FunctionBase(true), NonlinearFunction()
{
}// LinearFunction

LinearFunction::LinearFunction
    (std::auto_ptr<const FunctionTree>  apFunction,
     const wxString&                    textForm)
    : FunctionBase(true), NonlinearFunction(apFunction, textForm, true)
{
}// LinearFunction

/* virtual */
LinearFunction::~LinearFunction()
{
}// ~LinearFunction


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
LinearFunction::Clone() const
{
    return auto_ptr<FunctionBase>(new LinearFunction(*this));
}// Clone

//============================= ACCESS     ===================================
/* virtual */ wxString
LinearFunction::ToString(bool full) const
{
    if (full)
    {
        const VariableNames& variables = mpProblem->Variables();
        const RowVector& coefficients = Coefficients();
        wxString description;

        for (int i = 1; i <= coefficients.ncols(); ++i)
        {
            if (i != 1)
            {
               if (coefficients(i) >= 0)    description << wxT("+ ");
               else                         description << wxT("- ");
            }
            else if (coefficients(i) >= 0)
            {
                description << wxT(" ");
            }

            description << wxString::Format(wxT("%g"),
                            i == 1 ? coefficients(i) : fabs(coefficients(i)))
                        << wxT("*")
                        << variables[i - 1]
                        << wxT(" ");
        }
        if (A0() != 0)
        {
            if (A0() > 0)   description << wxT("+ ");
            else            description << wxT("- ");
            description << wxString::Format(wxT("%g"), fabs(A0()));
        }

        LOG(DEBUG, wxT("description: ") << description);
        return description;
    }
    else
    {
        LOG(DEBUG, wxT("just the function expression: ") << NonlinearFunction::ToString(false));
        return NonlinearFunction::ToString(false);
    }

}// ToString

/* virtual */ RowVector
LinearFunction::Coefficients() const
{
    assert(mpProblem != NULL);

    // Tworzenie wspczynnikw poprzez wyliczanie wartoci funkcji

    const VariableNames& variables = mpProblem->Variables();

    RowVector coefficients(variables.GetCount());
    ColumnVector point(variables.GetCount());
    double a0 = A0();

    point = 0;

    for (size_t i = 1; i <= variables.GetCount(); ++i)
    {
        point(i) = 1;
        coefficients(i) = Evaluate(point) - a0;
        point(i) = 0;
    }

    LOG(DEBUG, wxT("Coefficients:\n") << coefficients);

    return coefficients;
} // Coefficients

/* virtual */ double
LinearFunction::A0() const
{
    assert(mpProblem != NULL);

    const VariableNames& variables = mpProblem->Variables();
    ColumnVector point(variables.GetCount());

    point = 0;

    return Evaluate(point);
}// A0

//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////
LinearFunction::LinearFunction(const LinearFunction& from)
    : FunctionBase(from), NonlinearFunction(from)
{
}

/////////////////////////////// PRIVATE    ///////////////////////////////////
