#include <precomp.h>

#include "Logger.h"
#include "LinearFunction.h"
#include "LinearParser.h"  // class implemented

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

LinearParser::LinearParser()
{
}// LinearParser

LinearParser::~LinearParser()
{
}// ~LinearParser


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
LinearParser::CreateFunction(const wxString&    input,
                             VariableNames&     rVariables,
                             bool               resetVariables)
    const throw(ParseError)
{
    VariableNames mTempVariables;

    if (!resetVariables)    mTempVariables = rVariables;

    Tokenizer tokenizer(input, mTempVariables, resetVariables);
    auto_ptr<const FunctionTree> ap_function_tree;

    ap_function_tree = Expression(tokenizer);

    if (tokenizer.Peek() != Tokenizer::END)
    {
        LOG(DEBUG, wxT("Parse error"));
        throw ParseError(input, _("End of input expected"),
                tokenizer.Position());
    }
    LOG(DEBUG, wxT("Parsed successfully"));

    if (resetVariables)     rVariables = mTempVariables;

    return auto_ptr<FunctionBase>
        (new LinearFunction(ap_function_tree, input));
}// CreateFunction

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

std::auto_ptr<const FunctionTree>
LinearParser::Expression(Tokenizer& rTokenizer) const
{
    LOG(DEBUG, wxT("Entering Expression"));

    auto_ptr<const FunctionTree> ap_left;

    ap_left = Product(rTokenizer, true);

    Tokenizer::TokenType token = rTokenizer.Peek();

    while ((token == Tokenizer::PLUS) || (token == Tokenizer::MINUS))
    {
        auto_ptr<const FunctionTree> ap_right;

        rTokenizer.Consume();
        ap_right = Product(rTokenizer);

        if (token == Tokenizer::PLUS)
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorAdd, ap_left, ap_right));
        }
        else
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorSubtract,
                        ap_left, ap_right));
        }

        token = rTokenizer.Peek();
    }

    LOG(DEBUG, wxT("Leaving Expression"));
    return ap_left;
}// Expression

std::auto_ptr<const FunctionTree>
LinearParser::Product(Tokenizer& rTokenizer, bool first) const
{
    LOG(DEBUG, wxT("Entering Product"));

    Tokenizer::TokenType token;
    bool has_variable = false;
    bool negation = false;
    auto_ptr<const FunctionTree> ap_function;

    token = rTokenizer.Peek();

    if (token == Tokenizer::MINUS)
    {
        rTokenizer.Consume();
        negation = true;
        token = rTokenizer.Peek();
    }

    while ((token == Tokenizer::NUMBER) || (token == Tokenizer::VARIABLE))
    {
        LOG(DEBUG, wxT("Token type: ") << token);
        if (token == Tokenizer::VARIABLE)
        {
            if  (has_variable)
            {
                // TODO: lepszy komunikat
                throw ParseError(rTokenizer.Input(), _("Nonlinear element"),
                        rTokenizer.Position());
            }
            else                has_variable = true;
        }

        if (ap_function.get() == NULL)
        {
            ap_function = rTokenizer.Consume();
            if (negation)
            {
                ap_function = auto_ptr<const FunctionTree>
                    (new FunctionTree(Parser::OperatorNegate, ap_function));
                negation = false;       // w sumie zbdne
            }
        }
        else
        {
            ap_function = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorMultiply, ap_function,
                        auto_ptr<const FunctionTree>(rTokenizer.Consume())));
        }

        if (rTokenizer.Peek() != Tokenizer::MULTIPLY)
        {
            LOG(DEBUG, wxT("Leaving Product"));
            return ap_function;     // EXIT
        }
        else    rTokenizer.Consume();

        token = rTokenizer.Peek();
    }

    throw ParseError(rTokenizer.Input(), _("Number or variable expected"),
            rTokenizer.Position());

    // Tu nigdy nie dojdzie
    LOG(DEBUG, wxT("Leaving Product"));
    return ap_function;
}// Product
