#include <precomp.h>

//#include <wx/intl.h>

#include "LinearParser.h"

#include "LinearProblem.h"  // class implemented

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//LinearProblem::LinearProblem(const wxString& function)
//    : ProblemBase(), mConstraints()
//{
//    ParseFunction(function);
//}// LinearProblem

LinearProblem::~LinearProblem()
{
}// ~LinearProblem


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ auto_ptr<ProblemBase>
LinearProblem::Clone() const
{
    return auto_ptr<ProblemBase>(new LinearProblem(*this));
}// Clone

/* virtual */ wxString
LinearProblem::ToString() const
{
    Constraint::Container::const_iterator i;
    wxString string(Name());

    string << wxT("\n") << _("Function:") << wxT("\n");
    string << ((mcpFunction.Get() != NULL) ?
            mcpFunction->ToString() : wxString(_("None"))) << wxT("\n");
    string << _("Constraints:") << wxT("\n");

    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
    {
        string << i->ToString() << wxT("\n");
    }

    return string;
}// ToString

//============================= ACCESS     ===================================

//void
//LinearProblem::ParseFunction(const wxString& input)
//{
//    LinearParser parser;
//
//    mcpVariables.Reset(new VariableNames());
//
//    auto_ptr<FunctionBase> ap_function;
//    ap_function = parser.CreateFunction(input, *mcpVariables, true);
//    ap_function->SetProblem(this);
//
//    mcpFunction = auto_ptr<const FunctionBase>(ap_function);
//}// ParseFunction

void
LinearProblem::AddConstraint(const Constraint& constraint)
{
    mConstraints.push_back(constraint);
} // AddConstraint

/* virtual */ const Constraint::Container&
LinearProblem::Constraints() const
{
    return mConstraints;
}// Constraints

//list<const Constraint*>&
//LinearProblem::rConstraints()
//{
//    return mConstraints;
//}// rConstraints

const Constraint::Container
LinearProblem::ConstraintsWithAxes() const
{
    assert(mConstraints.size() > 0);

    Constraint::Container copy(mConstraints);
    VariableNames variables = Variables();
    LinearParser parser;
    auto_ptr<FunctionBase> ap_constraint_function;

    for (size_t i = 1; i <= variables.GetCount(); ++i)
    {
        ap_constraint_function = auto_ptr<FunctionBase>(
                parser.CreateFunction(variables.Item(i - 1),
                                      variables,
                                      false));
        ap_constraint_function->SetProblem(this);
        copy.push_back(Constraint(auto_ptr<const FunctionBase>(
                                            ap_constraint_function),
                                  Constraint::GREATER_EQUAL_THAN));
    }

    return copy;
}// ConstraintsWithAxes

int
LinearProblem::ConstraintCount() const
{
    return mConstraints.size();
}// ConstraintCount

/* virtual */ RowVector
LinearProblem::Coefficients() const
{
    return dynamic_cast<const LinearFunction&>(*mcpFunction).Coefficients();
}// Coefficients

/* virtual */ const double
LinearProblem::A0() const
{
    return dynamic_cast<const LinearFunction&>(*mcpFunction).A0();
}

/* virtual */ wxString
LinearProblem::Name() const
{
    return wxGetTranslation(mName);
}// Name

/* static */ ProblemIdType
LinearProblem::ClassId()
{
    const static ProblemIdType LinearProblemId = GenerateId();

    return LinearProblemId;
}// ClassId

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

LinearProblem::LinearProblem(bool doRegister)
    : ProblemBase(doRegister), mConstraints()
{
}// LinearProblem

LinearProblem::LinearProblem(const LinearProblem& from)
    : ProblemBase(from), mConstraints(from.mConstraints)
{
    Constraint::Container::iterator i;
    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
        *i = i->DeepCopy();

    Constraint::ProblemSetter set_problem(this);
    for_each(mConstraints.begin(), mConstraints.end(), set_problem);
}// LinearProblem


/* virtual */ ProblemIdType
LinearProblem::Id() const
{
    return LinearProblem::ClassId();
}// MyId

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ const wxString
    LinearProblem::mName(_("Linear Programming"));

LinearProblem LinearProblem::mLinearProblem(true);
