/**  Zadanie programowania liniowego.
 *
 * @todo RemoveConstraint
 * @todo Edit/ChangeConstraint
 */

#ifndef _LinearProblem_h
#define _LinearProblem_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
//#include <newmat/newmat.h>

// LOCAL INCLUDES
//
#include "Constraint.h"
#include "ProblemBase.h"
#include "LinearFunction.h"

// FORWARD REFERENCES
//


class LinearProblem
    : public ProblemBase
{
public:
    friend class LinearProblemPanel;

// LIFECYCLE

//    /** Konstruktor.
//     *
//     * @param function Cig znakw reprezentujcy wzr funkcji.
//     * @throws ParseError Bd parsowania.
//     * @deprecated Patrz ProblemBase::SetFunction.
//     */
//    LinearProblem(const wxString& function);


    /** Destruktor.
     */
    virtual ~LinearProblem(void);


// OPERATORS
// OPERATIONS

    /** Tworzy kopi problemu.
     */
    virtual std::auto_ptr<ProblemBase>  Clone() const;

    /** Tworzy opis tekstowy problemu.
     */
    virtual wxString                    ToString() const;

// ACCESS

//    /** Ustawia funkcj celu na podstawie reprezentacji tekstowej.
//     *
//     * @deprecated Naley uywa SetFunction.
//     */
//    void        ParseFunction(const wxString& input);

    /** Dodaje ograniczenie.
     */
    void        AddConstraint(const Constraint& Constraint);

    /** Zwraca na list ogranicze.
     */
    virtual const Constraint::Container& Constraints() const;

    const Constraint::Container  ConstraintsWithAxes() const;

//    /** Zwraca referencj na list ogranicze.
//     */ // TODO: moe by niepotrzebne
//    list<const Constraint*>&            rConstraints();

    /** Liczba ogranicze.
     */
    virtual int                         ConstraintCount() const;

    /** Wzpczynniki przy poszczeglnych zmiennych funkcji celu.
     */
    virtual RowVector                   Coefficients() const;

    /** Wyraz wolny funkcji celu.
     */
    virtual const double                A0() const;

    /** Zwraca nazw.
     */
    virtual wxString                    Name() const;

    /** Zwraca ID problemu.
     */
    virtual ProblemIdType               Id() const;

    /** Zwraca ID klasy problemu.
     */
    static ProblemIdType                ClassId();

// INQUIRY

protected:

    /** Konstruktor domylny.
     *
     * @param doRegister Czy zarejestrowac ten problem jako prototyp.
     */
    LinearProblem(bool doRegister = false);


    /** Konstruktor kopiujcy.
     *
     * UWAGA! Nie kopiuje funkcji ani ogranicze!
     *
     * @see ProblemBase::ProblemBase
     */
    LinearProblem(const LinearProblem& from);


private:
    Constraint::Container   mConstraints;   ///< Ograniczenia

    static const wxString   mName;          ///< Nazwa
    static LinearProblem    mLinearProblem; ///< Prototyp
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _LinearProblem_h_
