#include <precomp.h>

#include "Logger.h"
#include "LogarithmicInnerPenalty.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

LogarithmicInnerPenalty::LogarithmicInnerPenalty()
    : InnerPenalty()
{
}// LogarithmicInnerPenalty

LogarithmicInnerPenalty::LogarithmicInnerPenalty(CountedPtr<const FunctionBase> cpFunction)
    : InnerPenalty(cpFunction)
{
}// LogarithmicInnerPenalty

LogarithmicInnerPenalty::~LogarithmicInnerPenalty()
{
}// ~LogarithmicInnerPenalty


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
LogarithmicInnerPenalty::Clone() const
{
    return auto_ptr<FunctionBase>(new LogarithmicInnerPenalty(*this));
}

/* virtual */ double
LogarithmicInnerPenalty::PenaltyValue(double arg) const
{
    assert(arg < 0);

    double result = -1 * log(-1 * arg);

    LOG(DEBUG, wxString::Format(wxT("-ln(%g) = %g"),-1 * arg,result));

    return result;
}

//============================= ACCESS     ===================================

/* virtual */ wxString
LogarithmicInnerPenalty::ToString(bool full) const
{
    wxString result(InnerFunction().ToString());

    if (full)
    {
        result << wxT("\n")
          << _("Penalty function: inner, logarithmic,")
          << wxT("\n")
          << _("based on following constraints:")
          << wxT("\n");

        result << ConstraintsToString();

        result << InnerPenalty::ToString(full);

        result << wxT("\n");
    }

    return result;
}// ToString

/* virtual *//* FunctionIdType
LogarithmicInnerPenalty::Id() const
{
    return LogarithmicInnerPenalty::ClassId();
}// Id
*/
/* static *//* FunctionIdType
LogarithmicInnerPenalty::ClassId()
{
    const static FunctionIdType LogarithmicInnerPenaltyId = GenerateId();

    return LogarithmicInnerPenaltyId;
}// ClassId
*/
//============================= INQUIRY    ===================================

/* virtual */ int
LogarithmicInnerPenalty::FunctionId() const
{
    return 2;   // TODO: Jejku, co to?
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

LogarithmicInnerPenalty::LogarithmicInnerPenalty(
        const LogarithmicInnerPenalty&  from)
    : InnerPenalty(from)
{
}// LogarithmicInnerPenalty

/////////////////////////////// PRIVATE    ///////////////////////////////////
