/** Logarytmiczna funkcja bariery.
 *
 * @see InnerPenalty, HyperbolicInnerPenalty
 */

#ifndef _LogarithmicInnerPenalty_h
#define _LogarithmicInnerPenalty_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Error.h"
#include "InnerPenalty.h"

// FORWARD REFERENCES
//


class LogarithmicInnerPenalty
    : public InnerPenalty
{
public:
// LIFECYCLE

    /** Konstruktor domyslny.
     */
    LogarithmicInnerPenalty();

    /** Konstruktor.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj, do ktrej dodana
     * zostanie warto kary.
     */
    LogarithmicInnerPenalty(CountedPtr<const FunctionBase> cpFunction);

    /** Destruktor.
     */
    ~LogarithmicInnerPenalty(void);


// OPERATORS
    virtual std::auto_ptr<FunctionBase> Clone() const;

    /** Warto kary.
     *
     * Funkcja zwraca warto kary w zalenoci od odlegoci od ograniczenia wg
     * wzoru: \f$ -\ln(-g(x)) \f$.
     *
     * @param arg przekroczenie ograniczenia.
     */
    virtual double                  PenaltyValue(double arg) const;

// OPERATIONS

// ACCESS

    virtual wxString                ToString(bool full = true) const;
/*
    static FunctionIdType           ClassId();

    virtual FunctionIdType          Id() const;
*/

// INQUIRY

    virtual int                     FunctionId() const;

protected:
    LogarithmicInnerPenalty(const LogarithmicInnerPenalty& from);

private:

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _LogarithmicInnerPenalty_h
