#include <precomp.h>

#include "Logger.h"  // class implemented

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Logger::Logger(wxOutputStream&   targetOutputStream,
                   const wxString&   unitName,
                   const LevelType   displayLevel)
    : wxTextOutputStream(targetOutputStream),
      mUnitName(unitName),
      mDisplayLevel(displayLevel),
      mMessageLevel(displayLevel),
      mNewLine(true)
{
    mCurrentPrefix = wxString();
    mCurrentPrefix << mUnitName << wxT(" : ");
    mCurrentPrefix << Logger::LevelName(mMessageLevel) << wxT(" : ");

}

Logger::~Logger()
{
}// ~wxLogger

/*static*/ wxFFileOutputStream&
Logger::CommonFfos()
{
    // TODO:zrobi gdzie sta (w pliku .cc, eby nie trzeba byo caoci
    // przekompilowywa).
    static wxFFileOutputStream file_output_os(wxT("log.txt"));

    return file_output_os;
}

/*static*/ wxFFileOutputStream&
Logger::CommonStderrOs()
{
    static wxFFileOutputStream file_output_os(stderr);

    return file_output_os;
}

//============================= OPERATORS ====================================
// Podstawowe operatory s zdefiniowane w klasie wxTextOutputStream.
//============================= OPERATIONS ===================================

void
Logger::WriteString(const wxString& string)
{
    if (mMessageLevel >= mDisplayLevel)
    {
        int         index;
        wxString    in(string);

        do
        {
            if (mNewLine)
            {
                wxTextOutputStream::WriteString(mCurrentPrefix);
                mNewLine = false;
            }

            index = in.Find(_T('\n'));

            wxTextOutputStream::WriteString
                (in.Left((index != -1) ? (index + 1) : in.Length()));

            if (index == -1) in.Empty();
            else
            {
                mNewLine = true;
                in = in.Mid(index + 1);
            }

        } while (!in.IsEmpty());
    }
}

//============================= ACCESS     ===================================

void
Logger::MessageLevel(const LevelType messageLevel)
{
    mMessageLevel = messageLevel;
    mCurrentPrefix.Empty();
    mCurrentPrefix << mUnitName << _T(" - ");
    mCurrentPrefix << Logger::LevelName(mMessageLevel) << _T(" : ");
}

void
Logger::DisplayLevel(const LevelType displayLevel)
{
    mDisplayLevel = displayLevel;
}

//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/*static*/ const wxString&
Logger::LevelName(LevelType level)
{
    static wxString level_names[] =
    {
        wxT("DEBUG"),
        wxT("INFO"),
        wxT("WARNING"),
        wxT("ERROR"),
        wxT("FATAL")
    };

    return level_names[level];
}

/*static*//* wxTextCtrlOutputStream&
Logger::CommonTCOS()
{
    static int count;

    static wxFrame* frame = new wxFrame(NULL,-1,wxString(wxT("Logger output")),
                                        wxPoint(50,50));
    static wxTextCtrl* tc = new wxTextCtrl(frame,-1,wxString(wxT("")),
                                           wxDefaultPosition,wxSize(600,400),
                                           wxTE_MULTILINE | wxTE_READONLY);
    static wxTextCtrlOutputStream tcos(tc);

    ++count;

    if (count < 2)
    {
        frame->SetAutoLayout(true);
        frame->SetClientSize(tc->GetSize());
        frame->Show(true);
    }

    return tcos;
}*/


