#include <precomp.h>

#include "Logger.h"
#include "Method.h"  // class implemented

LOGGER(INFO);

using namespace std;


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


Method::~Method()
{
}// ~Method


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


std::auto_ptr<Result>
Method::StartOptimize(const ProblemBase& problem) const throw(OptimizationError)
{
    Prestart();
    return this->Optimize(problem);
}// StartOptimize

void
Method::Abort() const
{
    mAbort = true;
}// Abort

void
Method::Prestart() const
{
    mPreviousTime = wxGetLocalTimeMillis();
    mAbort = false;
}// Prestart

/* static */ void
Method::Init()
{
    vector<Method *>& methods = Method::rMethods();

    // Przetumacz nazwy (taki hak, bo zapomnielimy, e s inicjowane
    // statycznie, a wxLocale aduje si dopiero pniej)
    vector<Method *>::iterator i;
    for (i = methods.begin(); i != methods.end(); ++i)
    {
        (*i)->rName() = wxGetTranslation((*i)->Name());
    }

    // ...i posortuj
    sort(methods.begin(), methods.end(), NameLesser);
}// Init


//============================= ACCESS     ===================================

// TODO: usun, jak nie bdzie potrzebna
/* virtual */ wxString
Method::ToString(bool standalone) const
{
    wxString string(Name());

    string << wxT("\nTODO\n");

    return string;
}// ToString

/* static */ const vector<Method*>&
Method::Methods()
{
    return Method::rMethods();
}// Methods

/* static */ Method*
Method::GetById(MethodIdType id)
{
    vector<Method*>::const_iterator position;
    const vector<Method*>& methods = Method::Methods();

    position = find_if(methods.begin(), methods.end(),
            bind2nd(ptr_fun(Method::CompareId), id));

    assert(position  != methods.end());
    assert(*position != NULL);

    return *position;
}// GetById

/* static */ MethodIdType
Method::ClassId()
{
    return ABSTRACT_METHOD_ID;
}// ClassId

//============================= INQUIRY    ===================================
/* virtual */ StopConditions
Method::AllowedStopConditions() const
{
    StopConditions conditions;

    return conditions;
}
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
Method::NotifyYield() const throw(Error)
{
    LOG(DEBUG, wxT("NotifyYield"));
    NotifyObservers();

    if (mAbort) throw Error(_("Aborted by user"));
}// NotifyYield

Method::Method()
{
    // Sortowania nie mona zrobi tutaj, gdy w konstruktorach nie mona
    // wywoywa metod wirtualnych (a tak jest Name()).

    Method::rMethods().push_back(this);
}// Method

/* static */ MethodIdType
Method::GenerateId()
{
    // Za kadym razem zwraca wiksz liczb, zaczynajc od ABSTRACT_METHOD_ID +
    // 1
    static MethodIdType method_id = ABSTRACT_METHOD_ID;

    return ++method_id;
}// GenerateId

/////////////////////////////// PRIVATE    ///////////////////////////////////



/* static */ vector<Method*>&
Method::rMethods()
{
    static vector<Method*> methods;

    return methods;
}// rMethods

/* static */ bool
Method::CompareName(const Method* pMethod, const wxString *pName)
{
    LOG(DEBUG, wxT("Name: ") << *pName);
    LOG(DEBUG, wxT("Method Name: ") << pMethod->Name());

    return (pMethod->Name() == *pName);
}// CompareName

/* static */ bool
Method::CompareId(const Method* pMethod, MethodIdType id)
{
    return (pMethod->Id() == id);
}// CompareId

/* static */ bool
Method::NameLesser(const Method* method1, const Method* method2)
{
    return (method1->Name() < method2->Name());
}// CompareNames
