#include <precomp.h>

#include "Logger.h"
#include "MethodWithLineSearch.h"  // class implemented
#include "text-util.h"
#include "DirectionalUnivariateAdapter.h"

LOGGER(WARNING);

using namespace std;


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

MethodWithLineSearch::~MethodWithLineSearch()
{
}// ~MethodWithLineSearch


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
MethodWithLineSearch::AttachObserver(Observer* pObserver) const
{
    assert(mcpLineSearch.Get() != NULL);

    mcpLineSearch->AttachObserver(pObserver);
    Observable::AttachObserver(pObserver);
}// AttachObserver

/* virtual */ void
MethodWithLineSearch::DetachObserver(Observer* pObserver) const
{
    assert(mcpLineSearch.Get() != NULL);

    mcpLineSearch->DetachObserver(pObserver);
    Observable::DetachObserver(pObserver);
}// DetachObserver

/* virtual */ void
MethodWithLineSearch::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
MethodWithLineSearch::ResetStartingConditions() const
{
    LOG(DEBUG, wxT("mStartingPoint") << mStartingPoint);
    LOG(DEBUG, wxT("mInitialStartingPoint") << mInitialStartingPoint);
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================
/* virtual */ wxString
MethodWithLineSearch::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << wxT('\n');
    description << LineSearchDescription(mcpLineSearch->ToString(false));
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

ColumnVector
MethodWithLineSearch::StepInDirection(const FunctionBase&   function,
                                      const ColumnVector&   point,
                                      const ColumnVector&   direction,
                                      double* const         pStepLength)
                                const throw (Error)
{
    double step_length = 0;
    ColumnVector  normalised_direction = direction / direction.norm_Frobenius();

    try
    {
        step_length = mcpLineSearch->FindOptimalStep
          (DirectionalUnivariateAdapter(function, point, normalised_direction));
    }
    catch (const Error& error)
    {
        wxString    description(_("Error in line search:\n"));

        description << error.What();

        throw Error(description);
    }

    if (pStepLength != NULL)
    {
        *pStepLength = step_length / direction.norm_Frobenius();
    }

    return point + step_length * normalised_direction;
}

MethodWithLineSearch::MethodWithLineSearch()
    : Method(),
      mStartingPoint(0),
      mInitialStartingPoint(0),
      mcpStopCondition()
{
}// MethodWithLineSearch

MethodWithLineSearch::MethodWithLineSearch(const MethodWithLineSearch& from)
    : Method(from),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mcpStopCondition(from.mcpStopCondition)
{
    if (from.mcpLineSearch.Get() != NULL)
    {
        mcpLineSearch = CountedPtr<const LineSearch>
            (aptr_dynamic_cast<const LineSearch>(from.mcpLineSearch->Clone()));
    }
}

/////////////////////////////// PRIVATE    ///////////////////////////////////
