/** Warunek stopu oparty na najlepszym i najgorszym
 * punkcie w sympleksie.
 *
 * Porwnuje z epsilon odlego,
 * wartoci funkcji lub nachylenie funkcji pomidzy
 * najgorszym i najlepszym punktem w sympleksie.
 */

#ifndef _NMBestWorstCondition_h
#define _NMBestWorstCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "NMStopCondition.h"

// FORWARD REFERENCES
//


class NMBestWorstCondition
    : public NMStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    NMBestWorstCondition(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NMBestWorstCondition(const NMBestWorstCondition& from);

    /** Konstruktor.
     *
     * Pozwala ustawi jedn z metod porwnania najgorszego i najlepszego
     * punktu: porwnanie odlegoci (obiekt klasy DistanceCondition),
     * porwnanie wartoci funkcji (ValueCondition), porwnanie nachylenia
     * funkcji (SlopeCondition).
     *
     * @param cpComparison Obiekt dokonujcy porwnania.
     */
    NMBestWorstCondition(CountedPtr<StandardStopCondition> cpComparison);

    /** Destruktor.
     */
    ~NMBestWorstCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see NMStopCondition::IsMet
     */
    virtual bool        IsMet(const PointWithValue&     bestPoint,
                              const PointWithValue&     worstPoint,
                              const PointWithValue&     previousWorstPoint,
                              const PointWithValue&     worstReplacement,
                              double                    circumference) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NMBestWorstCondition_h
