/** Warunek stopu oparty na najgorszych punktach
 * w sympleksie w biecej i poprzedniej iteracji.
 *
 * Porwnuje z epsilon odlego,
 * wartoci funkcji lub nachylenie funkcji pomidzy
 * najgorszymi punktami w sympleksie w biecej
 * i poprzedniej iteracji.
 */

#ifndef _NMPreviousWorstCondition_h
#define _NMPreviousWorstCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "NMStopCondition.h"

// FORWARD REFERENCES
//


class NMPreviousWorstCondition
    : public NMStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    NMPreviousWorstCondition(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NMPreviousWorstCondition(const NMPreviousWorstCondition& from);

    /** Konstruktor.
     *
     * Pozwala ustawi jedn z metod porwnania punktw:
     * porwnanie odlegoci (obiekt klasy DistanceCondition),
     * porwnanie wartoci funkcji (ValueCondition), porwnanie nachylenia
     * funkcji (SlopeCondition).
     *
     * @param cpComparison Obiekt dokonujcy porwnania.
     */
    NMPreviousWorstCondition(CountedPtr<StandardStopCondition> cpComparison);

    /** Destruktor.
     */
    ~NMPreviousWorstCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see NMStopCondition::IsMet
     */
    virtual bool        IsMet(const PointWithValue&     bestPoint,
                              const PointWithValue&     worstPoint,
                              const PointWithValue&     previousWorstPoint,
                              const PointWithValue&     worstReplacement,
                              double                    circumference) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NMPreviousWorstCondition_h
