/** Warunek stopu dla metody Neldera-Meada.
 */

#ifndef _NMStopCondition_h
#define _NMStopCondition_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "FunctionBase.h"
#include "PointWithValue.h"
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class NMStopCondition
    : public StopConditionBase
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    NMStopCondition(void);

    /** Konstruktor.
     *
     * @param cpComparison Obiekt dokonujcy porwnania podawanych
     * w podklasach wartoci z epsilon.
     */
    NMStopCondition(CountedPtr<StandardStopCondition> cpComparison);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NMStopCondition(const NMStopCondition& from);


    /** Destruktor.
     */
    virtual ~NMStopCondition(void);


// OPERATORS

// OPERATIONS

// ACCESS

    /** Zwraca warto minimaln <em>epsilon</em>.
     */
    virtual DoubleParameter         Epsilon() const;

    /** Zwraca referencj na <em>epsilon</em>.
     */
    virtual DoubleParameter&        rEpsilon();

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @param bestPoint Najlepszy punkt w sympleksie.
     * @param worstPoint Najgorszy punkt w sympleksie.
     * @param previousWorstPoint Najgorszy punkt w sympleksie
     * w poprzedniej iteracji.
     * @param worstReplacement Punkt, ktry zastpi najgorszy
     * punkt w biecej iteracji.
     * @param improvementOfWorst Zmiana wartoci funkcji w najgorszym
     * punkcie.
     * @param circumference Obwd sympleksu.
     *
     * @return Czy warunek stopu jest speniony.
     */
    virtual bool        IsMet(const PointWithValue&     bestPoint,
                              const PointWithValue&     worstPoint,
                              const PointWithValue&     previousWorstPoint,
                              const PointWithValue&     worstReplacement,
                              double                    circumference)
                                                                    const = 0;

// PUBLIC MEMBERS

protected:
    /** Obiekt dokonujcy porwnania podanych punktw lub wartoci
     * z epsilon.
     */
    CountedPtr<StandardStopCondition>    mcpComparison;
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NMStopCondition_h
