/** Warunek stopu oparty na najgorszym punkcie w sympleksie
 * i punkcie, ktry zastpi go w biecej iteracji.
 *
 * Porwnuje z epsilon odlego,
 * wartoci funkcji lub nachylenie funkcji pomidzy
 * najgorszym punktem w sympleksie a punktem, ktry
 * zastpi go w biecej iteracji.
 */

#ifndef _NMWorstReplacementCondition_h
#define _NMWorstReplacementCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "NMStopCondition.h"

// FORWARD REFERENCES
//


class NMWorstReplacementCondition
    : public NMStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    NMWorstReplacementCondition(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NMWorstReplacementCondition(const NMWorstReplacementCondition& from);

    /** Konstruktor.
     *
     * Pozwala ustawi jedn z metod porwnania punktw:
     * porwnanie odlegoci (obiekt klasy DistanceCondition),
     * porwnanie wartoci funkcji (ValueCondition), porwnanie nachylenia
     * funkcji (SlopeCondition).
     *
     * @param cpComparison Obiekt dokonujcy porwnania.
     */
    NMWorstReplacementCondition(CountedPtr<StandardStopCondition> cpComparison);

    /** Destruktor.
     */
    ~NMWorstReplacementCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see NMStopCondition::IsMet
     */
    virtual bool        IsMet(const PointWithValue&     bestPoint,
                              const PointWithValue&     worstPoint,
                              const PointWithValue&     previousWorstPoint,
                              const PointWithValue&     worstReplacement,
                              double                    circumference) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NMWorstReplacementCondition_h
