#include <precomp.h>

#include "NonlinearFunction.h"  // class implemented
#include "ProblemBase.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NonlinearFunction::NonlinearFunction()
    : FunctionBase(true),
      mapFunctionTree(NULL),
      mapOptimizedFunctionTree(NULL),
      mOptimized(false),
      mTextForm(wxT(""))
{
}// NonlinearFunction


NonlinearFunction::NonlinearFunction(
        std::auto_ptr<const FunctionTree>   apFunction,
        const wxString&                     textForm,
        bool                                differentiable)
    : FunctionBase(differentiable),
      mapFunctionTree(apFunction),
      mapOptimizedFunctionTree(NULL),
      mOptimized(false),
      mTextForm(textForm)
{
}// NonlinearFunction

NonlinearFunction::~NonlinearFunction()
{
}// ~NonlinearFunction


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
NonlinearFunction::Clone() const
{
    return auto_ptr<FunctionBase>(new NonlinearFunction(*this));
}

/* virtual */ void
NonlinearFunction::BeginFixed() const
{
    assert (mpProblem != NULL);

    mapOptimizedFunctionTree =
        mapFunctionTree->Optimize(mpProblem->Variables());
    mOptimized = true;
}// BeginFixed

/* virtual */ void
NonlinearFunction::EndFixed() const
{
    mOptimized = false;
    mapOptimizedFunctionTree.reset(NULL);
}// EndFixed

//============================= ACCESS     ===================================

/* virtual */ wxString
NonlinearFunction::ToString(bool full) const
{
    return mTextForm;
}// ToString

/* virtual */ VariableNames
NonlinearFunction::PresentVariables() const
{
    assert(mapFunctionTree.get() != NULL);

    return mapFunctionTree->PresentVariables();
}// PresentVariables

//============================= INQUIRY    ===================================

/* virtual */ bool
NonlinearFunction::ContainsUnknown(const VariableNames& variables) const
{
    assert(mapFunctionTree.get() != NULL);

    return mapFunctionTree->ContainsUnknown(variables);
}// ContainsUnknown


/////////////////////////////// PROTECTED  ///////////////////////////////////

NonlinearFunction::NonlinearFunction(const NonlinearFunction& from)
    : FunctionBase(from),
      mapFunctionTree(new FunctionTree(*from.mapFunctionTree)),
      mapOptimizedFunctionTree(NULL),
      mOptimized(false),
      mTextForm(from.mTextForm)
{
}// NonlinearFunction

/* virtual */ double
NonlinearFunction::DoEvaluateNoThrow(const ColumnVector&     point) const
    throw()
{
    // TODO: to mona nieco poprawi
    if (mOptimized)
    {
        return mapOptimizedFunctionTree->Evaluate(point, mpProblem->Variables());
    }
    else
    {
        return mapFunctionTree->Evaluate(point, mpProblem->Variables());
    }
}// DoEvaluateNoThrow

/////////////////////////////// PRIVATE    ///////////////////////////////////
