/**  Funkcja nieliniowa opisana wzorem.
 */

#ifndef _NonlinearFunction_h
#define _NonlinearFunction_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "FunctionBase.h"
#include "FunctionTree.h"
#include "VariableNames.h"

// FORWARD REFERENCES
//


class NonlinearFunction
    : public virtual FunctionBase
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    NonlinearFunction(void);


    /** Konstruktor.
     *
     * @param apFunction Wskanik na drzewo funkcji.
     * @param textForm Reprezentacja tekstowa funkcji (na potrzeby
     * wywietlania).
     * @param differentiable Czy funkcja jest rniczkowalna.
     */
    NonlinearFunction(std::auto_ptr<const FunctionTree> apFunction,
                      const wxString&                   textForm = wxT(""),
                      bool                              differentiable = true);

    /** Destruktor.
     */
    ~NonlinearFunction(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase> Clone() const;

    /** Rozpoczyna fragment kodu, w ktrym zmienne ani funkcja nie zmieniaj
     * si.
     *
     * Metoda powoduje znalezienie i zapamitanie indeksw zmiennych, dziki
     * czemu nie trzeba ich bdzie wyszukiwa przy kadym obliczeniu wartoci
     * funkcji.
     *
     * Do momentu wywoania EndFixed zmienne ani funkcja (poprzez inne
     * wskaniki) nie mog ulec zmianie.
     */
    virtual void            BeginFixed() const;

    virtual void            EndFixed() const;

// ACCESS

    virtual wxString            ToString(bool full = true) const;

    virtual VariableNames       PresentVariables() const;

// INQUIRY

    virtual bool                ContainsUnknown(const VariableNames& variables)
                                    const;

protected:
    NonlinearFunction(const NonlinearFunction& from);

    /** Oblicza warto funkcji w podanym punkcie.
     *
     * Wymaga, aby funkcja miaa przydzielone zadanie (patrz
     * FunctionBase::SetProblem()).
     *
     * @exception EvaluationError Gdy wynik nie jest liczb skoczon.
     */
    virtual double          DoEvaluateNoThrow(const ColumnVector& point) const
                                    throw();

    std::auto_ptr<const FunctionTree>   mapFunctionTree; ///< Drzewo funkcji.

    /// Zoptymalizowane drzewo funkcji.
    mutable std::auto_ptr<const FunctionTree>   mapOptimizedFunctionTree;

    /// Czy funkcja zostaa zoptymalizowana.
    mutable bool                                mOptimized;
private:
    const wxString                      mTextForm;       ///< Forma tekstowa
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NonlinearFunction_h_
