/** Interface obiektu obserwowanego.
 *
 * Razem z klas Observer tworz wzorzec projektowy obserwatora, a
 * przynajmniej co na jego ksztat.
 *
 * @see Observer
 */

#ifndef _Observable_h
#define _Observable_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Observer.h"

// FORWARD REFERENCES
//

class Observable
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    Observable(void);


    /** Destruktor.
     *
     * Odcza wszyskie obserwatory, co wie si z ustawieniem ich wewntrznych
     * wskakikw na obiekt obserwowany na <code>NULL</code>.
     */
    virtual ~Observable(void);


// OPERATORS

// OPERATIONS

    /** Przycza obserwator.
     *
     * Ustawia wewntrzny wskanik obserwatora na obiekt obserwowany.
     *
     * Obserwatory s przechowywane w kontenerze, ktry jest
     * <code>mutable</code>, wic mog by przyczane nawet do obiektu
     * <code>const</code>.
     */
    virtual void            AttachObserver(Observer* pObserver) const;

    /** Odcza obserwator.
     *
     * Ustawia wewntrzny wskanik obserwatora na obiekt obserwowany na
     * <code>NULL</code>.
     *
     * Obserwatory s przechowywane w kontenerze, ktry jest
     * <code>mutable</code>, wic mog by odczane nawet od obiektu
     * <code>const</code>.
     */
    virtual void            DetachObserver(Observer* pObserver) const;


// ACCESS
// INQUIRY

protected:
    /** Powiadamia obserwatory o zmianie stanu.
     *
     * Wywouje metod Observer::Update dla wszystkich zarejestrowanych
     * obserwatorw.
     */
    void                    NotifyObservers() const;

private:
    mutable std::vector<Observer*> mObservers;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Observable_h
