/** Bd optymalizacji.
 *
 * Klasa zawiera informacje o bdz optymalizacji:
 * - opis,
 * - czciowy wynik (przebieg procesu do momentu wystpienia bdu).
 */

#ifndef _OptimizationError_h
#define _OptimizationError_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "Error.h"
#include "Result.h"

// FORWARD REFERENCES
//


class OptimizationError
    : public Error
{
public:
// LIFECYCLE

//    /** Konstruktor domylny.
//     */
//    OptimizationError(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    OptimizationError(const OptimizationError& from);

    /** Konstruktor.
     *
     * @param description Opis/komunikat bdu.
     * @param apResult <code>auto_ptr</code> na (czciowy) wynik.
     */
    OptimizationError(const wxString&       description,
                      std::auto_ptr<Result> apResult = std::auto_ptr<Result>());

    /** Destruktor.
     */
    ~OptimizationError(void);


// OPERATORS

    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    OptimizationError&      operator=(const OptimizationError& from);

// OPERATIONS
// ACCESS

    /** Referencja na wynik.
     *
     * Wymaga, aby wyjtek mia wynik, inaczej wykonana zostanie dereferencja
     * wskanika <code>NULL</code>.
     *
     * @see HasResult
     */
    Result&                 rResult();

    /** Zwraca <code>CountedPtr</code> na wynik.
     */
    CountedPtr<Result>      GiveResult() const;

// INQUIRY

    /** Sprawdza, czy wyjtek ma wynik.
     */
    bool                    HasResult() const;

protected:
private:
    CountedPtr<Result>  mcpResult;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _OptimizationError_h
