#include <precomp.h>

#include "Logger.h"
#include "OuterPenalty.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

OuterPenalty::OuterPenalty()
    : PenaltyFunction()
{
}// OuterPenalty

OuterPenalty::OuterPenalty(CountedPtr<const FunctionBase> cpFunction)
    : PenaltyFunction(cpFunction)
{
    mInitialPenaltyParam      = DoubleParameter(1, 0, true);
    mPenaltyParamMultiplier   = DoubleParameter(2, 1, true);
    mTolerance                = DoubleParameter(0.0001, 0.0, false);
}// OuterPenalty

OuterPenalty::~OuterPenalty()
{
}// ~OuterPenalty


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
OuterPenalty::Clone() const
{
    return auto_ptr<FunctionBase>(new OuterPenalty(*this));
}

//============================= ACCESS     ===================================

/* virtual */ wxString
OuterPenalty::ToString(bool full) const
{
    wxString result(InnerFunction().ToString());

    if (full)
    {
        // TODO: zastanowi si nad tym sformuowaniem
        result << wxT("\n")
            << _("Penalty function: outer,")
            << wxT("\n")
            << _("based on following constraints:")
            << wxT("\n");

        result << ConstraintsToString();
    }

    return result;
}// ToString

/* virtual */ FunctionIdType
OuterPenalty::Id() const
{
    return OuterPenalty::ClassId();
}// Id

/* static */ FunctionIdType
OuterPenalty::ClassId()
{
    const static FunctionIdType OuterPenaltyId = GenerateId();

    return OuterPenaltyId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
OuterPenalty::IsValid(const ColumnVector& point) const
{
    Constraint::Container::const_iterator i;

    // Sprawd IsValid dla wszystkich ogranicze
    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
    {
        if (i->IsValid(point, true, mTolerance) == false)
        {
            return false;
        }
    }

    return true;
}// IsValid

/* virtual */ bool
OuterPenalty::IsValidStartingPoint(const ColumnVector& point) const
{
    // TODO: Czy nie da sprawdzania Isinf?. Funkcja kary szybko ronie, mona
    // by to sprawdzi.
    return  true;
}

/* virtual */ int
OuterPenalty::FunctionId() const
{
    return 0;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

OuterPenalty::OuterPenalty(const OuterPenalty& from)
    : PenaltyFunction(from)
{
}

// TODO: zrobi adnie
/* virtual */ double
OuterPenalty::DoEvaluate(const ColumnVector& point) const throw(EvaluationError)
{
    Constraint::Container::const_iterator i;
    const Constraint::Container& constraints = Constraints();
    double value;

    LOG(DEBUG,wxT("Point               : ") << point);
    LOG(DEBUG,wxT("penalty parameter   : ") << PenaltyParam());

    value = InnerFunction().Evaluate(point);

    //LOG(DEBUG, wxT("Inner Function value: ") << value);

    for (i = constraints.begin(); i != constraints.end(); ++i)
    {
        double constraint_value = i->Value(point);

        if (constraint_value > 0.0)
        {
            value += PenaltyParam() * (constraint_value * constraint_value);
        }
    }
    LOG(DEBUG, wxT("After adding constraints value: ") << value);

    return value;
}// DoEvaluate

/* virtual */ double
OuterPenalty::DoEvaluateNoThrow(const ColumnVector& point) const throw()
{
    Constraint::Container::const_iterator i;
    const Constraint::Container& constraints = Constraints();
    double value;

    LOG(DEBUG,wxT("Point               : ") << point);
    LOG(DEBUG,wxT("penalty parameter   : ") << PenaltyParam());

    value = InnerFunction().EvaluateNoThrow(point);

    //LOG(DEBUG, wxT("Inner Function value: ") << value);

    for (i = constraints.begin(); i != constraints.end(); ++i)
    {
        double constraint_value = i->Value(point);

        if (constraint_value > 0.0)
        {
            value += PenaltyParam() * (constraint_value * constraint_value);
        }
    }
    LOG(DEBUG, wxT("After adding constraints value: ") << value);

    return value;
}// DoEvaluateNoThrow

/////////////////////////////// PRIVATE    ///////////////////////////////////
