/** Zewntrzna funkcja kary.
 *
 * @see InnerPenalty, PenaltyFunction
 */

#ifndef _OuterPenalty_h
#define _OuterPenalty_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Error.h"
#include "PenaltyFunction.h"

// FORWARD REFERENCES
//


class OuterPenalty
    : public PenaltyFunction
{
public:
// LIFECYCLE

    /** Konstruktor domyslny.
     */
    OuterPenalty();

    /** Konstruktor.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj, do ktrej dodana
     * zostanie warto kary.
     */
    OuterPenalty(CountedPtr<const FunctionBase> cpFunction);


    /** Destruktor.
     */
    ~OuterPenalty(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase> Clone() const;

// ACCESS

    virtual wxString                ToString(bool full = true) const;

    static FunctionIdType           ClassId();

    /** Zwraca ID funkcji kary.
     */
    virtual FunctionIdType          Id() const;

// INQUIRY

    virtual bool    IsValid(const ColumnVector& point) const;

    virtual bool    IsValidStartingPoint(const ColumnVector& point) const;

    virtual int     FunctionId() const;

protected:

    OuterPenalty(const OuterPenalty& from);

    /** Zwraca warto funkcji celu powikszon o warto funkcji kary.
     *
     * @exception Error Gdy wynik lub wartoci porednie w trakcie oblicze
     * przestan by skoczone.
     */
    virtual double   DoEvaluate(const ColumnVector& point) const
                                            throw(EvaluationError);

    virtual double   DoEvaluateNoThrow(const ColumnVector& point) const
                                            throw();

private:

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _OuterPenalty_h
