#include <precomp.h>

#include "Logger.h"
#include "PenaltyFunction.h"  // class implemented

using namespace std;

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PenaltyFunction::PenaltyFunction()
    : FunctionBase(false),
      FunctionDecorator(),
      mOutOfBoundsReaction(MAX_VALUE),
      mConstraints(),
      mPenaltyParam(1)
{
}// PenaltyFunction

PenaltyFunction::PenaltyFunction(CountedPtr<const FunctionBase> cpFunction)
    : FunctionBase(cpFunction->IsDifferentiable()),
      FunctionDecorator(cpFunction),
      mOutOfBoundsReaction(MAX_VALUE),
      mConstraints(),
      mPenaltyParam(1)
{
}// PenaltyFunction

PenaltyFunction::~PenaltyFunction()
{
}// ~PenaltyFunction


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void
PenaltyFunction::ModifyPenalty()
{
    mPenaltyParam *= mPenaltyParamMultiplier;
}// IncreasePenalty

void
PenaltyFunction::ResetPenalty()
{
    mPenaltyParam = mInitialPenaltyParam;
    InnerFunction().ResetCounter();
}// ResetPenalty

//============================= ACCESS     ===================================

void
PenaltyFunction::AddConstraint(const Constraint& constraint)
{
    mConstraints.push_back(constraint);
}// AddConstraint

const Constraint::Container&
PenaltyFunction::Constraints() const
{
    return mConstraints;
}// Constraints

Constraint::Container&
PenaltyFunction::rConstraints()
{
    return mConstraints;
}// Constraints

PenaltyFunction::OutOfBoundsReactionType&
PenaltyFunction::rOutOfBoundsReaction()
{
    return mOutOfBoundsReaction;
};

PenaltyFunction::OutOfBoundsReactionType
PenaltyFunction::OutOfBoundsReaction() const
{
    return mOutOfBoundsReaction;
};

double
PenaltyFunction::PenaltyParam() const
{
    return mPenaltyParam;
}

double&
PenaltyFunction::rPenaltyParam() const
{
    return mPenaltyParam;
}

/* virtual */ unsigned
PenaltyFunction::EvaluationCount() const
{
    return InnerFunction().EvaluationCount();
    LOG(ERROR,InnerFunction().EvaluationCount());
}// EvaluationCount

wxString
PenaltyFunction::ConstraintsToString() const
{
    wxString result;

    Constraint::Container::const_iterator i;
    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
    {
        result << i->ToString() << wxT("\n");
    }
    return result;
}// ConstraintsToString

/* virtual */ void
PenaltyFunction::SetProblem(const ProblemBase* pProblem) const
{
    InnerFunction().SetProblem(pProblem);

    Constraint::ProblemSetter set_problem(pProblem);
    for_each(mConstraints.begin(), mConstraints.end(), set_problem);
//    transform(mConstraints.begin(), mConstraints.end(), mConstraints.begin(),
//            set_problem);

    mpProblem = pProblem;
}// SetProblem

/* virtual */ VariableNames
PenaltyFunction::PresentVariables() const
{
    return InnerFunction().PresentVariables();
}// PresentVariables

/* static */ FunctionIdType
PenaltyFunction::ClassId()
{
    const FunctionIdType AbstractPenaltyFunctionId = 0;

    return AbstractPenaltyFunctionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
PenaltyFunction::ContainsUnknown(const VariableNames& variables) const
{
    return InnerFunction().ContainsUnknown(variables);
}// ContainsUnknown

/////////////////////////////// PROTECTED  ///////////////////////////////////

PenaltyFunction::PenaltyFunction(const PenaltyFunction& from)
    : FunctionBase(from),
      FunctionDecorator(from),
      mInitialPenaltyParam(from.mInitialPenaltyParam),
      mPenaltyParamMultiplier(from.mPenaltyParamMultiplier),
      mTolerance(from.mTolerance),
      mOutOfBoundsReaction(from.mOutOfBoundsReaction),
      mConstraints(from.mConstraints),
      mPenaltyParam(from.mPenaltyParam)
{
    Constraint::Container::iterator i;
    for (i = mConstraints.begin(); i != mConstraints.end(); ++i)
        *i = i->DeepCopy();
}// PenaltyFunction

/* static */ FunctionIdType
PenaltyFunction::GenerateId()
{
    static FunctionIdType id = 1;

    return id++;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

