/** Klasa reprezentujca populacj osobnikw w iteracji ewolucyjnej
 * metody optymalizacji
 *
 * @see EvolutionaryMethod
 */

#ifndef _Population_h
#define _Population_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "FunctionBase.h"
#include "PointWithValue.h"

// FORWARD REFERENCES
//

#define NOT_CHOSEN -1

class Population
{
public:
// LIFECYCLE

    typedef std::vector<PointWithValue> ChromosomeContainer;
    typedef PointWithValue              ChromosomeType;

    /** Konstruktor tworzcy populacj o okrelonej wielkoci
     *
     * @param numberOfChromosomes liczba osobnikw
     */
    Population(const FunctionBase&  rFunction,
                     unsigned       numberOfChromosomes,
               const ColumnVector&  rangeMinBounds,
               const ColumnVector&  rangeMaxBounds);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Population(const Population& from);

    /** Destruktor.
     */
    ~Population(void);

// OPERATORS

    PointWithValue&         operator[](size_t   index);

// OPERATIONS

    void                    Generate(      size_t           numberOfChromosomes,
                                     const ColumnVector&    rangeMinBounds,
                                     const ColumnVector&    rangeMaxBounds);

    void                    CalculateGenerationFitness();

    void                    ReevaluateChromosome(size_t     index);

    void                    GatherInfo(bool findBest = true);

// ACCESS

    const PointWithValue&   Best() const;

    size_t                  BestIndex() const;

    size_t                  Size() const;

    size_t                  DesiredSize() const;

    void                    AddChromosome(const ColumnVector& newChromosome);

    void                    Erase(size_t index);

// INQUIRY

    ColumnVector            AverageChromosome() const;
    double                  AverageValue() const;

    double                  ChromosomeStandardDeviation() const;
    double                  ValueStandardDeviation() const;

protected:
private:

    void                    FindBest();
    void                    CalculateAverages();
    void                    CalculateDeviations();



    const FunctionBase&     mFunction;

    ChromosomeContainer     mChromosomes;
    size_t                  mDesiredSize;
    int                     mBestIndex;

    ColumnVector            mAverageChromosome;
    double                  mAverageValue;
    double                  mChromosomeStandardDeviation;
    double                  mValueStandardDeviation;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Population_h
