/**  Zadanie optymalizacji z funkcj kary.
 *
 * Ograniczenia w zadaniu tego typu s wymuszane za pomoc funkcji kary
 * (zewntrznej, bd wewntrznej bariery). Dziki temu mona stosowa metody
 * przewidziane dla optymalizacji bez ogranicze.
 *
 * Optymalizacja przebiega wieloetapowo: po kadym przebiegu algorytmu
 * sprawdzane jest, czy wynik spenia ograniczenia. Jeli nie, to zwikszany
 * jest wspczynnik kary i algorytm optymalizacji uruchamiany jest ponownie.
 * Wynikiem tego problemu jest lista punktw wskazanych przez metod
 * optymalizacji przy kolejnych iteracjach (w przeciwiestwie do kolejnych
 * krokw metody optymalizacji).
 */

#ifndef _ProblemWithPenalty_h
#define _ProblemWithPenalty_h


// SYSTEM INCLUDES
//
//#include <memory>

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "PenaltyFunction.h"
#include "UnconstrainedProblem.h"

// FORWARD REFERENCES
//


class ProblemWithPenalty
    : public UnconstrainedProblem
{
public:

    friend class    ProblemWithPenaltyPanel;
    friend class    ProblemWithPenaltyValidator;
    friend class    PwpAdvancedDialog;

// LIFECYCLE

    /** Destruktor.
     */
    ~ProblemWithPenalty(void);


// OPERATORS

// OPERATIONS
    /** Tworzy kopi obieku.
     */
    std::auto_ptr<ProblemBase>          Clone() const;

// ACCESS

    /** Referencja na wsk na funkcj kary.
     */
    //TODO: Usun
    PenaltyFunction*&           rpPenalty();

    /** Zwraca funkcj kary.
     */
    const PenaltyFunction&      Penalty() const;

    /** Zwraca oryginaln funkcj celu.
     *
     * Funkcja zwraca oryginaln funkcj celu bez uwzgldnienia
     * wartoci funkcji kary.
     *
     * @return Referencja na oryginaln funkcj celu.
     */
    virtual const FunctionBase& BaseFunction() const;

    /** Zwraca nazw problemu.
     */
    virtual wxString            Name() const;

    /** Zwraca ID klasy problemu.
     */
    static ProblemIdType        ClassId();

    /** Zwraca ID.
     */
    virtual ProblemIdType       Id() const;

    virtual void            SetFunction(
                                auto_ptr<FunctionBase>  cpFunction,
                                auto_ptr<VariableNames> cpVariables);
// INQUIRY

    bool                        ReuseStartingPoint() const;

protected:

    /** Konstruktor.
     *
     * @param doRegister Czy zarejestrowac ten problem jako prototyp.
     */
    ProblemWithPenalty(bool doRegister = false);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    ProblemWithPenalty(const ProblemWithPenalty& from);

private:

    /** Rozwizuje zadanie optymalizacji.
     */
    virtual std::auto_ptr<Result> Solve() const throw(OptimizationError);

    /** Znajduje funkcj kary w funkcji celu i ustawia na ni wskanik.
     */
    void                        SetPenaltyFunction();

    static const wxString       mName;                  ///< Nazwa
    static ProblemWithPenalty   mProblemWithPenalty;    ///< Prototyp

    DoubleParameter             mEpsilon;   ///< dopuszczalny bd
    bool                        mReuseStartingPoint;

    mutable PenaltyFunction*    mpPenaltyFunction; ///< Wsk. na funkcj kary.

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ProblemWithPenalty_h_
