/** Opisuje prostoktny obszar.
 *
 * Jednostki wsprzdnych s liczbami zmiennoprzecinkowymi.
 */

#ifndef _Region_h
#define _Region_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//

// FORWARD REFERENCES
//


class Region
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    Region(void);

    /** Konstruktor.
     */
    Region(double x1, double y1, double x2, double y2);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Region(const Region& from);


    /** Destruktor.
     */
    ~Region(void);


// OPERATORS

    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    Region&        operator=(const Region& from);

    /** Operator przypisania sumy.
     *
     * Powiksza obszar tak, aby obejmowa on take <code>from</code>.
     *
     * @param from Drugi operand.
     * @return Obiekt, na ktrym operuje.
     */
    Region&         operator+=(const Region& from);

    /** Operator sumy.
     *
     * @param from Drugi operand.
     * @return Najmniejszy obszar, ktry obejmuje oba operandy.
     */
    Region          operator+(const Region& from) const;

// OPERATIONS

    /** Poszerza obszar, aby obejmowa punkt o danych wsprzdnych.
     */
    void            Extend(double x, double y);

// ACCESS

    /// Zwraca skadow x lewego-grnego rogu.
    inline double   X1() const;

    /// Zwraca skadow y lewego-grnego rogu.
    inline double   Y1() const;

    /// Zwraca skadow x prawego-dolnego rogu.
    inline double   X2() const;

    /// Zwraca skadow y prawego-dolnego rogu.
    inline double   Y2() const;

// INQUIRY

    /** Sprawdza, czy obiekt jest poprawny (zdefiniowany).
     */
    inline bool     IsValid() const;

    /** Sprawdza, czy obszar jest mniejszy od <code>other_region<code>.
     */
    inline bool     IsSmaller(const Region& other_region) const;

    /** Sprawdza, czy obszar znajduje si wewntrz <code>other_region<code>.
     *
     * Obejmuje rwnie krawdzie, czyli dla siebie samego zwraca
     * <code>true</code>.
     */
    inline bool     IsInside(const Region& other_region) const;

protected:
private:
    double  mX1;
    double  mX2;
    double  mY1;
    double  mY2;

    bool    mValid;
};

// INLINE METHODS
//
inline double
Region::X1() const
{
    return mX1;
}// X1

inline double
Region::X2() const
{
    return mX2;
}// X2

inline double
Region::Y1() const
{
    return mY1;
}// Y1

inline double
Region::Y2() const
{
    return mY2;
}// Y2

inline bool
Region::IsValid() const
{
    return mValid;
}// IsValid

inline bool
Region::IsSmaller(const Region& other_region) const
{
    return (mX2 - mX1 < other_region.mX2 - other_region.mX1)
          && (mY2 - mY1 < other_region.mY2 - other_region.mY1);
}// IsSmaller

inline bool
Region::IsInside(const Region& other_region) const
{
    return (mX1 >= other_region.mX1) && (mX2 <= other_region.mX2)
        && (mY1 >= other_region.mY1) && (mY2 <= other_region.mY2);
}// IsInside


// EXTERNAL REFERENCES
//

#endif  // _Region_h
