#include <precomp.h>

#include "ProblemBase.h"    // eby nie byo warningw o destruktorze auto_ptr
#include "Result.h"  // class implemented
#include "util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Result::Result(unsigned firstIndex)
    : mFinalResult(),
      mapProblem(),
      mEvaluationCount(0),
      mFirstIndex(firstIndex),
      mAborted(false),
      mErrorMessage(_("No error"))
{
}// Result


Result::~Result()
{
}// ~Result


//============================= OPERATORS ====================================

/* friend */ wxTextOutputStream&
operator<<(wxTextOutputStream& rOutput, const Result& result)
{
    result.Print(rOutput);

    return rOutput;
}// operator<<

bool
Result::operator==(const Result& right) const
{
    return (mFinalResult == right.mFinalResult)
        && (mEvaluationCount == right.mEvaluationCount);
}// ==

//============================= OPERATIONS ===================================

void
Result::AddIteration(auto_ptr<const IterationData> apIteration)
{
    mIterations.push_back(CountedPtr<const IterationData>(apIteration));
}// AddIteration

// TODO: dummy, usun.
/* virtual */ void
Result::Print(wxTextOutputStream& rOutput) const
{
}// Print

//============================= ACCESS     ===================================

const ProblemBase&
Result::Problem() const
{
    assert (mapProblem.get() != NULL);

    return *mapProblem;
}// Problem

void
Result::SetProblem(std::auto_ptr<const ProblemBase> apProblem)
{
    mapProblem = apProblem;
}// SetProblem


unsigned
Result::FirstIterationNumber() const
{
    return mFirstIndex;
}// FirstIterationNumber

unsigned
Result::LastIterationNumber() const
{
    return mFirstIndex + mIterations.size() - 1;
}// LastIterationNumber

const Result::IterationContainer&
Result::Iterations() const
{
    return mIterations;
}// Iterations

const IterationData&
Result::Iteration(unsigned number) const
{
    assert (number >= mFirstIndex);

    return *mIterations[number - mFirstIndex];
}// Iteration

unsigned
Result::IterationCount() const
{
    unsigned iteration_count = mIterations.size();

    // Odejmij iteracj 0.
    if ((iteration_count > 0) && (mFirstIndex == 0))    --iteration_count;

    return iteration_count;
}// IterationCount

const ColumnVector&
Result::FinalResult() const
{
    return mFinalResult;
}// FinalResult

ColumnVector&
Result::rFinalResult()
{
    return mFinalResult;
}// rFinalResult

unsigned&
Result::rEvaluationCount()
{
    return mEvaluationCount;
}// rEvaluationCount

unsigned
Result::EvaluationCount() const
{
    return mEvaluationCount;
}// EvaluationCount

void
Result::SetError(const wxString& message)
{
    mFinalResult = NullPoint;
    mErrorMessage = message;
}// SetError

const wxString&
Result::ErrorMessage() const
{
    return mErrorMessage;
}// ErrorMessage

bool
Result::IsAborted() const
{
    return mFinalResult == NullPoint;
}// IsAborted

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
