#ifndef _Result_h
#define _Result_h


// SYSTEM INCLUDES
//
//#include <memory>

// PROJECT INCLUDES

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "IterationData.h"

// FORWARD REFERENCES
//
class ProblemBase;


/**  Wynik dziaania metody optymalizacji.
 *
 * Wynik zawiera, oczywicie, punkt, ktry metoda uznaa za rozwizanie problemu
 * optymalizacji, oraz list kolejnych iteracji.
 */
class Result
{
public:
    /// Kontener na iteracje.
    typedef vector<CountedPtr<const IterationData> >    IterationContainer;

// LIFECYCLE

    /** Konstruktor.
     *
     * @param firstIndex Numer pierwszej iteracji, najczsciej 1.
     */
    Result(unsigned firstIndex);

    /** Destruktor.
     */
    virtual ~Result(void);


// OPERATORS

    /** Operator porwnania.
     *
     * Porwnywany jest tylko wynik, nie dodatkowe skadowe (np. zadanie
     * optymalizacji).
     */
    bool            operator==(const Result& right) const;

// OPERATIONS

    // TODO: usun
    /** Drukuje wynik do strumienia tekstowego.
     */
    virtual void                Print(wxTextOutputStream& rOutput) const;

    // TODO: usun
    /** Operator wypisywania do strumienia.
     */
    friend wxTextOutputStream&  operator<<(wxTextOutputStream&  rOutput,
                                           const Result&        result);

    // TODO: po co tu jest alokacja pamici?
    void            AddIteration(auto_ptr<const IterationData> apIteration);

// ACCESS

    /** Zwraca zadanie optymalizacji.
     */
    const ProblemBase&          Problem() const;

    /** Przekazuje zadanie optymalizacji do wyniku.
     */
    void       SetProblem(std::auto_ptr<const ProblemBase> apProblem);

    /** Zwraca numer pierwszej iteracji.
     *
     * Najczciej 1 albo 0.
     */
    unsigned                    FirstIterationNumber() const;

    /** Zwraca numer ostatniej iteracji.
     */
    unsigned                    LastIterationNumber() const;

    /** Zwraca kontener iteracji.
     */
    const IterationContainer&   Iterations() const;

    /** Zwraca iteracj o podanym numerze.
     *
     * @see FirstIterationNumber
     */
    const IterationData&        Iteration(unsigned number) const;

    /** Zwraca liczb iteracji.
     *
     * Nie bierze pod uwag iteracji zerowej, moe wic by o 1 mniejsze od
     * liczby obiektw iteracji w kontenerze.
     */
    unsigned                    IterationCount() const;


    /** Zwraca wynik optymalizacji (tzn. uzyskany punkt minimum).
     *
     * Wynik jest zdefinowany tylko wtedy, gdy optymalizacja nie zostaa
     * przerwana wystpieniem bedu.
     *
     * @see IsAborted
     */
    const ColumnVector&         FinalResult() const;

    /** Referencja na wynik optymalizacji.
     */
    ColumnVector&               rFinalResult();

    /** Zwraca liczb wylicze wartoci funkcji celu.
     */
    unsigned                    EvaluationCount() const;

    /** Referencja na liczb wylicze wartoci funkcji celu.
     */
    unsigned&                   rEvaluationCount();

    /** Ustawia komunikat o bdzie oraz odp. flag.
     * @see IsAborted.
     */
    void                        SetError(const wxString& message);

    /** Zwraca komunikat o bdzie.
     */
    const wxString&             ErrorMessage() const;

    /** Sprawdza, czy optymalizacja zostaa przerwana (np. na skutek bdu
     * oblicze).
     */
    bool                        IsAborted() const;

// INQUIRY

protected:
private:
    /** Ostateczny wynik optymalizacji w postaci jednego punktu.
     */
    ColumnVector mFinalResult;

    /** Problem optymalizacji, ktrego rozwizaniem jest ten obiekt.
     */
    std::auto_ptr<const ProblemBase> mapProblem;

    /** Liczba wylicze wartoci funkcji celu.
     */
    unsigned    mEvaluationCount;

    /// Poszczeglne iteracje
    IterationContainer  mIterations;

    /// Numer pierwszej iteracji
    unsigned            mFirstIndex;

    /// Czy przerwano (na skutek bdu).
    bool                mAborted;

    /// Komunikat bdu.
    wxString            mErrorMessage;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Result_h_
