#include <precomp.h>

#include "Logger.h"
#include "RouletteSelection.h"  // class implemented
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RouletteSelection::~RouletteSelection()
{
}// ~RouletteSelection


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<SelectionMethod>
RouletteSelection::Clone() const
{
    return auto_ptr<SelectionMethod>(new RouletteSelection(*this));
}// Clone

/* virtual */ void
RouletteSelection::Select
                    (Population&                     population,
                     double                          survivalRate,
                     EvolutionaryMethodIteration&    iteration) const
{
    unsigned number_of_chromosomes_to_leave = static_cast<unsigned>
        (ceil(survivalRate * static_cast<double>(population.Size())));

    double death_probabilities[population.Size()];
    double sum_of_probabilities = 0.0;
    double biggest_probability = -DBL_MIN;
    unsigned worst_index;
    unsigned i;

    // Dodawanie do wartoci funkcji przystosowania dla poszczeglnych
    // chromosomw najmniejszej z tych wartoci w celu wyeliminowania ujemnych
    // prawdopodobiestw.
    LOG(DEBUG, wxT("adding constant:"));
    for (i = 0; i < population.Size(); ++i)
    {
        death_probabilities[i] = population[i].mValue -
                                    population.Best().mValue;
        sum_of_probabilities += death_probabilities[i];
        LOG(DEBUG, wxString::Format(wxT("%d. %g"), i, death_probabilities[i]));
    }
    LOG(DEBUG, wxString::Format(wxT("sum of modified values: %g"),
                                sum_of_probabilities));

    // Tworzenie prawdopodobiestw z przedziau [0; 1) i znajdowanie
    // najwikszego z nich.
    LOG(DEBUG, wxT("dividing by sum:"));
    for (i = 0; i < population.Size(); ++i)
    {
        death_probabilities[i] /= sum_of_probabilities;

        LOG(DEBUG, wxString::Format(wxT("%d. %g"), i, death_probabilities[i]));

        if (death_probabilities[i] > biggest_probability)
        {
            biggest_probability = death_probabilities[i];
            worst_index = i;

            LOG(DEBUG, wxString::Format(wxT("biggest_probability: %g"),
                                        biggest_probability));
        }
    }


    // Normalizacja.
    LOG(DEBUG, wxT("normalization:"));
    const double normalization_multiplier = 1.0 / biggest_probability;
    for (i = 0; i < population.Size(); ++i)
    {
        death_probabilities[i] *= normalization_multiplier;
        LOG(DEBUG, wxString::Format(wxT("%d. %g"), i, death_probabilities[i]));
    }


    i = 0;
    while (i < population.Size())
    {
        double rand = RandomDouble(0.0, 1.0);

        if (rand < death_probabilities[i])
        {
            iteration.AddDeceased(population[i].mPoint);
            population.Erase(i);
            death_probabilities[i] = death_probabilities[population.Size()];

            --i;
        }

        ++i;
    }

    population.GatherInfo(true);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
RouletteSelection::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
RouletteSelection::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

RouletteSelection::RouletteSelection()
    : SelectionMethod()
{
}// RouletteSelection

RouletteSelection::RouletteSelection(const RouletteSelection& from)
    : SelectionMethod(from)
{
}// RouletteSelection

/* static */ RouletteSelection
                    RouletteSelection::mRouletteSelection;
const /* static */ wxString
                    RouletteSelection::mName(_("Roulette selection"));
