/** Klasa reprezentujca selekcj proporcjonaln.
 *
 * Metoda selekcji ruletkowej w swej istocie polega na umoliwieniu lepszym
 * chromosomom namnoenia si z wikszym prawdopodobiestwem ni chromosomom
 * gorszym. W niniejszej klasie nie zaimplementowano tej metody w jej oglnie
 * przyjtej wersji w celu uniknicia utworzenia kilku kopii tego samego
 * chromosomu. Oglne zaoenie metody jest zachowane: lepsze chromosomy maj
 * wiksz szans na znalezienie si w nastpnej generacji.
 *
 * W pierwszym kroku algorytmu kademu chromosomowi nadawane jest
 * prawdopodobniestwo jego eliminacji z populacji proporcjonalne do funkcji
 * celu dla tego chromosomu.
 * To wstpne pradopodobiestwo wyliczane jest ze wzoru:
 * \f[ p^t(x) = \frac{f(x)}{ \sum_{y \in P} f(y)} \f]
 * Otrzymane wstpne prawdopodobiestwa modyfikujemy nastpnie tak, by
 * najwiksze z nich byo rwne 1, a proporcje midzy nimi pozostay zachowane:
 * \f[ p^k(x) = p^t(x) * \frac{1}{\max_{y \in P} p(y)} \f]
 * Gdybymy nie dokonali tej modyfikacji prawdopodobiestwo usunicia nawet dla
 * najgorszego chromosomu w populacji byoby znikome i ciar odpowiedzialnoci
 * za podanie algorytmu w kierunku minimum spadby na operator mutacji, co
 * praktycznie uniemoliwiaoby znalezienie dobrego rozwizania w rozsdnym
 * czasie.
 *
 * @see EvolutionaryMethod
 */

#ifndef _RouletteSelection_h
#define _RouletteSelection_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "SelectionMethod.h"

// FORWARD REFERENCES
//


class RouletteSelection
    : public SelectionMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~RouletteSelection(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<SelectionMethod>  Clone() const;

    /** Funkcja dokonujca selekcji populacji bazowej.
     *
     * Funkcja dokonujca selekcji populacji metod ruletkow (proporcjonaln).
     *
     * @param population Populacja bazowa, z ktrej osobniki bd wybierane.
     * @param survivalRate Wspczynnik selekcji ignorowany w tej metodzie.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void    Select(Population&                  population,
                           double                       partOfPopulationToLeave,
                           EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Przedstaw opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    RouletteSelection(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    RouletteSelection(const RouletteSelection& from);

    /** Egzemplarz metody
     */
    static RouletteSelection        mRouletteSelection;
    /** Nazwa metody
     */
    static const wxString           mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RouletteSelection_h
