/** Klasa bazowa dla klas selekcji osobnikw populacji.
 *
 * Zadaniem metod selekcji osobnikw populacji bazowej w procesie optymalizacji
 * genetycznej jest wybr najlepszych osobnikw do nastpnej generacji.
 *
 * @see TournamentSelection, RouletteSelection, EvolutionaryMethod
 */

#ifndef _SelectionMethod_h
#define _SelectionMethod_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "methods/EvolutionaryMethodIteration.h"
#include "Population.h"

// FORWARD REFERENCES
//

class SelectionMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    virtual ~SelectionMethod(void);


// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach selekcji.
     *
     * @return Kopia prototypu metody
     */
    virtual std::auto_ptr<SelectionMethod>      Clone() const = 0;

    /** Funkcja dokonujca selekcji populacji bazowej.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach selekcji.
     *
     * @param population Populacja bazowa, z ktrej osobniki bd wybierane.
     * @param survivalRate Wspczynnik selekcji interpretowany rnie w rnych
     * metodach selekcji.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void            Select
                            (Population&                  population,
                             double                       survivalRate,
                             EvolutionaryMethodIteration& iteration) const = 0;

// ACCESS

    /** Zwr referencj na kontener metod selekcji.
     *
     * @return referencja na konterer metod selekcji.
     */
    static const std::vector<SelectionMethod*>& SelectionMethods();

    /** Przedstaw opis metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach selekcji.
     *
     * @return Opis metody.
     */
    virtual wxString                            ToString() const = 0;

    /** Zwr nazw metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach selekcji.
     *
     * @return Nazwa metody.
     */
    virtual wxString                            Name() const = 0;

    /** Zwraca nazw klasy metod manipulacji populacj.
     *
     * Zwraca nazw klasy metod manipulacji populacj. W tym przypadku "Metoda
     * selekcji" lub odpowiednik w innym jezyku.
     *
     * @return Nazwa klasy manipulacji populacj.
     */
    static wxString                             BaseClassName();

// INQUIRY

protected:

    /** Konstruktor domylny.
     */
    SelectionMethod(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SelectionMethod(const SelectionMethod& from);

private:

    /** Zwraca referencj na kontener metod selekcji.
     *
     * @return referencja na kontener metod selekcji.
     */
    static std::vector<SelectionMethod*>&       rSelectionMethods();

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SelectionMethod_h
