#include <precomp.h>

#include "SimpleUnivariateAdapter.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//SimpleUnivariateAdapter::SimpleUnivariateAdapter
//                            (CountedPtr<const FunctionBase> cpFunction)
//    : FunctionBase(cpFunction->IsDifferentiable()),
//      UnivariateFunction(cpFunction->IsDifferentiable()),
//      mcpFunction(cpFunction)
SimpleUnivariateAdapter::SimpleUnivariateAdapter
                            (std::auto_ptr<FunctionBase> apFunction)
    : FunctionBase(apFunction->IsDifferentiable()),
      UnivariateFunction(apFunction->IsDifferentiable()),
      mapFunction(apFunction)
{
}// SimpleUnivariateAdapter

SimpleUnivariateAdapter::~SimpleUnivariateAdapter()
{
}// ~SimpleUnivariateAdapter


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
SimpleUnivariateAdapter::Clone() const
{
    return auto_ptr<FunctionBase>(new SimpleUnivariateAdapter(*this));
}// Clone


//============================= ACCESS     ===================================

const FunctionBase&
SimpleUnivariateAdapter::InnerFunction() const
{
    return *mapFunction;
}// InnerFunction

FunctionBase&
SimpleUnivariateAdapter::rInnerFunction()
{
    return *mapFunction;
}

unsigned
SimpleUnivariateAdapter::EvaluationCount() const
{
    return mapFunction->EvaluationCount();
}// EvaluationCount

/* virtual */ wxString
SimpleUnivariateAdapter::ToString(bool full) const
{
    wxString result;

    result << mapFunction->ToString(full);

    return result;
}// ToString

/* virtual */ VariableNames
SimpleUnivariateAdapter::PresentVariables() const
{
    return mapFunction->PresentVariables();
}// PresentVariables

//============================= INQUIRY    ===================================

/* virtual */ bool
SimpleUnivariateAdapter::ContainsUnknown(const VariableNames& variables) const
{
    return mapFunction->ContainsUnknown(variables);
}// ContainsUnknown

/////////////////////////////// PROTECTED  ///////////////////////////////////

SimpleUnivariateAdapter::SimpleUnivariateAdapter(
        const SimpleUnivariateAdapter& from)
    : FunctionBase(from),
      UnivariateFunction(from),
      mapFunction(from.mapFunction->Clone())
{
};

/* virtual */ double
SimpleUnivariateAdapter::DoEvaluateNoThrow(const ColumnVector& point) const
    throw()
{
    assert(point.nrows() == 1);

    return mapFunction->EvaluateNoThrow(point);
}

/* virtual */ double
SimpleUnivariateAdapter::DoEvaluate(const ColumnVector& point) const
    throw(EvaluationError)
{
    assert(point.nrows() == 1);

    return mapFunction->Evaluate(point);
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

