/** Opakowuje funkcj tak, by moga by traktowana jak funkcja jednej zmiennej.
 *
 * UWAGA: Klasa nie moe by wykorzystywana do minimalizacji w kierunku. Jej
 * zadaniem jej opakowanie funkcji gdy jest rzecz pewn, i jest ona funkcj
 * jednej zmiennej (Np. w przypadku funkcji kary w problemie jednowymiarowym).
 * Do umoliwienia minimalizacji w kierunku przeznaczona jest klasa
 * <code>DirectionalUnivariateAdapter</code>.
 *
 * @see DirectionalUnivariateAdapter
 */

#ifndef _SimpleUnivariateAdapter_h
#define _SimpleUnivariateAdapter_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "UnivariateFunction.h"

// FORWARD REFERENCES
//


class SimpleUnivariateAdapter
    : public UnivariateFunction
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param function Funkcja podstawowa. Staje si ona wasnoci obiektu.
     *
     */
//    SimpleUnivariateAdapter(CountedPtr<const FunctionBase> function);
    SimpleUnivariateAdapter(auto_ptr<FunctionBase> function);

    /** Destruktor.
     */
    virtual ~SimpleUnivariateAdapter(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase> Clone() const;

// ACCESS

    /** Zwraca funkcj wewntrzn.
     */
    const FunctionBase& InnerFunction() const;

    /** Zwraca referencj na funkcj wewntrzn.
     *
     * Uwanie korzysta!
     */
    FunctionBase&       rInnerFunction();

    /** Zwraca liczb wylicze funkcji.
     */
    unsigned            EvaluationCount() const;

    /** Zwraca tekstowy opis funkcji.
     */
    virtual wxString    ToString(bool full = true) const;

    /** Zwraca pust list zmiennych.
     */
    virtual VariableNames   PresentVariables() const;

// INQUIRY

//    /** Sprawdza, czy funkcja jest rniczkowalna.
//     */
//    virtual bool        IsDifferentiable() const;

    /** Zwraca <code>false</code>
     */
    virtual bool        ContainsUnknown(const VariableNames& variables) const;

protected:
    SimpleUnivariateAdapter(const SimpleUnivariateAdapter& from);

    /** Oblicza warto funkcji w punkcie.
     */
    virtual double      DoEvaluateNoThrow(const ColumnVector& point) const
                                                    throw();

    virtual double      DoEvaluate(const ColumnVector& point) const
                                                    throw(EvaluationError);

//    CountedPtr<const FunctionBase>  mcpFunction;   ///< Funkcja podstawowa
    std::auto_ptr<FunctionBase>  mapFunction;   ///< Funkcja podstawowa


private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SimpleUnivariateAdapter_h
