/**  Klasa bazowa dla metod optymalizacji w kierunku, ktre polegaj na
 * poszukiwaniu coraz lepszych pojedynczych punktw.
 *
 * Metody wywodzce si z tej klasy
 * w kadej iteracji znajduj tylko pojedynczy punkt w przestrzeni
 * jednej zmiennej. Najczciej jest on wybierany na podstawie wartoci
 * pochodnych w punkcie poprzednim
 *
 * W przypadku gdy metoda jest wykorzystywana do optymalizacji w kierunku przez
 * metod optymalizacji wielu zmiennych, wynikiem jest pojedynczy punkt.
 * Gdy metoda dziaa samodzielnie, wynikiem jest cay przebieg optymalizacji.
 */

#ifndef _SinglePointLineSearch_h
#define _SinglePointLineSearch_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//

#include "DoubleParameter.h"
#include "LineSearch.h"

// FORWARD REFERENCES
//


class SinglePointLineSearch
    : public LineSearch
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~SinglePointLineSearch(void);


// OPERATORS

// OPERATIONS

    virtual void        UpdateStartingConditions(const Result* result) const;

    virtual void        ResetStartingConditions() const;

// ACCESS
//
    void                AddCommonParameterDescriptions
                                               (wxArrayString& descriptions,
                                                bool standalone) const;

    virtual wxString    ToString(bool standalone = true) const;

// INQUIRY
    virtual bool        CanAssist(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    SinglePointLineSearch(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SinglePointLineSearch(const SinglePointLineSearch& from);

    mutable ColumnVector    mStartingPoint; ///< Punkt startowy
    ColumnVector            mInitialStartingPoint;

    /** Dokadno (\f$\varepsilon\f$).
     */
    DoubleParameter         mEpsilon;

private:

    /** Optymalizuj.
     *
     * Przeprowadza optymalizacj za pomoc algorytmu, ktry realizowa ma dana
     * klasa.
     *
     * @param problem Problem optymalizacji.
     * @return Wynik oraz przebieg optymalizacji.
     */
    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    /** Znajd optymaln dugo kroku.
     *
     * Wywoywana z FindOptimalStep.
     *
     * @param function Funkcja celu
     * @see FindOptimalStep
     */
    virtual double  Optimize(const UnivariateFunction& function)
                               const;

    /** Znajd minimum w kierunku.
     *
     * Funkcja znajduje minimum funkcji jednowymiarowej.
     * Jest wykorzystywana zarwno gdy minimalizujemy funkcj jednowymiarow w
     * samodzielnym problemie minimalizacji, jak i gdy minimalizacja w kierunku
     * jest etapem minimalizacji w wielu wymairach.
     *
     * @param function Funkcja celu  (\f$f\f$).
     * @param startingPoint Punkt pocztkowy \f$ x^0 \f$.
     * @param generateResult Czy generowa przebieg optymalizacji w wyniku.
     * @return <code>auto_ptr</code> na przebieg optymalizacji. Jeli
     * <code>generateResult</code> jest <code>false</code> to nie zawiera
     * adnych iteracji (ale wynik kocowy tak).
     */
    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      const ColumnVector&       startingPoint,
                                      bool                      generateResult)
                          const throw(OptimizationError) = 0;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SinglePointLineSearch_h_
