#include <precomp.h>

#include "Logger.h"
#include "util.h"

#include "SlopeCondition.h"  // class implemented

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SlopeCondition::SlopeCondition()
    : StandardStopCondition()
{
    mName = wxT("| f(x k+1) - f(x k) | / || x k+1 - x k ||");
}// SlopeCondition

SlopeCondition::SlopeCondition(const DoubleParameter& epsilon)
    : StandardStopCondition(epsilon)
{
}// SlopeCondition

SlopeCondition::SlopeCondition(const SlopeCondition& from)
    : StandardStopCondition(from)
{
}// SlopeCondition

SlopeCondition::~SlopeCondition()
{
}// ~SlopeCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
SlopeCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new SlopeCondition(*this));
}// Clone

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

/* virtual */ bool
SlopeCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      double                previousValue,
                      double                lastValue,
                      double                customValue) const
{
    LOG(DEBUG, wxT("previousValue: ") << previousValue);
    LOG(DEBUG, wxT("lastValue: ")     << lastValue);

    // Warunek nie jest speniony w pierwszej iteracji.
    // W metodach sprawdzajcych warunek stopu przed wykonaniem
    // kadej iteracji (np. w metodach gradientowych) naley inicjowa
    // poprzedni punkt sta NullPoint.
    if (previousPoint == NullPoint)     { return false; }

    double distance = (lastPoint - previousPoint).norm_Frobenius();

    LOG(DEBUG, wxT("distance: ")     << distance);

    // Zakadamy, e jeli oba kroki s w tym samym miejscu, to warunek stopu
    // jest speniony:
    if (distance == 0) return true;

    return (fabs(previousValue - lastValue)/distance < mEpsilon);
}// IsMet

/* virtual */ bool
SlopeCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      double                previousValue,
                      const FunctionBase&   function,
                      double                customValue) const
{
    return IsMet(previousPoint, lastPoint, previousValue,
                 function.Evaluate(lastPoint));
}// IsMet

/* virtual */ bool
SlopeCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      const FunctionBase&   function,
                      double                customValue) const
{
    // Warunek nie jest speniony w pierwszej iteracji.
    // W metodach sprawdzajcych warunek stopu przed wykonaniem
    // kadej iteracji (np. w metodach gradientowych) naley inicjowa
    // poprzedni punkt sta NullPoint.
    if (previousPoint == NullPoint)     { return false; }

    return IsMet(previousPoint, lastPoint, function.Evaluate(previousPoint),
                 function.Evaluate(lastPoint));
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
