/** Warunek stopu oparty na przyblionym nachyleniu funkcji pomidzy dwoma
 * punktami.
 *
 * Sprawdza, czy nachylenie funkcji dla podanych dwch punktw jest mniejsze
 * od wartoci granicznej (<em>epsilon</em>):
 * \f[ \frac{|f(x^2) - f(x^1)|}{\|x^2 - x^1\|} < \varepsilon . \f]
 */

#ifndef _SlopeCondition_h
#define _SlopeCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class SlopeCondition
    : public StandardStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    SlopeCondition(void);


    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     */
    SlopeCondition(const DoubleParameter& epsilon);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SlopeCondition(const SlopeCondition& from);


    /** Destruktor.
     */
    ~SlopeCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS
// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousSlope,
                              double                lastSlope,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousSlope,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SlopeCondition_h
