#include <precomp.h>

#include "Logger.h"

#include "SquaredCustomValueCondition.h"  // class implemented

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SquaredCustomValueCondition::SquaredCustomValueCondition()
    : StandardStopCondition(),
      mMetWhenGreater(false)
{
    mName = wxT("(custom value)^2");
}// SquaredCustomValueCondition

SquaredCustomValueCondition::SquaredCustomValueCondition(const wxString& name,
                                                         bool  metWhenGreater)    : StandardStopCondition(),
    mMetWhenGreater(metWhenGreater)
{
    mName = name;
}// SquaredCustomValueCondition

SquaredCustomValueCondition::SquaredCustomValueCondition(
                const DoubleParameter&  epsilon,
                bool                    metWhenGreater)
    : StandardStopCondition(epsilon),
      mMetWhenGreater(metWhenGreater)
{
}// SquaredCustomValueCondition

SquaredCustomValueCondition::SquaredCustomValueCondition
    (const SquaredCustomValueCondition& from)
    : StandardStopCondition(from),
      mMetWhenGreater(from.mMetWhenGreater)
{
}// SquaredCustomValueCondition

SquaredCustomValueCondition::~SquaredCustomValueCondition()
{
}// ~SquaredCustomValueCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
SquaredCustomValueCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>
        (new SquaredCustomValueCondition(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString
SquaredCustomValueCondition::ToString() const
{
    wxString description;

    description << Name();
    if (!mMetWhenGreater)   description << wxT(" < ");
    else                    description << wxT(" > ");

    description << wxString::Format(wxT("%g"), static_cast<double>(Epsilon()));

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
SquaredCustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                                   const ColumnVector&    lastPoint,
                                   double                 previousValue,
                                   double                 lastValue,
                                   double                 customValue) const
{
    if (!mMetWhenGreater)   return ((customValue * customValue) < mEpsilon);
    else                    return ((customValue * customValue) > mEpsilon);
}// IsMet

/* virtual */ bool
SquaredCustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                          const ColumnVector&    lastPoint,
                          double                 previousValue,
                          const FunctionBase&    function,
                          double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0, customValue);
}// IsMet

/* virtual */ bool
SquaredCustomValueCondition::IsMet(const ColumnVector&    previousPoint,
                          const ColumnVector&    lastPoint,
                          const FunctionBase&    function,
                          double                 customValue) const
{
    return IsMet(previousPoint, lastPoint, 0.0, 0.0, customValue);
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
