/** Warunek stopu oparty na kwadracie podanej liczby.
 *
 * Sprawdza, czy kwadrat podanej liczby (moe ni by np. norma gradientu) jest
 * mniejszy lub wikszy od wartoci granicznej (<em>epsilon</em>).
 */

#ifndef _SquaredCustomValueCondition_h
#define _SquaredCustomValueCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class SquaredCustomValueCondition
    : public StandardStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    SquaredCustomValueCondition(void);

    /** Konstruktor.
     *
     * @param name Nazwa sprawdzanego parametru.
     * @param metWhenGreater Informacja, czy warunek jest speniony,
     * gdy warto jest wiksza ni epsilon (false - speniony gdy mniejsza).
     */
    SquaredCustomValueCondition(const wxString& name,
                                bool  metWhenGreater = false);

    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     * @param metWhenGreater Informacja, czy warunek jest speniony,
     * gdy warto jest wiksza ni epsilon (false - speniony gdy mniejsza).
     */
    SquaredCustomValueCondition(const DoubleParameter& epsilon,
                                bool  metWhenGreater = false);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SquaredCustomValueCondition(const SquaredCustomValueCondition& from);


    /** Destruktor.
     */
    ~SquaredCustomValueCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS

    /** Zwraca tekstowy opis warunku stopu.
     */
    virtual wxString        ToString() const;

// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              double                lastValue,
                              double                customValue) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              const FunctionBase&   function,
                              double                customValue) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue) const;

protected:
private:
bool mMetWhenGreater;   /// Informacja, czy warunek jest speniony,
                        /// gdy warto jest wiksza ni epsilon
                        /// (false - speniony gdy mniejsza).
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SquaredCustomValueCondition_h
