/** Klasa bazowa dla warunkw stopu.
 */

#ifndef _StopConditionBase_h
#define _StopConditionBase_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "DoubleParameter.h"

// FORWARD REFERENCES
//
class StopConditionBase;
typedef vector< CountedPtr<StopConditionBase> > StopConditions;

class StopConditionBase
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    StopConditionBase(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    StopConditionBase(const StopConditionBase& from);


    /** Destruktor.
     */
    virtual ~StopConditionBase(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const = 0;

// ACCESS

    /** Zwraca nazw warunku stopu.
     */
    virtual const wxString& Name() const;

    /** Zwraca tekstowy opis warunku stopu.
     */
    virtual wxString        ToString() const;

    /** Zwraca referencj na nazw warunku stopu.
     */
    virtual wxString& rName();

    /** Zwraca warto minimaln <em>epsilon</em>.
     */
    virtual DoubleParameter         Epsilon() const = 0;

    /** Zwraca referencj na <em>epsilon</em>.
     */
    virtual DoubleParameter&        rEpsilon() = 0;


// INQUIRY

    /** Zwraca wektor podstawowych warunkw stopu.
     *
     * Zwracany wektor zawiera wskaniki na trzy warunki stopu:
     * DistanceCondition, ValueCondition i SlopeCondition
     */
    static StopConditions   StandardStopConditions();

    /** Zwraca wektor warunkw stopu dla metod gradientowych.
     *
     * Zwracany wektor zawiera wskaniki na trzy podstawowe
     * warunki stopu: DistanceCondition, ValueCondition
     * i SlopeCondition oraz oraz na CustomValueCondition(
     * "|| grad f(x k) ||") i SquaredCustomValueCondition(
     * "|| grad f(x k) ||^2").
     */
    static StopConditions   GradientStopConditions();

    /** Zwraca wektor warunkw stopu dla metody
     * Neldera-Meada.
     */
    static StopConditions   NelderMeadStopConditions();

    /** Zwraca wektor warunkw stopu dla metody ewolucyjnej.
     */
    static StopConditions   EvolutionaryMethodStopConditions();

// PUBLIC MEMBERS

protected:
    wxString mName;
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _StopConditionBase_h
