#include <precomp.h>

#include "ThreeParticipantsCrossover.h"  // class implemented
#include "Logger.h"
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ThreeParticipantsCrossover::~ThreeParticipantsCrossover()
{
}// ~ThreeParticipantsCrossover

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<CrossoverMethod>
ThreeParticipantsCrossover::Clone() const
{
    return auto_ptr<CrossoverMethod>(new ThreeParticipantsCrossover(*this));
}// Clone

/* virtual */ void
ThreeParticipantsCrossover::Crossover
                             (Population&                   population,
                              double                        crossoverRate,
                              EvolutionaryMethodIteration&  iteration) const
{
    while (population.Size() < population.DesiredSize())
    {
        size_t  parent_index_1 = RandomIndex(0, population.Size());
        size_t  parent_index_2 = RandomIndex(1, population.Size());
        size_t  parent_index_3 = RandomIndex(0, population.Size());

        if (parent_index_1 == parent_index_2) --parent_index_1;

        LOG(DEBUG, wxString::Format(wxT("First parent: ch%d: "),
                                    parent_index_1)
                << population[parent_index_1].mPoint.as_row()
                << wxString::Format(wxT("value = %g"),
                                    population[parent_index_1].mValue));
        LOG(DEBUG, wxString::Format(wxT("Second parent: ch%d: "),
                                    parent_index_2)
                << population[parent_index_2].mPoint.as_row()
                << wxString::Format(wxT("value = %g"),
                                    population[parent_index_2].mValue));
        LOG(DEBUG, wxString::Format(wxT("Nanny: ch%d: "),
                                    parent_index_3)
                << population[parent_index_3].mPoint.as_row()
                << wxString::Format(wxT("value = %g"),
                                    population[parent_index_3].mValue));

        ColumnVector child = population[parent_index_3].mPoint +
                           ((population[parent_index_1].mPoint -
                             population[parent_index_2].mPoint) *
                                                    crossoverRate);

        LOG(DEBUG, wxT("Child: ") << child.as_row());

        iteration.AddChild(child);

        population.AddChromosome(child);
    }

    population.GatherInfo(false);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
ThreeParticipantsCrossover::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
ThreeParticipantsCrossover::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

ThreeParticipantsCrossover::ThreeParticipantsCrossover()
    : CrossoverMethod()
{
}// ThreeParticipantsCrossover

ThreeParticipantsCrossover::ThreeParticipantsCrossover(const ThreeParticipantsCrossover& from)
    : CrossoverMethod(from)
{
}// ThreeParticipantsCrossover

/* static */ ThreeParticipantsCrossover
        ThreeParticipantsCrossover::mThreeParticipantsCrossover;
/* static */ const wxString
        ThreeParticipantsCrossover::mName(_("Three participants crossover"));
