/** Klasa reprezentujca metod krzyowania z udziaem trzech osobnikw.
 *
 * Klasa dokonuje krzyowania chromosomw tworzc nowy
 * chromosom-dziecko poprzez dodanie do wsprzdnych losowo wybranego
 * chromosomu rnicy wsprzdnych pomidzy rodzicami pomniejszonej o
 * wspczynnik krzyowania.
 *
 * @see EvolutionaryMethod
 */

#ifndef _ThreeParticipantsCrossover_h
#define _ThreeParticipantsCrossover_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CrossoverMethod.h"

// FORWARD REFERENCES
//


class ThreeParticipantsCrossover
    : public CrossoverMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~ThreeParticipantsCrossover(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<CrossoverMethod>  Clone() const;

    /* Przeprowad krzyowanie.
     *
     * Funkcja tworzy nowe chromosomy dodaj odcinek czcy dwa osobiniki
     * wylosowane z ppopulacji do wsprzdnych trzeciego osobnika.
     *
     * @param population Populacja, ktrej osobniki maj by poddane procesowi
     * krzyowania.
     * @param crossoverRate Wspczynnik krzyowania mnoony przez odcinek
     * czcy chromosomy-rodzicw
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void        Crossover
                              (Population&                  population,
                               double                       crossoverRate,
                               EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Przedstaw opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    ThreeParticipantsCrossover(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    ThreeParticipantsCrossover(const ThreeParticipantsCrossover& from);

    /** Egzemplarz metody
     */
    static ThreeParticipantsCrossover   mThreeParticipantsCrossover;
    /** Nazwa metody
     */
    static const wxString               mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ThreeParticipantsCrossover_h
