// TODO: Peny opis
/** Klasa dzielca cig znakw na tokeny.
 *
 */

#ifndef _Tokenizer_h
#define _Tokenizer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "FunctionTree.h"
#include "ParseError.h"

// FORWARD REFERENCES
//


class Tokenizer
{
public:

    enum TokenType
    {
        NONE,               ///< Brak, warto specjalna
        NUMBER,             ///< Liczba
        VARIABLE,           ///< Zmienna
        PLUS,               ///< '+'
        MINUS,              ///< '-' (minus bd negacja)
        MULTIPLY,           ///< '*'
        DIVIDE,             ///< '/'
        POWER,              ///< '^'
        FUNCTION,           ///< Funkcja
        OPEN_PAREN,         ///< '('
        CLOSE_PAREN,        ///< ')'
        END                 ///< Koniec cigu znakw
    };

    /// Informacja o funkcji matematycznej.
    struct FunctionInfo {
        FunctionTree::UnaryOperator function;       ///< Wskanik na funkcj
        bool                        differentiable; ///< Czy rniczkowalna
    };

    /// Mapa rozpoznawanych funkcji.
    typedef std::map<wxString, FunctionInfo>    FunctionMap;

// LIFECYCLE

    /** Konstruktor.
     *
     * @param input Tekst do podziau na tokeny.
     * @param rVariables Referencja na list zmiennych.
     * @param addVariables Czy dodawa nowe zmienne.
     */
    Tokenizer(const wxString&   input,
              VariableNames&    rVariables,
              bool              addVariables,
              FunctionMap       functions = FunctionMap());

    /** Destruktor.
     */
    ~Tokenizer(void);


// OPERATORS

// OPERATIONS

    /** Sprawd nastpny token.
     *
     * Zwraca typ nastpnego tokenu i zapamituje go. Wielokrotne wywoanie
     * zwraca ten sam wynik, dopki nie zostanie wywoane Consume.
     *
     * @see Consume
     */
    TokenType               Peek() throw(ParseError);

    /** Wczytaj token.
     *
     * Zjada zaznaczony przez Peek token. Kolejne wywoanie Peek znajdzie
     * nastpny token.
     *
     * Dla tokenw typu: <code>NUMBER</code>, <code>VARIABLE</code> oraz
     * <code>FUNCTION</code> zwracany jest wze drzewa funkcji odpowiadajcy
     * wartoci tokenu (liczba, zmienna).
     *
     * @see FunctionTree, Peek
     */
    auto_ptr<FunctionTree>  Consume();

    /** Wczytaj token podanego typu.
     *
     * Jeli nastpny token jest taki, jak <code>expectedToken</code>, to
     * wywoywane jest Consume, w przeciwnym razie zgaszany jest wyjtek.
     */
    void                    Expect(TokenType expectedToken) throw(ParseError);

// ACCESS

    /** Zwraca parsowany cig znakw.
     */
    const wxString&         Input() const;

    /** Zwraca ilo przetworzonych znakw.
     *
     * Inaczej: pozycj zaczynajc od 0.
     */
    unsigned                Position() const;

// INQUIRY

    /** Czy dotychczas odczytane tokeny skadaj si na funkcj rniczkowaln?
     */
    bool                    AllDifferentiable() const;

protected:
private:

    /** Obcina pocztkowe spacje z cigu wejciowego.
     */
    void                TrimInput();

    /** Czy znak <code>c</code> naley do znakw, z ktrych skadaj si liczby
     * rzeczywiste.
     */
    static bool         IsDoubleChar(wxChar c);

    /** Czy znak <code>c</code> naley do znakw, z ktrych skadaj si
     * identyfikatory (nazwy funkcji, zmiennych).
     */
    static bool         IsIdentifierChar(wxChar c);

    /// Cig znakw, na ktrym pracuje tokenizer, niszczony w trakcie pracy.
    const wxString          mInput;         ///< Oryginalny cig wejciowy.
    wxString                mWorkingInput;  ///< Robocza kopia wejcia.
    unsigned                mPosition;      ///< Ilo przetworzonych znakw.
    int                     mTokenLength;   ///< Dugo rozpoznanego tokenu.
    TokenType               mTokenType;     ///< Typ rozpoznanego tokenu.
    auto_ptr<FunctionTree>  mapTokenNode;
    /// Czy dotychczasowe tokeny skadaj si na funkcj rniczkowaln.
    bool                    mDifferentiable;

    VariableNames&          mrVariables;    ///< Lista zmiennych.
    bool                    mAddVariables;  ///< Czy dodawa nowe zmienne.
    FunctionMap             mFunctionMap;   ///< Lista funkcji.

    static wxChar mDecimalSeparator;    ///< Znak oddzielajcy cz uamkow.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Tokenizer_h
