#include <precomp.h>

#include "Logger.h"
#include "TournamentSelection.h"  // class implemented
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TournamentSelection::~TournamentSelection()
{
}// ~TournamentSelection


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<SelectionMethod>
TournamentSelection::Clone() const
{
    return auto_ptr<SelectionMethod>(new TournamentSelection(*this));
}// Clone

/* virtual */ void
TournamentSelection::Select
                    (Population&                     population,
                     double                          survivalRate,
                     EvolutionaryMethodIteration&    iteration) const
{
    unsigned number_of_chromosomes_to_leave = static_cast<unsigned>
        (ceil(survivalRate * static_cast<double>(population.Size())));

    assert(number_of_chromosomes_to_leave > 0);

    while (population.Size() > number_of_chromosomes_to_leave)
    {
        unsigned    combatant_index_1
                                = RandomIndex(0, population.Size());
        unsigned    combatant_index_2
                                = RandomIndex(1, population.Size());
        if (combatant_index_1 == combatant_index_2) --combatant_index_1;
        unsigned    chromosome_to_kill;

        LOG(DEBUG, wxString::Format(wxT("First chromosome:  %d\n"),
                                    combatant_index_1)
                   << population[combatant_index_1].mPoint.as_row()
                   << wxString::Format(wxT(" -> %g"),
                                       population[combatant_index_1].mValue))
        LOG(DEBUG, wxString::Format(wxT("Second chromosome: %d\n"),
                                    combatant_index_2)
                   << population[combatant_index_2].mPoint.as_row()
                   << wxString::Format(wxT(" -> %g"),
                                       population[combatant_index_2].mValue))

        chromosome_to_kill = (population[combatant_index_1].mValue <=
                                population[combatant_index_2].mValue) ?
                                    combatant_index_2 :
                                    combatant_index_1;

        LOG(DEBUG, wxString::Format(wxT("Loser: %d"), chromosome_to_kill));

        iteration.AddDeceased(population[chromosome_to_kill].mPoint);

        population.Erase(chromosome_to_kill);

        LOG(DEBUG, wxString::Format(wxT("Current size: %d\n"),
                                    population.Size()));
    }

    population.GatherInfo(true);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
TournamentSelection::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
TournamentSelection::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

TournamentSelection::TournamentSelection()
    : SelectionMethod()
{
}// TournamentSelection

TournamentSelection::TournamentSelection(const TournamentSelection& from)
    : SelectionMethod(from)
{
}// TournamentSelection

/* static */ TournamentSelection
                    TournamentSelection::mTournamentSelection;
const /* static */ wxString
                    TournamentSelection::mName(_("Tournament selection"));
