/** Klasa reprezentujca selekcj turniejowej.
 *
 * W selekcji turniejowej z populacji wybierana jest grupa turniejowa o
 * okrelonej wielkoci. Z grypu tej do nastepnej generacji wybierany jest
 * osobnik o najlepszej funkcji przystosowania. Proces jest powtarzany do
 * momentu gdy populacja zmniejszy si do zadanych z gry rozmiarw. W naszej
 * implementacji metody grupa turniejowa ma wielko dwch osobnikw.
 *
 * @see EvolutionaryMethod
 */

#ifndef _TournamentSelection_h
#define _TournamentSelection_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "SelectionMethod.h"

// FORWARD REFERENCES
//


class TournamentSelection
    : public SelectionMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~TournamentSelection(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<SelectionMethod>  Clone() const;

    /** Funkcja dokonujca selekcji populacji bazowej.
     *
     * Funkcja dokonujca selekcji populacji metod turniejow.
     *
     * @param population Populacja bazowa, z ktrej osobniki bd wybierane.
     * @param survivalRate Wspczynnik selekcji informujcy metod jaka cz
     * pocztkowej populacji ma pozosta przy yciu.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void    Select(Population&                  population,
                           double                       partOfPopulationToLeave,
                           EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Przedstaw opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    TournamentSelection(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    TournamentSelection(const TournamentSelection& from);

    /** Egzemplarz metody
     */
    static TournamentSelection      mTournamentSelection;
    /** Nazwa metody
     */
    static const wxString           mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _TournamentSelection_h
