#include <precomp.h>

#include "Logger.h"
#include "UnconstrainedProblem.h"  // class implemented

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


UnconstrainedProblem::~UnconstrainedProblem()
{
}// ~UnconstrainedProblem


//============================= OPERATORS ====================================


//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<ProblemBase>
UnconstrainedProblem::Clone() const
{
    return std::auto_ptr<ProblemBase>(new UnconstrainedProblem(*this));
}// Clone

/* virtual */ wxString
UnconstrainedProblem::ToString() const
{
    wxString string(Name());

    string << wxT("\n") << _("Function:") << wxT("\n");
    string << ((mcpFunction.Get() != NULL) ?
        mcpFunction->ToString() : wxString(_("None"))) << wxT("\n");

    return string;
}// ToString


//============================= ACCESS     ===================================

/* virtual */ wxString
UnconstrainedProblem::Name() const
{
    LOG(DEBUG, wxGetTranslation(mName));
    return wxGetTranslation(mName);
}// Name

/* virtual */ ProblemIdType
UnconstrainedProblem::Id() const
{
    return UnconstrainedProblem::ClassId();
}// Id

/* static */ ProblemIdType
UnconstrainedProblem::ClassId()
{
    const static ProblemIdType UnconstrainedProblemId = GenerateId();

    return UnconstrainedProblemId;
}// ClassId

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

UnconstrainedProblem::UnconstrainedProblem(bool doRegister)
    : ProblemBase(doRegister)
{
}// UnconstrainedProblem

UnconstrainedProblem::UnconstrainedProblem(const UnconstrainedProblem& from)
    : ProblemBase(from)
{
}// UnconstrainedProblem


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ const wxString
        UnconstrainedProblem::mName(_("Unconstrained Problem"));

/* static */ UnconstrainedProblem
        UnconstrainedProblem::mUnconstrainedProblem(true);
