#include <precomp.h>

#include "Logger.h"
#include "UniformMutation.h"  // class implemented
#include "util.h"

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UniformMutation::~UniformMutation()
{
}// ~UniformMutation


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<MutationMethod>
UniformMutation::Clone() const
{
    return auto_ptr<MutationMethod>(new UniformMutation(*this));
}// Clone

/* virtual */ void
UniformMutation::Mutate(Population&                  population,
                       double                       radioactivity,
                       double                       mutationRate,
                       EvolutionaryMethodIteration& iteration) const
{
    if (population.Size() == 0) return;

    LOG(DEBUG, wxString::Format(wxT("Radioactivity:     %g"), radioactivity));

    unsigned dimensions = population[0].mPoint.nrows();

    for (unsigned index = 0; index < population.Size(); ++index)
    {
        double random = RandomDouble(0.0, 1.0);

        //TODO: Moe jednak da moliwo mutacji najlepszego chromosomu...
        if (index != population.BestIndex() && random < radioactivity)
        {
            for (unsigned j = 1; j <= dimensions; ++j)
            {
                population[index].mPoint(j) +=
                                    RandomDouble(-1.0, 1.0) * mutationRate;
            }

            population.ReevaluateChromosome(index);

            iteration.AddMutant(population[index].mPoint);
        }
        else
            iteration.AddSurvivor(population[index].mPoint);
    }

    population.GatherInfo(false);
}

//============================= ACCESS     ===================================

/* virtual */ wxString
UniformMutation::ToString() const
{
    wxString text;

    text << Name();

    return text;
}

/* virtual */ wxString
UniformMutation::Name() const
{
    return wxGetTranslation(mName);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

UniformMutation::UniformMutation()
    : MutationMethod()
{
}// UniformMutation

UniformMutation::UniformMutation(const UniformMutation& from)
    : MutationMethod(from)
{
}// UniformMutation

/* static */ UniformMutation
                    UniformMutation::mUniformMutation;
const /* static */ wxString
                    UniformMutation::mName(_("Uniform distribution mutation"));
