/** Klasa reprezentujca mutacj z rozkadem jendostajnym.
 *
 * Mutacja w algortymie optymalizacji ewolucyjnej polega na wybraniu z pewnym
 * prawdopodobiestwem kandydatw do przeprowadzenia mutacji z puli chromosomw,
 * ktre przetrway etap selekcji. Do wsprzenych wybranych chromosomw w tej
 * konkretnej metodzie mutacji dodawana jest warto zmiennej losowej o
 * rozkadzie jednorodnym w przedziale /f$[-1; 1]/f$. Przedzia moe by
 * modyfikowany za pomoc parametru <code>mutationRate</code> przekazywanego do
 * metody Mutate.
 *
 * @see EvolutionaryMethod
 */

#ifndef _UniformMutation_h
#define _UniformMutation_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "MutationMethod.h"

// FORWARD REFERENCES
//


class UniformMutation
    : public MutationMethod
{
public:
// LIFECYCLE

    /** Destruktor.
     */
    ~UniformMutation(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     */
    virtual std::auto_ptr<MutationMethod>  Clone() const;

    /** Funkcja dokonujca mutacji wybranych chromosomw populacji bazowej.
     *
     * Funkcja z prawdopodobiestwem przekazanym w parametrze
     * <code>mutation</code> przydziela chromosom do grupy przeznaczonych do
     * mutacji, ktra polega na dodaniu do wsprzdnych chromosomy wartoci
     * zmiennej losowej o jednorodnym rozkadzie w przedziale \f$[-1; 1]\f$.
     *
     * @param population Populacja bazowa, ktrej osobniki bd mutowane.
     * @param radioactivity natenie zjawiska mutacji - prawdopodobiestwo
     * przeznaczenia chromosomu do mutacji.
     * @param mutationRate wspczynnik mnoony przez wylosowan warto
     * zmiennej losowej,
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void            Mutate
                            (Population&                  population,
                             double                       radioactivity,
                             double                       mutationRate,
                             EvolutionaryMethodIteration& iteration) const;

// ACCESS

    /** Zwr opis metody.
     *
     * @return Opis metody.
     */
    virtual wxString                ToString() const;

    /** Zwr nazw metody.
     *
     * @return Nazwa metody.
     */
    virtual wxString                Name() const;

// INQUIRY

protected:
private:

    /** Konstruktor domylny.
     */
    UniformMutation(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    UniformMutation(const UniformMutation& from);

    /** Egzemplarz metody
     */
    static UniformMutation      mUniformMutation;
    /** Nazwa metody
     */
    static const wxString       mName;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _UniformMutation_h
