#include <precomp.h>

#include "Logger.h"

#include "ValueCondition.h"  // class implemented

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ValueCondition::ValueCondition()
    : StandardStopCondition()
{
    mName = wxT("| f(x k+1) - f(x k) |");
}// ValueCondition

ValueCondition::ValueCondition(const DoubleParameter& epsilon)
    : StandardStopCondition(epsilon)
{
}// ValueCondition

ValueCondition::ValueCondition(const ValueCondition& from)
    : StandardStopCondition(from)
{
}// ValueCondition

ValueCondition::~ValueCondition()
{
}// ~ValueCondition


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

std::auto_ptr<StopConditionBase>
ValueCondition::Clone() const
{
    return std::auto_ptr<StopConditionBase>(new ValueCondition(*this));
}// Clone

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

/* virtual */ bool
ValueCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      double                previousValue,
                      double                lastValue,
                      double                customValue) const
{
    LOG(DEBUG, wxT("previousPoint: ") << previousPoint);
    LOG(DEBUG, wxT("previousValue: ") << previousValue);
    LOG(DEBUG, wxT("lastPoint: ")     << lastPoint);
    LOG(DEBUG, wxT("lastValue: ")     << lastValue);

    // Warunek nie jest speniony w pierwszej iteracji.
    // W metodach sprawdzajcych warunek stopu przed wykonaniem
    // kadej iteracji (np. w metodach gradientowych) naley inicjowa
    // poprzedni punkt sta NullPoint.
    if (previousPoint == NullPoint)     { return false; }

    return (fabs(previousValue - lastValue) < mEpsilon);
}// IsMet

/* virtual */ bool
ValueCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      double                previousValue,
                      const FunctionBase&   function,
                      double                customValue) const
{
    return IsMet(previousPoint, lastPoint, previousValue,
                 function.Evaluate(lastPoint));
}// IsMet

/* virtual */ bool
ValueCondition::IsMet(const ColumnVector&   previousPoint,
                      const ColumnVector&   lastPoint,
                      const FunctionBase&   function,
                      double                customValue) const
{
    // Warunek nie jest speniony w pierwszej iteracji.
    // W metodach sprawdzajcych warunek stopu przed wykonaniem
    // kadej iteracji (np. w metodach gradientowych) naley inicjowa
    // poprzedni punkt sta NullPoint.
    if (previousPoint == NullPoint)     { return false; }

    return IsMet(previousPoint, lastPoint, function.Evaluate(previousPoint),
                 function.Evaluate(lastPoint));
}// IsMet

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
