/** Warunek stopu oparty na rnicy wartoci funkcji w dwch punktach.
 *
 * Sprawdza, czy rnica wartoci funkcji dla podanych dwch punktw
 * jest mniejsza od wartoci granicznej (<em>epsilon</em>):
 * \f[ |f(x^2) - f(x^1)| < \varepsilon\ . \f]
 */

#ifndef _ValueCondition_h
#define _ValueCondition_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "StandardStopCondition.h"

// FORWARD REFERENCES
//


class ValueCondition
    : public StandardStopCondition
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    ValueCondition(void);

    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     */
    ValueCondition(const DoubleParameter& epsilon);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    ValueCondition(const ValueCondition& from);

    /** Destruktor.
     */
    ~ValueCondition(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<StopConditionBase>    Clone() const;

// ACCESS
// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              double                lastValue,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

    /** Sprawd warunek stopu.
     *
     * @see StopCondition::IsMet
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue = 0) const;

protected:
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ValueCondition_h
