#include <precomp.h>

#include "AlphaDivisionIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//AlphaDivisionIteration::AlphaDivisionIteration()
//    : mA(0), mB(0), mMu(0), mLambda(0)
//{
//}// AlphaDivisionIteration

AlphaDivisionIteration::AlphaDivisionIteration(double   a,
                                               double   b,
                                               double   lambda,
                                               double   mu,
                                               bool     mu_better)
    : mA(a), mB(b), mLambda(lambda), mMu(mu), mMuBetter(mu_better)
{
}// AlphaDivisionIteration

AlphaDivisionIteration::~AlphaDivisionIteration()
{
}// ~AlphaDivisionIteration


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ ColumnVector
AlphaDivisionIteration::EndValuePoint() const
{
    ColumnVector end_value_point(1);

    end_value_point(1) = (mA + mB) / 2;
    return end_value_point;
}// EndValuePoint

/* virtual */ vector<IterationData::IterationDataIdType>
AlphaDivisionIteration::ClassIds() const
{
    // Identyfikatory rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), AlphaDivisionIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
AlphaDivisionIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

double
AlphaDivisionIteration::LeftBound() const
{
    return mA;
}// LeftBound

double
AlphaDivisionIteration::RightBound() const
{
    return mB;
}// RightBound

double
AlphaDivisionIteration::LeftStep() const
{
    return mLambda;
}// LeftStep

double
AlphaDivisionIteration::RightStep() const
{
    return mMu;
}// RightStep

double
AlphaDivisionIteration::BetterStep() const
{
    if (mMuBetter)  return mMu;
    else            return mLambda;
}// BetterStep

double
AlphaDivisionIteration::WorseStep() const
{
    if (mMuBetter)  return mLambda;
    else            return mMu;
}// WorseStep

bool
AlphaDivisionIteration::IsMuBetter() const
{
    return mMuBetter;
}// IsMuBetter

double
AlphaDivisionIteration::Length() const
{
    return mB - mA;
}// Length

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
