/** Dane pojedynczej iteracji metody alfa-podziau.
 *
 * @see AlphaDivision, Result
 */

#ifndef _AlphaDivisionIteration_h
#define _AlphaDivisionIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class AlphaDivisionIteration
    : public IterationData
{
public:
// LIFECYCLE

//    /** Konstruktor domylny.
//     */
//    AlphaDivisionIteration(void);

    /** Konstruktor.
     */
    AlphaDivisionIteration(double a,
                           double b,
                           double lambda,
                           double mu,
                           bool   mu_better);

    /** Destruktor.
     */
    ~AlphaDivisionIteration(void);

// OPERATORS

// OPERATIONS


// ACCESS

    virtual ColumnVector  EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    /** Zwraca lewy kraniec przedziau pocztkowego.
     */
    double      LeftBound() const;

    /** Zwraca prawy kraniec przedziau pocztkowego.
     */
    double      RightBound() const;

    /** Zwraca lewy punkt prbny (\f$\lambda\f$).
     */
    double      LeftStep() const;

    /** Zwraca prawy punkt prbny (\f$\mu\f$).
     */
    double      RightStep() const;

    /** Zwraca punkt prbny, w ktrym warto funkcji jest nisza.
     */
    double      BetterStep() const;

    /** Zwraca punkt prbny, w ktrym warto funkcji jest wysza.
     */
    double      WorseStep() const;

    /** Zwraca dugo pocztkowego odcinka.
     */
    double      Length() const;

// INQUIRY

    bool        IsMuBetter() const;

protected:
private:

    double  mA;     ///< Lewy kraniec przedziau pocztkowego.
    double  mB;     ///< Prawy kraniec przedziau pocztkowego.

    double  mLambda;///< Lewy punkt prbny.
    double  mMu;    ///< Prawy punkt prbny.

    bool    mMuBetter;  ///< Czy w punkcie mi warto funkcji jest nisza.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _AlphaDivisionIteration_h
