#include <precomp.h>

#include "DFPIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DFPIteration::DFPIteration()
    : GradientMethodIteration(),
      mD(0),
      mGradientInEndPoint(NullPoint),
      mPointDifference(NullPoint),
      mGradientDifference(NullPoint),
      mNewD(0),
      mNewDirection(NullPoint)
{
}// DFPIteration

DFPIteration::DFPIteration(
        const ColumnVector&     startingPoint,
        const ColumnVector&     gradient,
        const ColumnVector&     direction,
        const SquareMatrix&     D)
    : GradientMethodIteration(startingPoint, gradient, direction),
      mD(D),
      mGradientInEndPoint(NullPoint),
      mPointDifference(NullPoint),
      mGradientDifference(NullPoint),
      mNewD(0),
      mNewDirection(NullPoint)

{
}// DFPIteration

DFPIteration::~DFPIteration()
{
}// ~DFPIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
DFPIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(GradientMethodIteration::ClassIds());

    base_ids.insert(base_ids.begin(), DFPIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
DFPIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

const SquareMatrix&
DFPIteration::D() const
{
    return mD;
}// D

SquareMatrix&
DFPIteration::rD()
{
    return mD;
}// rD

const ColumnVector&
DFPIteration::GradientInEndPoint() const
{
    return mGradientInEndPoint;
}// GradientInEndPoint

ColumnVector&
DFPIteration::rGradientInEndPoint()
{
    return mGradientInEndPoint;
}// rGradientInEndPoint

const ColumnVector&
DFPIteration::PointDifference() const
{
    return mPointDifference;
}// PointDifference

ColumnVector&
DFPIteration::rPointDifference()
{
    return mPointDifference;
}// rPointDifference

const ColumnVector&
DFPIteration::GradientDifference() const
{
    return mGradientDifference;
}// GradientDifference

ColumnVector&
DFPIteration::rGradientDifference()
{
    return mGradientDifference;
}// rGradientDifference

const SquareMatrix&
DFPIteration::NewD() const
{
    return mNewD;
}// NewD

SquareMatrix&
DFPIteration::rNewD()
{
    return mNewD;
}// rNewD

const ColumnVector&
DFPIteration::NewDirection() const
{
    return mNewDirection;
}// NewDirection

ColumnVector&
DFPIteration::rNewDirection()
{
    return mNewDirection;
}// rNewDirection

//============================= INQUIRY    ===================================
bool
DFPIteration::BackToIdentityMatrix() const
{
    return mPointDifference == NullPoint;
}// BackToIdentityMatrix
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
