/** Dane pojedynczej iteracji metody Davidona-Fletchera-Powella.
 *
 * @see GradientMethodIteration, DFP, Result
 */

#ifndef _DFPIteration_h
#define _DFPIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../GradientMethodIteration.h"

// FORWARD REFERENCES
//


class DFPIteration
    : public GradientMethodIteration
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    DFPIteration(void);

     /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param gradient Gradient funkcji w punkcie wyjcia.
     * @param direction Kierunek, w ktrym ma zosta wykonany krok
     * (\f$d\f$).
     * @param D Macierz estymujca odwrotn do
     * hesjanu (\f$D\f$).
     */
    DFPIteration(const ColumnVector&    staringPoint,
                 const ColumnVector&    gradient,
                 const ColumnVector&    direction,
                 const SquareMatrix&    D);

    /** Destruktor.
     */
    ~DFPIteration(void);

// OPERATORS

// OPERATIONS

// ACCESS

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const SquareMatrix&         D() const;
    SquareMatrix&               rD();

    const ColumnVector&         GradientInEndPoint() const;
    ColumnVector&               rGradientInEndPoint();

    const ColumnVector&         PointDifference() const;
    ColumnVector&               rPointDifference();

    const ColumnVector&         GradientDifference() const;
    ColumnVector&               rGradientDifference();

    const SquareMatrix&         NewD() const;
    SquareMatrix&               rNewD();

    const ColumnVector&         NewDirection() const;
    ColumnVector&               rNewDirection();

// INQUIRY
    bool                BackToIdentityMatrix() const;

protected:
private:
    SquareMatrix    mD;                     ///< Macierz aproksymujca
                                            /// odwrotn do hesjanu (\f$D\f$).

    ColumnVector    mGradientInEndPoint;    ///< Gradient w punkcie otrzymanym
                                            /// po wykonaniu kroku.

    ColumnVector    mPointDifference;       ///< Odlego midzy otrzymanym
                                            /// punktem a punktem bazowym
                                            /// (\f$p\f$).

    ColumnVector    mGradientDifference;    ///< Rnica midzy gradientem
                                            /// w punkcie otrzymanym
                                            /// a gradientem w punkcie bazowym
                                            /// (\f$q\f$).

    SquareMatrix    mNewD;                  ///< Nowa macierz \f$D\f$.

    ColumnVector    mNewDirection;          ///< Kierunek nastpnego kroku.

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DFPIteration_h
