/** Metoda pochodnej.
 *
 * Metoda pochodnej jest prost metod rozwizywania problemw jednowymiarowych
 * wykorzystujc warto pochodnej funkcji w punkcie.
 *
 * Do algorytmu wprowadzono usprawnienie dziki ktremu niemoliwe jest
 * zaptlenie si procesu poszukiwa.
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ x^k \f$ - punkt w <i>k</i>-tej iteracji<br>
 * \f$ x_{kand} \f$ - kandydat na punkt w nastpnej iteracji<br>
 * \f$ \alpha^k \f$ - wspczynnik wpywu wartoci pochodnej w punkcie na
 * przesunicie punktu roboczego w <i>k</i>-tej iteracji<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ x^0 \f$ - punkt poczatkowy<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 * \f$ \alpha^0 \f$ - pocztkowa warto \f$ \alpha_i \f$ w kadej iteracji
 * (moe si zmniejsza gdy wykonywane kroki s za dugie).<br>
 * \f$ \gamma \f$ - wspczynnik zmniejszania wspczynnika \f$ \alpha \f$ w
 * przypadku, gdy wykonanie kroku daoby gorszy wynik<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Podstawi \f$ k=0 \f$ oraz \f$ x^1=x^0 \f$.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Podstawi \f$ k=k+1 \f$ oraz \f$ \alpha^k = \alpha^0 \f$.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Obliczy \f$ x_{kand} = x^{k-1} - \alpha^i f'(x^{k-1}) \f$.<br>
 * Jeli \f$ f(x_{kand}) < f(x^k) \f$, podstawi \f$ x^k=x_{kand} \f$ i
 * przej do kroku 5. W przeciwnym wypadku przej do kroku 4.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Zmniejszy \f$ \alpha_i \f$ zgodnie ze wzorem:
 * \f$ \alpha^k = \gamma \alpha^k \f$, a nastpnie przejs do kroku 3.<br>
 * <br>
 * <b>Krok 5</b>:<br>
 * Jeli speniony jest warunek zbienoci, zakoczy dziaanie algorytmu i
 * zwrci \f$ x^k \f$ jako wynik.
 * W przeciwnym wypadku przej do kroku 2.<br>
 * <br>
 *
 * Algorytm zaimplementowano na podstawie pracy dyplomowej:<br>
 * <i>Grabowski J.: Numeryczne metody optymalizacji, miejsce i data obrony:
 * Politechnika Wrocawska, TODO</i>
 *
 * @see Linesearch
 **/

#ifndef _Derivative_h
#define _Derivative_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../SinglePointLineSearch.h"
#include "../ProblemBase.h"
#include "../UnivariateFunction.h"
#include "../ui/DerivativeLineSearchPanel.h"

// FORWARD REFERENCES
//


class Derivative
    : public SinglePointLineSearch
{
public:
    friend class DerivativeLineSearchPanel<Derivative>;

// LIFECYCLE


    /** Destruktor.
     */
    ~Derivative(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    Derivative(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Derivative(const Derivative& from);

    /** Wspczynnik wpywu wartoci pochodnej na pooenie nastpnego odcinka.
     */
    DoubleParameter             mAlpha;
    /** Wspczynnik zmniejszania wartoci mAlpha w przypadku braku zbienoci.
     */
    DoubleParameter             mGamma;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      const ColumnVector&       startingPoint,
                                      bool                      generateResult)
                          const throw(OptimizationError);

    /** Nazwa metody
     */
    static wxString       mName;
    /** Egzemplarz metody
     */
    static Derivative           mDerivative;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Derivative_h_
