/** Dane pojedynczej iteracji metody podziau dychotomicznego.
 *
 * @see DichotomousDivision, Result
 */

#ifndef _DichotomousDivisionIteration_h
#define _DichotomousDivisionIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class DichotomousDivisionIteration
    : public IterationData
{
public:

    typedef enum
    {
        XL,
        XM,
        XR
    } NextMiddleType;

    typedef enum
    {
        LEFT,
        RIGHT
    } BetterPointType;

// LIFECYCLE

    /** Konstruktor.
     */
    DichotomousDivisionIteration(double             a,
                                 double             xL,
                                 double             xM,
                                 double             xR,
                                 double             b);

    /** Destruktor.
     */
    ~DichotomousDivisionIteration(void);

// OPERATORS
// OPERATIONS
// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    /** Zwraca lewy kraniec przedziau pocztkowego.
     */
    double              LeftBound() const;

    /** Zwraca prawy kraniec przedziau pocztkowego.
     */
    double              RightBound() const;

    /** Zwraca lewy punkt prbny (\f$\lambda\f$).
     */
    double              LeftStep() const;

    /** Zwraca rodkowy punkt prbny (rodek przedziau)
     */
    double              MiddleStep() const;

    /** Zwraca prawy punkt prbny (\f$\mu\f$).
     */
    double              RightStep() const;

    /** Ktry punkt zosta lewym kracem przedziau w nastpnej iteracji.
     */
    NextMiddleType      NextMiddle() const;
    NextMiddleType&     rNextMiddle();

    BetterPointType     BetterPoint() const;
    BetterPointType&    rBetterPoint();

    /** Zwraca dugo pocztkowego odcinka.
     */
    double              Length() const;


// INQUIRY

protected:
private:

    double  mA;     ///< Lewy kraniec przedziau pocztkowego.
    double  mXl;    ///< Lewy punkt prbny.
    double  mXm;    ///< rodkowy punkt punkt prbny.
    double  mXr;    ///< Prawy punkt prbny
    double  mB;     ///< Prawy kraniec przedziau pocztkowego.

    /** Ktry punkt zosta lewym kracem przedziau w nastpnej iteracji.
     */
    NextMiddleType  mNextMiddle;
    BetterPointType mBetterPoint;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DichotomousDivisionIteration_h
