#include <precomp.h>

#include "../util.h"
#include "EvolutionaryMethodIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////
//============================= LIFECYCLE ====================================

EvolutionaryMethodIteration::EvolutionaryMethodIteration()
    : mDeceased(),
      mSurvivors(),
      mMutants(),
      mChildren(),
      mStartingBest(NullPoint),
      mStartingAverageChromosome(NullPoint),
      mStartingAverageValue(DBL_MAX),
      mStartingChromosomeStandardDeviation(DBL_MAX),
      mStartingValueStandardDeviation(DBL_MAX),
      mEndBest(NullPoint),
      mEndAverageChromosome(NullPoint),
      mEndAverageValue(DBL_MAX),
      mEndChromosomeStandardDeviation(DBL_MAX),
      mEndValueStandardDeviation(DBL_MAX)
{
}// EvolutionaryMethodIteration

EvolutionaryMethodIteration::~EvolutionaryMethodIteration()
{
}// ~EvolutionaryMethodIteration

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
EvolutionaryMethodIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), EvolutionaryMethodIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
EvolutionaryMethodIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
EvolutionaryMethodIteration::EndValuePoint() const
{
    return mEndBest;
}// EndValue



const EvolutionaryMethodIteration::ChromosomeContainer&
EvolutionaryMethodIteration::Deceased() const
{
    return mDeceased;
}

const EvolutionaryMethodIteration::ChromosomeContainer&
EvolutionaryMethodIteration::Survivors() const
{
    return mSurvivors;
}

const EvolutionaryMethodIteration::ChromosomeContainer&
EvolutionaryMethodIteration::Mutants() const
{
    return mMutants;
}

const EvolutionaryMethodIteration::ChromosomeContainer&
EvolutionaryMethodIteration::Children() const
{
    return mChildren;
}

const ColumnVector&
EvolutionaryMethodIteration::StartingBest() const
{
    return mStartingBest;
}

const ColumnVector&
EvolutionaryMethodIteration::StartingAverageChromosome() const
{
    return mStartingAverageChromosome;
}

double
EvolutionaryMethodIteration::StartingAverageValue() const
{
    return mStartingAverageValue;
}

double
EvolutionaryMethodIteration::StartingChromosomeStandardDeviation() const
{
    return mStartingChromosomeStandardDeviation;
}

double
EvolutionaryMethodIteration::StartingValueStandardDeviation() const
{
    return mStartingValueStandardDeviation;
}

const ColumnVector&
EvolutionaryMethodIteration::EndBest() const
{
    return mEndBest;
}

const ColumnVector&
EvolutionaryMethodIteration::EndAverageChromosome() const
{
    return mEndAverageChromosome;
}

double
EvolutionaryMethodIteration::EndAverageValue() const
{
    return mEndAverageValue;
}

double
EvolutionaryMethodIteration::EndChromosomeStandardDeviation() const
{
    return mEndChromosomeStandardDeviation;
}

double
EvolutionaryMethodIteration::EndValueStandardDeviation() const
{
    return mEndValueStandardDeviation;
}

void
EvolutionaryMethodIteration::AddDeceased(const ColumnVector& deceased)
{
    mDeceased.push_back(deceased);
}

void
EvolutionaryMethodIteration::AddSurvivor(const ColumnVector& survivor)
{
    mSurvivors.push_back(survivor);
}

void
EvolutionaryMethodIteration::AddMutant(const ColumnVector& mutant)
{
    mMutants.push_back(mutant);
}

void
EvolutionaryMethodIteration::AddChild(const ColumnVector& child)
{
    mChildren.push_back(child);
}

void
EvolutionaryMethodIteration::SetStartingStatistics
                            (const ColumnVector& best,
                             const ColumnVector& averageChromosome,
                                   double        averageValue,
                                   double        chromosomeStandardDeviation,
                                   double        valueStandardDeviation)
{
    mStartingBest                           = best;
    mStartingAverageChromosome              = averageChromosome;
    mStartingAverageValue                   = averageValue;
    mStartingChromosomeStandardDeviation    = chromosomeStandardDeviation;
    mStartingValueStandardDeviation         = valueStandardDeviation;
}

void
EvolutionaryMethodIteration::SetEndStatistics
                            (const ColumnVector& best,
                             const ColumnVector& averageChromosome,
                                   double        averageValue,
                                   double        chromosomeStandardDeviation,
                                   double        valueStandardDeviation)
{
    mEndBest                        = best;
    mEndAverageChromosome           = averageChromosome;
    mEndAverageValue                = averageValue;
    mEndChromosomeStandardDeviation = chromosomeStandardDeviation;
    mEndValueStandardDeviation      = valueStandardDeviation;
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
