/** Dane iteracji ewolucyjnej metody optymalizacji
 *
 * @see EvolutionaryMethodDiscrete, Result
 */

#ifndef _EvolutionaryMethodIteration_h
#define _EvolutionaryMethodIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"
#include "../Region.h"

// FORWARD REFERENCES
//


class EvolutionaryMethodIteration
    : public IterationData
{
public:

    typedef std::list<ColumnVector> ChromosomeContainer;

    /** Konstruktor domylny.
     */
    EvolutionaryMethodIteration(void);

    /** Destruktor.
     */
    ~EvolutionaryMethodIteration(void);

// OPERATORS

// OPERATIONS

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();



    const ChromosomeContainer&      Deceased() const;
    const ChromosomeContainer&      Survivors() const;
    const ChromosomeContainer&      Mutants() const;
    const ChromosomeContainer&      Children() const;

    const ColumnVector&             StartingBest() const;
    const ColumnVector&             StartingAverageChromosome() const;
    double                          StartingAverageValue() const;
    double                          StartingChromosomeStandardDeviation() const;
    double                          StartingValueStandardDeviation() const;

    const ColumnVector&             EndBest() const;
    const ColumnVector&             EndAverageChromosome() const;
    double                          EndAverageValue() const;
    double                          EndChromosomeStandardDeviation() const;
    double                          EndValueStandardDeviation() const;

    void                    AddDeceased(const ColumnVector& removedChromosome);
    void                    AddSurvivor(const ColumnVector& removedChromosome);
    void                    AddMutant(const ColumnVector& removedChromosome);
    void                    AddChild(const ColumnVector& childChromosome);

    void                    SetStartingStatistics
                               (const ColumnVector& best,
                                const ColumnVector& averageChromosome,
                                      double        averageValue,
                                      double        chromosomeStandardDeviation,
                                      double        valueStandardDeviation);

    void                    SetEndStatistics
                               (const ColumnVector& best,
                                const ColumnVector& averageChromosome,
                                      double        averageValue,
                                      double        chromosomeStandardDeviation,
                                      double        valueStandardDeviation);

// INQUIRY


protected:
private:

    ChromosomeContainer             mDeceased;
    ChromosomeContainer             mSurvivors;
    ChromosomeContainer             mMutants;
    ChromosomeContainer             mChildren;

    ColumnVector                    mStartingBest;
    ColumnVector                    mStartingAverageChromosome;
    double                          mStartingAverageValue;
    double                          mStartingChromosomeStandardDeviation;
    double                          mStartingValueStandardDeviation;

    ColumnVector                    mEndBest;
    ColumnVector                    mEndAverageChromosome;
    double                          mEndAverageValue;
    double                          mEndChromosomeStandardDeviation;
    double                          mEndValueStandardDeviation;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _EvolutionaryMethodIteration_h
