#include <precomp.h>

#include "../Logger.h"
#include "FibonacciDivision.h"  // class implemented
#include "FibonacciDivisionIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


FibonacciDivision::~FibonacciDivision()
{
}// ~FibonacciDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
FibonacciDivision::Clone() const
{
    return auto_ptr<Method>(new FibonacciDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
FibonacciDivision::Name() const
{
    return FibonacciDivision::mName;
}// Name

/* virtual */ wxString&
FibonacciDivision::rName()
{
    return FibonacciDivision::mName;
}// rName

/* virtual */ MethodIdType
FibonacciDivision::Id() const
{
    return FibonacciDivision::ClassId();
}// Id

/* static */ MethodIdType
FibonacciDivision::ClassId()
{
    const static MethodIdType FibonacciDivisionId = Method::GenerateId();

    return FibonacciDivisionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
FibonacciDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

FibonacciDivision::FibonacciDivision()
    : SectionLineSearch()
{
    //TODO: przywrci?
//    mEpsilon = 0.01;
}// FibonacciDivision

FibonacciDivision::FibonacciDivision(const FibonacciDivision& from)
    : SectionLineSearch(from)
{
}// FibonacciDivision

vector<unsigned int>
FibonacciDivision::GenerateFibonacciSequence(double a,
                                             double b,
                                             double epsilon) const
{
    vector<unsigned int>    result;

    result.push_back(1);
    result.push_back(1);

    while (result.back() < ((b - a) / epsilon))
    {
        result.push_back(result.back() + result[result.size() - 2]);
    }

    return result;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
FibonacciDivision::Solve(const UnivariateFunction&  function,
                         double                     a,
                         double                     b,
                         bool                       generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result>                        ap_result(new Result(1));
    auto_ptr<FibonacciDivisionIteration>    ap_iteration;

    try
    {
        if (a > b)
            swap(a, b);

        unsigned int    step_count = 0;
        double lambda;
        double mu;
        double value_in_lambda;
        double value_in_mu;
        // 3 kolejne, potrzebne w iteracjach elementy ciagu Fibonacciego.
        double el1, el2, el3;
        vector<unsigned int>    fibonacci_sequence;


        fibonacci_sequence = GenerateFibonacciSequence(a, b, mEpsilon);
        const unsigned steps_needed = fibonacci_sequence.size() - 1;

        /* Ponisza asercja jest tosama ze sprawdzaniem czy epsilon jest
         * mniejszy od pocztkowego przedziau.
         */
        assert(fibonacci_sequence.size() >= 3);

        LOG(DEBUG,wxT("Steps needed: ") << steps_needed);

        if (steps_needed == 2)
        {
            lambda          = (a + b) / 2;
            value_in_lambda = function.Evaluate(lambda);
        }


        for (step_count = 1;
             step_count < (steps_needed - 1);
             ++step_count)
        {

            el1 = static_cast<double>
                     (fibonacci_sequence[steps_needed - step_count - 1]);
            el2 = static_cast<double>
                     (fibonacci_sequence[steps_needed - step_count]);
            el3 = static_cast<double>
                     (fibonacci_sequence[steps_needed - step_count + 1]);

            LOG(DEBUG,wxT("Krok nr ") << step_count);
            LOG(DEBUG,wxT("Wspolczynnik do lambdy : ") << el1 / el3);
            LOG(DEBUG,wxT("Wspolczynnik do mu     : ") << el2 / el3);

            if (step_count == 1)
            {
                lambda  = a + ( el1 / el3 ) * (b - a);
                mu      = a + ( el2 / el3 ) * (b - a);

                value_in_lambda = function.Evaluate(lambda);
                value_in_mu     = function.Evaluate(mu);
            }

            ap_iteration.reset(new FibonacciDivisionIteration(a,
                                                              lambda,
                                                              mu,
                                                              b));

            if (value_in_lambda > value_in_mu)
            {
                a               = lambda;
                lambda          = mu;
                value_in_lambda = value_in_mu;

                mu              = a + ( el2 / el3 ) * (b - a);
                value_in_mu     = function.Evaluate(mu);

                ap_iteration->rNextRightBound()
                                    = FibonacciDivisionIteration::B;
            }
            else
            {
                b               = mu;
                mu              = lambda;
                value_in_mu     = value_in_lambda;

                lambda          = a + ( el1 / el3 ) * (b - a);
                value_in_lambda = function.Evaluate(lambda);

                ap_iteration->rNextRightBound()
                                    = FibonacciDivisionIteration::MU;
            }

            ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));



            LOG(DEBUG,wxT("a:      ") << a);
            LOG(DEBUG,wxT("lambda: ") << lambda << wxT(" value = ")
                                            << value_in_lambda);
            LOG(DEBUG,wxT("mu:     ") << mu << wxT(" value = ")
                                            << value_in_mu);
            LOG(DEBUG,wxT("b:      ") << b);

            LOG(DEBUG,wxT("Dlugosc przedzialu: ") << (b - a));

            // TODO: AddStep
            //
            LOG(DEBUG,wxT("-----------------------------------"));

            NotifyYield();
        }


        LOG(DEBUG,wxT("Krok nr ") << step_count);

        /* W kroku rwnym K-1 nie mozemy wygenerowa punktw
         * lambda i mu w normalny sposb, gdy oba
         * punkty znalazyby si w tym samym miejscu na osi (pierwsze dwa
         * elementy cigu Fibonacciego rwnaj si 1.
         */
        mu          = lambda + mEpsilon;
        value_in_mu = function.Evaluate(mu);

        ap_iteration.reset(new FibonacciDivisionIteration(a,
                                                          lambda,
                                                          mu,
                                                          b));

        if (value_in_lambda > value_in_mu)
        {
            a   = lambda;

            ap_iteration->rNextRightBound()
                                = FibonacciDivisionIteration::B;
        }
        else
        {
            b   = lambda;

            ap_iteration->rNextRightBound()
                                = FibonacciDivisionIteration::LAMBDA;
        }

        ap_iteration->rLast() = true;
        ap_result->AddIteration
                    (auto_ptr<const IterationData>(ap_iteration));


        LOG(DEBUG,wxT("a:      ") << a);
        LOG(DEBUG,wxT("lambda: ") << lambda);
        LOG(DEBUG,wxT("mu:     ") << mu);
        LOG(DEBUG,wxT("b:      ") << b);

        LOG(DEBUG,wxT("Dlugosc przedzialu: ") << (b - a));

        LOG(DEBUG,wxT("-----------------------------------"));

        ap_result->rFinalResult().resize(1);
        ap_result->rFinalResult()(1) = (b + a) / 2;
    }// try
    catch (BaseException)
    {// Wyjtek newmata.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    return ap_result;
} // Solve


wxString FibonacciDivision::mName(_("Fibonacci Division"));

// Utwrz i zarejestruj prototyp.
FibonacciDivision FibonacciDivision::mFibonacciDivision;
