#include <precomp.h>

#include "FibonacciDivisionIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FibonacciDivisionIteration::FibonacciDivisionIteration
                                   (double              a,
                                    double              lambda,
                                    double              mu,
                                    double              b)
    : mA(a),
      mLambda(lambda),
      mMu(mu),
      mB(b),
      mLast(false)
{
}// FibonacciDivisionIteration

FibonacciDivisionIteration::~FibonacciDivisionIteration()
{
}// ~FibonacciDivisionIteration


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ ColumnVector
FibonacciDivisionIteration::EndValuePoint() const
{
    ColumnVector end_value_point(1);

    end_value_point(1) = (mA + mB) / 2;
    return end_value_point;
}// EndValuePoint

/* virtual */ vector<IterationData::IterationDataIdType>
FibonacciDivisionIteration::ClassIds() const
{
    // Identyfikatory rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), FibonacciDivisionIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
FibonacciDivisionIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

double
FibonacciDivisionIteration::LeftBound() const
{
    return mA;
}// LeftBound

double
FibonacciDivisionIteration::RightBound() const
{
    return mB;
}// RightBound

double
FibonacciDivisionIteration::Lambda() const
{
    return mLambda;
}// Lambda

double
FibonacciDivisionIteration::Mu() const
{
    return mMu;
}// Mu

double
FibonacciDivisionIteration::BetterPoint() const
{
    switch (mNextRightBound)
    {
        case FibonacciDivisionIteration::LAMBDA : return mLambda;
        case FibonacciDivisionIteration::MU :     return mLambda;
        case FibonacciDivisionIteration::B :      return mMu;
        default : assert(false);
    }
}// BetterPoint

double
FibonacciDivisionIteration::WorsePoint() const
{
    switch (mNextRightBound)
    {
        case FibonacciDivisionIteration::LAMBDA : return mMu;
        case FibonacciDivisionIteration::MU :     return mMu;
        case FibonacciDivisionIteration::B :      return mLambda;
        default : assert(false);
    }
}// WorsePoint

double
FibonacciDivisionIteration::LambdaCoefficient() const
{
    return (mLambda - mA) / (mB - mA);
}// LambdaCoefficient

double
FibonacciDivisionIteration::MuCoefficient() const
{
    return (mMu - mA) / (mB - mA);
}// MuCoefficient

FibonacciDivisionIteration::NextRightBoundType
FibonacciDivisionIteration::NextRightBound() const
{
    return mNextRightBound;
}// NextRightBound

FibonacciDivisionIteration::NextRightBoundType&
FibonacciDivisionIteration::rNextRightBound()
{
    return mNextRightBound;
}// rNextRightBound

bool
FibonacciDivisionIteration::Last() const
{
    return mLast;
}

bool&
FibonacciDivisionIteration::rLast()
{
    return mLast;
}

double
FibonacciDivisionIteration::Length() const
{
    return mB - mA;
}// Length

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
