/** Dane pojedynczej iteracji metody podziau dychotomicznego.
 *
 * @see FibonacciDivision, Result
 */

#ifndef _FibonacciDivisionIteration_h
#define _FibonacciDivisionIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class FibonacciDivisionIteration
    : public IterationData
{
public:

    typedef enum
    {
        LAMBDA,
        MU,
        B
    } NextRightBoundType;

// LIFECYCLE

    /** Konstruktor.
     */
    FibonacciDivisionIteration(double       a,
                               double       lambda,
                               double       mu,
                               double       b);

    /** Destruktor.
     */
    ~FibonacciDivisionIteration(void);

// OPERATORS
// OPERATIONS
// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    /** Zwraca lewy kraniec przedziau pocztkowego.
     */
    double              LeftBound() const;

    /** Zwraca prawy kraniec przedziau pocztkowego.
     */
    double              RightBound() const;

    /** Zwraca lewy punkt prbny (\f$\lambda\f$).
     */
    double              Lambda() const;

    /** Zwraca rodkowy punkt prbny (rodek przedziau)
     */
    double              Mu() const;

    double              BetterPoint() const;
    double              WorsePoint() const;

    double              MuCoefficient() const;
    double              LambdaCoefficient() const;

/*
    PartChosenType      PartChosen() const;
    PartChosenType&     rPartChosen();
*/

    NextRightBoundType  NextRightBound() const;
    NextRightBoundType& rNextRightBound();

    bool                Last() const;
    bool&               rLast();

    /** Zwraca dugo pocztkowego odcinka.
     */
    double              Length() const;

// INQUIRY

protected:
private:

    double  mA;         ///< Lewy kraniec przedziau pocztkowego.
    double  mLambda;    ///< Lewy punkt prbny.
    double  mMu;        ///< Prawy punkt prbny.
    double  mB;         ///< Prawy kraniec przedziau pocztkowego.
    bool    mLast;

    NextRightBoundType  mNextRightBound;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FibonacciDivisionIteration_h
