#include <precomp.h>

#include "HookeJeevesIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HookeJeevesIteration::HookeJeevesIteration()
    : mStartingPoint(),
      mOldBasePoint(),
      mTrialPoints(),
      mBasePoint(),
      mEndPoint(),
      mCancelStep(false),
      mOldTau(0.0),
      mNewTau(0.0)
{
}// HookeJeevesIteration

HookeJeevesIteration::HookeJeevesIteration(
        const ColumnVector& startingPoint)
    : mStartingPoint(startingPoint),
      mOldBasePoint(),
      mTrialPoints(),
      mBasePoint(),
      mEndPoint(),
      mCancelStep(false),
      mOldTau(0.0),
      mNewTau(0.0)
{
}// HookeJeevesIteration

HookeJeevesIteration::~HookeJeevesIteration()
{
}// ~HookeJeevesIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
HookeJeevesIteration::AddTrialPoint(const ColumnVector& point,
                                    bool                better)
{
    mTrialPoints.push_back(TrialPoint(point, better));
}// AddTrialPoint

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
HookeJeevesIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), HookeJeevesIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
HookeJeevesIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
HookeJeevesIteration::EndValuePoint() const
{
    return mEndPoint;
}// EndValue


const ColumnVector&
HookeJeevesIteration::StartingPoint() const
{
    return mStartingPoint;
}// StartingPoint

ColumnVector&
HookeJeevesIteration::rStartingPoint()
{
    return mStartingPoint;
}// rStrartingPoint

const ColumnVector&
HookeJeevesIteration::OldBasePoint() const
{
    return mOldBasePoint;
}// OldBasePoint

ColumnVector&
HookeJeevesIteration::rOldBasePoint()
{
    return mOldBasePoint;
}// rStrartingPoint


const HookeJeevesIteration::TrialPointContainer&
HookeJeevesIteration::TrialPoints() const
{
    return mTrialPoints;
}// TrialPoints

const ColumnVector&
HookeJeevesIteration::BasePoint() const
{
    return mBasePoint;
}// BasePoint

ColumnVector&
HookeJeevesIteration::rBasePoint()
{
    return mBasePoint;
}// rStrartingPoint

ColumnVector
HookeJeevesIteration::StepVector() const
{
    return mBasePoint - mOldBasePoint;
}// StepVector

const ColumnVector&
HookeJeevesIteration::EndPoint() const
{
    return mEndPoint;
}// EndPoint

ColumnVector&
HookeJeevesIteration::rEndPoint()
{
    return mEndPoint;
}// rStrartingPoint


double
HookeJeevesIteration::OldTau() const
{
    assert (mOldTau > 0.0);
    return mOldTau;
}// OldTau

double&
HookeJeevesIteration::rOldTau()
{
    return mOldTau;
}// rOldTau

double
HookeJeevesIteration::NewTau() const
{
    assert (mNewTau > 0.0);
    return mNewTau;
}// NewTau

double&
HookeJeevesIteration::rNewTau()
{
    return mNewTau;
}// rNewTau

bool&
HookeJeevesIteration::rStepCancelled()
{
    return mCancelStep;
}// rIsStepCancelled

//============================= INQUIRY    ===================================

bool
HookeJeevesIteration::IsStepCancelled() const
{
    return mCancelStep;
}// IsStepCancelled

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
