/** Dane pojedynczej iteracji metody Hooke'a Jeevesa.
 *
 * @see HookeJeevesDiscrete, Result
 */

#ifndef _HookeJeevesIteration_h
#define _HookeJeevesIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class HookeJeevesIteration
    : public IterationData
{
public:
    struct TrialPoint
    {
        TrialPoint(const ColumnVector& point, bool better)
            : mPoint(point), mBetter(better)    { };

        ColumnVector    mPoint;
        bool            mBetter;
    };

    /// Kontener punktw prbnych.
    typedef vector<TrialPoint>  TrialPointContainer;

// LIFECYCLE

    /** Konstruktor domylny.
     */
    HookeJeevesIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     */
    HookeJeevesIteration(const ColumnVector& staringPoint);

    /** Destruktor.
     */
    ~HookeJeevesIteration(void);

// OPERATORS

// OPERATIONS

    void                AddTrialPoint(const ColumnVector& point, bool better);


// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const ColumnVector& StartingPoint() const;
    ColumnVector&       rStartingPoint();

    const ColumnVector& OldBasePoint() const;
    ColumnVector&       rOldBasePoint();

    const TrialPointContainer&  TrialPoints() const;

    const ColumnVector& BasePoint() const;
    ColumnVector&       rBasePoint();

    ColumnVector        StepVector() const;

    const ColumnVector& EndPoint() const;
    ColumnVector&       rEndPoint();

    double              OldTau() const;
    double&             rOldTau();
    double              NewTau() const;
    double&             rNewTau();

    bool&               rStepCancelled();

// INQUIRY
    bool                IsStepCancelled() const;

protected:
private:
    ColumnVector    mStartingPoint; ///< Punkt pocztkowy iteracji.
    ColumnVector    mOldBasePoint;  ///< Dotychczasowy punkt bazowy

    TrialPointContainer  mTrialPoints;   ///< Wektor punktw prbnych.

    ColumnVector    mBasePoint;     ///< Wyliczony w tej iteracji punkt bazowy.
    ColumnVector    mEndPoint;      ///< Nowy punkt.

    bool    mCancelStep;    ///< Czy cofnito krok do bazowego?
    double  mOldTau;        ///< Stara warto wsp. tau (tylko, gdy cofnito).
    double  mNewTau;        ///< Nowa warto wsp. tau (tylko, gdy cofnito).
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _HookeJeevesIteration_h
