#include <precomp.h>

#include "MarquardtIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

MarquardtIteration::MarquardtIteration(const ColumnVector&      startingPoint,
                                       const ColumnVector&      gradient,
                                       const SymmetricMatrix&   hessian,
                                             double             firstLambda)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mHessian(hessian),
      mFirstLambda(firstLambda)
{
}// MarquardtIteration

MarquardtIteration::~MarquardtIteration()
{
}// ~MarquardtIteration


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ ColumnVector
MarquardtIteration::EndValuePoint() const
{
    return mTrialPoints.back();
}// EndValuePoint

/* virtual */ vector<IterationData::IterationDataIdType>
MarquardtIteration::ClassIds() const
{
    // Identyfikatory rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), MarquardtIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
MarquardtIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId


const ColumnVector&
MarquardtIteration::StartingPoint() const
{
    return mStartingPoint;
}// LeftBound

const ColumnVector&
MarquardtIteration::Gradient() const
{
    return mGradient;
}// Gradient

const SymmetricMatrix&
MarquardtIteration::Hessian() const
{
    return mHessian;
}// Hessian

const std::vector<ColumnVector>&
MarquardtIteration::MarquardtSteps() const
{
    return mMarquardtSteps;
}// MarquardtSteps

std::vector<ColumnVector>&
MarquardtIteration::rMarquardtSteps()
{
    return mMarquardtSteps;
}// rMarquardtSteps

const std::vector<ColumnVector>&
MarquardtIteration::TrialPoints() const
{
    return mTrialPoints;
}// TrialPoints

std::vector<ColumnVector>&
MarquardtIteration::rTrialPoints()
{
    return mTrialPoints;
}// rTrialPoints

double
MarquardtIteration::FirstLambda() const
{
    return mFirstLambda;
}// FirstLambda

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
