/** Dane pojedynczej iteracji metody podziau dychotomicznego.
 *
 * @see Marquardt, Result
 */

#ifndef _MarquardtIteration_h
#define _MarquardtIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class MarquardtIteration
    : public IterationData
{
public:

// LIFECYCLE

    /** Konstruktor.
     */
    MarquardtIteration(const ColumnVector&      startingPoint,
                       const ColumnVector&      gradient,
                       const SymmetricMatrix&   hessian,
                             double             firstLambda);

    /** Destruktor.
     */
    ~MarquardtIteration(void);

// OPERATORS
// OPERATIONS
// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const ColumnVector&                 StartingPoint() const;

    const ColumnVector&                 Gradient() const;

    const SymmetricMatrix&              Hessian() const;

    const std::vector<ColumnVector>&    MarquardtSteps() const;
    std::vector<ColumnVector>&          rMarquardtSteps();

    const std::vector<ColumnVector>&    TrialPoints() const;
    std::vector<ColumnVector>&          rTrialPoints();

    double                              FirstLambda() const;

// INQUIRY

protected:
private:

    ColumnVector                    mStartingPoint; ///< Punkt startowy iteracji
    ColumnVector                    mGradient; ///< Gradient w punkcie
    SymmetricMatrix                 mHessian; ///< Hesjan w punkcie

    /** Kolejne prbne kroki iteracji metody Marquardta wyl;iczane na podstawie
     * macierzy hesjanu i gradientu. Dodanie ostatniego z krokw do punktu
     * pocztkowego daje nam punkt pocztkowy nastepnej iteracji.
     */
    std::vector<ColumnVector>       mMarquardtSteps;
    /** Punkty prbne w iteracji (ostatni jest zawsze zaakceptowanym)
     */
    std::vector<ColumnVector>       mTrialPoints;

    double                          mFirstLambda;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _MarquardtIteration_h
