#include <precomp.h>

#include "NelderMeadIteration.h"  // class implemented

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================
/*
NelderMeadIteration::NelderMeadIteration()
    : mSimplex()
{
    mReflected  = NullPoint;
    mExpanded   = NullPoint;
    mContracted = NullPoint;
}// NelderMeadIteration
*/
NelderMeadIteration::NelderMeadIteration
                        (const vector<PointWithValue>& initialSimplex)
    : mInitialSimplex(initialSimplex),
      mNewSimplex(),
      mReflected(NullPoint),
      mExpanded(NullPoint),
      mContracted(NullPoint)
{
}// NelderMeadIteration

NelderMeadIteration::~NelderMeadIteration()
{
}// ~NelderMeadIteration

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void
NelderMeadIteration::AddReflected(const PointWithValue& reflected)
{
    mReflected  = reflected;
}

void
NelderMeadIteration::AddExpanded(const PointWithValue& expanded)
{
    mExpanded   = expanded;
}

void
NelderMeadIteration::AddContracted(const PointWithValue& contracted)
{
    mContracted = contracted;
}

void
NelderMeadIteration::AddNewSimplex(const vector<PointWithValue>& simplex)
{
    mNewSimplex = simplex;
}

void
NelderMeadIteration::SetSimplexModification
                        (const SimplexModificationType simplexModification)
{
    mSimplexModification    = simplexModification;
}

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
NelderMeadIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), NelderMeadIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
NelderMeadIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
NelderMeadIteration::EndValuePoint() const
{
    return mInitialSimplex.front().mPoint;
}// EndValue


const vector<PointWithValue>&
NelderMeadIteration::InitialSimplex() const
{
    return  mInitialSimplex;
}

const vector<PointWithValue>&
NelderMeadIteration::NewSimplex() const
{
    return  mNewSimplex;
}

const PointWithValue&
NelderMeadIteration::Reflected() const
{
    return  mReflected;
}

const PointWithValue&
NelderMeadIteration::Expanded() const
{
    return  mExpanded;
}

const PointWithValue&
NelderMeadIteration::Contracted() const
{
    return  mContracted;
}
/*
const std::vector<ColumnVector>
NelderMeadIteration::Shrinked() const
{
    //TODO: Dokoczy jeli to bdzie rzeczywicie tak zrobione.
    vector<ColumnVector>    shrinked_simplex;

    return  shrinked_simplex;
}
*/
NelderMeadIteration::SimplexModificationType
NelderMeadIteration::SimplexModification() const
{
    return  mSimplexModification;
}


//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
