/** Dane pojedynczej iteracji metody Neldera-Meada.
 *
 * @see HookeJeevesDiscrete, Result
 */

#ifndef _NelderMeadIteration_h
#define _NelderMeadIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"
#include "../PointWithValue.h"

// FORWARD REFERENCES
//

class NelderMeadIteration
    : public IterationData
{
public:

    typedef enum
    {
        REFLECTION = 1,
        EXPANSION,
        CONTRACTION,
        SHRINKAGE
    } SimplexModificationType;

// LIFECYCLE

    /** Konstruktor domylny.
     */
    //NelderMeadIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     */
    NelderMeadIteration(const std::vector<PointWithValue>& initialSimplex);

    /** Destruktor.
     */
    ~NelderMeadIteration(void);

// OPERATORS

// OPERATIONS

    void    AddReflected(const PointWithValue&          reflected);
    void    AddExpanded(const PointWithValue&           expanded);
    void    AddContracted(const PointWithValue&         contracted);
    void    AddNewSimplex(const vector<PointWithValue>& simplex);
    void    SetSimplexModification
                        (const SimplexModificationType  simplexModification);

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const std::vector<PointWithValue>&  InitialSimplex() const;
    const std::vector<PointWithValue>&  NewSimplex() const;
    const PointWithValue&               Reflected() const;
    const PointWithValue&               Expanded() const;
    const PointWithValue&               Contracted() const;
    SimplexModificationType             SimplexModification() const;
    //const std::vector<ColumnVector>     Shrinked() const;

// INQUIRY

protected:
private:

    /** Pocztkowy sympleks iteracji.
     */
    std::vector<PointWithValue> mInitialSimplex;
    /** Sympleks kocowy iteracji.
     */
    std::vector<PointWithValue> mNewSimplex;
    PointWithValue              mReflected;
    PointWithValue              mExpanded;
    PointWithValue              mContracted;
    SimplexModificationType     mSimplexModification;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NelderMeadIteration_h
