#include <precomp.h>

#include "NewtonIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NewtonIteration::NewtonIteration(const ColumnVector&     startingPoint,
                                 const ColumnVector&     gradient,
                                 const SymmetricMatrix&  hessian,
                                 const ColumnVector&     endPoint,
                                       bool              stepValid)
    : mStartingPoint(startingPoint),
      mGradient(gradient),
      mHessian(hessian),
      mEndPoint(endPoint),
      mStepValid(stepValid)
{
}// NewtonIteration

NewtonIteration::~NewtonIteration()
{
}// ~NewtonIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
NewtonIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), NewtonIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
NewtonIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
NewtonIteration::EndValuePoint() const
{
    return mEndPoint;
}// EndValue


const ColumnVector&
NewtonIteration::StartingPoint() const
{
    return mStartingPoint;
}// StartingPoint

const ColumnVector&
NewtonIteration::Gradient() const
{
    return mGradient;
}// Gradient

const SymmetricMatrix&
NewtonIteration::Hessian() const
{
    return mHessian;
}// Hessian

const ColumnVector&
NewtonIteration::EndPoint() const
{
    return mEndPoint;
}// EndPoint

bool
NewtonIteration::StepValid() const
{
    return mStepValid;
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
