/** Klasa danych iteracji metody Newtona.
 *
 * @see Result
 */

#ifndef _NewtonIteration_h
#define _NewtonIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"
#include "../util.h"

// FORWARD REFERENCES
//


class NewtonIteration
    : public IterationData
{
public:

// LIFECYCLE

    /** Konstruktor.
     *
     * @param staringPoint Punkt pocztkowy danej iteracji.
     * @param gradient Gradient funkcji w punkcie startingPoint.
     * @param gradient macierz hessianu funkcji w punkcie startingPoint.
     * @param endPoint kocowy punkt danej iteracji.
     */
    NewtonIteration(const ColumnVector&     startingPoint,
                    const ColumnVector&     gradient,
                    const SymmetricMatrix&  hessian,
                    const ColumnVector&     endPoint,
                          bool              stepValid);

    /** Destruktor.
     */
    ~NewtonIteration(void);

// OPERATORS

// OPERATIONS

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();


    const ColumnVector&                 StartingPoint() const;

    const ColumnVector&                 Gradient() const;

    const SymmetricMatrix&              Hessian() const;

    const ColumnVector&                 EndPoint() const;

    bool                                StepValid() const;

// INQUIRY

protected:
private:

    ColumnVector    mStartingPoint; ///< Punkt pocztkowy iteracji.
    ColumnVector    mGradient;      ///< Gradient funkcji w punkcie pocztkowym.
    SymmetricMatrix mHessian;       ///< Hesjan funkcji w punkcie pocztkowym.
    ColumnVector    mEndPoint;      ///< Kocowy punkt iteracji
    bool            mStepValid;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NewtonIteration_h
