/** Metoda Polaka-Ribiere'a.
 *
 * Matoda Polaka-Ribiere'a naley do klasy metod gradientw sprzonych, co
 * oznacza, e do otrzymywania kierunkw sprzonych wykorzystano w niej
 * wartoci skadnikw gradientw.
 *
 * Metoda Polaka-Ribiere'a powinna znale minimum dowolnej funkcji
 * kwadratowej w liczbie krokw co najwyej rwnej wymiarowi problemu.
 *
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ k \f$ - numer aktualnej iteracji<br>
 * \f$x^k\f$ - punkt w \f$k\f$-tej iteracji,<br>
 * \f$d^k\f$ - kierunek poszukiwa w \f$k\f$-tej iteracji,<br>
 * \f$\gamma^k\f$ -wspczynnik zapewniajcy sprzenie nowego
 * kierunku \f$d^k\f$ z poprzednim,<br>
 * \f$ \tau \f$ - dugo kroku<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja celu<br>
 * \f$ x^0 \f$ - punkt startowy<br>
 * \f$ \varepsilon \f$ - wymagana dokadno<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|\nabla f(x^k)\| < \varepsilon\f$</li>
 * <li>\f$\|\nabla f(x^k)\|^2 < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$.</li>
 * </ul>
 * <br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * Oblicz \f$ \nabla f(x^0) \f$.<br>
 * Podstaw \f$ k=0 \f$.<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Jeli speniony jest warunek stopu, to przerwij dziaanie algorytmu i jako
 * wynik przedstaw \f$ x^k \f$. W przeciwnym wypadku przejd do kroku 2.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Jeli \f$ k=0 \f$, przejd do kroku 3. W przeciwnym wypadku przejd do
 * kroku 4.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Podstaw:
 * \f[ d^k = -\nabla(f(x^k)) \f]
 * Przejd do kroku 5.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Podstaw:
 * \f[ d^{(k)} = -\nabla(f(x^k)) + \gamma^{k-1} d^{k-1} \f]
 * gdzie:<br>
 * \f[ \gamma^k = \frac{(\nabla f(x^k) - \nabla f(x^{k-1}))^T
 * \nabla f(x^k)}{\|\nabla f(x^{k-1}))\|^2} \f]
 * Przejd do kroku 5.<br>
 * <br>
 * <b>Krok 5</b>:<br>
 * \f[ x^{k+1} = x^k + \tau d^k , \f]
 * gdzie \f$ \tau \f$ dobierane jest tak, by funkcja \f$ f(x) \f$ osigaa
 * minimum w kierunku \f$ d^k \f$.<br>
 * Podstaw: \f$ k = k+1 \f$.<br>
 * Przejd do kroku 1.<br>
 * <br>
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 */

#ifndef _PolakRibiere_h
#define _PolakRibiere_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../MethodWithLineSearch.h"
#include "../ProblemBase.h"
#include "../ui/MethodWithLineSearchPanel.h"

// FORWARD REFERENCES
//

class PolakRibiere
    : public MethodWithLineSearch
{
public:

    friend class MethodWithLineSearchPanel<PolakRibiere>;

// LIFECYCLE

    /** Destruktor.
     */
    ~PolakRibiere(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
//

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    /** Czy metoda nadaje si do rozwizania zadanego problemu
     */
    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    PolakRibiere(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    PolakRibiere(const PolakRibiere& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Optimize(const ProblemBase& problem) const
                                                    throw(OptimizationError);

    /** Nazwa metody
     */
    static wxString           mName;
    /** Egzemplarz metody
     */
    static PolakRibiere             mPolakRibiere;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _PolakRibiere_h
