#include <precomp.h>

#include "PowellIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PowellIteration::PowellIteration()
    : mNewD(0),
      mStartingPoint(),
      mD(),
      mTrialPoints(),
      mEndPoint(),
      mTrialDistance(),
      mNewDirection(NullPoint),
      mNewPoint()
{
}// PowellIteration

PowellIteration::PowellIteration(
        const ColumnVector& startingPoint,
        const SquareMatrix& D)
    : mNewD(0),
      mStartingPoint(startingPoint),
      mD(D),
      mTrialPoints(),
      mEndPoint(),
      mTrialDistance(),
      mNewDirection(NullPoint),
      mNewPoint()
{
}// PowellIteration

PowellIteration::~PowellIteration()
{
}// ~PowellIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
void
PowellIteration::AddTrialPoint(const ColumnVector&   point,
                               double                tau)
{
    mTrialPoints.push_back(TrialPoint(point, tau));
}// AddTrialPoint

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
PowellIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), PowellIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
PowellIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
PowellIteration::EndValuePoint() const
{
    if (StepInNewDirection())   return mNewPoint.mPoint;
    else                        return mEndPoint;
}// EndValue

const ColumnVector&
PowellIteration::StartingPoint() const
{
    return mStartingPoint;
}// StartingPoint

ColumnVector&
PowellIteration::rStartingPoint()
{
    return mStartingPoint;
}// rStrartingPoint

const SquareMatrix&
PowellIteration::D() const
{
    return mD;
}// D

SquareMatrix&
PowellIteration::rD()
{
    return mD;
}// rD

const PowellIteration::TrialPointContainer&
PowellIteration::TrialPoints() const
{
    return mTrialPoints;
}// TrialPoints

const ColumnVector&
PowellIteration::EndPoint() const
{
    return mEndPoint;
}// EndPoint

ColumnVector&
PowellIteration::rEndPoint()
{
    return mEndPoint;
}// rEndPoint

const ColumnVector&
PowellIteration::TrialDistance() const
{
    return mTrialDistance;
}// TrialDistance

ColumnVector&
PowellIteration::rTrialDistance()
{
    return mTrialDistance;
}// rTrialDistance

const ColumnVector&
PowellIteration::NewDirection() const
{
    return mNewDirection;
}// NewDirection

ColumnVector&
PowellIteration::rNewDirection()
{
    return mNewDirection;
}// rNewDirection

const PowellIteration::TrialPoint&
PowellIteration::NewPoint() const
{
    return mNewPoint;
}// NewPoint

PowellIteration::TrialPoint&
PowellIteration::rNewPoint()
{
    return mNewPoint;
}// rNewPoint

const SquareMatrix&
PowellIteration::NewD() const
{
    return mNewD;
}// NewD

SquareMatrix&
PowellIteration::rNewD()
{
    return mNewD;
}// rNewD

//============================= INQUIRY    ===================================
bool
PowellIteration::PreliminaryIteration() const
{
    return (mTrialPoints.size() == 1);
}// BaseChanged

bool
PowellIteration::StepInNewDirection() const
{
    return (mNewDirection != NullPoint);
}// BaseChanged

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
