/** Klasa bazowa dla klas danych iteracji metod Powella.
 *
 * @see PowellVariant1, PowellVariant2, Result
 */

#ifndef _PowellIteration_h
#define _PowellIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"
#include "../util.h"

// FORWARD REFERENCES
//


class PowellIteration
    : public IterationData
{
public:

// LIFECYCLE

    /** Konstruktor domylny.
     */
    PowellIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param D Baza kierunkw, wzdu ktrych wykonywane bd
     * kroki prbne w tej iteracji.
     */
    PowellIteration(const ColumnVector& staringPoint,
                    const SquareMatrix& D);

    /** Destruktor.
     */
    ~PowellIteration(void);

struct TrialPoint
    {
        TrialPoint(void)
            : mPoint(NullPoint),
              mTau(0.0)
              { };

        TrialPoint(const ColumnVector&      point,
                   double                   tau)
            : mPoint(point),
              mTau(tau)
              { };

        ColumnVector    mPoint;
        double          mTau;
    };

    /// Kontener punktw prbnych.
    typedef vector<TrialPoint>  TrialPointContainer;

// OPERATORS

// OPERATIONS
    void                        AddTrialPoint(const ColumnVector&   point,
                                              double                tau);

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const ColumnVector&         StartingPoint() const;
    ColumnVector&               rStartingPoint();

    const SquareMatrix&         D() const;
    SquareMatrix&               rD();

    const TrialPointContainer&  TrialPoints() const;

    const ColumnVector&         EndPoint() const;
    ColumnVector&               rEndPoint();

    const ColumnVector&         TrialDistance() const;
    ColumnVector&               rTrialDistance();

    const ColumnVector&         NewDirection() const;
    ColumnVector&               rNewDirection();

    const TrialPoint&           NewPoint() const;
    TrialPoint&                 rNewPoint();

    const SquareMatrix&         NewD() const;
    SquareMatrix&               rNewD();

// INQUIRY
    bool                        PreliminaryIteration() const;

    bool                        StepInNewDirection() const;

protected:
    SquareMatrix    mNewD;          ///< Nowa baza kierunkw.

private:
    ColumnVector    mStartingPoint; ///< Punkt pocztkowy iteracji.
    SquareMatrix    mD;             ///< Baza kierunkw dla iteracji.

    TrialPointContainer  mTrialPoints;   ///< Wektor punktw prbnych.

    ColumnVector    mEndPoint;   ///< Ostatni z punktw prbnych.

    ColumnVector    mTrialDistance; ///< Odlego midzy ostatnim punktem
                                    /// prbnym a punktem bazowym.

    ColumnVector    mNewDirection;  ///< Nowy kierunek sprzony.
    TrialPoint      mNewPoint;      ///< Punkt otrzymany w wyniku kroku
                                    ///  wzdu nowego kierunku.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockIteration_h
