#include <precomp.h>

#include "QuadraticApproximationIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//QuadraticApproximationIteration::QuadraticApproximationIteration()
//    : mA(0), mB(0), mMu(0), mLambda(0)
//{
//}// QuadraticApproximationIteration

QuadraticApproximationIteration::QuadraticApproximationIteration(
                                               double   a,
                                               double   b,
                                               double   m,
                                               double   x,
                                               bool     xBetter)
    : mA(a), mB(b), mM(m), mX(x), mXBetter(xBetter)
{
}// QuadraticApproximationIteration

QuadraticApproximationIteration::~QuadraticApproximationIteration()
{
}// ~QuadraticApproximationIteration


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ ColumnVector
QuadraticApproximationIteration::EndValuePoint() const
{
    ColumnVector end_value_point(1);

    end_value_point(1) = BetterStep();

    return end_value_point;
}// EndValuePoint

/* virtual */ vector<IterationData::IterationDataIdType>
QuadraticApproximationIteration::ClassIds() const
{
    // Identyfikatory rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), QuadraticApproximationIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
QuadraticApproximationIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

double
QuadraticApproximationIteration::LeftBound() const
{
    return mA;
}// LeftBound

double
QuadraticApproximationIteration::RightBound() const
{
    return mB;
}// RightBound

double
QuadraticApproximationIteration::TrialPoint() const
{
    return mM;
}// TrialPoint

double
QuadraticApproximationIteration::ApproximationStationaryPoint() const
{
    return mX;
}// ApproximationStationaryPoint

QuadraticApproximationIteration::NextSectionType
QuadraticApproximationIteration::NextSection() const
{
    return mNextSection;
}// NextSection

QuadraticApproximationIteration::NextSectionType&
QuadraticApproximationIteration::rNextSection()
{
    return mNextSection;
}// rNextSection

bool
QuadraticApproximationIteration::XIsBetter() const
{
    return mXBetter;
}// XIsBetter

double
QuadraticApproximationIteration::BetterStep() const
{
    if (mXBetter)   return mX;
    else            return mM;
}// BetterStep

double
QuadraticApproximationIteration::WorseStep() const
{
    if (mXBetter)   return mM;
    else            return mX;
}// WorseStep

double
QuadraticApproximationIteration::Length() const
{
    return mB - mA;
}// Length

//============================= INQUIRY    ===================================
bool
QuadraticApproximationIteration::XOutOfBounds() const
{
    return (mX < mA || mX > mB);
}// XOutOfBounds

bool
QuadraticApproximationIteration::Succeeded() const
{
    return mNextSection != NONE_ERROR;
}// Succeeded

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
