/** Dane pojedynczej iteracji metody aproksymacji kwadratowej.
 *
 * @see QuadraticApproximation, Result
 */

#ifndef _QuadraticApproximationteration_h
#define _QuadraticApproximationIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class QuadraticApproximationIteration
    : public IterationData
{
public:
    /** Informacja o nastpnym odcinku.
     */
    typedef enum {
        AM,
        XB,
        MB,
        AX,
        NONE_OK,    ///< Nie wyznaczamy nastpnego odcinka, speniony
                    /// warunek stopu z odlegoci x od jednego z
                    /// punktw prbnych a, b, m.
        NONE_ERROR  ///< Nie wyznaczamy nastpnego odcinka, znaleziono
                    /// maksimum lub punkt poza przedziaem.
    } NextSectionType;


    // LIFECYCLE

    /** Konstruktor.
     */
    QuadraticApproximationIteration(double a,
                                    double b,
                                    double m,
                                    double x,
                                    bool   xBetter);

    /** Destruktor.
     */
    ~QuadraticApproximationIteration(void);

// OPERATORS

// OPERATIONS


// ACCESS

    virtual ColumnVector  EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    /** Zwraca lewy kraniec przedziau pocztkowego.
     */
    double      LeftBound() const;

    /** Zwraca prawy kraniec przedziau pocztkowego.
     */
    double      RightBound() const;

    /** Zwraca wewntrzny punkt prbny (\f$m\f$).
     */
    double      TrialPoint() const;

    /** Zwraca punkt zerowania si pierwszej pochodnej funkcji
     * aproksymujcej (\f$x\f$).
     */
    double      ApproximationStationaryPoint() const;

    /** Zwraca informacj o nastpnym odcinku.
     */
    NextSectionType     NextSection() const;

    /** Zwraca referencj na informacj o nastpnym odcinku.
     */
    NextSectionType&    rNextSection();

    /** Zwraca odpowied, czy lepszym punktem jest x.
     */
    bool        XIsBetter() const;

    /** Zwraca punkt prbny, w ktrym warto funkcji jest nisza.
     */
    double      BetterStep() const;

    /** Zwraca punkt prbny, w ktrym warto funkcji jest wysza.
     */
    double      WorseStep() const;

    /** Zwraca dugo pocztkowego odcinka.
     */
    double      Length() const;

// INQUIRY
    /** Zwraca odpowied, czy x jest poza badanym przedziaem.
     */
    bool        XOutOfBounds() const;

    /** Zwraca odpowied, czy iteracja zakoczya si
     * powodzeniem (znaleziono minimum i znajduje si
     * ono w badanym przedziale).
     */
    bool        Succeeded() const;

protected:
private:

    double  mA;     ///< Lewy kraniec przedziau pocztkowego.
    double  mB;     ///< Prawy kraniec przedziau pocztkowego.

    double  mM;     ///< Lewy punkt prbny.
    double  mX;     ///< Prawy punkt prbny.

    bool    mXBetter;  ///< Czy w punkcie x warto funkcji jest nisza.

    NextSectionType mNextSection;   ///< Informacja o nastpnym odcinku.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _QuadraticApproximationIteration_h
