#include <precomp.h>

#include "RosenbrockDiscreteIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RosenbrockDiscreteIteration::RosenbrockDiscreteIteration()
    : RosenbrockIteration(),
      mTrialPoints(),
      mTauMin(0.0)
{
}// RosenbrockDiscreteIteration

RosenbrockDiscreteIteration::RosenbrockDiscreteIteration(
        const ColumnVector& startingPoint,
        const SquareMatrix& D)
    : RosenbrockIteration(startingPoint, D),
      mTrialPoints(),
      mTauMin(0.0)
{
}// RosenbrockDiscreteIteration

RosenbrockDiscreteIteration::~RosenbrockDiscreteIteration()
{
}// ~RosenbrockDiscreteIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
void
RosenbrockDiscreteIteration::AddTrialPoint(const ColumnVector&   point,
                                           bool                  better,
                                           double                tau,
                                           double                newTau)
{
    mTrialPoints.push_back(TrialPoint(point, better, tau, newTau));
}// AddTrialPoint

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
RosenbrockDiscreteIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(RosenbrockIteration::ClassIds());

    base_ids.insert(base_ids.begin(), RosenbrockDiscreteIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
RosenbrockDiscreteIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

const RosenbrockDiscreteIteration::TrialPointContainer&
RosenbrockDiscreteIteration::TrialPoints() const
{
    return mTrialPoints;
}// TrialPoints

double
RosenbrockDiscreteIteration::TauMin() const
{
    return mTauMin;
}// TauMin

double&
RosenbrockDiscreteIteration::rTauMin()
{
    return mTauMin;
}// rTauMin

//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
