/** Dane pojedynczej iteracji metody Rosenbrocka z krokiem dyskretnym.
 *
 * @see RosenbrockIteration, RosenbrockDiscrete, Result
 */

#ifndef _RosenbrockDiscreteIteration_h
#define _RosenbrockDiscreteIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "RosenbrockIteration.h"

// FORWARD REFERENCES
//


class RosenbrockDiscreteIteration
    : public RosenbrockIteration
{
public:
    struct TrialPoint
    {
        TrialPoint(const ColumnVector&      point,
                   bool                     better,
                   double                   tau,
                   double                   newTau)
            : mPoint(point),
              mBetter(better),
              mTau(tau),
              mNewTau(newTau)
              { };

        ColumnVector    mPoint;
        bool            mBetter;
        double          mTau;
        double          mNewTau;
    };

    /// Kontener punktw prbnych.
    typedef vector<TrialPoint>  TrialPointContainer;

// LIFECYCLE

    /** Konstruktor domylny.
     */
    RosenbrockDiscreteIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param D Baza kierunkw, wzdu ktrych wykonywane bd
     * kroki prbne w tej iteracji.
     */
    RosenbrockDiscreteIteration(const ColumnVector& staringPoint,
                                const SquareMatrix& D);

    /** Destruktor.
     */
    ~RosenbrockDiscreteIteration(void);

// OPERATORS

// OPERATIONS

    void                AddTrialPoint(const ColumnVector&   point,
                                      bool                  better,
                                      double                tau,
                                      double                newTau);

// ACCESS

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const TrialPointContainer&  TrialPoints() const;

    double              TauMin() const;
    double&             rTauMin();

// INQUIRY

protected:
private:
    TrialPointContainer  mTrialPoints;   ///< Wektor punktw prbnych.

    double  mTauMin;        ///< Najmniejsza (po wszystkich kierunkach)
                            /// dugo kroku dla nastpnej iteracji.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockDiscreteIteration_h
