#include <precomp.h>

#include "RosenbrockIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RosenbrockIteration::RosenbrockIteration()
    : mStartingPoint(),
      mD(),
      mLambda(),
      mEndPoint(),
      mA(0),
      mNewD(0)
{
}// RosenbrockIteration

RosenbrockIteration::RosenbrockIteration(
        const ColumnVector& startingPoint,
        const SquareMatrix& D)
    : mStartingPoint(startingPoint),
      mD(D),
      mLambda(),
      mEndPoint(),
      mA(0),
      mNewD(0)
{
}// RosenbrockIteration

RosenbrockIteration::~RosenbrockIteration()
{
}// ~RosenbrockIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
RosenbrockIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), RosenbrockIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
RosenbrockIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
RosenbrockIteration::EndValuePoint() const
{
    return mEndPoint;
}// EndValue


const ColumnVector&
RosenbrockIteration::StartingPoint() const
{
    return mStartingPoint;
}// StartingPoint

ColumnVector&
RosenbrockIteration::rStartingPoint()
{
    return mStartingPoint;
}// rStrartingPoint

const SquareMatrix&
RosenbrockIteration::D() const
{
    return mD;
}// D

SquareMatrix&
RosenbrockIteration::rD()
{
    return mD;
}// rD

const ColumnVector&
RosenbrockIteration::Lambda() const
{
    return mLambda;
}// Lambda

ColumnVector&
RosenbrockIteration::rLambda()
{
    return mLambda;
}// rLambda

const ColumnVector&
RosenbrockIteration::EndPoint() const
{
    return mEndPoint;
}// EndPoint

ColumnVector&
RosenbrockIteration::rEndPoint()
{
    return mEndPoint;
}// rEndPoint


const SquareMatrix&
RosenbrockIteration::A() const
{
    return mA;
}// A

SquareMatrix&
RosenbrockIteration::rA()
{
    return mA;
}// rA

const SquareMatrix&
RosenbrockIteration::NewD() const
{
    return mNewD;
}// NewD

SquareMatrix&
RosenbrockIteration::rNewD()
{
    return mNewD;
}// rNewD

//============================= INQUIRY    ===================================
bool
RosenbrockIteration::BaseChanged() const
{
    return (mNewD.ncols() != 0);
}// BaseChanged

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
