/** Klasa bazowa dla klas danych iteracji metod Rosenbrocka z krokiem
 * dyskretnym i optymalnym.
 *
 * @see RosenbrockDiscrete, RosenbrockOptimal, Result
 */

#ifndef _RosenbrockIteration_h
#define _RosenbrockIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../IterationData.h"

// FORWARD REFERENCES
//


class RosenbrockIteration
    : public IterationData
{
public:

// LIFECYCLE

    /** Konstruktor domylny.
     */
    RosenbrockIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param D Baza kierunkw, wzdu ktrych wykonywane bd
     * kroki prbne w tej iteracji.
     */
    RosenbrockIteration(const ColumnVector& staringPoint,
                        const SquareMatrix& D);

    /** Destruktor.
     */
    ~RosenbrockIteration(void);

// OPERATORS

// OPERATIONS

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const ColumnVector& StartingPoint() const;
    ColumnVector&       rStartingPoint();

    const SquareMatrix& D() const;
    SquareMatrix&       rD();

    const ColumnVector& Lambda() const;
    ColumnVector&       rLambda();

    const ColumnVector& EndPoint() const;
    ColumnVector&       rEndPoint();

    const SquareMatrix& A() const;
    SquareMatrix&       rA();

    const SquareMatrix& NewD() const;
    SquareMatrix&       rNewD();

// INQUIRY
    bool                BaseChanged() const;

protected:
private:
    ColumnVector    mStartingPoint; ///< Punkt pocztkowy iteracji.
    SquareMatrix    mD;             ///< Baza kierunkw dla iteracji.

    ColumnVector    mLambda;     ///< Wektor lambda po wykonaniu
                                 ///  wszystkich krokw prbnych.
    ColumnVector    mEndPoint;   ///< Najlepszy punkt znaleziony w iteracji.

    SquareMatrix    mA;         ///< Macierz kierunkw poprawy.
    SquareMatrix    mNewD;      ///< Nowa baza kierunkw.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockIteration_h
