/** Dane pojedynczej iteracji metody Rosenbrocka z krokiem optymalnym.
 *
 * @see RosenbrockIteration, RosenbrockOptimal, Result
 */

#ifndef _RosenbrockOptimalIteration_h
#define _RosenbrockOptimalIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "RosenbrockIteration.h"

// FORWARD REFERENCES
//


class RosenbrockOptimalIteration
    : public RosenbrockIteration
{
public:
    struct TrialPoint
    {
        TrialPoint(const ColumnVector&      point,
                   double                   tau)
            : mPoint(point),
              mTau(tau)
              { };

        ColumnVector    mPoint;
        double          mTau;
    };

    /// Kontener punktw prbnych.
    typedef vector<TrialPoint>  TrialPointContainer;

// LIFECYCLE

    /** Konstruktor domylny.
     */
    RosenbrockOptimalIteration(void);

    /** Konstruktor.
     *
     * @param staringPoint Punkt wyjcia dla tej iteracji.
     * @param D Baza kierunkw, wzdu ktrych wykonywane bd
     * kroki prbne w tej iteracji.
     */
    RosenbrockOptimalIteration(const ColumnVector& staringPoint,
                                const SquareMatrix& D);

    /** Destruktor.
     */
    ~RosenbrockOptimalIteration(void);

// OPERATORS

// OPERATIONS

    void                AddTrialPoint(const ColumnVector&   point,
                                      double                tau);

// ACCESS

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();

    const TrialPointContainer&  TrialPoints() const;

// INQUIRY

protected:
private:
    TrialPointContainer  mTrialPoints;   ///< Wektor punktw prbnych.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockOptimalIteration_h
