/** Dane pojedynczej iteracji metody Simplex.
 *
 * @see Simplex, Result
 */

#ifndef _SimplexIteration_h
#define _SimplexIteration_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../AugmentedForm.h"
#include "../IterationData.h"

// FORWARD REFERENCES
//


class SimplexIteration
    : public IterationData
{
public:

// LIFECYCLE
    typedef enum {
        FIRST_TRIVIAL,
        FIRST_EXTENDED,
        FIRST_MAINPHASE,
        LAST_EXTENDED,
        OTHER
    } SimplexIterationType;

    /** Konstruktor domylny.
     */
    //SimplexIteration(void);

    /** Konstruktor.
     *
     * @param cpMainProblemData CountedPtr na posta standardow problemu
     * waciwego.
     * @param cpExtendedProblemData CountedPtr na posta standardow
     * problemu rozszerzonego (dla iteracji fazy waciwej na NULL).
     * @param iB Indeksy kolumn bazowych.
     * @param iN Indeksy kolumn niebazowych.
     */
    SimplexIteration(CountedPtr<const AugmentedForm>    cpMainProblemData,
                     CountedPtr<const AugmentedForm>    cpExtendedProblemData,
                     const vector<int>&                 iB,
                     const vector<int>&                 iN);

    /** Destruktor.
     */
    ~SimplexIteration(void);

// OPERATORS

// OPERATIONS
    void AddMatricesData(const SquareMatrix&        B,
                         const Matrix&              N);

    void AddReducedCostData(const ColumnVector&     d,
                            int                     s,
                            int                     prevS);

    void AddAllowedShiftData(const ColumnVector&    h,
                             int                    q,
                             int                    prevQ,
                             const ColumnVector&    sigma);

// ACCESS

    virtual ColumnVector                EndValuePoint() const;

    virtual vector<IterationDataIdType> ClassIds() const;

    static IterationDataIdType          ClassId();


    SimplexIterationType    Type() const;
    SimplexIterationType&   rType();


    const Matrix&       MainA() const;

    const ColumnVector& MainB() const;

    const ColumnVector& MainC() const;

    const Matrix&       ExtendedA() const;

    const ColumnVector& ExtendedB() const;

    const ColumnVector& ExtendedC() const;


    const vector<int>&  BasicIndices() const;

    const vector<int>&  NonbasicIndices() const;

    const SquareMatrix& Base() const;
    const Matrix&       N() const;

    const ColumnVector& X() const;
    ColumnVector&       rX();


    const ColumnVector& D() const;

    int            S() const;

    int            PreviousS() const;


    const ColumnVector& H() const;

    const ColumnVector& Sigma() const;

    int                 Q() const;

    int                 PreviousQ() const;


    double              Alpha() const;

    const ColumnVector& NewX() const;
    ColumnVector&       rNewX();

// INQUIRY
    bool                MainPhase() const;

    ColumnVector        ActualPoint() const;

    ColumnVector        FullPoint() const;

    int                 ActualVariables() const;
protected:
private:
    /** Posta standardowa problemu waciwego.
     */
    CountedPtr<const AugmentedForm> mcpMainProblemData;

    /** Posta standardowa problemu rozszerzonego.
     */
    CountedPtr<const AugmentedForm> mcpExtendedProblemData;

    SimplexIterationType    mType;  ///< Typ iteracji.

    vector<int>     miB;            ///< Indeksy kolumn bazowych.
    vector<int>     miN;            ///< Indeksy kolumn niebazowych.

    SquareMatrix    mB;             ///< Bazowe kolumny macierzy A.
    Matrix          mN;             ///< Niebazowe kolumny macierzy A.

    ColumnVector    mX;             ///< Bazowe wsprzdne biecego punktu.

    ColumnVector    mD;             ///< Wektor kosztw ograniczonych.
    int             mS;             ///< Indeks zmiennej wprowadzanej do bazy.
    int             mPrevS;         ///< Indeks zmiennej wprowadzonej do bazy
                                    /// w poprzedniej iteracji.

    ColumnVector    mH;             ///< Wektor przesunicia.
    ColumnVector    mSigma;         ///< Wektor maksymalnych dopuszczalnych
                                    /// dugoci kroku.
    int             mQ;             ///< Indeks zmiennej wyprowadzanej z bazy.
    int             mPrevQ;         ///< Indeks zmiennej wyprowadzonej z bazy
                                    /// w poprzedniej iteracji.

    ColumnVector    mNewX;          ///< Nastpny punkt bazowy.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SimplexIteration_h
