#include <precomp.h>


#include "../Configuration.h"
#include "../Logger.h"
#include "UniformDivision.h"  // class implemented
#include "UniformDivisionIteration.h"

LOGGER(INFO);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


UniformDivision::~UniformDivision()
{
}// ~UniformDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
UniformDivision::Clone() const
{
    return auto_ptr<Method>(new UniformDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
UniformDivision::Name() const
{
    return UniformDivision::mName;
}// Name

/* virtual */ wxString&
UniformDivision::rName()
{
    return UniformDivision::mName;
}// rName

/* virtual */ MethodIdType
UniformDivision::Id() const
{
    return UniformDivision::ClassId();
}// Id

/* static */ MethodIdType
UniformDivision::ClassId()
{
    const static MethodIdType UniformDivisionId = Method::GenerateId();

    return UniformDivisionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
UniformDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

UniformDivision::UniformDivision()
    : SectionLineSearch()
{
    mEpsilon = 0.01;
}// UniformDivision

UniformDivision::UniformDivision(const UniformDivision& from)
    : SectionLineSearch(from)
{
}// UniformDivision

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
UniformDivision::Solve(const UnivariateFunction&    function,
                       double                       a,
                       double                       b,
                       bool                         generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));
    if (a > b) swap(a, b);

    double x = a;
    double x_min = x;
    double value;
    double min = DBL_MAX;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;

    try
    {
        x = a;

        while (x <= b && step_count < STEP_LIMIT)
        {
            ++step_count;

            value = function.Evaluate(x);

            if (generateResult)
            {
                ap_result->AddIteration(auto_ptr<const IterationData>(
                            new UniformDivisionIteration(a, b, x)));
            }

            if (value < min)
            {
                min = value;
                x_min = x;
            }

            x += mEpsilon;

            NotifyYield();
        } // while
    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }

    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }

    ColumnVector final_result(1);
    final_result(1) = x_min;
    ap_result->rFinalResult() = final_result;

    return ap_result;
} // Solve

wxString UniformDivision::mName(_("Uniform Division"));

// Utwrz i zarejestruj prototyp.
UniformDivision UniformDivision::mUniformDivision;
