/**  Metoda podziau rwnomiernego.
 *
 * Metoda optymalizacji w kierunku polegajca na stopniowym
 * skracaniu zadanego odcinka (patrz SectionLineSearch).
 *
 * Dokonuje przegldu zupenego punktw lecych na odcinku \f$[a, b]\f$,
 * z dokadnoci do \f$\varepsilon\f$.
 *
 * Wynikiem jest oczywicie ten z punktw, dla ktrego warto funkcji
 * jest najmniejsza.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$f(x)\f$ - minimalizowana funkcja jednej zmiennej,<br>
 * \f$a\f$ - lewy kraniec przedziau poszukiwa,<br>
 * \f$b\f$ - prawy kraniec przedziau poszukiwa,<br>
 * \f$\varepsilon\f$ - wymagana dokadno (dugo kroku).<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$x^k\f$ - aktualny punkt prbny,<br>
 * \f$x_{min}\f$ - najlepszy z dotychczas zbadanych punktw prbnych,<br>
 * \f$F_{min}\f$ - warto funkcji w punkcie \f$x_{min}\f$.
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ x^1 = a , \f]
 * \f[ x_{min} = x^1 , \f]
 * \f[ F_{min} = f(x^1). \f]
 *
 * <br><b>Krok 1:</b><br>
 * Podstawiamy:
 * \f[ x^{k+1} = x^k + \varepsilon . \f]
 * Sprawdzamy warunek stopu:<br>
 * Jeli \f$ x^{k+1} > b \f$, to
 * STOP. W przeciwnym wypadku przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Jeli \f$ f(x^{k+1}) < F_{min} \f$, podstawiamy:
 * \f[ x_{min} = x^{k+1} , \f]
 * \f[ F_{min} = f(x^{k+1}). \f]
 * Podstawiamy \f$k = k + 1\f$ i przechodzimy do kroku 1.
 *
 */

#ifndef _UniformDivision_h
#define _UniformDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"

// FORWARD REFERENCES
//


class UniformDivision
    : public SectionLineSearch
{
public:

    friend class UniformDivisionPanel;

// LIFECYCLE


    /** Destruktor.
     */
    ~UniformDivision(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* problem) const;

protected:

    /** Konstruktor domylny.
     */
    UniformDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    UniformDivision(const UniformDivision& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);

    static wxString      mName;                   ///< Nazwa metody.
    static UniformDivision mUniformDivision;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _UniformDivision_h_
