#include <precomp.h>

#include <newmat/newmatrc.h>        // Nie ma w precomp.h, uywany tylko tu
#include <wx/tokenzr.h>

#include "Logger.h"
#include "Symbols.h"
#include "util.h"
#include "text-util.h"

LOGGER(DEBUG);

// To mona byo zrobi duo prociej na dwch tablicach stringw: opisw
// i wartoci. Oszczdzioby si szukania ':', ew. problemw z dwoma dwukropkami
// w tekcie (albo brakiem) itd.
wxString
AlignedParameterDescriptions(const wxArrayString& descriptions)
{
    size_t count;
    size_t i;
    int length;
    int max = 0;
    wxStringTokenizer tokenizer;
    wxString token;
    wxString description;

    count = descriptions.GetCount();
    for (i = 0; i < count; ++i)
    {
        wxString item = descriptions.Item(i);

        length = item.Find(wxT(':'));
        if (length > max)     max = length;
    }

    for (i = 0; i < count; ++i)
    {
        tokenizer.SetString(descriptions.Item(i), wxT(":"));

        token = tokenizer.GetNextToken();
        token << wxT(':');
        token = token.Pad(max - token.Length() + 1);
        description << token;

        token = tokenizer.GetNextToken();
        description << token;
    }

    return description;
}// AlignedParameterDescriptions

wxString
AlphaDescriptionAD(const DoubleParameter& alpha)
{
    wxString description;

    description << _("Section ratio (")
                << Symbols::GetInstance()[Symbols::ALPHA] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(alpha))
                << wxT('\n');

   return description;
}// AlphaDescriptionAD

wxString
AlphaDescriptionDerivative(const DoubleParameter& alpha)
{
    wxString description;

    description << _("Initial point shift factor (")
                << Symbols::GetInstance()[Symbols::ALPHA] << wxT(" 0): ")
                << wxString::Format(wxT("%g"), static_cast<double>(alpha))
                << wxT('\n');

   return description;
}// AlphaDescriptionDerivative

wxString
AlphaDescriptionRD(const DoubleParameter& alpha)
{
    wxString description;

    description << _("Expansion factor (")
                << Symbols::GetInstance()[Symbols::ALPHA]
                << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(alpha))
                << wxT('\n');

   return description;
}// AlphaDescriptionRD


wxString
BetaDescriptionHJD(const DoubleParameter& beta)
{
    wxString description;

    description << _("Contraction factor (")
                << Symbols::GetInstance()[Symbols::BETA] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(beta))
                << wxT('\n');

   return description;
}// BetaDescriptionHJD


wxString
BetaDescriptionRD(const DoubleParameter& beta)
{
    wxString description;

    description << _("Contraction factor (")
                << Symbols::GetInstance()[Symbols::BETA]
                << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(beta))
                << wxT('\n');

   return description;
}// BetaDescriptionRD

wxString
CenterDescriptionNM(const ColumnVector& center)
{
    wxString description;

    description << _("Starting simplex center: ")
                << center
                << wxT('\n');
    return description;
}// CenterDescriptionNM


wxString
ContractionDescriptionNM(const DoubleParameter& contractionFactor)
{
    wxString description;

    description << _("Contraction factor (c): ")
                << wxString::Format(wxT("%g"), static_cast<double>(
                                                    contractionFactor))
                << wxT('\n');
    return description;
}// ContractionDescriptionNM


wxString
DeltaDescriptionRD(const DoubleParameter& delta)
{
    wxString description;

    description << _("Minimal step length in single direction (")
                << Symbols::GetInstance()[Symbols::DELTA] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(delta))
                << wxT('\n');

   return description;
}// DeltaDescriptionRD


wxString
EpsilonDescription(const DoubleParameter& epsilon)
{
    wxString description;

    description << _("Precision (")
                << Symbols::GetInstance()[Symbols::EPSILON] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(epsilon))
                << wxT('\n');
    return description;
}// EpsilonDescription

wxString
ExpansionDescriptionNM(const DoubleParameter& expansionFactor)
{
    wxString description;

    description << _("Expansion factor (e): ")
                << wxString::Format(wxT("%g"), static_cast<double>(
                                                    expansionFactor))
                << wxT('\n');
    return description;

}// ExpansionDescriptionNM

wxString
GammaDescriptionDerivative(const DoubleParameter& gamma)
{
    wxString description;
    const Symbols& symbols = Symbols::GetInstance();

    description << EmbedString(_("Decrease of %s factor"),
                    symbols[Symbols::ALPHA])
                << wxT(" (")
                << symbols[Symbols::GAMMA] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(gamma))
                << wxT('\n');

   return description;
}// GammaDescriptionDerivative


wxString
GoldenSectionDescription(bool goldenSection)
{
    wxString description;

    description << _("Golden section: ");
    if (goldenSection)  description << _("yes");
    else                description << _("no");
    description << wxT('\n');

    return description;
}// GoldenSectionDescription


wxString
InitialSectionLengthDescription(double length)
{
    wxString description;

    description << _("Initial section length: ")
                << wxString::Format(wxT("%g"), length)
                << wxT('\n');

    return description;
}// InitialSectionLengthDescription

wxString
LambdaDescriptionMarquardt(const DoubleParameter& lambda)
{
    wxString description;

    description << _("Initial identity matrix weight (")
                << Symbols::GetInstance()[Symbols::LAMBDA] << wxT(") : ")
                << wxString::Format(wxT("%g"), static_cast<double>(lambda))
                << wxT('\n');

    return description;
}// LambdaDescriptionMarquardt

wxString
LineSearchDescription(const wxString& lineSearchString)
{
    wxString description;

    description << _("Line search method:\n") << lineSearchString <<
        wxT('\n');

    return description;
}// LineSearchDescription

wxString
MethodName(const wxString& name)
{
    wxString description;

    description << _("Name:\t") << name << wxT('\n');

    return description;
}// MethodName

wxString
ReflectionDescriptionNM(const DoubleParameter& reflectionFactor)
{
    wxString description;

    description << _("Reflection factor (r): ")
                << wxString::Format(wxT("%g"), static_cast<double>(
                                                    reflectionFactor))
                << wxT('\n');
    return description;
}// ReflectionDescriptionNM

wxString
SearchSectionDescription(const DoubleParameter& a,
                         const DoubleParameter& b)
{
    wxString description;

    description << _("Search section: [ ")
                << wxString::Format(wxT("%g"), static_cast<double>(a))
                << wxT("; ")
                << wxString::Format(wxT("%g"), static_cast<double>(b))
                << wxT(" ]")
                << wxT('\n');

    return description;
}// InitialSectionDescription

wxString
ShrinkageDescriptionNM(const DoubleParameter& shrinkageFactor)
{
    wxString description;

    description << _("Shrinkage factor (s): ")
                << wxString::Format(wxT("%g"), static_cast<double>(
                                                    shrinkageFactor))
                << wxT('\n');
    return description;
}// ShrinkageDescriptionNM

wxString
SideLengthDescriptionNM(const DoubleParameter& edgeLength)
{
    wxString description;

    description << _("Starting simplex edge length: ")
                << wxString::Format(wxT("%g"), static_cast<double>(edgeLength))
                << wxT('\n');
    return description;
}// SideLengthDescriptionNM

wxString
StartingPointDescription(const ColumnVector& startingPoint)
{
    wxString description;

    description << _("Starting point:\t") << startingPoint << wxT('\n');

    return description;
}// StartingPointDescription

wxString
StartingRangeDescription(const ColumnVector& minBounds,
                         const ColumnVector& maxBounds)
{
    wxString description;

    description << _("Bounds of population generation range") << wxT(":\n");
    description << _("Lower: ");
    description << minBounds;
    description << wxT('\n');
    description << _("Upper: ");
    description << maxBounds;

    return description;
}

wxString
StartingPointDescriptionUnivariate(const ColumnVector& startingPoint)
{
    wxString description;

    description << _("Starting point: ")
                << wxString::Format(wxT("%g"), startingPoint(1))
                << wxT('\n');

    return description;
}// StartingPointDescriptionUnivariate

wxString
StopConditionDescription(const wxString& stopConditionString)
{
    wxString description;

    description << _("Stop condition:\t") << stopConditionString << wxT('\n')
        << wxT('\n');

    return description;
}// StopConditionDescription

wxString
StopConditionDescriptionHJD(const DoubleParameter& epsilon)
{
    wxString description;

     description << _("Stop condition (")
                 << Symbols::GetInstance()[Symbols::EPSILON] << wxT("):\t")
                 << wxString::Format(wxT("%g"), static_cast<double>(epsilon))
                 << wxT('\n') << wxT('\n');

    return description;
}// StopConditionDescriptionHJD


wxString
TauDescriptionHJD(const DoubleParameter& tau)
{
    wxString description;

    description << _("Step length (")
                << Symbols::GetInstance()[Symbols::TAU] << wxT("): ")
                << wxString::Format(wxT("%g"), static_cast<double>(tau))
                << wxT('\n');

   return description;
}// TauDescriptionHJD

wxString
TauDescriptionRD(const ColumnVector& tau)
{
    wxString description;

    description << _("Initial step length in directions (")
                << Symbols::GetInstance()[Symbols::TAU] << wxT("): ")
                << tau
                << wxT('\n');

   return description;
}// TauDescriptionRD



wxString LabelSymbol(const wxString& symbol)
{
    return wxString(wxT(" (") + symbol + wxString(wxT("):")));
}

wxString
EmbedStrings(const wxString& template_string, const wxArrayString& arguments)
{
    wxString template_left = template_string;
    wxString result;
    unsigned argument_index = 0;

    LOG(DEBUG, wxT("Template string: '") << template_string << wxT("'"));
    LOG(DEBUG, wxT("Argument count:   ") << arguments.GetCount());
    int index = template_left.Find(wxT('%'));
    while (index != wxNOT_FOUND)
    {
        result << template_left.Left(index); // character at index not included

        if (index < template_left.Length() - 1)
        {
            wxChar template_specifier = template_left[index + 1];
            // TODO: da mi to pusty string, jeli na kocu jest "%s" ?
            template_left = template_left.Mid(index + 2);

            switch(template_specifier)
            {
                case wxT('%') :
                        result << wxT('%');
                        break;
                case wxT('s') :
                        if (argument_index < arguments.GetCount())
                        {
                            result << arguments[argument_index];
                            ++argument_index;
                        }
                        else
                        {
                            LOG(ERROR, wxT("More placeholders than arguments"));
                        }
                        break;
                default :
                        LOG(WARNING, wxT("Unknown specifier: ") <<
                                    template_specifier);
            }
            index = template_left.Find(wxT('%'));
        }
        else    index = wxNOT_FOUND;
    }

    result << template_left;

    return result;
}// EmbedStrings

wxString
EmbedString(const wxString& template_string, const wxString& value)
{
    wxArrayString strings;

    strings.Add(value);
    return EmbedStrings(template_string, strings);
}// EmbedString


wxString PrintDouble(double  value)
{
    wxString string;

    // Przy przekroczeniu ogranicze funkcja kary zwraca warto DBL_MAX.
    // Chcemy by dugi cig cyfr nie niszczy wydruku.
    if (value == DBL_MAX)
        string = Symbols::GetInstance()[Symbols::INFINITY];
    else
        string = wxString::Format(wxT("%12.7f"), value);

    return string;
}

wxString PrintPoint(const ColumnVector& point)
{
    wxString string;

    string << point;

    return string;
}// PrintPoint

wxString PrintPoint(const wxString&     pointSymbol,
                    const ColumnVector& point)
{
    wxString string;

    string << pointSymbol << wxT(" = ");
    string << PrintPoint(point);

    return string;
}// PrintPoint

wxString PrintFunctionValue(double value)
{
    wxString string;

    string << wxT("f = ");
    string << PrintDouble(value);

    return string;
}// PrintFunctionValue


wxString PrintFunctionValue(const wxString&     pointSymbol,
                            double              value)
{
    wxString string;

    string << wxT("f(")
           << pointSymbol
           << wxT(") = ")
           << PrintDouble(value);

    return string;
}// PrintFunctionValue


wxString PrintFunctionValue(const ColumnVector& point,
                            const FunctionBase& function)
{
    wxString string;

    string << PrintFunctionValue(function.EvaluateNoThrow(point));

    return string;
}// PrintFunctionValue

wxString PrintFunctionValue(const wxString&     pointSymbol,
                            const ColumnVector& point,
                            const FunctionBase& function)
{
    wxString string;

    string << PrintFunctionValue(pointSymbol, function.EvaluateNoThrow(point));

    return string;
}// PrintFunctionValue

wxString PrintFunctionValue(const wxString&     pointSymbol,
                            double              point,
                            const FunctionBase& function)
{
    wxString string;
    ColumnVector point_vector(1);
    point_vector(1) = point;

    string << PrintFunctionValue(pointSymbol, function.Evaluate(point_vector));

    return string;
}// PrintFunctionValue

wxString PrintDerivativeInPoint(const wxString&     pointSymbol,
                                double              point,
                                double              value)
{
    wxString string;

    string << wxT("f'(")
           << pointSymbol
           << wxT(")  = ")
           << PrintDouble(value);

    return string;
}

wxString PrintDerivativeInPoint(const wxString&     pointSymbol,
                                double              point,
                                const FunctionBase& function)
{
    wxString        string;
    ColumnVector    point_vector(1);

    point_vector(1) = point;

    string << PrintDerivativeInPoint
                               (pointSymbol,
                                point,
                                function.Gradient(point_vector).as_scalar());

    return string;
}

wxString PrintSecondDerivativeInPoint(const wxString&     pointSymbol,
                                            double              point,
                                            double              value)
{
    wxString string;

    string << wxT("f''(")
           << pointSymbol
           << wxT(") = ")
           << PrintDouble(value);

    return string;
}

wxString PrintSecondDerivativeInPoint(const wxString&     pointSymbol,
                                            double              point,
                                            const FunctionBase& function)
{
    wxString        string;
    ColumnVector    point_vector(1);

    point_vector(1) = point;

    string << PrintSecondDerivativeInPoint
                               (pointSymbol,
                                point,
                                function.Hessian(point_vector).as_scalar());

    return string;
}

wxString PrintPointWithValue(const ColumnVector& point,
                             const FunctionBase& function)
{
    wxString string;
    string << PrintPoint(point)
           << wxT('\t')
           << PrintFunctionValue(point, function)
           << wxT('\t');

    return string;
}// PrintPointWithValue

wxString PrintPointWithValue(const wxString&     pointSymbol,
                             const ColumnVector& point,
                             const FunctionBase& function)
{
    wxString string;

    string << PrintPoint(pointSymbol, point)
           << wxT("  ")
           << PrintFunctionValue(pointSymbol, point, function);

    return string;
}// PrintPointWithValue

wxString PrintGradient(const wxString&      pointSymbol,
                       const ColumnVector&  gradient)
{
    wxString string;

    string << wxT("grad f(")
           << pointSymbol
           << wxT(") = ")
           << gradient;

    return string;
}// PrintGradient

wxString PrintPointWithValueAndGradient(const wxString&         pointSymbol,
                                        const ColumnVector&     point,
                                        const ColumnVector&     gradient,
                                        const FunctionBase&     function)
{
    wxString string;

    string << PrintPointWithValue(pointSymbol, point, function)
           << wxT('\t')
           << PrintGradient(pointSymbol, gradient);

    return string;
}// PrintPointWithValueAndGradient

wxString PrintPointWithValueAndGradient(const wxString&         pointSymbol,
                                        const ColumnVector&     point,
                                        const FunctionBase&     function)
{
    wxString string;

    string << PrintPointWithValueAndGradient(pointSymbol,
                                             point,
                                             function.Gradient(point),
                                             function);

    return string;
}// PrintPointWithValueAndGradient

wxString PrintDoubleWithValue(const wxString&     pointSymbol,
                              double              point,
                              double              value)
{
    wxString string;

    string << pointSymbol
           << wxT(" = ")
           << PrintDouble(point)
           << wxT('\t')
           << PrintFunctionValue(pointSymbol, value);

    return string;
}// PrintDoubleWithValue

wxString PrintDoubleWithValue(const wxString&       pointSymbol,
                              double                point,
                              const FunctionBase&   function)
{
    wxString string;

    ColumnVector vector_point(1);

    vector_point(1) = point;

    string << PrintDoubleWithValue(pointSymbol, point, function.Evaluate(
                vector_point));

    return string;
}// PrintDoubleWithValue

wxString PrintOptimalStep(double                tau,
                          const ColumnVector&   point,
                          const FunctionBase&   function)
{
    wxString string;

    string << Symbols::GetInstance()[Symbols::TAU]
           << wxString::Format(wxT(" = %g"), tau)
           << wxT("  ")
           << PrintPointWithValue(wxT("x"), point, function);

    return string;
}// PrintOptimalStep

wxString PrintSection(double leftBound,
                      double rightBound,
                      bool   printLength)
{
    wxString string;

    string << PrintCustomSection(wxT("a"), wxT("b"), leftBound, rightBound,
            printLength);

    return string;
}// PrintSection

wxString PrintSection(double                leftBound,
                      double                rightBound,
                      const FunctionBase&   function,
                      bool                  printLength)
{
    return PrintCustomSection(wxT("a"), wxT("b"), leftBound, rightBound,
            function, printLength);
}// PrintSection

wxString PrintCustomSection(const wxString& leftBoundSymbol,
                            const wxString& rightBoundSymbol,
                            double          leftBound,
                            double          rightBound,
                            bool            printLength)
{
    wxString string;

    string << wxT("[ ")
           << leftBoundSymbol
           << wxT(" ")
           << rightBoundSymbol
           << wxT(" ] = [ ")
           << wxString::Format(wxT("%g"), leftBound)
           << wxT("  ")
           << wxString::Format(wxT("%g"), rightBound)
           << wxT(" ]");

    if (printLength)
    {
        string << wxT("\t| ")
               << leftBoundSymbol
               << wxT(" ")
               << rightBoundSymbol
               << wxT(" | = ")
               << wxString::Format(wxT("%g"), rightBound - leftBound);
    }

    return string;
}// PrintCustomSection

wxString PrintCustomSection(const wxString&     leftBoundSymbol,
                            const wxString&     rightBoundSymbol,
                            double              leftBound,
                            double              rightBound,
                            const FunctionBase& function,
                            bool                printLength)
{
    wxString string(PrintCustomSection(leftBoundSymbol,
                                       rightBoundSymbol,
                                       leftBound,
                                       rightBound,
                                       printLength));

    string << wxT('\t')
           << PrintFunctionValue(leftBoundSymbol, leftBound, function)
           << wxT('\t')
           << PrintFunctionValue(rightBoundSymbol, rightBound, function);

    return string;
}
