#ifndef _text_util_h
#define _text_util_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "DoubleParameter.h"
#include "FunctionBase.h"

// FORWARD REFERENCES
//

// Funkcje tworzce standardowe opisy parametrw. Wywoywane z ToString metod.
wxString AlignedParameterDescriptions(const wxArrayString& descriptions);

wxString AlphaDescriptionAD(const DoubleParameter& alpha);

wxString AlphaDescriptionDerivative(const DoubleParameter& alpha);

wxString AlphaDescriptionRD(const DoubleParameter& alpha);

wxString BetaDescriptionHJD(const DoubleParameter& beta);

wxString BetaDescriptionRD(const DoubleParameter& beta);

wxString CenterDescriptionNM(const ColumnVector& center);

wxString ContractionDescriptionNM(const DoubleParameter& contractionFactor);

wxString DeltaDescriptionRD(const DoubleParameter& delta);

wxString EpsilonDescription(const DoubleParameter& epsilon);

wxString ExpansionDescriptionNM(const DoubleParameter& expansionFactor);

wxString GammaDescriptionDerivative(const DoubleParameter& gamma);

wxString GoldenSectionDescription(bool goldenSection);

wxString InitialSectionLengthDescription(double length);

wxString LambdaDescriptionMarquardt(const DoubleParameter& lambda);

wxString LineSearchDescription(const wxString& lineSearchString);

wxString MethodName(const wxString& name);

wxString ReflectionDescriptionNM(const DoubleParameter& reflectionFactor);

wxString SearchSectionDescription(const DoubleParameter& a,
                                  const DoubleParameter& b);

wxString ShrinkageDescriptionNM(const DoubleParameter& shrinkageFactor);

wxString SideLengthDescriptionNM(const DoubleParameter& sideLength);

wxString StartingPointDescription(const ColumnVector& startingPoint);

wxString StartingRangeDescription(const ColumnVector& minBounds,
                                  const ColumnVector& maxBounds);

wxString StartingPointDescriptionUnivariate(const ColumnVector& startingPoint);

wxString StopConditionDescription(const wxString& stopConditionString);

wxString StopConditionDescriptionHJD(const DoubleParameter& epsilon);

wxString TauDescriptionHJD(const DoubleParameter& tau);

wxString TauDescriptionRD(const ColumnVector& tau);

// Zwraca napis " (<argument>):" do etykiet.
wxString LabelSymbol(const wxString& symbol);

// Wstawia wxStringi w miejsce "%s" w template_string.
wxString EmbedStrings(const wxString&       template_string,
                      const wxArrayString&  arguments);

// Wstawia wxString w miejsce "%s" w tamplate_string.
wxString EmbedString(const wxString&    template_string,
                     const wxString&    argument);

// Funkcje bardziej oglne, wykorzystywane gwnie w opisach iteracji.

wxString PrintDouble(double  value);

wxString PrintPoint(const ColumnVector&  point);

wxString PrintPoint(const wxString&      pointSymbol,
                    const ColumnVector&  point);

wxString PrintFunctionValue(double               value);

wxString PrintFunctionValue(const wxString&      pointSymbol,
                            double               value);

wxString PrintFunctionValue(const ColumnVector&  point,
                            const FunctionBase&  function);

wxString PrintFunctionValue(const wxString&      pointSymbol,
                            const ColumnVector&  point,
                            const FunctionBase&  function);

wxString PrintFunctionValue(const wxString&      pointSymbol,
                            double               point,
                            const FunctionBase&  function);

wxString PrintDerivativeInPoint(const wxString&     pointSymbol,
                                double              point,
                                double              value);

wxString PrintDerivativeInPoint(const wxString&     pointSymbol,
                                double              point,
                                const FunctionBase& function);

wxString PrintSecontDerivativeInPoint(const wxString&     pointSymbol,
                                            double              point,
                                            double              value);

wxString PrintSecondDerivativeInPoint(const wxString&     pointSymbol,
                                            double              point,
                                            const FunctionBase& function);

wxString PrintPointWithValue(const ColumnVector& point,
                             const FunctionBase& function);

wxString PrintPointWithValue(const wxString&     pointSymbol,
                             const ColumnVector& point,
                             const FunctionBase& function);

wxString PrintGradient(const wxString&       pointSymbol,
                       const ColumnVector&   gradient);

wxString PrintPointWithValueAndGradient(const wxString&         pointSymbol,
                                        const ColumnVector&     point,
                                        const ColumnVector&     gradient,
                                        const FunctionBase&     function);

wxString PrintPointWithValueAndGradient(const wxString&         pointSymbol,
                                        const ColumnVector&     point,
                                        const FunctionBase&     function);

wxString PrintDoubleWithValue(const wxString&     pointSymbol,
                              double              point,
                              double              value);

wxString PrintDoubleWithValue(const wxString&        pointSymbol,
                              double                 point,
                              const FunctionBase&    function);

wxString PrintOptimalStep(double                tau,
                          const ColumnVector&   point,
                          const FunctionBase&   function);

wxString PrintSection(double leftBound,
                      double rightBound,
                      bool   printLength = false);

wxString PrintSection(double                 leftBound,
                      double                 rightBound,
                      const FunctionBase&    function,
                      bool                   printLength = false);

wxString PrintCustomSection(const wxString& leftBoundSymbol,
                            const wxString& rightBoundSymbol,
                            double          leftBound,
                            double          rightBound,
                            bool            printLength = false);

wxString PrintCustomSection(const wxString& leftBoundSymbol,
                            const wxString& rightBoundSymbol,
                            double          leftBound,
                            double          rightBound,
                            const FunctionBase&    function,
                            bool            printLength = false);

#endif  // _text_util_h
