#include <precomp.h>

#include "../methods/AlphaDivisionIteration.h"

#include "AlphaDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

AlphaDivisionLayer::AlphaDivisionLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// AlphaDivisionLayer

AlphaDivisionLayer::AlphaDivisionLayer(const AlphaDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// AlphaDivisionLayer

AlphaDivisionLayer::~AlphaDivisionLayer()
{
}// ~AlphaDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
AlphaDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
AlphaDivisionLayer::Create(const PlotControl&       plotControl,
                           const Result&            result,
                           CountedPtr<PlotLayer>    cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new AlphaDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
AlphaDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const AlphaDivisionIteration& iteration =
        dynamic_cast<const AlphaDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
AlphaDivisionLayer::DrawIteration(wxDC&             dc,
                                  unsigned          iterationNumber,
                                  bool              highlight) const
{
    const AlphaDivisionIteration& iteration =
        dynamic_cast<const AlphaDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if (highlight)
    {   // Punkty
        DrawBigPoint(dc, iteration.LeftBound(),
                AttributeDatabase::NORMAL);
        DrawBigPoint(dc, iteration.RightBound(),
                AttributeDatabase::NORMAL);

        DrawBigPoint(dc, iteration.BetterStep(), AttributeDatabase::POINT_BEST);
        DrawBigPoint(dc, iteration.WorseStep(), AttributeDatabase::POINT_WORST);
    }

}// DrawIteration

void
AlphaDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const AlphaDivisionIteration& iteration =
        dynamic_cast<const AlphaDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* AlphaDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            AlphaDivisionIteration::ClassId(),
            AlphaDivisionLayer::Create);
