#include <precomp.h>

#include "AttributeDatabase.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

AttributeDatabase::~AttributeDatabase()
{
}// ~AttributeDatabase


//============================= OPERATORS ====================================

//AttributeDatabase&
//AttributeDatabase::operator=(const AttributeDatabase& from)
//{
//    return *this;
//}// =

const wxColour&
AttributeDatabase::Colour(AttributeDatabase::AttributeType attribute) const
{
    AttributeContainer::const_iterator position;

    // Nie uywamy [], bo nie chcemy tworzy nowych wartoci.
    position = mAttributes.find(attribute);

    assert(position != mAttributes.end());

    return position->second.mForegroundColour;
}// Colour

wxColour
AttributeDatabase::TextColour(AttributeDatabase::AttributeType attribute) const
{
    const double DARKEN_FACTOR = 0.75;

    const wxColour& colour = Colour(attribute);
    return wxColour(lround(colour.Red() * DARKEN_FACTOR),
                    lround(colour.Green() * DARKEN_FACTOR),
                    lround(colour.Blue() * DARKEN_FACTOR));
}// TextColour

wxPen
AttributeDatabase::Pen(AttributeDatabase::AttributeType attribute) const
{
    return wxPen(Colour(attribute));
}// Pen

wxBrush
AttributeDatabase::Brush(AttributeDatabase::AttributeType attribute) const
{
    return wxBrush(Colour(attribute));
}// Brush

wxTextAttr
AttributeDatabase::TextAttribute(AttributeDatabase::AttributeType attribute)
        const
{
    wxTextAttr text_attribute(mBaseTextAttribute);
    AttributeContainer::const_iterator position;

    // Nie uywamy [], bo nie chcemy tworzy nowych wartoci.
    position = mAttributes.find(attribute);

    assert(position != mAttributes.end());

    const AttributeDataType& attr = position->second;

    if (attr.mForegroundColour != wxNullColour)
    {
        // TODO: niepotrzebne szukanie
        text_attribute.SetTextColour(TextColour(attribute));
    }
    else
    {
        text_attribute.SetTextColour(wxNullColour);
    }

    text_attribute.SetBackgroundColour(attr.mBackgroundColour);

    if (attr.mBold != UNDEFINED)
    {
        wxFont new_font = text_attribute.GetFont();
        // TODO: nie wycza si pogrubienie
        new_font.SetWeight(attr.mBold ?
                wxFONTWEIGHT_BOLD : wxFONTWEIGHT_NORMAL);
//        new_font.SetUnderlined(attr.mBold ? true: false);
        text_attribute.SetFont(new_font);
        assert(text_attribute.HasFont());
    }

    return text_attribute;
}// TextAttribute

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* static */ const AttributeDatabase&
AttributeDatabase::GetInstance()
{
    static AttributeDatabase instance;

    return instance;
}// GetInstance

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

AttributeDatabase::AttributeDatabase()
    : mAttributes(),
      // TODO: uy staej dla rozmiaru
      mBaseTextAttribute(*wxBLACK, *wxWHITE, wxFont(9, wxFONTFAMILY_TELETYPE,
                  wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL))
{
//  Najwidoczniej taby nie s zaimplementowane w wxWidgets.
//    wxArrayInt tab_stops;
//    tab_stops.Add(5);
//    tab_stops.Add(500);
//    tab_stops.Add(5000);
//    mBaseTextAttribute.SetTabs(tab_stops);

    // Atrybuty dodajemy tutaj. Ta klasa nie jest jaka genialna, no i trudno.
    mAttributes[ALERT] =
        AttributeDataType(wxColour(189, 27, 27), wxNullColour, UNDEFINED);
    mAttributes[DEFAULT] =
        AttributeDataType(*wxBLACK, *wxWHITE, UNDEFINED);
    mAttributes[NORMAL] =
        AttributeDataType(wxColour(0, 0, 0), wxNullColour, UNDEFINED);
    mAttributes[INACTIVE] =
        AttributeDataType(wxColour(150, 150, 150), wxNullColour, UNDEFINED);
    mAttributes[HEADER] =
        AttributeDataType(wxNullColour, wxNullColour, YES);

    mAttributes[PLOT_FUNCTION] =
        AttributeDataType(wxColour(100, 100, 100), wxNullColour, UNDEFINED);
    mAttributes[PLOT_BACKGROUND] =
        AttributeDataType(wxColour(255, 255, 255), wxNullColour, UNDEFINED);
    mAttributes[PLOT_OUTSIDE_BG] =
        AttributeDataType(wxColour(245, 245, 245), wxNullColour, UNDEFINED);
    mAttributes[PLOT_OUTSIDE_BG2] =
        AttributeDataType(wxColour(230, 230, 230), wxNullColour, UNDEFINED);
    mAttributes[PLOT_OUTSIDE_BG3] =
        AttributeDataType(wxColour(215, 215, 215), wxNullColour, UNDEFINED);
    mAttributes[PLOT_NEW_SECTION_BG] =
        AttributeDataType(wxColour(245, 248, 255), wxNullColour, UNDEFINED);

    mAttributes[PLOT_CONSTRAINT1] =
        AttributeDataType(wxColour(198, 93, 93), wxNullColour, UNDEFINED);
    mAttributes[PLOT_CONSTRAINT2] =
        AttributeDataType(wxColour(138, 93, 198), wxNullColour, UNDEFINED);
    mAttributes[PLOT_CONSTRAINT3] =
        AttributeDataType(wxColour(98, 163, 96), wxNullColour, UNDEFINED);
    mAttributes[PLOT_CONSTRAINT4] =
        AttributeDataType(wxColour(163, 136, 96), wxNullColour, UNDEFINED);
    mAttributes[PLOT_CONSTRAINT5] =
        AttributeDataType(wxColour(205, 42, 159), wxNullColour, UNDEFINED);

    mAttributes[POINT_START] =
        AttributeDataType(wxColour(255, 255, 255), wxNullColour, UNDEFINED);
    mAttributes[POINT_RESULT] =
        AttributeDataType(wxColour(0, 0, 0), wxNullColour, UNDEFINED);
    mAttributes[POINT_BASE] =
//        AttributeDataType(wxColour(174, 121, 29), wxNullColour, UNDEFINED);
        AttributeDataType(wxColour(234, 163, 39), wxNullColour, UNDEFINED);
    mAttributes[POINT_BEST] =
        AttributeDataType(wxColour(104, 227, 21), wxNullColour, UNDEFINED);
    mAttributes[POINT_MIDDLE] =
        AttributeDataType(wxColour(227, 215, 21), wxNullColour, UNDEFINED);
    mAttributes[POINT_OUT_OF_BOUNDS] =
        AttributeDataType(wxColour(245, 100, 0), wxNullColour, UNDEFINED);
    mAttributes[POINT_WORST] =
        AttributeDataType(wxColour(236, 67, 0), wxNullColour, UNDEFINED);
    mAttributes[POINT_X0] =
        AttributeDataType(wxNullColour, wxNullColour, UNDEFINED);

    mAttributes[SIMPLEX_B] =
        AttributeDataType(wxColour(150, 100, 45), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_BASE] =
        AttributeDataType(wxColour(35, 25, 230), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_C] =
        AttributeDataType(wxColour(255, 128, 64), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_N] =
        AttributeDataType(wxColour(150, 230, 250), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_POINT_EXTENDED] =
        AttributeDataType(wxColour(255, 255, 255), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_POINT_MAIN] =
        AttributeDataType(wxColour(127, 127, 127), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_VAR_ENTERING] =
        AttributeDataType(wxColour(104, 227, 21), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_VAR_EXTENDING] =
        AttributeDataType(wxNullColour, wxColour(250, 250, 0), UNDEFINED);
    mAttributes[SIMPLEX_VAR_LEAVING] =
        AttributeDataType(wxColour(236, 67, 0), wxNullColour, UNDEFINED);
    mAttributes[SIMPLEX_VAR_SLACK] =
        AttributeDataType(wxNullColour, wxColour(180, 180, 180), UNDEFINED);

    mAttributes[STEP_IN_DIRECTION] =
//        AttributeDataType(wxColour(46, 120, 157), wxNullColour, UNDEFINED);
//        AttributeDataType(wxColour(22, 141, 201), wxNullColour, UNDEFINED);
        AttributeDataType(wxColour(134, 196, 14), wxNullColour, UNDEFINED);
    mAttributes[STEP_PRELIMINARY] =
        AttributeDataType(wxColour(0, 200, 200), wxNullColour, UNDEFINED);
    mAttributes[STEP_TRIAL_D1] =
//        AttributeDataType(wxColour(17, 38, 134), wxNullColour, UNDEFINED);
        AttributeDataType(wxColour(105, 135, 213), wxNullColour, UNDEFINED);
    mAttributes[STEP_TRIAL_D2] =
//        AttributeDataType(wxColour(89, 38, 144), wxNullColour, UNDEFINED);
        AttributeDataType(wxColour(194, 44, 231), wxNullColour, UNDEFINED);
    mAttributes[CH_STARTING_BEST] =
        AttributeDataType(wxColour(53, 206, 122), wxNullColour, UNDEFINED);
    mAttributes[CH_END_BEST] =
        AttributeDataType(wxColour(104, 227, 21), wxNullColour, UNDEFINED);
    mAttributes[CH_DECEASED] =
        AttributeDataType(wxColour(253, 134, 146), wxNullColour, UNDEFINED);
    mAttributes[CH_SURVIVOR] =
        AttributeDataType(wxColour(0, 0, 0), wxNullColour, UNDEFINED);
    mAttributes[CH_MUTANT] =
        AttributeDataType(wxColour(155, 149, 234), wxNullColour, UNDEFINED);
    mAttributes[CH_CHILD] =
        AttributeDataType(wxColour(245, 151, 56), wxNullColour, UNDEFINED);
}// AttributeDatabase

AttributeDatabase::AttributeDatabase(const AttributeDatabase& from)
    : mAttributes(from.mAttributes),
      mBaseTextAttribute(from.mBaseTextAttribute)
{
}// AttributeDatabase

AttributeDatabase::AttributeDataType::AttributeDataType()
    : mForegroundColour(), mBackgroundColour(), mBold(UNDEFINED)
{
}

AttributeDatabase::AttributeDataType::AttributeDataType(
        const wxColour&                         foreground,
        const wxColour&                         background,
        AttributeDatabase::YesNoUndefinedType   bold)
    : mForegroundColour(foreground),
      mBackgroundColour(background),
      mBold(bold)
{
}

AttributeDatabase::AttributeDataType::AttributeDataType(
        const AttributeDataType& from)
    : mForegroundColour(from.mForegroundColour),
      mBackgroundColour(from.mBackgroundColour),
      mBold(from.mBold)
{
}

AttributeDatabase::AttributeDataType&
AttributeDatabase::AttributeDataType::operator=(
        const AttributeDatabase::AttributeDataType& from)
{
    mForegroundColour = from.mForegroundColour;
    mBackgroundColour = from.mBackgroundColour;
    mBold             = from.mBold;

    return *this;
}
