/** Baza kolorw i atrybutw (tekstu i nie tylko)
 */

#ifndef _AttributeDatabase_h
#define _AttributeDatabase_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//

// FORWARD REFERENCES
//


class AttributeDatabase
{
public:

    enum AttributeType {
        ALERT,                  ///< Drukowanie komunikatw o bdach
        DEFAULT,                ///< Domylny
        NORMAL,                 ///< Zwyky, czarny
        INACTIVE,               ///< Kolor nieaktywnej iteracji, szary
        HEADER,                 ///< Nagwek w tekcie
        PLOT_FUNCTION,          ///< Wykres funkcji 1-wymiarowej
        PLOT_BACKGROUND,        ///< To wykresu
        PLOT_OUTSIDE_BG,        ///< To obszaru poza...
        PLOT_OUTSIDE_BG2,       ///< To obszaru poza, bardziej intensywne.
        PLOT_OUTSIDE_BG3,       ///< To obszaru poza, jeszcze b. intensywne.
        PLOT_NEW_SECTION_BG,    ///< To obszaru nowego (wybranego) przedziau.

        // Atrybuty ogranicze musz by te dodane w ui/XyConstraintsLayer.cc
        // oraz ui/UnivariateConstraintsLayer.cc (jeli bdzie).
        PLOT_CONSTRAINT1,       ///< Wykres ograniczenia, kolor 1.
        PLOT_CONSTRAINT2,       ///< Wykres ograniczenia, kolor 2.
        PLOT_CONSTRAINT3,       ///< Wykres ograniczenia, kolor 3.
        PLOT_CONSTRAINT4,       ///< Wykres ograniczenia, kolor 4.
        PLOT_CONSTRAINT5,       ///< Wykres ograniczenia, kolor 5.

        POINT_START,            ///< Punkt startowy przebiegu oraz iteracji
        POINT_RESULT,           ///< Ostatecznie uzyskany punkt
        POINT_BASE,             ///< Punkt bazowy
        POINT_BEST,             ///< Najlepszy z porwnywanych 2 lub 3 punktw
        POINT_MIDDLE,           ///< redni z porwnywanych 2 lub 3 punktw
        POINT_OUT_OF_BOUNDS,    ///< Minimum aproks. kwadratowej poza odcinkiem  / punkt poza ograniczeniami
        POINT_WORST,            ///< Najgorszy z porwnywanych 2 lub 3 punktw
        POINT_X0,               ///< Punkt startowy
        SIMPLEX_B,              ///< Wektor ogranicze b (kolor tekstu)
        SIMPLEX_BASE,           ///< Baza (kolor tekstu)
        SIMPLEX_C,              ///< Wektor kosztw c (kolor tekstu)
        SIMPLEX_N,              ///< Macierz kolumn niebazowych (kolor tekstu)
        SIMPLEX_POINT_EXTENDED, ///< Punkt uzyskany w fazie wstpnej
        SIMPLEX_POINT_MAIN,     ///< Punkt uzyskane w fazie waciwej
        SIMPLEX_VAR_ENTERING,   ///< Zmienna wchodzca (kolor tekstu)
        SIMPLEX_VAR_EXTENDING,  ///< Zmienna rozszerzajca (to tekstu)
        SIMPLEX_VAR_LEAVING,    ///< Zmienna opuszczajca (kolor tekstu)
        SIMPLEX_VAR_SLACK,      ///< Zmienna dopeniajca (to tekstu)
        STEP_IN_DIRECTION,      ///< Krok w wyznaczonym wczeniej kierunku
        STEP_PRELIMINARY,       ///< Krok wstpny (Powell)
        STEP_TRIAL_D1,          ///< Krok prbny w kierunku 1
        STEP_TRIAL_D2,          ///< Krok prbny w kierunku 2
        CH_STARTING_BEST,       ///< Najlepszy chromosom na poczatku iteracji
        CH_END_BEST,            ///< Najlepszy chromosom pod koniec iteracji
        CH_DECEASED,            ///< Chromosom odrzucony w procesie selekcji
        CH_SURVIVOR,            ///< Chromosom, ktry przey selekcj
        CH_MUTANT,              ///< Chromosom zmutowany
        CH_CHILD                ///< Chromosom powstay w prosesie krzyowania
     };

// LIFECYCLE

    /** Destruktor.
     */
    ~AttributeDatabase(void);


// OPERATORS

// OPERATIONS
// ACCESS

    static const AttributeDatabase& GetInstance();

    /** Zwraca kolor gwny dla danego atrybutu.
     */
    const wxColour&     Colour(AttributeType attribute) const;

    /** Zwraca kolor gwny danego atrybutu dla tekstu (odp. ciemniejszy).
     */
    wxColour            TextColour(AttributeType attribute) const;

    /** Zwraca piro o kolorze gwnym danego atrybutu.
     */
    wxPen               Pen(AttributeType attribute) const;

    /** Zwraca wypenienie o kolorze gwnym (nie ta) danego atrybutu.
     */
    wxBrush             Brush(AttributeType attribute) const;

    /** Zwraca atrybut tekstu.
     *
     * Kolor czcionki jest pobierany za pomoc TextColour, kolor ta jest brany
     * wprost z definicji atrybutu. Uwzgldnia take pogrubienie.
     */
    wxTextAttr          TextAttribute(AttributeType attribute) const;

// INQUIRY

protected:
private:
    /** Konstruktor domylny.
     */
    AttributeDatabase(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    AttributeDatabase(const AttributeDatabase& from);

    /** Typ logiczny z wartoci nieokrelon.
     *
     * Jeli nie jest UNDEFINED, to mona z powodzeniem uywa jak inta
     * true/false.
     */
    enum YesNoUndefinedType {
        YES = 1,
        NO  = 0,
        UNDEFINED = 2
    };

    struct AttributeDataType {
        // Czemu nie mona w C++ inicjowa atwo struktur?
        AttributeDataType();
        AttributeDataType(const wxColour&       foreground,
                          const wxColour&       background,
                          YesNoUndefinedType    bold);
        AttributeDataType(const AttributeDataType& from);
        AttributeDataType& operator=(const AttributeDataType& from);

        wxColour            mForegroundColour;
        wxColour            mBackgroundColour;
        YesNoUndefinedType  mBold;
    };

    typedef std::map<AttributeType, AttributeDataType>   AttributeContainer;

    AttributeContainer  mAttributes;
    // To powinna by staa, ale nie mona by wtedy ustawi tabw.
    wxTextAttr    mBaseTextAttribute;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _AttributeDatabase_h
