/** Okienko wyboru metody.
 *
 * Funkcjonalno rni si nieco w zalenoci od tego, dla jakiej klasy
 * tworzony jest template (patrz CanHandle):
 * - Method - wybr metody do rozwizania zadania optymalizacji, przydatno
 *   metody jest sprawdzana za pomoc Method::CanHandle,
 * - LineSearch - wybr metody optymalizacji w kierunku stosowanej, przez gwn
 *   metod optymalizacji funkcji wielu zmiennych; przydatno metody jest
 *   sprawdzana za pomoc LineSearch::CanAssist.
 */

#ifndef _AutoChoice_h
#define _AutoChoice_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../Method.h"
#include "../LineSearch.h"

// FORWARD REFERENCES
//

// TODO: poprawi nazw,
template<class T>
class AutoChoice
    : public wxChoice
{
public:
// LIFECYCLE

    /** Konstruktor
     */
    AutoChoice()
        : wxChoice()
    {
    }// AutoChoice

    /** Konstruktor.
     */
    AutoChoice(wxWindow             *pParent,
               wxWindowID           id,
               const ProblemBase*   pProblem = NULL,
               const wxPoint&       pos = wxDefaultPosition,
               const wxSize&        size = wxDefaultSize,
               long                 style = 0,
               const wxString&      name = wxT(""))
        : wxChoice(pParent, id, pos, size, 0, NULL, style, wxDefaultValidator,
                name)
    {
        FillIn(pProblem);
    }// AutoChoice

    /** Destruktor.
     */
    ~AutoChoice()
    {
    }// ~AutoChoice

// OPERATORS

// OPERATIONS

    /** Tworzenie kontrolki w sytuacji, gdy wywoano konstruktor
     * bezparametrowy.
     */
    bool        Create(wxWindow          *pParent,
                       wxWindowID        id,
                       const wxPoint&    pos,
                       const wxSize&     size,
                       long              style,
                       const wxString&   name)
    {
        bool result;

        result = wxChoice::Create(pParent, id, pos, size, 0, NULL,
                wxDefaultValidator, name);

        if (result) FillIn(NULL);

        return result;
    }


    /** Zwraca wskanik na wybrany obiekt.
     */
    T*          GetSelected() const
    {
        T* p_selected_item
            = static_cast<T*>(GetClientData(GetSelection()));

        return p_selected_item;
    }// GetSelected

//    std::auto_ptr<T> CreateSelected() const
//    {
//        return GetSelected()->Clone();
//    }// CreateSelected

    /** Resetuje zawarto kontrolki po zmianie problemu.
     *
     * @param pProblem Wskanik na nowe zadanie optymalizacji.
     */
    bool        Reset(const ProblemBase* pProblem);

// INQUIRY

protected:

private:

    /** Wypenia kontrolk metodami, ktre nadaj si do rozwizania danego
     * zadania optymalizacji.
     */
    void FillIn(const ProblemBase* pProblem);

    /** Sprawdza, czy metoda nadaje si do rozwizania zadania optymalizacji.
     *
     * W zalenoci od specjalizacji template'u wywouje albo CanHandle (Method)
     * albo CanAssist (LineSearch).
     */
    bool CanHandle(const Method* pMethod, const ProblemBase* pProblem) const;
};

// IMPLEMENTATION


template<class T>
void
AutoChoice<T>::FillIn(const ProblemBase* pProblem)
{
    const vector<Method*> container = Method::Methods();
    typename vector<Method*>::const_iterator i;

    Clear();

    for (i = container.begin(); i != container.end(); ++i)
    {
        if ((dynamic_cast<T*>(*i) != NULL) &&
                (pProblem == NULL || CanHandle(*i, pProblem)))
        {
            Append((*i)->Name(), static_cast<void *>(*i));
        }
    }
    SetSelection(0);
}// FillIn


template<class T>
bool
AutoChoice<T>::Reset(const ProblemBase* pProblem)
{
    const T* p_old_selection = GetSelected();

    FillIn(pProblem);

    if (CanHandle(p_old_selection, pProblem))
    {   // Stara metoda moe poradzi sobie z nowym problemem - zostawiamy
        assert(GetCount() > 0);

        // Znajdz p_old_selection
        int i = 0;
        while (GetClientData(i) != p_old_selection) ++i;

        SetSelection(i);

        return false;
    }
    else return true;
}// Reset

#endif
