/**Szablon panelu dla metod optymalizacji funkcji wielu zmiennych.
 *
 * Uniwersalny panel dla metod posiadajcych 2 parametry:
 * warunek stopu, punkt startowy.
 *
 * Panele dla poszczeglnych metod s rejestrowane
 * w BasicMethodPanel.cc.
 */

#ifndef BASICMETHODPANEL_H
#define BASICMETHODPANEL_H

#include "DoubleParameterValidator.h"
#include "MethodPanelInfo.h"
#include "MethodPanelFactory.h"
#include "PointControl.h"
#include "StopConditionControl.h"
#include "StopConditionValidator.h"

template <class T>
class BasicMethodPanel: public wxPanel
{
public:

    /** Konstruktor.
     */
    BasicMethodPanel(wxWindow*             parent,
                     int                   id,
                     T*                    pMethod,
                     const ProblemBase&    problem,
                     const wxPoint&        pos=wxDefaultPosition,
                     const wxSize&         size=wxDefaultSize,
                     long                  style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*     CreatePanel(MethodPanelInfo     panelInfo);

private:
    void do_layout();

protected:
    StopConditionControl* mpStopCondition;
    PointControl* mpStartingPoint;
};

template <class T>
BasicMethodPanel<T>::BasicMethodPanel(wxWindow*             parent,
                                      int                   id,
                                      T*                    pMethod,
                                      const ProblemBase&    problem,
                                      const wxPoint&        pos,
                                      const wxSize&         size,
                                      long                  style)
    : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{
    // Tworzenie kontrolek.
    mpStopCondition = new StopConditionControl(this, -1);
    mpStartingPoint = new PointControl(this, -1, wxDefaultPosition,
            wxDefaultSize, 0, _("Starting point"));

    do_layout();

    // Wypenianie listy warunkw stopu.
    mpStopCondition->SetList(pMethod->AllowedStopConditions());

    // Walidatory.
    mpStopCondition->SetValidator(StopConditionValidator(
                reinterpret_cast<CountedPtr<const StopConditionBase>&>(
                    pMethod->mcpStopCondition)));

    // Wypenianie kontrolki punktu startowego.
    mpStartingPoint->Init(problem.Variables(),
            /*pMethod->mStartingPoint*/pMethod->mInitialStartingPoint);

}// BasicMethodPanel

template <class T>
/* static */ wxPanel*
BasicMethodPanel<T>::CreatePanel(MethodPanelInfo panelInfo)
{
    return new BasicMethodPanel(
                panelInfo.pParent(),
                -1,
                dynamic_cast<T*>(panelInfo.pMethod()),
                *panelInfo.pProblem());

}// CreatePanel

template <class T>
void
BasicMethodPanel<T>::do_layout()
{
    wxBoxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);
    p_main_sizer->Add(mpStopCondition, 0, wxEXPAND, 0);
    p_main_sizer->Add(mpStartingPoint, 1, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);
}// do_layout

#endif // BASICMETHODPANEL_H
