#include <precomp.h>

#include "../methods/BolzanoDivisionIteration.h"

#include "BolzanoDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

BolzanoDivisionLayer::BolzanoDivisionLayer
                        (const PlotControl&             plotControl,
                         const Result&                  result,
                               CountedPtr<PlotLayer>    cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// BolzanoDivisionLayer

BolzanoDivisionLayer::BolzanoDivisionLayer
                       (const BolzanoDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// BolzanoDivisionLayer


BolzanoDivisionLayer::~BolzanoDivisionLayer()
{
}// ~BolzanoDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
BolzanoDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);

}// Draw

/* static */ std::auto_ptr<ResultLayer>
BolzanoDivisionLayer::Create(const PlotControl&     plotControl,
                                 const Result&          result,
                                 CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new BolzanoDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
BolzanoDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const BolzanoDivisionIteration& iteration =
        dynamic_cast<const BolzanoDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
BolzanoDivisionLayer::DrawIteration(wxDC&             dc,
                                        unsigned          iterationNumber,
                                        bool              highlight) const
{
    const BolzanoDivisionIteration& iteration =
        dynamic_cast<const BolzanoDivisionIteration&>
            (mResult.Iteration(iterationNumber));


    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if (!highlight) return;

    //TODO: Moe jednak zastosowa jako kolorki...
    //AttributeDatabase::AttributeType attr_left, attr_middle, attr_right;
    AttributeDatabase::AttributeType attr = AttributeDatabase::NORMAL;

    DrawBigPoint(dc, iteration.LeftBound(), attr);
    DrawBigPoint(dc, iteration.Middle(), attr);
    DrawBigPoint(dc, iteration.RightBound(), attr);

}// DrawIteration

void
BolzanoDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const BolzanoDivisionIteration& iteration =
        dynamic_cast<const BolzanoDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ void* BolzanoDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            BolzanoDivisionIteration::ClassId(),
            BolzanoDivisionLayer::Create);
