/**  wxValidator dla kontrolki ustawiania funkcji i ogranicze.
 *
 * @see wxValidator
 */

#ifndef _ConstraintsValidator_h
#define _ConstraintsValidator_h

// SYSTEM INCLUDES
//
#include <vector>

// PROJECT INCLUDES
//
#include <wx/validate.h>

// LOCAL INCLUDES
//
#include "../Constraint.h"

// FORWARD REFERENCES
//


class ConstraintsValidator
    : public wxValidator
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    ConstraintsValidator(CountedPtr<const FunctionBase>&    rcpFunction,
                         CountedPtr<const VariableNames>&   rcpVariables,
                         vector<Constraint>&                rConstraints);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    ConstraintsValidator(const ConstraintsValidator& from);

    /** Destruktor.
     */
    ~ConstraintsValidator(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual wxObject*       Clone() const;

    virtual bool            TransferToWindow();

    virtual bool            TransferFromWindow();

    virtual bool            Validate(wxWindow* pParent);

// ACCESS
// INQUIRY

protected:
private:
    /** Sprawdza, ktre ograniczenia zawieraj zmienne nieobecne na podanej
     * licie.
     *
     * @param constraints Kontener ogranicze do sprawdzenia.
     * @param variables Dopuszczalne zmienne.
     *
     * @return Tablica indeksw ogranicze, ktre naley usun.
     */
    wxArrayInt  FindInvalidConstraints(
                        const Constraint::Container&    constraints,
                        const VariableNames&            variables) const;

    /** Usuwa ograniczenia o indeksach znajdujcych si w tablicy
     * <code>indexes</code>.
     *
     * @param rConstraints Referencja na kontener ogranicze.
     * @param indexes Tablica indeksw do usunicia z <code>rConstraints</code>
     */
    void        RemoveInvalidConstraints(
                        Constraint::Container&  rConstraints,
                        const wxArrayInt&       indexes) const;


    CountedPtr<const FunctionBase>&     mrcpFunction;
    CountedPtr<const VariableNames>&    mrcpVariables;
    vector<Constraint>&                 mrConstraints;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ConstraintsValidator_h
