#ifndef _ContourPlotter_h
#define _ContourPlotter_h

#include "PlotControl.h"

// Klasa zbierajca odcinki i czca je w poziomice.
class ContourPlotter
{
public:
// CONSTANTS

    /// Styl dodatkowej linii przerywanej uywanej do ogranicze nierwn.
    const static unsigned   DASH_STYLE;
    /// Odlego dodatkowej linii od waciwego konturu.
    const static unsigned   OUTLINE_DISTANCE;

// LIFECYCLE
    ContourPlotter(unsigned   levelCount);

// OPERATORS
// OPERATIONS
    void            AddSegment(double   x1,
                               double   y1,
                               double   x2,
                               double   y2,
                               unsigned level);

    /** Rysuje poziomice.
     *
     * @param dc Drawing Context.
     * @param shading Czy cieniowa poziomice. Jeli nie, to rysowane jest
     * aktualnym pirem.
     */
    void            Draw(wxDC& dc, bool shading) const;

    /** Rysuje poziomice z obwdk po zewntrznej stronie (uywane dla
     * ogranicze nierwnociowych.
     */
    void            Draw(wxDC&                  dc,
                         const FunctionBase&    function,
                         unsigned               variableIndexX,
                         unsigned               variableIndexY,
                         unsigned               variableCount,
                         const PlotControl&     plotControl,
                         bool                   greaterThanConstraint) const;

    // TODO: zmieni nazw
    void            Collapse();
// ACCESS


// INQUIRY

// DEBUG
    void            DumpStats() const;

private:
    class Contour
    {
        public:
            Contour();
            Contour(const Contour& from);
            Contour(wxPoint a, wxPoint b);

            bool        AddSegment(wxPoint a, wxPoint b);
            bool        AddContour(const Contour& contour);

            void        ToPointArray(wxPoint points[]) const;
            // TODO: wyznaczy stron, w ktr ma by przesunity obrys
            // TODO: zmieni nieco nazw (to nie jest proste przesunicie).
            void        ShiftedToArray(wxPoint      points[],
                                       unsigned&    count,
                                       bool         counterClockwise) const;

            /** Sprawdza, ktra strona poziomicy jest zewntrzna, a ktra
             * wewntrzna.
             *
             * Test polega na sprawdzeniu nachylenia w pierwszym punkcie
             * amanej.
             *
             * @return True, jeli strona zewntrzna jest w kierunku zgodnym
             * z ruchem wskazwek zegara.
             *
             * @see ShiftedToArray
             */
            bool        FindOuterSide(const FunctionBase&   function,
                                      unsigned              variableIndexX,
                                      unsigned              variableIndexY,
                                      unsigned              variableCount,
                                      const PlotControl&    plotControl) const;

            unsigned    SegmentCount() const;
            bool        IsClosed() const;

        private:
            typedef list<wxPoint>     Polyline;

            Polyline    mPolyline;

            wxPoint     mBegin;     ///< Punkt pocztkowy
            wxPoint     mEnd;       ///< Punkt kocowy

            bool        mClosed;    ///< amana zamknita?
    };

    typedef vector<Contour>             ContourContainer;
    typedef vector<ContourContainer>    LevelContainer;

    void            DoDraw(wxDC&                dc,
                           const FunctionBase*  pFunction,
                           unsigned             variableIndexX,
                           unsigned             variableIndexY,
                           unsigned             variableCount,
                           const PlotControl*   pPlotControl,
                           bool                 greaterThanConstraint,
                           unsigned             shadeDistance) const;
    unsigned            MaxSegmentLength() const;
    ContourContainer    Collapse(const ContourContainer& contour);

    const static unsigned LIGHTEST_SHADE;
    const static unsigned DARKEST_SHADE;

    LevelContainer  mLevels;
};

#endif
