/**Szablon panelu dla metod pochodnej i znaku pochodnej.
 *
 * Panel dla dwch metod posiadajcych jednakowe parametry:
 * epsilon, punkt startowy, wspczynnik wpywu pochodnej
 * na przesunicie punktu prbnego (alfa), wspczynnik
 * zmniejszania alfa po nieudanym kroku (gamma).
 */


#ifndef DERIVATIVELINESEARCHPANEL_H
#define DERIVATIVELINESEARCHPANEL_H

#include "../Symbols.h"
#include "../text-util.h"

#include "DoubleParameterValidator.h"
#include "MethodPanelInfo.h"
#include "PointControl.h"
#include "ui_util.h"

template <class T>
class DerivativeLineSearchPanel: public wxPanel
{
public:

    DerivativeLineSearchPanel(wxWindow*     parent,
                              int           id,
                              T*            pMethod,
                              const         wxPoint& pos=wxDefaultPosition,
                              const         wxSize& size=wxDefaultSize,
                              long          style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*         CreatePanel(MethodPanelInfo     panelInfo);

private:
    void do_layout();

protected:
    wxStaticText* mpAlphaLabel;
    wxTextCtrl* mpAlphaTextCtrl;
    wxStaticText* mpGammaLabel;
    wxTextCtrl* mpGammaTextCtrl;
    wxStaticText* mpEpsilonLabel;
    wxTextCtrl* mpEpsilonTextCtrl;

};

template <class T>
DerivativeLineSearchPanel<T>::DerivativeLineSearchPanel
            (wxWindow*            parent,
             int                  id,
             T*                   pMethod,
             const                wxPoint& pos,
             const                wxSize& size,
             long                 style)
            : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{
    const Symbols& symbols = Symbols::GetInstance();
    mpAlphaLabel = new wxStaticText(this, -1,
            _("Initial point shift factor")
            + LabelSymbol(symbols[Symbols::ALPHA]));
    mpAlphaTextCtrl = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition,
            wxDefaultSize, wxTE_RIGHT);
    mpGammaLabel = new wxStaticText(this, -1,
            EmbedString(_("Decrease of %s factor"), symbols[Symbols::ALPHA])
            + LabelSymbol(symbols[Symbols::GAMMA]));
    mpGammaTextCtrl = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition,
            wxDefaultSize, wxTE_RIGHT);
    mpEpsilonLabel = new wxStaticText(this, -1,
            _("Stop condition") + LabelSymbol(symbols[Symbols::EPSILON]));
    mpEpsilonTextCtrl = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition,
            wxDefaultSize, wxTE_RIGHT);

    do_layout();

    // Walidatory
    mpAlphaTextCtrl->SetValidator(DoubleParameterValidator(pMethod->mAlpha));
    mpGammaTextCtrl->SetValidator(DoubleParameterValidator(pMethod->mGamma));
    mpEpsilonTextCtrl->SetValidator(
            DoubleParameterValidator(pMethod->mEpsilon));

}// DerivativeLineSearchPanel

template <class T>
void DerivativeLineSearchPanel<T>::do_layout()
{
    wxBoxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);
    wxGridSizer* p_grid_sizer = new wxGridSizer(3, 2, 0, 0);
    p_grid_sizer->Add(mpAlphaLabel, 0,
                    wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_grid_sizer->Add(mpAlphaTextCtrl, 0,
                    wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_grid_sizer->Add(mpGammaLabel, 0,
                    wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_grid_sizer->Add(mpGammaTextCtrl, 0,
                    wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);    p_grid_sizer->Add(mpEpsilonLabel, 0,
                    wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_grid_sizer->Add(mpEpsilonTextCtrl, 0,
                    wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_main_sizer->Add(p_grid_sizer, 0, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);
}// do_layout

template <class T>
/* static */ wxPanel*
DerivativeLineSearchPanel<T>::CreatePanel(MethodPanelInfo   panelInfo)
{
    T* p_method = dynamic_cast<T*>(panelInfo.pMethod());

    if (panelInfo.PanelType() == MethodPanelInfo::STANDALONE)
    {
        // Tworzy panel z panelu metody w kierunku oraz kontrolki
        // ustawiania odcinka pocztkowego.
        PointControl* p_starting_point = new PointControl(panelInfo.pParent(),
                -1, wxDefaultPosition, wxDefaultSize, 0, _("Starting point"));
        p_starting_point->Init((panelInfo.pProblem())->Variables(),
               /*p_method->mStartingPoint*/p_method->mInitialStartingPoint);

        return CreatePanelOfTwo(
                panelInfo.pParent(),
                new DerivativeLineSearchPanel<T>(panelInfo.pParent(), -1,
                                                 p_method),
                p_starting_point);
    }
    else
    {
        return new DerivativeLineSearchPanel(panelInfo.pParent(), -1, p_method);
    }
}// CreatePanel

#endif // DERIVATIVELINESEARCHPANEL_H
