#include <precomp.h>

//#include <cassert>
//
//#include <wx/intl.h>
//#include <wx/msgdlg.h>
//#include <wx/textctrl.h>

#include "../Logger.h"
#include "../util.h"

#include "DoubleParameterValidator.h"  // class implemented

LOGGER(INFO);

BEGIN_EVENT_TABLE(DoubleParameterValidator, wxValidator)
    EVT_CHAR(DoubleParameterValidator::OnChar)
END_EVENT_TABLE()


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DoubleParameterValidator::DoubleParameterValidator(
        DoubleParameter& rParameter, const wxString& format)
    : mrParameter(rParameter), mFormat(format)
{
}// DoubleParameterValidator

DoubleParameterValidator::DoubleParameterValidator(
        const DoubleParameterValidator& from)
    : mrParameter(from.mrParameter), mFormat(from.mFormat)
{
}// DoubleParameterValidator

DoubleParameterValidator::~DoubleParameterValidator()
{
}// ~DoubleParameterValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
DoubleParameterValidator::Clone() const
{
    return new DoubleParameterValidator(*this);
}// Clone

/* virtual */ bool
DoubleParameterValidator::TransferToWindow()
{
    dynamic_cast<wxTextCtrl*>(GetWindow())
            ->SetValue(wxString::Format(mFormat, static_cast<double>(mrParameter)));

    return true;
}// TransferToWindow

/* virtual */ bool
DoubleParameterValidator::TransferFromWindow()
{
    double tmp;
    bool result;
    wxString value = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    result = value.ToDouble(&tmp);

    if (result) mrParameter = tmp;

    return result;
}// TransferFromWindow

/* virtual */ bool
DoubleParameterValidator::Validate(wxWindow* pParent)
{
    double dummy;
    wxString value = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    bool result = value.ToDouble(&dummy);

    if (!result)
    {
        wxMessageDialog dialog(pParent, _("Please enter a number!"),
                _("Invalid input"), wxOK);
        dialog.ShowModal();
        GetWindow()->SetFocus();
    }
    else if (!mrParameter.IsValid(dummy))
    {
        result = false;

        wxString message;
        if (mrParameter.HasIntegerType())
            message = _("The entered value must be an integer in ");
        else
            message = _("The entered value must be in ");
        message << mrParameter.BoundaryToString();

        wxMessageDialog dialog(pParent, message, _("Invalid input"), wxOK);
        dialog.ShowModal();

        GetWindow()->SetFocus();
    }

    return result;
}// Validate

void
DoubleParameterValidator::OnChar(wxKeyEvent& event)
{
    if (mDecimalSeparator == '\0')
    {
        mDecimalSeparator = DiscoverDecimalSeparator();
    }

    static wxString numeric_chars(
            wxString(wxT("0123456789-+")) + mDecimalSeparator);

    // Skip(true) oznacza, e event bdzie propagowany dalej :)
    int key_code = event.GetKeyCode();

    if ((key_code < WXK_SPACE || key_code == WXK_DELETE || key_code > WXK_START)
        || (-1 < numeric_chars.Find(event.GetKeyCode())))
    {
        event.Skip(true);
    } else if (!wxValidator::IsSilent()) wxBell();
}// OnChar

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

// \0 oznacza, e trzeba odkry, patrz OnChar
wxChar DoubleParameterValidator::mDecimalSeparator('\0');
