#include <precomp.h>

#include "../Logger.h"
#include "DoubleValidator.h"  // class implemented

LOGGER(INFO);

BEGIN_EVENT_TABLE(DoubleValidator, wxValidator)
    EVT_CHAR(DoubleValidator::OnChar)
END_EVENT_TABLE()


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DoubleValidator::DoubleValidator(double& rValue, const wxString& format)
    : mrValue(rValue), mFormat(format)
{
}// DoubleValidator

DoubleValidator::DoubleValidator(const DoubleValidator& from)
    : mrValue(from.mrValue), mFormat(from.mFormat)
{
}// DoubleValidator

DoubleValidator::~DoubleValidator()
{
}// ~DoubleValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
DoubleValidator::Clone() const
{
    return new DoubleValidator(*this);
}// Clone

/* virtual */ bool
DoubleValidator::TransferToWindow()
{
    dynamic_cast<wxTextCtrl*>(GetWindow())
            ->SetValue(wxString::Format(mFormat, mrValue));

    return true;
}// TransferToWindow

/* virtual */ bool
DoubleValidator::TransferFromWindow()
{
    wxString value = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    return value.ToDouble(&mrValue);
}// TransferFromWindow

/* virtual */ bool
DoubleValidator::Validate(wxWindow* pParent)
{
    double dummy;
    wxString value = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    bool result = value.ToDouble(&dummy);

    if (!result)
    {
        wxMessageDialog dialog(pParent, _("Please enter a number!"),
                _("Bad input"), wxOK);
        dialog.ShowModal();
        GetWindow()->SetFocus();
    }

    return result;
}// Validate

void
DoubleValidator::OnChar(wxKeyEvent& event)
{
    if (mDecimalSeparator == '\0')
    {
        mDecimalSeparator = DiscoverDecimalSeparator();
    }

    static wxString numeric_chars(
            wxString(wxT("0123456789-+")) + mDecimalSeparator);

    // Skip(true) oznacza, e event bdzie propagowany dalej :)
    int key_code = event.GetKeyCode();

    if ((key_code < WXK_SPACE || key_code == WXK_DELETE || key_code > WXK_START)
        || (-1 < numeric_chars.Find(event.GetKeyCode())))
    {
        event.Skip(true);
    } else if (!wxValidator::IsSilent()) wxBell();
}// OnChar

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

wxChar
DoubleValidator::DiscoverDecimalSeparator() const
{
    // W duchu prawdziwych odkrywcw...
    wxString text(wxString::Format(wxT("%3.1g"), 0.5));

    LOG(DEBUG, text);

    if ((text[1] == ',') || (text[1] == '.'))   return text[1];
    else
    {
        LOG(ERROR, wxT("Decimal separator doesn't seem to be neither period nor comma.\nFalling back to period."));
        return '.';
    }
}// DiscoverDecimalSeparator

// \0 oznacza, e trzeba odkry, patrz OnChar
wxChar DoubleValidator::mDecimalSeparator('\0');
