/** Warstwa wykresu rysujca przebieg optymalizacji metod ewolucyjn.
 *
 * @see PlotControl
 */

#ifndef _EvolutionaryMethodLayer_h
#define _EvolutionaryMethodLayer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../methods/EvolutionaryMethodIteration.h"
#include "PlotControl.h"
#include "XyResultLayer.h"

// FORWARD REFERENCES
//

class EvolutionaryMethodLayer
    : public XyResultLayer
{
public:

    friend class DrawDeceasedAction;
    friend class OnlyBestAction;

// LIFECYCLE
/*
    struct PointInRegion
    {
        PointInRegion(const ColumnVector& point)
            : mPoint(point)
        {
        };

        ColumnVector    mPoint;
        int             mBoxPartX;
        int             mBoxPartY;
    };
*/
    /** Konstruktor
     */
    EvolutionaryMethodLayer(const PlotControl&          plotControl,
                            const Result&               result,
                                  CountedPtr<PlotLayer> cpFunctionLayer);

    /** Konstruktor kopiujcy.
     */
    EvolutionaryMethodLayer(const EvolutionaryMethodLayer& from);


    /** Destruktor.
     */
    ~EvolutionaryMethodLayer(void);


// OPERATORS

// OPERATIONS

    virtual void    Draw(wxDC& dc) const;

    virtual vector<ToolbarTool> CreateTools(PlotControl* pPlotControl);

    virtual Region  IterationBoundingBox(unsigned iterationNumber) const;

    /** Metoda fabryki.
     */
    static std::auto_ptr<ResultLayer>   Create(
                          const PlotControl&            plotControl,
                          const Result&                 result,
                                CountedPtr<PlotLayer>   cpFunctionLayer);
// ACCESS

// INQUIRY


protected:
private:

    void            DrawIteration(wxDC&     dc,
                                  unsigned  iterationNumber,
                                  bool      highlight) const;

    void            DrawDeceased
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const;

    void            DrawSurvivors
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const;

    void            DrawMutants
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const;

    void            DrawChildren
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const;

    void            DrawStartingBest
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const;

    void            DrawEndBest
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const;

    void            DrawImprovementLine
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const;

    void            DrawChromosomeContainer(wxDC&           dc,
    const EvolutionaryMethodIteration::ChromosomeContainer& container,
          AttributeDatabase::AttributeType                  attribute) const;
/*
    void            RemoveOutliers(const Region&            boundingBox,
                                   vector<PointInRegion>&   points) const;

*/
    bool            mDrawDeceased;
    bool            mDrawOnlyBest;

    const static void*  mRegistrationDummy;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _EvolutionaryMethodLayer_h
