#include <precomp.h>

//#include <wx/valgen.h>

#include "../Logger.h"

#include "DoubleParameterValidator.h"
#include "MethodPanelFactory.h"
#include "EvolutionaryMethodPanel.h"
#include "GeneticOperatorValidator.h"
#include "InitialRangeValidator.h"
#include "IntegerValidator.h"
#include "StopConditionValidator.h"

LOGGER(WARNING)

EvolutionaryMethodPanel::EvolutionaryMethodPanel(wxWindow* parent,
                                 int                    id,
                                 EvolutionaryMethod*            pMethod,
                                 const ProblemBase&     problem,
                                 const                  wxPoint& pos,
                                 const                  wxSize& size,
                                 long                   style):
    wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{

    mpInitialPopulationSizer_staticbox
                    = new wxStaticBox(this, -1, _("Initial population"));
    mpSelectionMethodPanel
                    = new GeneticOperatorControl<SelectionMethod>(this, -1);
    mpMutationMethodPanel
                    = new GeneticOperatorControl<MutationMethod>(this, -1);
    mpCrossoverMethodPanel
                    = new GeneticOperatorControl<CrossoverMethod>(this, -1);
    mpNumberOfChromosomesLabel
                    = new wxStaticText(this, -1, _("Number of chromosomes:"));
    mpNumberOfChromosomesTextCtrl = new wxTextCtrl(this, -1, wxT(""),
                                                   wxDefaultPosition,
                                                   wxDefaultSize,
                                                   wxTE_RIGHT);
    mpSurvivalRateLabel = new wxStaticText(this, -1, _("Survival rate:"));
    mpSurvivalRateCtrl  = new wxTextCtrl(this, -1, wxT(""),
                                         wxDefaultPosition,
                                         wxDefaultSize,
                                         wxTE_RIGHT);
    mpRadioactivityLabel = new wxStaticText(this, -1, _("Radioactivity:"));
    mpRadioactivityCtrl  = new wxTextCtrl(this, -1, wxT(""),
                                          wxDefaultPosition,
                                          wxDefaultSize,
                                          wxTE_RIGHT);
    mpMutationRateLabel = new wxStaticText(this, -1, _("Mutation rate:"));
    mpMutationRateCtrl  = new wxTextCtrl(this, -1, wxT(""),
                                         wxDefaultPosition,
                                         wxDefaultSize,
                                         wxTE_RIGHT);
    mpCrossoverRateLabel = new wxStaticText(this, -1, _("Crossover rate:"));
    mpCrossoverRateCtrl  = new wxTextCtrl(this, -1, wxT(""),
                                          wxDefaultPosition,
                                          wxDefaultSize,
                                          wxTE_RIGHT);
    mpStopCondition = new StopConditionControl(this, -1);
    mpStartingRange = new RangeControl(this, -1,
                                            wxDefaultPosition,
                                            wxDefaultSize,
                                            0, _("Range"));

    set_properties();
    do_layout();

    // Inicjowanie kontrolek
    mpSelectionMethodPanel->SetList(SelectionMethod::SelectionMethods());
    mpMutationMethodPanel->SetList(MutationMethod::MutationMethods());
    mpCrossoverMethodPanel->SetList(CrossoverMethod::CrossoverMethods());

    mpStopCondition->SetList(pMethod->AllowedStopConditions());

    mpStartingRange->Init(problem.Variables(),
                          pMethod->mInitialRangeMinBounds,
                          pMethod->mInitialRangeMaxBounds);

    // Walidatory.
    mpSelectionMethodPanel->SetValidator
      (GeneticOperatorValidator<SelectionMethod>(pMethod->mcpSelectionMethod));
    mpMutationMethodPanel->SetValidator
      (GeneticOperatorValidator<MutationMethod>(pMethod->mcpMutationMethod));
    mpCrossoverMethodPanel->SetValidator
      (GeneticOperatorValidator<CrossoverMethod>(pMethod->mcpCrossoverMethod));

    mpSurvivalRateCtrl->SetValidator(
                        DoubleParameterValidator(pMethod->mSurvivalRate));
    mpRadioactivityCtrl->SetValidator(
                        DoubleParameterValidator(pMethod->mRadioactivity));
    mpMutationRateCtrl->SetValidator(
                        DoubleParameterValidator(pMethod->mMutationRate));
    mpCrossoverRateCtrl->SetValidator(
                        DoubleParameterValidator(pMethod->mCrossoverRate));
    // TODO: tu te uproci, adnych reinterpret_castw!
    mpStopCondition->SetValidator(StopConditionValidator(
                    reinterpret_cast<CountedPtr<const StopConditionBase>&>(
                        pMethod->mcpStopCondition)));

    mpNumberOfChromosomesTextCtrl->SetValidator(IntegerValidator
        (reinterpret_cast<long&>(pMethod->mNumberOfChromosomes), 10l, 1000l));
    mpStartingRange->SetValidator(InitialRangeValidator
                               (dynamic_cast<EvolutionaryMethod*>(pMethod),
                                pMethod->mInitialRangeMinBounds,
                                pMethod->mInitialRangeMaxBounds));

}

void EvolutionaryMethodPanel::set_properties()
{
}

void EvolutionaryMethodPanel::do_layout()
{

    wxStaticBoxSizer* p_params_box_sizer = new wxStaticBoxSizer(wxHORIZONTAL,
                            this, wxString(_("Genetic operators parameters")));
    wxFlexGridSizer* p_params_grid_sizer = new wxFlexGridSizer(4, 2, 0, 0);
    p_params_grid_sizer->AddGrowableCol(0, 2);
    p_params_grid_sizer->AddGrowableCol(1, 1);
    p_params_grid_sizer->Add(mpSurvivalRateLabel, 0,
                  wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpSurvivalRateCtrl, 0,
                  wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpRadioactivityLabel, 0,
                  wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpRadioactivityCtrl, 0,
                  wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpMutationRateLabel, 0,
                  wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpMutationRateCtrl, 0,
                  wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpCrossoverRateLabel, 0,
                  wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_grid_sizer->Add(mpCrossoverRateCtrl, 0,
                  wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_params_box_sizer->Add(p_params_grid_sizer, 1, wxEXPAND);

    wxFlexGridSizer* p_start_grid_sizer = new wxFlexGridSizer(1, 2, 0, 0);
    p_start_grid_sizer->AddGrowableCol(0, 2);
    p_start_grid_sizer->AddGrowableCol(1, 1);
    p_start_grid_sizer->Add(mpNumberOfChromosomesLabel, 0,
                  wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    p_start_grid_sizer->Add(mpNumberOfChromosomesTextCtrl, 0,
                  wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);

    wxStaticBoxSizer* p_initial_population_sizer
                    = new wxStaticBoxSizer(mpInitialPopulationSizer_staticbox,
                                           wxVERTICAL);
    p_initial_population_sizer->Add(p_start_grid_sizer, 0, wxEXPAND, 0);
    p_initial_population_sizer->Add(mpStartingRange, 1, wxEXPAND, 0);

    wxBoxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);
    p_main_sizer->Add(mpSelectionMethodPanel, 0, wxEXPAND, 0);
    p_main_sizer->Add(mpMutationMethodPanel, 0, wxEXPAND, 0);
    p_main_sizer->Add(mpCrossoverMethodPanel, 0, wxEXPAND, 0);
    p_main_sizer->Add(p_params_box_sizer, 1, wxEXPAND, 0);
    p_main_sizer->Add(mpStopCondition, 0, wxEXPAND, 0);
    p_main_sizer->Add(p_initial_population_sizer, 1, wxEXPAND, 0);

    SetAutoLayout(true);
    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);

}

/* static */ wxPanel*
EvolutionaryMethodPanel::CreatePanel(MethodPanelInfo   panelInfo)
{
    return new EvolutionaryMethodPanel(
            panelInfo.pParent(),
            -1,
            dynamic_cast<EvolutionaryMethod*>(panelInfo.pMethod()),
            *panelInfo.pProblem());
}// CreatePanel

// Zarejestruj metod tworzc panel
const void* EvolutionaryMethodPanel::mpRegistrationDummy =
        MethodPanelFactory::GetInstance().Register(
                EvolutionaryMethod::ClassId(),
                EvolutionaryMethodPanel::CreatePanel);

