#include <precomp.h>

#include "../methods/FibonacciDivisionIteration.h"

#include "FibonacciDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FibonacciDivisionLayer::FibonacciDivisionLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// FibonacciDivisionLayer

FibonacciDivisionLayer::FibonacciDivisionLayer(const FibonacciDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// FibonacciDivisionLayer

FibonacciDivisionLayer::~FibonacciDivisionLayer()
{
}// ~FibonacciDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
FibonacciDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
FibonacciDivisionLayer::Create(const PlotControl&           plotControl,
                               const Result&                result,
                                     CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new FibonacciDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
FibonacciDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const FibonacciDivisionIteration& iteration =
        dynamic_cast<const FibonacciDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
FibonacciDivisionLayer::DrawIteration(wxDC&             dc,
                                  unsigned          iterationNumber,
                                  bool              highlight) const
{
    const FibonacciDivisionIteration& iteration =
        dynamic_cast<const FibonacciDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if (highlight)
    {   // Punkty
        DrawBigPoint(dc, iteration.LeftBound(),
                AttributeDatabase::NORMAL);
        DrawBigPoint(dc, iteration.RightBound(),
                AttributeDatabase::NORMAL);

        DrawBigPoint(dc, iteration.BetterPoint(),
                                            AttributeDatabase::POINT_BEST);
        DrawBigPoint(dc, iteration.WorsePoint(),
                                            AttributeDatabase::POINT_WORST);
    }

}// DrawIteration

void
FibonacciDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const FibonacciDivisionIteration& iteration =
        dynamic_cast<const FibonacciDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ void* FibonacciDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            FibonacciDivisionIteration::ClassId(),
            FibonacciDivisionLayer::Create);
