/** Klasa odpowiedzialna za wizualizacj wynikw na wykresie.
 *
 * Wybrana iteracja przebiegu optymalizacji jest pokazywana szczegowo wraz z
 * opisem tekstowym.
 */

#ifndef _FullPlotVisualizer_h
#define _FullPlotVisualizer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "PlotControl.h"
#include "PlotStatus.h"
#include "PlotToolbar.h"
#include "ResultLayer.h"
#include "TextIterationVisualizer.h"
#include "Visualizer.h"

// FORWARD REFERENCES
//


class FullPlotVisualizer
    : public Visualizer
{
    enum {
        ID_LIST_ITERATIONS = 0,
        ID_BUTTON_NEXT,
        ID_BUTTON_PREVIOUS
    };
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param pParent Okno rodzica.
     * @param result Wynik optymalizacji.
     * @param resultLayerCreator Funkcja tworzca odpowiedni warstw
     * wizualizacji dla wykresu.
     * @param apTextIterationVisualizer Wizualizator tekstowy iteracji.
     */
    FullPlotVisualizer(
            wxWindow*                                   pParent,
            const Result&                               result,
            const ResultLayer::Creator                  resultLayerCreator,
            auto_ptr<const TextIterationVisualizer>     apTextIterationVisualizer);


    /** Destruktor.
     */
    ~FullPlotVisualizer(void);


    void OnIterationSelect(wxCommandEvent& event);

    void OnNext(wxCommandEvent& event);
    void OnPrevious(wxCommandEvent& event);

// OPERATORS

// OPERATIONS

    void    SetFitIteration(bool toggle);

// ACCESS

    /** Zwraca nazw tego typu wizualizacji.
     */
    virtual wxString        Name() const;

// INQUIRY

    bool        IsFitIteration() const;

protected:
private:

    /** Zaznacza iteracj.
     *
     * @param selectionIndex Pozycja iteracji na licie (zaczynajc od 0).
     * @param select Czy ustawi zaznaczenie na kontrolce listy.
     */
    void        SelectIteration(unsigned selectionIndex, bool select);

    /** Ustawia viewport tak, aby zaznaczona iteracja bya jak najbardziej
     * powikszona.
     */
    void        FitIteration();

    void        FillIterationList(wxListBox* pListBox, const Result& result)
                        const;

    void        SetUpStatusAndIterationControlPanel(
                        wxPanel*        pPanel,
                        PlotStatus*     pPlotStatus,
                        wxStaticText*   pStaticIteration);

    wxPanel*    SetUpPlotPanel(wxWindow*            pParent,
                               ResultLayer::Creator resultLayerCreator);

    void        SetUpToolbarTools(PlotToolbar *pToolbar);

    CountedPtr<ResultLayer>                     mcpResultPlotLayer;
    CountedPtr<PlotLayer>                       mcpFunctionPlotLayer;
    auto_ptr<const TextIterationVisualizer>     mapTextIterationVisualizer;

    wxSplitterWindow*   mpTopBottomSplitter;    ///< Podzia gra-d.
    wxSplitterWindow*   mpPlotListSplitter;     ///< Podzia wykres-lista (gra)
    wxSplitterWindow*   mpStatusTextSplitter;   ///< Podzia status-tekst (d)

    PlotControl*        mpPlotControl;          ///< Kontrolka z wykresem.
    wxListBox*          mpListIterations;       ///< Lista numerw iteracji.
    wxTextCtrl*         mpTextIteration;        ///< Opis zaznaczonej iteracji.
    PlotStatus*         mpPlotStatus;           ///< Pozycja kursora na wykresie
    wxStaticText*       mpStaticIteration;      ///< Tekst z numerem iteracji.

    wxButton*           mpButtonNext;       ///< Przycisk "nastpna iteracja".
    wxButton*           mpButtonPrevious;   ///< Przycisk "poprzednia iteracja".

    const static wxString   mName;              ///< Nazwa wizualizatora.
    const static unsigned   mIterationListWidth;///< Szeroko listy iteracji.
    /// Pocztkowa wysoko okienka z tekstowym opisem iteracji.
    const static unsigned   mTextIterationDefaultHeight;

    bool        mFitIteration;  ///< Czy automatycznie zoomowa do zazn. iter.

    DECLARE_EVENT_TABLE();
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FullPlotVisualizer_h
