/** Kontrolka wyboru operatora genetycznego
 */

#ifndef _GeneticOperatorControl_h
#define _GeneticOperatorControl_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "ObjectChoice.h"

// FORWARD REFERENCES
//

template <class T>
class GeneticOperatorControl
    : public wxPanel
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    GeneticOperatorControl(wxWindow*        pParent,
                           wxWindowID       id,
                           const wxPoint&   pos = wxDefaultPosition,
                           const wxSize&    size = wxDefaultSize,
                           long             style = 0)
        : wxPanel(pParent, id, pos, size, wxTAB_TRAVERSAL)
    {
        wxSizer* p_sizer_main = new wxStaticBoxSizer(wxVERTICAL,
                                                     this,
                                                     T::BaseClassName());

        mpOperatorChoice = new ObjectChoice<T>(this, -1);
        p_sizer_main->Add(mpOperatorChoice, 0, wxALL|wxEXPAND, 2);

        SetAutoLayout(true);
        SetSizer(p_sizer_main);
        p_sizer_main->Fit(this);
        p_sizer_main->SetSizeHints(this);

        Layout();
    }

    /** Destruktor.
     */
    ~GeneticOperatorControl(void)
    {
    }

// OPERATORS
// OPERATIONS

    /** Dodanie operatora genetycznego.
     */
    void        Insert(CountedPtr<T> cpOperator)
    {
        const wxString& name = cpOperator->Name();

        mpOperatorChoice->Insert(name, cpOperator);
    }// Insert

    /** Dodanie do kontrolki listy operatorw.
     */
    void        SetList(const std::vector<T*>& operatorsContainer)
    {
        CountedPtr<T>                   cp_operator;
        typename std::vector<T*>::const_iterator iter;

        for (iter = operatorsContainer.begin();
             iter != operatorsContainer.end();
             iter++)
        {
            cp_operator.Reset(((*iter)->Clone()).release());
            const wxString& name = cp_operator->Name();
            mpOperatorChoice->Insert(name, cp_operator);
        }
    }

    /** Tworzy operator genetyczny na podstawie wyboru uytkownika
     *
     * Tworzony jest przez klonowanie obiekt operatora genetycznego
     */
    std::auto_ptr<T>    CreateSelected()
    {
        return mpOperatorChoice->GetSelected()->Clone();
    }// CreateSelected

    /** Ustawia zaznaczenie kontrolki na pozycj, ktra odpowiada podanej
     * nazwie.
     *
     * @return Pozycja na licie (liczc od 0) albo <code>wxNOT_FOUND</code>
     */
    int         SelectName(const wxString& name)
    {
        int position = mpOperatorChoice->FindString(name);

        if (position != wxNOT_FOUND)
                mpOperatorChoice->SetSelection(position);

        return position;
    }// SelectName

// ACCESS
// INQUIRY

protected:
private:

    ObjectChoice<T>*  mpOperatorChoice;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _GeneticOperatorControl_h
