#include <precomp.h>

#include "../GradientMethodIteration.h"

#include "GradientMethodLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

GradientMethodLayer::GradientMethodLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// GradientMethodLayer

GradientMethodLayer::GradientMethodLayer(const GradientMethodLayer& from)
    : XyUnconstrainedLayer(from)
{
}// GradientMethodLayer

GradientMethodLayer::~GradientMethodLayer()
{
}// ~GradientMethodLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
GradientMethodLayer::Create(const PlotControl&          plotControl,
                            const Result&               result,
                            CountedPtr<PlotLayer>       cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new GradientMethodLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================


/* virtual */ Region
GradientMethodLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const GradientMethodIteration& iteration =
        dynamic_cast<const GradientMethodIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));

    if (iteration.StepMade())
    {
        bounding_box.Extend(iteration.EndPoint()(mVariableXIndex),
                            iteration.EndPoint()(mVariableYIndex));
    }

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
GradientMethodLayer::DrawIteration(wxDC&       dc,
                                      unsigned    iterationNumber,
                                      bool        highlight) const
{
    const GradientMethodIteration& iteration =
        dynamic_cast<const GradientMethodIteration&>
            (mResult.Iteration(iterationNumber));

    PointList   points_to_draw;

    ColumnVector    base_point = iteration.StartingPoint();

    if (highlight || iterationNumber == mResult.FirstIterationNumber())
    {
        if (iterationNumber == mResult.FirstIterationNumber())
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(base_point),
                                        AttributeDatabase::POINT_START,
                                        SQUARE));
        }
        else
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(base_point),
                                        AttributeDatabase::POINT_START,
                                        CIRCLE));
        }
    }

    if (iteration.StepMade())
    {
        AttributeDatabase::AttributeType attribute =
            AttributeDatabase::INACTIVE;
        if (highlight)      attribute = AttributeDatabase::STEP_IN_DIRECTION;

        DrawLine(dc, base_point, iteration.EndPoint(), attribute);
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(
                                                    iteration.EndPoint()),
                                                    attribute));
    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* GradientMethodLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            GradientMethodIteration::ClassId(),
            GradientMethodLayer::Create);
