#include <precomp.h>

#include "../methods/HookeJeevesIteration.h"

#include "HookeJeevesLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HookeJeevesLayer::HookeJeevesLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// HookeJeevesLayer

HookeJeevesLayer::HookeJeevesLayer(const HookeJeevesLayer& from)
    : XyUnconstrainedLayer(from)
{
}// HookeJeevesLayer

HookeJeevesLayer::~HookeJeevesLayer()
{
}// ~HookeJeevesLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
HookeJeevesLayer::Create(const PlotControl&     plotControl,
                         const Result&          result,
                         CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new HookeJeevesLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
HookeJeevesLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const HookeJeevesIteration& iteration =
        dynamic_cast<const HookeJeevesIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));

    HookeJeevesIteration::TrialPointContainer::const_iterator i;
    for (i = iteration.TrialPoints().begin();
         i != iteration.TrialPoints().end();
         ++i)
    {
         bounding_box.Extend(i->mPoint(mVariableXIndex),
                             i->mPoint(mVariableYIndex));
    }

    if (iteration.IsStepCancelled())
    {
        bounding_box.Extend(iteration.OldBasePoint()(mVariableXIndex),
                            iteration.OldBasePoint()(mVariableYIndex));
    }
    else
    {
        bounding_box.Extend(iteration.EndPoint()(mVariableXIndex),
                            iteration.EndPoint()(mVariableYIndex));
    }

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
HookeJeevesLayer::DrawIteration(wxDC&       dc,
                                        unsigned    iterationNumber,
                                        bool        highlight) const
{
    const HookeJeevesIteration& iteration =
        dynamic_cast<const HookeJeevesIteration&>
            (mResult.Iteration(iterationNumber));

    PointList   points_to_draw;

    ColumnVector    current = iteration.StartingPoint();

    if (highlight || (iterationNumber == mResult.FirstIterationNumber()))
    {
        if (iterationNumber == mResult.FirstIterationNumber())
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                                AttributeDatabase::POINT_START,
                                                SQUARE));
        }
        else
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                                AttributeDatabase::POINT_START,
                                                CIRCLE));
        }
    }

    AttributeDatabase::AttributeType attribute = AttributeDatabase::INACTIVE;

    if (highlight)  attribute = AttributeDatabase::STEP_TRIAL_D1;

    unsigned step_no = 1;
    HookeJeevesIteration::TrialPointContainer::const_iterator i;
    for (i = iteration.TrialPoints().begin();
         i != iteration.TrialPoints().end();
         ++i, ++step_no)
    {
        if (i->mBetter)
        {
            DrawLine(dc, current, i->mPoint, attribute);
            current = i->mPoint;
        }
        if (highlight || ! i->mBetter)
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(i->mPoint),
                                                     attribute));
        }

        if (highlight && (i->mBetter || (step_no == 2)))
        {
            attribute = AttributeDatabase::STEP_TRIAL_D2;
        }
    }

    if (iteration.IsStepCancelled())
    {
        if (highlight)
        {
            points_to_draw.push_back(
                    ColourPointType(PlotToDcPoint(iteration.OldBasePoint()),
                                    AttributeDatabase::POINT_BASE));
        }
    }
    else
    {
        if (highlight)
        {// Zaznacz nowy bazowy i ustaw kolor dla kroku.
            points_to_draw.push_back(
                    ColourPointType(PlotToDcPoint(iteration.BasePoint()),
                                    AttributeDatabase::POINT_BASE));
            attribute = AttributeDatabase::STEP_IN_DIRECTION;
        }
        DrawLine(dc, current, iteration.EndPoint(), attribute);

        // Zaznacz punkt kocowy
        if (highlight)
        {
            points_to_draw.push_back(
                    ColourPointType(PlotToDcPoint(iteration.EndPoint()),
                                    attribute));
        }
    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* HookeJeevesLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            HookeJeevesIteration::ClassId(),
            HookeJeevesLayer::Create);
