#include <precomp.h>

//#include <cassert>
//
//#include <wx/intl.h>
//#include <wx/msgdlg.h>
//#include <wx/textctrl.h>

#include "../Logger.h"
#include "../util.h"

#include "InitialRangeValidator.h"  // class implemented

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////
//============================= LIFECYCLE ====================================

InitialRangeValidator::InitialRangeValidator
           (EvolutionaryMethod* pMethod,
            ColumnVector&       rRangeMinBounds,
            ColumnVector&       rRangeMaxBounds)
    : mpMethod(pMethod),
      mrRangeMinBounds(rRangeMinBounds),
      mrRangeMaxBounds(rRangeMaxBounds)
{
    assert(mpMethod != NULL);
}// InitialRangeValidator

InitialRangeValidator::InitialRangeValidator(const InitialRangeValidator& from)
    : mpMethod(from.mpMethod),
      mrRangeMinBounds(from.mrRangeMinBounds),
      mrRangeMaxBounds(from.mrRangeMaxBounds)
{
}// InitialRangeValidator

InitialRangeValidator::~InitialRangeValidator()
{
}// ~InitialRangeValidator

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ wxObject*
InitialRangeValidator::Clone() const
{
    LOG(INFO, wxT("Cloning InitialRangeValidator"));

    return new InitialRangeValidator(*this);
}// Clone

/* virtual */ bool
InitialRangeValidator::TransferToWindow()
{
    LOG(INFO, wxT("ToWindow"));
    GetWindow()->TransferDataToWindow();

    return true;
}// TransferToWindow

/* virtual */ bool
InitialRangeValidator::TransferFromWindow()
{
    LOG(INFO, wxT("FromWindow"));
    bool result = GetWindow()->TransferDataFromWindow();

    mpMethod->mInitialRangeMinBounds = mrRangeMinBounds;
    mpMethod->mInitialRangeMaxBounds = mrRangeMaxBounds;

    return result;
}// TransferFromWindow

/* virtual */ bool
InitialRangeValidator::Validate(wxWindow* pParent)
{
    LOG(INFO, wxT("Validate"));
    wxString message;

    bool result = GetWindow()->Validate() &&
                  GetWindow()->TransferDataFromWindow();

    if (result)
    {
        int     i = 1;

        while (result && i <= mrRangeMinBounds.nrows())
        {
            if (mrRangeMinBounds(i) >= mrRangeMaxBounds(i))
            {
                message = wxString::Format
        (_("Right bound of the variable no %d is smaller than left bound"), i);
                result = false;
            }
            else if (isinf(mrRangeMaxBounds(i) - mrRangeMinBounds(i)))
            {
                message = wxString::Format
                    (_("The range of variable %d is greater than DBL_MAX"), i);
                result = false;
            }

            ++i;
        }

        if (!result)
        {
            wxMessageDialog dialog(pParent, message, _("Invalid input"), wxOK);
            dialog.ShowModal();
            GetWindow()->SetFocus();
        }
    }

    return result;
}// Validate

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

