/** wxValidator dla obszaru, na ktrym ma powsta pocztkowa iteracja w metodzie
 * ewolucyjnej.
 *
 * @see RangeControl, wxValidator
 */

#ifndef _InitialRangeValidator_h
#define _InitialRangeValidator_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <wx/validate.h>

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../methods/EvolutionaryMethod.h"

// FORWARD REFERENCES
//

class InitialRangeValidator
    : public wxValidator
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    InitialRangeValidator(EvolutionaryMethod*   pMethod,
                          ColumnVector&         rRangeMinBounds,
                          ColumnVector&         rRangeMaxBounds);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    InitialRangeValidator(const InitialRangeValidator& from);

    /** Destruktor.
     */
    ~InitialRangeValidator(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi obiektu.
     */
    virtual wxObject*       Clone() const;

    /** Wprowadza dane ze zmiennej do kontrolki.
     */
    virtual bool            TransferToWindow();

    /** Wyprowadza dane z kontrolki do zmiennej.
     */
    virtual bool            TransferFromWindow();

    /** Sprawdza poprawno danych wprowadzonych do kontrolki.
     *
     * W przypadku nieprawidowoci wywietla okienko z komunikatem.
     */
    virtual bool            Validate(wxWindow* pParent);

// ACCESS
// INQUIRY

protected:

    EvolutionaryMethod*     mpMethod;
    ColumnVector&           mrRangeMinBounds;
    ColumnVector&           mrRangeMaxBounds;

private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _InitialRangeValidator_h
