#include <precomp.h>

#include "../Logger.h"
#include "../Symbols.h"
#include "../util.h"

#include "InitialSectionValidator.h"  // class implemented

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

InitialSectionValidator::InitialSectionValidator(
        SectionLineSearch*  pMethod,
        DoubleParameter& rLeftBound, DoubleParameter& rRightBound
        )
    : mpMethod(pMethod),
      mrLeftBound(rLeftBound), mrRightBound(rRightBound)
{
    assert(mpMethod != NULL);
}// InitialSectionValidator

InitialSectionValidator::InitialSectionValidator(
        const InitialSectionValidator& from)
    : mpMethod(from.mpMethod),
      mrLeftBound(from.mrLeftBound), mrRightBound(from.mrRightBound)
{
}// InitialSectionValidator

InitialSectionValidator::~InitialSectionValidator()
{
}// ~InitialSectionValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
InitialSectionValidator::Clone() const
{
    LOG(INFO, wxT("Clone"));
    LOG(INFO, wxString::Format(wxT("LB: %g, RB: %g"),
                static_cast<double>(mrLeftBound),
                static_cast<double>(mrRightBound)));

    return new InitialSectionValidator(*this);
}// Clone

/* virtual */ bool
InitialSectionValidator::TransferToWindow()
{
    LOG(INFO, wxT("ToWindow"));
    GetWindow()->TransferDataToWindow();

    return true;
}// TransferToWindow

/* virtual */ bool
InitialSectionValidator::TransferFromWindow()
{
    LOG(INFO, wxT("FromWindow"));
    bool result = GetWindow()->TransferDataFromWindow();

    LOG(INFO, wxString::Format(wxT("LB: %g, RB: %g"),
                static_cast<double>(mrLeftBound),
                static_cast<double>(mrRightBound)));

    mpMethod->mInitialA = mrLeftBound;
    mpMethod->mInitialB = mrRightBound;

    return result;
}// TransferFromWindow

/* virtual */ bool
InitialSectionValidator::Validate(wxWindow* pParent)
{
    LOG(INFO, wxT("Validate"));
    wxString message;

    bool result = GetWindow()->Validate() &&
                  GetWindow()->TransferDataFromWindow();

    if (result)
    {
        if (mrRightBound <= mrLeftBound)
        {
            message = _("Section end must be placed on the right of section beginning.");
            result = false;
        }
        else if (mrRightBound - mrLeftBound <= mpMethod->mEpsilon)
        {
            message = _("Initial section length must be bigger than precision")
                + wxString(wxT(" (")) + Symbols::GetInstance()[Symbols::EPSILON]
                + wxString(wxT(")."));
            result = false;
        }

        if (!result)
        {
            wxMessageDialog dialog(pParent, message, _("Invalid input"), wxOK);
            dialog.ShowModal();
            GetWindow()->SetFocus();
        }
    }

    return result;
}// Validate
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

