#include <precomp.h>

#include "../Logger.h"

#include "IntegerValidator.h"  // class implemented

LOGGER(DEBUG);

BEGIN_EVENT_TABLE(IntegerValidator, wxValidator)
    EVT_CHAR(IntegerValidator::OnChar)
END_EVENT_TABLE()

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

IntegerValidator::IntegerValidator(long&            value,
                                   long             min,
                                   long             max,
                                   const wxString&  format)
    : mrValue(value), mFormat(format), mMin(min), mMax(max)
{
    assert(min <= max); // hehe
}// IntegerValidator

IntegerValidator::IntegerValidator(const IntegerValidator& from)
    : mrValue(from.mrValue),
      mFormat(from.mFormat),
      mMin(from.mMin),
      mMax(from.mMax)
{
}// IntegerValidator

IntegerValidator::~IntegerValidator()
{
}// ~IntegerValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
IntegerValidator::Clone() const
{
    return new IntegerValidator(*this);
}// Clone


/* virtual */ bool
IntegerValidator::Validate(wxWindow* pParent)
{
    long value;
    wxString text = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    bool result = text.ToLong(&value);

    if (!result)    ShowError(pParent, _("Please enter a number"));
    else
    {
        if (value < mMin)
        {
            ShowError(pParent, _("Value must be at least: ")
                    + wxString::Format(mFormat, mMin));
            result = false;
        }
        else if (value > mMax)
        {
            ShowError(pParent, _("Value must be at most: ")
                    + wxString::Format(mFormat, mMax));
            result = false;
        }
    }

    return result;
}// Validate

/* virtual */ bool
IntegerValidator::TransferToWindow()
{
    dynamic_cast<wxTextCtrl*>(GetWindow())->SetValue(
            wxString::Format(mFormat, mrValue));

    LOG(DEBUG, wxT("TransferToWindow: ") << wxString::Format(mFormat, mrValue));

    return true;
}// TransferToWindow

/* virtual */ bool
IntegerValidator::TransferFromWindow()
{
    wxString text = dynamic_cast<wxTextCtrl*>(GetWindow())->GetValue();

    return text.ToLong(&mrValue);
}// TransferFromWindow

void
IntegerValidator::OnChar(wxKeyEvent& event)
{
    static wxString numeric_chars(wxString(wxT("0123456789+-")));
    int key_code = event.GetKeyCode();


    if ((key_code < WXK_SPACE || key_code == WXK_DELETE || key_code > WXK_START)
            || (-1 < numeric_chars.Find(key_code)))
    {
        event.Skip(true);
    } else if (!wxValidator::IsSilent()) wxBell();
}// OnChar


//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
IntegerValidator::ShowError(wxWindow* pParent, const wxString& message)
{
    wxMessageDialog dialog(pParent, message, _("Bad input"),
            wxOK | wxICON_ERROR);

    dialog.CenterOnParent();    // to chyba tylko dla Maca
    dialog.ShowModal();

    GetWindow()->SetFocus();
}// ShowError
