#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../IterationValueInterpolation.h"
#include "../Logger.h"

#include "BoundingBoxLayer.h"
#include "FitResultAction.h"
#include "IterationValueVisualizer.h"  // class implemented
#include "UnivariateFunctionLayer.h"
#include "VisualizerFactory.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

IterationValueVisualizer::IterationValueVisualizer(wxWindow*        pParent,
                                                   const Result&    result)
    : Visualizer(pParent, result),
      mapFunction(NULL)
{
    wxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);

    wxPanel* p_plot_panel = SetUpPlotPanel(this);
    p_main_sizer->Add(p_plot_panel, 1, wxEXPAND);

    SetSizer(p_main_sizer);
    p_main_sizer->SetSizeHints(this);

    LOG(DEBUG, wxT("Constructor"));
    wxCommandEvent event(PLOT_FITVIEW_REQUEST_EVENT, 0);
    mpPlotControl->AddPendingEvent(event);
//    mpPlotControl->FitView();
}// IterationValueVisualizer

IterationValueVisualizer::~IterationValueVisualizer()
{
}// ~IterationValueVisualizer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ wxString
IterationValueVisualizer::Name() const
{
    return wxGetTranslation(mName);
}// Name

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

// Podobne, jak w FullPlotVisualizer
wxPanel*
IterationValueVisualizer::SetUpPlotPanel(wxWindow*  pParent)
{
    wxPanel* p_panel = new wxPanel(pParent);
    wxSizer* p_sizer = new wxBoxSizer(wxVERTICAL);

    mapFunction = auto_ptr<const UnivariateFunction>
        (new IterationValueInterpolation(mResult));

    mpPlotControl = new UnivariatePlot(p_panel, wxID_ANY, *(mapFunction.get()),
            NULL, _("Iteration"), wxDefaultPosition, wxSize(400, 300), true);

    CountedPtr<PlotLayer> cp_function_layer
        (new UnivariateFunctionLayer(*mpPlotControl, *mapFunction.get()));
    CountedPtr<PlotLayer> cp_bounding_box_layer
        (new BoundingBoxLayer(*mpPlotControl,
                              Region(mResult.FirstIterationNumber(), 0,
                                     mResult.LastIterationNumber(), 0)));

    mpPlotControl->AppendLayer(cp_function_layer);
    mpPlotControl->AppendLayer(cp_bounding_box_layer);

    mpToolbar = new PlotToolbar(p_panel, wxID_ANY, mpPlotControl);

    // Dodaj narzdzia
    wxBitmap tmp_bitmap = wxXmlResource::Get()->LoadBitmap(wxT("TOOL_FIT"));
    LOG(DEBUG, wxT("bitmap ok? ") << tmp_bitmap.Ok());
    ToolbarTool fit_tool(
            auto_ptr<ClickToolAction>(new FitResultAction(mpPlotControl)),
            ToolbarTool::SET_VIEW,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_FIT")),
            _("Fit"), _("Fit view to whole plot"));
    mpToolbar->AddCustomTool(fit_tool);
    mpToolbar->Realize();

    // Wstaw do sizera
    // TODO: moe da si jako adniej?
    // Obejcie problemu z przyciskami zachodzcymi na wykres. Gruboci
    // marginesw dobrane eksperymentalnie.
#ifdef __WXMSW__
    p_sizer->Add(mpToolbar, 0, wxEXPAND | wxBOTTOM, 3);
#else
    p_sizer->Add(mpToolbar, 0, wxEXPAND);
#endif

    p_sizer->Add(mpPlotControl, 1, wxEXPAND);

    p_panel->SetSizer(p_sizer);
    p_sizer->SetSizeHints(p_panel);

    return p_panel;
}// SetUpPlotPanel

const /* static */ wxString IterationValueVisualizer::mName(
        _("Iteration-Value Plot"));
const /* static */ void*    IterationValueVisualizer::mpRegistrationDummy =
        VisualizerFactory::RegisterIterationValueVisualizer(
                IterationData::ClassId());
