#include <precomp.h>

#include "../Logger.h"

#include "DoubleParameterValidator.h"
#include "LineSearchControl.h"  // class implemented
#include "LineSearchSettingsDialog.h"

LOGGER(DEBUG)

BEGIN_EVENT_TABLE(LineSearchControl, wxPanel)
    EVT_BUTTON(ID_SETTINGS_BUTTON, LineSearchControl::OnSettings)
END_EVENT_TABLE()

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

LineSearchControl::LineSearchControl(
        wxWindow*          pParent,
        wxWindowID         id,
        const ProblemBase& problem,
        const wxPoint&     pos,
        const wxSize&      size,
        long               style)
    : wxPanel(pParent, id, pos, size, wxTAB_TRAVERSAL)
{
    wxSizer* p_sizer = new wxStaticBoxSizer(wxHORIZONTAL, this,
            _("Line Search"));

    // Utwrz kontrolki
    mpLineSearchChoice  = new AutoChoice<LineSearch>(this, -1, &problem);
    mpSettingsButton    = new wxButton(this, ID_SETTINGS_BUTTON,
            _("Settings..."));

    // Powstawiaj do sizera
    p_sizer->Add(mpLineSearchChoice, 1, wxALL | wxALIGN_CENTER_VERTICAL, 2);
    p_sizer->Add(mpSettingsButton, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);

    SetAutoLayout(true);
    SetSizer(p_sizer);
    p_sizer->Fit(this);
    p_sizer->SetSizeHints(this);

    Layout();
}// LineSearchControl

LineSearchControl::~LineSearchControl()
{
}// ~LineSearchControl


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

std::auto_ptr<const LineSearch>
LineSearchControl::CreateSelected() const
{
    return aptr_dynamic_cast<const LineSearch>
        (mpLineSearchChoice->GetSelected()->Clone());
}// CreateSelected

void
LineSearchControl::SelectName(const wxString& name)
{
    int pos = mpLineSearchChoice->FindString(name);

    assert(pos != wxNOT_FOUND);

    mpLineSearchChoice->SetSelection(pos);
}// SelectName

void
LineSearchControl::Reset(const ProblemBase* pProblem)
{
    mpLineSearchChoice->Reset(pProblem);
}// Reset


void
LineSearchControl::OnSettings(wxCommandEvent& event)
{
    LineSearchSettingsDialog dialog(this, -1, mpLineSearchChoice->GetSelected());
    dialog.CenterOnParent();

    dialog.ShowModal();
}// OnSettings

//============================= ACCESS     ===================================


//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
