#include <precomp.h>

#include "../HistoryManager.h"
#include "../LinearParser.h"

#include "ConstraintsValidator.h"
#include "GeneralValidator.h"
#include "LinearProblemPanel.h"
#include "ProblemPanelFactory.h"

LinearProblemPanel::LinearProblemPanel(
        wxWindow*               pParent,
        int                     id,
        LinearProblem*          pProblem,
        const wxPoint&          pos,
        const wxSize&           size,
        long                    style)
    : wxPanel(pParent, id, pos, size, wxTAB_TRAVERSAL)
{
    mpConstraintsPanel = new ConstraintsPanel(this, -1,
            CountedPtr<const Parser>(new LinearParser),
            pProblem,
            FunctionValidator::MULTIVARIATE_ONLY,
            HistoryManager::GetHistory(HistoryManager::LINEAR_FUNCTION),
            HistoryManager::GetHistory(HistoryManager::LINEAR_CONSTRAINT));

    wxBoxSizer* p_sizer = new wxBoxSizer(wxVERTICAL);
    p_sizer->Add(mpConstraintsPanel, 1, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(p_sizer);
    p_sizer->Fit(this);
    p_sizer->SetSizeHints(this);

    // Walidator
    mpConstraintsPanel->SetValidator(ConstraintsValidator(
                pProblem->mcpFunction,
                pProblem->mcpVariables,
                pProblem->mConstraints));
}// LinearProblemPanel

/* static */ wxPanel*
LinearProblemPanel::CreatePanel(ProblemBase* pProblem,
                                wxWindow*    pParent)
{
    wxPanel* p_panel = new LinearProblemPanel(pParent, -1,
            dynamic_cast<LinearProblem*>(pProblem));

    p_panel->SetValidator(GeneralValidator());

    return p_panel;
}// CreatePanel

// Zarejestruj metod tworzc panel
const void* LinearProblemPanel::mpRegistrationDummy =
        ProblemPanelFactory::GetInstance().Register(
                LinearProblem::ClassId(),
                LinearProblemPanel::CreatePanel);
