#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Eduoptim2.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"

#include "AboutDialog.h"
#include "ConfigurationDialog.h"
#include "MainFrame.h"
#include "MethodPanelFactory.h"
#include "ProblemDialog.h"
#include "ProgressDialog.h"
#include "VisualizationFrame.h"

LOGGER(DEBUG);

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_BUTTON(ID_BUTTON_PROBLEM, MainFrame::OnProblem)
    EVT_BUTTON(ID_BUTTON_CONFIGURE, MainFrame::OnConfigure)
    EVT_BUTTON(ID_BUTTON_OPTIMIZE, MainFrame::OnOptimize)
    EVT_BUTTON(ID_BUTTON_ABOUT, MainFrame::OnAbout)
    EVT_CHOICE(ID_METHOD_CHOICE, MainFrame::OnMethodChoice)
    EVT_CLOSE(MainFrame::OnClose)
END_EVENT_TABLE()

MainFrame::MainFrame(wxWindow* parent, int id, const wxString& title, const wxPoint& pos, const wxSize& size, long style)
    : wxFrame(parent, id, title, pos, size, wxDEFAULT_FRAME_STYLE),
      // Domylny problem.
      mpProblem(ProblemBase::GetById(UnconstrainedProblem::ClassId())),
      mNoProblem(true)
      //mpMethod(NULL)
{
    // Ikona
    wxIconBundle icons;
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM16")));
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM32")));
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM48")));
    SetIcons(icons);

    // Glowny panel i splitter
    mpMainPanel = new wxPanel(this, -1);
    mpSplitter = new wxSplitterWindow(mpMainPanel, -1, wxDefaultPosition, wxDefaultSize, 0);

    mpAboutButton = new wxButton(mpMainPanel, ID_BUTTON_ABOUT, _("About..."));
    mpConfigureButton = new wxButton(mpMainPanel, ID_BUTTON_CONFIGURE,
            _("Settings..."));
    mpOptimizeButton = new wxButton(mpMainPanel, ID_BUTTON_OPTIMIZE,
            _("Optimize..."));

    // Pane wprowadzania problemu
    mpProblemPane = new wxPanel(mpSplitter, -1);
    mpProblemStaticbox = new wxStaticBox(mpProblemPane, -1, _("Problem"));
    mpProblemText = new wxTextCtrl(mpProblemPane, -1, wxT(""),
            wxDefaultPosition, wxSize(200, 300),
            wxTE_MULTILINE|wxTE_READONLY|wxHSCROLL);
    mpProblemButton = new wxButton(mpProblemPane, ID_BUTTON_PROBLEM,
            _("Change problem..."));

    // Pane wyboru metody oraz jej opcji
    mpMethodPane = new wxPanel(mpSplitter, -1);
    mpMethodSizer = new wxBoxSizer(wxVERTICAL);
    mpMethodChoice = new AutoChoice<Method>(mpMethodPane, ID_METHOD_CHOICE);
    mpMethodPanel = new wxPanel(mpMethodPane, -1);
    mpMethodPane->SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);


    set_properties();
    do_layout();

    ChangeMethodPanel(mpMethodChoice->GetSelected());

    wxCommandEvent on_problem_event(wxEVT_COMMAND_BUTTON_CLICKED,
                                    ID_BUTTON_PROBLEM);
    AddPendingEvent(on_problem_event);
}


void MainFrame::set_properties()
{
    SetTitle(wxT("EduOptim v2.0.1"));
    mpSplitter->SetMinimumPaneSize(1);  // Nie pozwala usunac zadnego pane'a
    mpSplitter->SetSashGravity(0.3);
}

void MainFrame::do_layout()
{
    // TODO: przejrze i poprawi.
    wxBoxSizer* p_frame_sizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer* p_button_sizer = new wxBoxSizer(wxHORIZONTAL);

    // Ten sizer jest tylko po to, eby uzyska 2px merginesu dookoa ramki
    // p_problembox_sizera.
    wxBoxSizer* p_left_pane_sizer = new wxBoxSizer(wxVERTICAL);
    wxStaticBoxSizer* p_problembox_sizer = new wxStaticBoxSizer(mpProblemStaticbox, wxVERTICAL);

    // Wstaw opis problemu oraz przycisk jego edycji
    p_problembox_sizer->Add(mpProblemText, 1, wxALL|wxEXPAND, 2);
    p_problembox_sizer->Add(mpProblemButton, 0, wxALL|wxEXPAND, 2);

    p_left_pane_sizer->Add(p_problembox_sizer, 1, wxALL|wxEXPAND, 2);
    // Dopasuj rozmiary
    mpProblemPane->SetAutoLayout(true);
    mpProblemPane->SetSizer(p_left_pane_sizer);
    p_left_pane_sizer->Fit(mpProblemPane);
    p_left_pane_sizer->SetSizeHints(mpProblemPane);

    // Ustaw minimalny rozmiar panelu metody
    mpMethodPanel->SetMinSize(MethodPanelFactory::GetInstance().MinSize());
    // Wstaw wybr metody oraz panel ustawie
    mpMethodSizer->Add(mpMethodChoice, 0, wxEXPAND|wxALL, 2);
    mpMethodSizer->Add(mpMethodPanel, 1, wxEXPAND|wxALL, 2);

    mpMethodPane->SetAutoLayout(true);
    mpMethodPane->SetSizer(mpMethodSizer);
    mpMethodSizer->Fit(mpMethodPane);
    mpMethodSizer->SetSizeHints(mpMethodPane);

    // Przyciski
    p_button_sizer->Add(mpAboutButton, 0, wxALL, 2);
    p_button_sizer->AddStretchSpacer();
    p_button_sizer->Add(mpConfigureButton, 0, wxALL, 2);
    p_button_sizer->Add(mpOptimizeButton, 0, wxALL, 2);

    // Splitter z obydwoma pane'ami
    mpSplitter->SplitVertically(mpProblemPane, mpMethodPane);
    // Dodaj splitter oraz przycisk "Optymalizuj" do glownego sizera
    p_main_sizer->Add(mpSplitter, 1, wxEXPAND);
    p_main_sizer->Add(p_button_sizer, 0, wxEXPAND);
    mpMainPanel->SetAutoLayout(true);
    mpMainPanel->SetSizer(p_main_sizer);
    p_main_sizer->Fit(mpMainPanel);
    p_main_sizer->SetSizeHints(mpMainPanel);
    // Panel glowny jest sam w MainFrame (bo ponoc w Windowsie lepiej tak
    // wyglada :)
    p_frame_sizer->Add(mpMainPanel, 1, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(p_frame_sizer);
    p_frame_sizer->Fit(this);
    p_frame_sizer->SetSizeHints(this);

    Layout();
}

void MainFrame::OnProblem(wxCommandEvent& event)
{
    ProblemDialog dialog(this, -1, mpProblem, mNoProblem);

    dialog.CenterOnParent();

    long exit_status = dialog.ShowModal();
    if (wxID_OK == exit_status)
    {
        // TODO: sprawdzi, czy zmieni si problem, a jeli nie, to czy
        // zmienia si funkcja (wtedy wyzerowa punkt startowy)

        mpMethodChoice->Reset(mpProblem);
        ChangeMethodPanel(mpMethodChoice->GetSelected());

        LOG(INFO, wxT("Chosen problem"));
        LOG(INFO, mpProblem->ToString());
        LOG(INFO, wxT("------"));

        mpProblemText->Clear();
        mpProblemText->AppendText(mpProblem->ToString());

        // Mamy ju zadanie optymalizacji.
        mNoProblem = false;
    }
    else if (wxID_EXIT == exit_status)
    {   // Zakocz aplikacj (z wymuszeniem)
        Close(true);
    }
}// OnProblem

void MainFrame::OnMethodChoice(wxCommandEvent& event)
{
    ChangeMethodPanel(mpMethodChoice->GetSelected());
}// OnMethodChoice

void MainFrame::OnConfigure(wxCommandEvent& event)
{
    ConfigurationDialog configuration_dialog(this, -1, wxT(""));

    configuration_dialog.CenterOnParent();

    configuration_dialog.ShowModal();
}// OnConfigure

void MainFrame::OnAbout(wxCommandEvent& event)
{
    AboutDialog about_dialog(this, -1, _("About Eduoptim"));
    about_dialog.CenterOnParent();
    about_dialog.ShowModal();
}// OnAbout

void MainFrame::OnClose(wxCloseEvent& event)
{
    if (event.CanVeto())
    {
        wxMessageDialog dialog(this, _("Really quit?"), _("Quit"),
                wxYES_NO | wxICON_QUESTION);

        dialog.CenterOnParent();

        if (wxID_YES == dialog.ShowModal())
        {
            wxTheApp->ExitMainLoop();
        }
    }
    else    Destroy();
}// OnClose

void MainFrame::OnOptimize(wxCommandEvent& event)
{
    if (mpMethodPane->Validate())
    {
        ProgressDialog& r_progress_dialog = wxGetApp().rProgressDialog();

        r_progress_dialog.CenterOnParent();

        mpMethodPane->TransferDataFromWindow();

        // Klonuj problem i metod
        auto_ptr<const Method> ap_method(mpMethodChoice->GetSelected()->Clone());
        const Method* p_method = ap_method.get();    // TODO: przmyle to

        auto_ptr<ProblemBase> ap_problem = mpProblem->Clone();

        //p_method->ResetStartingConditions();
        p_method->AttachObserver(&r_progress_dialog);

        // Wstaw metod do problemu
        ap_problem->SetMethod(ap_method);

        // Musi by countedPtr, bo wynik jest pobierany take z wyjtku. A
        // wyjtki musz mc by kopiowane.
        CountedPtr<Result> cp_result;

        LOG(INFO, wxT("Selected method"));
        LOG(INFO, ap_problem->UsedMethod().ToString());


        wxBusyCursor busy;

        LOG(INFO, wxT("Problem created, solving..."));
        // Rozwi problem
        try
        {
            wxWindowDisabler disabler(&r_progress_dialog);

            // TODO: Uporzdkowa ten fragment
            r_progress_dialog.CenterOnParent();
            r_progress_dialog.MakeModal(true);
            r_progress_dialog.Show();

            cp_result = ap_problem->StartSolve();

            LOG(INFO, wxT("---- RESULT ----"));
            LOG(INFO, wxT("Number of iterations: ") <<
                    static_cast<int>(cp_result->Iterations().size()));

            LOG(INFO, wxT("Number of function evaluations: ") <<
                    cp_result->EvaluationCount());

            r_progress_dialog.MakeModal(false);
            r_progress_dialog.Hide();
            p_method->DetachObserver(&r_progress_dialog);
        }
        catch (const OptimizationError& error)
        {
            r_progress_dialog.MakeModal(false);
            r_progress_dialog.Hide();
            p_method->DetachObserver(&r_progress_dialog);

            wxString message;

            message << _("An error has occured during optimization:");
            message << wxT('\n') << error.What();
            message << wxT("\n\n") << _("Do you want to see partial results?");

            wxMessageDialog dialog(this, message, _("Error!"),
                   wxYES_NO | wxICON_ERROR);

            dialog.CenterOnParent();
            if (dialog.ShowModal() == wxID_NO) return;     //  EXIT

            // TODO: kiedy dobrze byoby zrefaktorowa zwracanie bdw i by
            // moe zrezygnowa z wyrzucania wyjtku przez metody.
            // Na razie jest takie obejcie.
            cp_result = error.GiveResult();
            cp_result->SetError(error.What());
        }
        catch (const Error& error)
        {   // To si nie powinno zdarza.
            r_progress_dialog.MakeModal(false);
            r_progress_dialog.Hide();
            p_method->DetachObserver(&r_progress_dialog);

            wxString message;

            message << _("An unexpected error has occured:");
            message << wxT('\n') << error.What();

            wxMessageDialog dialog(this, message, _("Error!"),
                    wxOK | wxICON_ERROR);

            dialog.CenterOnParent();
            dialog.ShowModal();

            return;     // EXIT
        }


        cp_result->SetProblem(auto_ptr<const ProblemBase>(ap_problem));

// Niech wylatuj wyjtki w wersji debugowej, bdzie atwiej analizowa.
#ifdef NODEBUG
        try
        {
#endif
            VisualizationFrame* p_visualization_frame =
                new VisualizationFrame(NULL, CountedPtr<const Result>(cp_result));

            p_visualization_frame->Show();
#ifdef NODEBUG
        }
        catch (const BaseException& error)
        {
            LOG(ERROR, wxT("Newmat exception:\n") << wxString::FromAscii(error.what()));
            wxMessageDialog dialog(this,
                    _("Unexpected exception:\n") +
                        wxString::FromAscii(error.what()),
                    _("Error!"), wxICON_ERROR | wxOK);
            dialog.CenterOnParent();
            dialog.ShowModal();
        }
        catch (const Error& error)
        {
            LOG(ERROR, wxT("Our exception: ") << error.What());
            wxMessageDialog dialog(this,
                    _("Unexpected exception:\n") + error.What(),
                    _("Error!"), wxICON_ERROR | wxOK);
            dialog.CenterOnParent();
            dialog.ShowModal();
        }
        catch (const exception& error)
        {
            LOG(ERROR, wxT("System exception: ") << wxString::FromAscii(error.what()));
            wxMessageDialog dialog(this,
                    _("Unexpected exception:\n") +
                        wxString::FromAscii(error.what()),
                    _("Error!"), wxICON_ERROR | wxOK);
            dialog.CenterOnParent();
            dialog.ShowModal();
        }
#endif
    }

}// OnOptimize

/////////////////////////////// PRIVATE  ////////////////////////////////////

void
MainFrame::ChangeMethodPanel(Method* pMethod)
{
    mpMethodSizer->Detach(mpMethodPanel);

    mpMethodPane->Freeze();

    mpMethodPane->RemoveChild(mpMethodPanel);
    mpMethodPanel->Destroy();

    // Utwrz panel na podstawie metody
    mpMethodPanel = MethodPanelFactory::GetInstance().
            CreatePanel(MethodPanelInfo(pMethod, mpMethodPane, mpProblem));

    // TODO: niektre panele maj walidatory, mam nadziej, e ten styl nie
    // przeszkadza
    mpMethodPanel->SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
    mpMethodPanel->TransferDataToWindow();

    // Dodaj do sizera
    mpMethodSizer->Insert(1, mpMethodPanel, 1, wxALL|wxEXPAND, 2);
    mpMethodSizer->Layout();

    Layout();

    mpMethodPane->Thaw();
}// ChangeMethodPanel
