#include <precomp.h>

#include "MatrixStyle.h"

MatrixStyle::MatrixStyle(unsigned width, unsigned height)
{
    Init(width, height);
}// MatrixStyle

MatrixStyle::MatrixStyle(const GeneralMatrix& matrix)
{
    Init(matrix.Ncols(), matrix.Nrows());
}// MatrixStyle

void
MatrixStyle::SetStyle(const wxTextAttr& style)
{
    SetColumnStyle(1, ColumnCount(), style);
}// SetStyle

void
MatrixStyle::SetStyle(unsigned x, unsigned y, const wxTextAttr& style)
{
    mStyles[y-1][x-1] = style;
}// SetStyle

void
MatrixStyle::SetColumnStyle(unsigned x, const wxTextAttr& style)
{
    for (unsigned y = 1; y <= RowCount(); ++y)      SetStyle(x, y, style);
}// SetColumnStyle

void
MatrixStyle::SetColumnStyle(unsigned first, unsigned last,
                                        const wxTextAttr& style)
{
    assert (first > 0);
    assert (last <= ColumnCount());

    for (unsigned column = first; column <= last; ++column)
    {
        SetColumnStyle(column, style);
    }
}// SetColumnStyle

void
MatrixStyle::SetRowStyle(unsigned y, const wxTextAttr& style)
{
    for (unsigned x = 1; x <= ColumnCount(); ++x)   SetStyle(x, y, style);
}// SetColumnStyle

void
MatrixStyle::AddStyle(const wxTextAttr& style)
{
    AddColumnStyle(1, ColumnCount(), style);
}// AddStyle

void
MatrixStyle::AddStyle(unsigned x, unsigned y, const wxTextAttr& style)
{
    mStyles[y-1][x-1] = wxTextAttr::Merge(mStyles[y-1][x-1], style);
}// AddStyle

void
MatrixStyle::AddColumnStyle(unsigned x, const wxTextAttr& style)
{
    for (unsigned y = 1; y <= RowCount(); ++y)      AddStyle(x, y, style);
}// AddColumnStyle

void
MatrixStyle::AddColumnStyle(unsigned first, unsigned last,
                                        const wxTextAttr& style)
{
    for (unsigned column = first; column <= last; ++column)
    {
        AddColumnStyle(column, style);
    }
}// AddColumnStyle

void
MatrixStyle::AddRowStyle(unsigned y, const wxTextAttr& style)
{
    for (unsigned x = 1; x <= ColumnCount(); ++x)   AddStyle(x, y, style);
}// AddColumnStyle


/* static */ MatrixStyle
MatrixStyle::Merge(const MatrixStyle& base, const MatrixStyle& overlaid)
{
    assert (base.ColumnCount() == overlaid.ColumnCount());
    assert (base.RowCount() == overlaid.RowCount());

    MatrixStyle result(base.ColumnCount(), base.RowCount());

    for (unsigned x = 0; x < base.ColumnCount(); ++x)
    {
        for (unsigned y = 0; y < base.RowCount(); ++y)
        {
            result.mStyles[y][x] = wxTextAttr::Merge(base.mStyles[y][x],
                                                     overlaid.mStyles[y][x]);
        }
    }

    return result;
}// Combine

void
MatrixStyle::Reset(const GeneralMatrix& matrix)
{
    mStyles.clear();
    Init(matrix.ncols(), matrix.nrows());
}// Reset

const wxTextAttr&
MatrixStyle::Style(unsigned x, unsigned y) const
{
    return mStyles[y-1][x-1];
}// Style

unsigned
MatrixStyle::ColumnCount() const
{
    return mStyles.front().size();
}// ColumnCount

unsigned
MatrixStyle::RowCount() const
{
    return mStyles.size();
}// RowCount

//////////////////////////////// PRIVATE ///////////////////////////////////
void
MatrixStyle::Init(unsigned width, unsigned height)
{
    assert (width > 0);
    assert (height > 0);

    for (unsigned i = 0; i < height; ++i)
    {
        StyleRow row;
        for (unsigned j = 0; j < width; ++j)
        {
            row.push_back(wxTextAttr());
        }
        mStyles.push_back(row);
    }
}// Init
