#ifndef _MatrixStyle_h
#define _MatrixStyle_h

/** Macierz styli.
 *
 * Klasa przechowuje style tekstu, ktrymi wywietlane s poszczeglne komrki
 * macierzy przy korzystaniu z klasy MultilineString.
 *
 * @see MultilineString
 */
class MatrixStyle
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    MatrixStyle(unsigned width, unsigned height);
    /** Konstruktor.
     *
     * Inicjuje macierz takich samych rozmiarw jak parametr <code>matrix</code>
     */
    MatrixStyle(const GeneralMatrix& matrix);

    // TODO: konstruktor kopiujcy

// OPERATORS
    // TODO: MatrixStyle& operator=

// OPERATIONS
    /** Ustawia styl dla caej macierzy.
     */
    void    SetStyle        (const wxTextAttr& style);
    /** Ustawia styl dla podanej komrki.
     */
    void    SetStyle        (unsigned x, unsigned y, const wxTextAttr& style);
    /** Ustawia styl dla komrek danego wiersza.
     */
    void    SetRowStyle     (            unsigned y, const wxTextAttr& style);
    /** Ustawia styl dla komrek danej kolumny.
     */
    void    SetColumnStyle  (unsigned x,             const wxTextAttr& style);
    /** Ustawia styl do komrek kolumn od first do last.
     */
    void    SetColumnStyle  (unsigned first, unsigned last,
                                                     const wxTextAttr& style);
    /** Dodaje styl dla caej macierzy.
     */
    void    AddStyle        (const wxTextAttr& style);
    /** Dodaje styl do podanej komrki.
     */
    void    AddStyle        (unsigned x, unsigned y, const wxTextAttr& style);
    /** Dodaje styl do komrek danego wiersza.
     */
    void    AddRowStyle     (            unsigned y, const wxTextAttr& style);
    /** Dodaje styl do komrek danej kolumny.
     */
    void    AddColumnStyle  (unsigned x,             const wxTextAttr& style);
    /** Dodaje styl do komrek kolumn od first do last.
     */
    void    AddColumnStyle  (unsigned first, unsigned last,
                                                     const wxTextAttr& style);

    /** czy dwie macierze styli.
     *
     * Wynikiem jest macierz styli, ktrej poszczeglne komrki posiadaj styl,
     * ktrego atrybuty pochodz z odp. komrek macierzy naoonej, a jeli nie
     * s zdefiniowane, to z macierzy podstawowej.
     *
     * @param base Podstawowa macierz styli.
     * @param overlaid Macierz styli nakadanych na podstawow.
     */
    static MatrixStyle      Merge(const MatrixStyle& base,
                                  const MatrixStyle& overlaid);

    /** Usuwa dotychczasowe style i inicjuje obiekt stylem domylnym
     * dla podanej macierzy.
     */
    void    Reset(const GeneralMatrix& matrix);
// ACCESS

    const wxTextAttr&   Style(unsigned x, unsigned y) const;
    unsigned            ColumnCount() const;
    unsigned            RowCount() const;

private:
    void    Init(unsigned width, unsigned height);

    typedef vector<wxTextAttr>  StyleRow;
    typedef vector<StyleRow>    StyleArray;

    StyleArray mStyles;
};

#endif
