#include <precomp.h>

#include "../Logger.h"
#include "../NonlinearParser.h"
#include "../UnconstrainedProblem.h"


#include "MethodPanelFactory.h"  // class implemented

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

MethodPanelFactory::~MethodPanelFactory()
{
}// ~MethodPanelFactory


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void*
MethodPanelFactory::Register(MethodIdType id, PanelCreator function)
{
    assert(function != NULL);

    mFactories.insert(pair<MethodIdType, PanelCreator>(id, function));

    return NULL;
}// Register

wxPanel*
MethodPanelFactory::CreatePanel(MethodPanelInfo panelInfo) const
{
    //    assert(mFactories.find(pMethod) != mFactories.end());

    map<MethodIdType, PanelCreator>::const_iterator i =
            mFactories.find(panelInfo.pMethod()->Id());

    // Jeli fabryka nie jest zarejestrowana, to twrz pusty panel
    if (i != mFactories.end())  return (i->second)(panelInfo);
    else
    {
        LOG(WARNING, wxT("No method configuration panel registered!")
                << wxT(" Creating empty panel"));
        return new wxPanel(panelInfo.pParent(), -1);
    }
}// CreatePanel

wxSize
MethodPanelFactory::MinSize()
{
    // Crude
    wxWindow* p_dummy_window = new wxFrame(NULL, -1, wxT(""));
//    UnconstrainedProblem *p_dummy_problem = dynamic_cast<UnconstrainedProblem*>
//            (ProblemBase::GetById(UnconstrainedProblem::ClassId()));
    auto_ptr<ProblemBase> ap_dummy_problem
            (ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
    NonlinearParser parser;
    std::auto_ptr<VariableNames>    ap_variables(new VariableNames);
    std::auto_ptr<FunctionBase>     ap_function;

    ap_function = parser.CreateFunction(wxT("dummy"), *ap_variables, true);
    ap_dummy_problem->SetFunction(ap_function, ap_variables);


    map<MethodIdType, PanelCreator>::iterator i;
    wxSize size(0, 0);

    for (i = mFactories.begin(); i != mFactories.end(); ++i)
    {
        MethodPanelInfo panelInfo(Method::GetById(i->first),
                                  p_dummy_window,
                                  ap_dummy_problem.get());
        wxPanel *p_panel = i->second(panelInfo);

        size.IncTo(p_panel->GetMinSize());
        p_dummy_window->RemoveChild(p_panel);
        delete p_panel;
    }

    delete p_dummy_window;

    return size;
}// MinSize

//============================= ACCESS     ===================================

/* static */ MethodPanelFactory&
MethodPanelFactory::GetInstance()
{
    static MethodPanelFactory instance;

    return instance;
}// GetInstance

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

MethodPanelFactory::MethodPanelFactory()
    : mFactories()
{
}// MethodPanelFactory

