/**Szablon panelu dla metod z optymalizacj w kierunku.
 *
 * Uniwersalny panel dla metod posiadajcych 3 parametry:
 * metoda optymalizacji w kierunku, warunek stopu, punkt startowy.
 *
 * Panele dla poszczeglnych metod s rejestrowane
 * w MethodWithLineSearchPanel.cc.
 */

#ifndef METHODWITHLINESEARCHPANEL_H
#define METHODWITHLINESEARCHPANEL_H

#include "DoubleParameterValidator.h"
#include "LineSearchControl.h"
#include "LineSearchValidator.h"
#include "MethodPanelInfo.h"
#include "MethodPanelFactory.h"
#include "PointControl.h"
#include "StopConditionControl.h"
#include "StopConditionValidator.h"

template <class T>
class MethodWithLineSearchPanel
    : public wxPanel
{
public:

    /** Konstruktor.
     */
    MethodWithLineSearchPanel(wxWindow*             parent,
                              int                   id,
                              T*                    pMethod,
                              const ProblemBase&    problem,
                              const wxPoint&        pos=wxDefaultPosition,
                              const wxSize&         size=wxDefaultSize,
                              long                  style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*     CreatePanel(MethodPanelInfo     panelInfo);

private:
protected:

    void do_layout();

    wxBoxSizer*             mpMainSizer;

    LineSearchControl*      mpLineSearch;
    StopConditionControl*   mpStopCondition;
    PointControl*           mpStartingPoint;
};

template <class T>
MethodWithLineSearchPanel<T>::MethodWithLineSearchPanel
                    (wxWindow*             parent,
                    int                   id,
                    T*                    pMethod,
                    const ProblemBase&    problem,
                    const wxPoint&        pos,
                    const wxSize&         size,
                    long                  style)
    : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{
    // Tworzenie kontrolek.
    mpLineSearch = new LineSearchControl(this, -1, problem);
    mpStopCondition = new StopConditionControl(this, -1);
    mpStartingPoint = new PointControl(this, -1, wxDefaultPosition,
            wxDefaultSize, 0, _("Starting point"));

    do_layout();

    // Wypenianie listy warunkw stopu.
    mpStopCondition->SetList(pMethod->AllowedStopConditions());

    // Walidatory.
    mpLineSearch->SetValidator(LineSearchValidator(pMethod->mcpLineSearch,
                &problem));
    mpStopCondition->SetValidator(StopConditionValidator(
                reinterpret_cast<CountedPtr<const StopConditionBase>&>(
                    pMethod->mcpStopCondition)));

    // Wypenianie kontrolki punktu startowego.
    mpStartingPoint->Init(problem.Variables(),
                 /*pMethod->mStartingPoint*/pMethod->mInitialStartingPoint);

}// MethodWithLineSearchPanel


template <class T>
/* static */ wxPanel*
MethodWithLineSearchPanel<T>::CreatePanel(MethodPanelInfo panelInfo)
{
    return new MethodWithLineSearchPanel(
                panelInfo.pParent(),
                -1,
                dynamic_cast<T*>(panelInfo.pMethod()),
                *panelInfo.pProblem());

} // CreatePanel

template <class T>
void
MethodWithLineSearchPanel<T>::do_layout()
{

    mpMainSizer = new wxBoxSizer(wxVERTICAL);
    mpMainSizer->Add(mpLineSearch, 0, wxEXPAND, 0);
    mpMainSizer->Add(mpStopCondition, 0, wxEXPAND, 0);
    mpMainSizer->Add(mpStartingPoint, 1, wxEXPAND, 0);

    SetAutoLayout(true);
    SetSizer(mpMainSizer);
    mpMainSizer->Fit(this);
    mpMainSizer->SetSizeHints(this);

}

#endif // METHODWITHLINESEARCHPANEL_H
