#ifndef _MultilineString_h
#define _MultilineString_h

// LOCAL INCLUDES
#include "MatrixStyle.h"

/** Klasa suca drukowaniu tekstu, ktrego wiersz rozciga si na kilka linii.
 *
 * Do obiektu tej klasy mog by wstawiane napisy (jednowierszowe, bez znaku
 * koca linii) oraz macierze i wektory. Tekst jest odpowiednio formatowany, aby
 * wszystkie elementy byy wyrodkowane w pionie (w granicach moliwoci).
 *
 * Tekst oraz macierze mog mie przypisane atrybuty tekstu (kolor, czcionka
 * itd.).
 */
class MultilineString
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param style Domylny styl tekstu.
     */
    MultilineString(const wxTextAttr& style = wxTextAttr());

// OPERATORS

    /** Operator dodawania tekstu.
     */
    MultilineString&    operator<<(const wxString& text);

    /** Operator dodawania macierzy.
     */
    MultilineString&    operator<<(const BaseMatrix& X);

    /** Operator dodawania macierzy.
     */
    MultilineString&    operator<<(const GeneralMatrix& matrix);

// OPERATIONS

    /** Dodawanie tekstu.
     */
    void                Append(const wxString& text);

    /** Dodawanie tekstu wraz z atrybutami formatowania.
     *
     * @param text Dodawany cig znakw.
     * @param style Styl.
     */
    void                Append(const wxString& text, const wxTextAttr& style);

    /** Dodawanie macierzy.
     *
     * @param matrix Macierz/wektor.
     * @param style Macierz styli dla poszczeglnych komrek.
     * @param upperIndex Tekst umieszczany w grnym prawym rogu macierzy, suy
     *        do oznaczania transpozycji bd operacji odwracania macierzy ("T",
     *        "-1").
     */
    void                Append(const GeneralMatrix&    matrix,
                               const MatrixStyle&      style,
                               const wxString&         upperIndex = wxT(""));

    /** Drukuj zawarto do okienka tekstowego.
     */
    void                Print(wxTextCtrl*  pTextCtrl) const;

    /** Utwrz wxString ze wszystkich wierszy.
     */
    wxString            ToString() const;

    /** Usuwa zawarto.
     */
    void                Clear();

    /** Drukuje zawarto do kontrolki, a nastpnie usuwa j
     * z obiektu.
     */
    void                Flush(wxTextCtrl*   pTargetCtrl);

    /** Dodaje styl do domylnego.
     *
     * Wpywa to na cay tekst nie posiadajcy ustalonego stylu, rwnie dodany
     * przed wywoaniem tej metody.
     */
    void                AddToDefaultStyle(const wxTextAttr& style);
// ACCESS

    /** Zwraca domylny styl.
     */
    const wxTextAttr&   DefaultStyle() const;

// DEBUG
    void                Dump() const;

private:

    /// Styl aplikowany do zakresu cigu znakw.
    class TextStyleRange
    {
    public:
        TextStyleRange(unsigned first, unsigned length, const wxTextAttr& style)
            : mFirst(first), mLength(length), mStyle(style) { }

        unsigned    mFirst;
        unsigned    mLength;
        wxTextAttr  mStyle;
    };

    typedef vector<TextStyleRange>  TextStyleRanges;
    typedef vector<TextStyleRanges> TextStyleRows;

    /** Przygotowuje do wprowadzenia elementu tekstu o podanej liczbie wierszy.
     *
     * Dodawane s ew. nowe wiersze, dokonywane jest wypenianie spacjami.
     */
    void        Allocate(unsigned rowCount);

    /// Zwraca indeks pierwszego wiersza, dla elementu o podanej wysokoci.
    unsigned    FirstRow(unsigned rowCount) const;

    wxArrayString   mStrings;       ///< Tablica wierszy.
    unsigned        mLength;        ///< Dugo tekstu.

    TextStyleRows   mTextStyles;    ///< Tablica styli.

    wxTextAttr      mDefaultStyle;
};

#endif
