/** Warstwa wykresu odpowiedzialna za rysowanie wykresu funkcji wielu zmiennych.
 *
 * @see PlotControl
 */
#ifndef _MultivariateFunctionLayer_h
#define _MultivariateFunctionLayer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../FunctionBase.h"
#include "IntervalCountLimiter.h"
#include "PlotLayer.h"

// FORWARD REFERENCES
//

class MultivariateFunctionLayer
    : public PlotLayer
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    MultivariateFunctionLayer(const PlotControl&    targetPlot,
                              const FunctionBase&   function,
                              unsigned              variable1Index = 0,
                              unsigned              variable2Index = 1);

    /** Konstruktor kopiujcy.
     */
    MultivariateFunctionLayer(const MultivariateFunctionLayer& from);

    /** Destruktor.
     */
    ~MultivariateFunctionLayer(void);


// OPERATORS
// OPERATIONS

    virtual void    Draw(wxDC& dc) const;

// ACCESS
// INQUIRY

protected:
private:
    /// IntervalCountLimiter zwracajcy zawsze tyle samo.
    class FixedCountLimiter
        : public IntervalCountLimiter
    {
    public:
        FixedCountLimiter(unsigned maximumCount);

        virtual unsigned operator()(double intervalLength) const;
    private:
        unsigned mMaximumCount;
    };


    void        DrawPlotLine(wxDC&  dc,
                             double x1,
                             double y1,
                             double x2,
                             double y2) const;

    double*     ComputeContourLevels(double     plotMin,
                                     double     plotMax,
                                     unsigned&  levelCount) const;

    void        conrec(double** d,
                       int      ilb,
                       int      iub,
                       int      jlb,
                       int      jub,
                       double*  x,
                       double*  y,
                       int      nc,
                       double*  z,
                       wxDC&    dc) const;



    const FunctionBase& mFunction;

    unsigned            mVariableXIndex;
    unsigned            mVariableYIndex;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _MultivariateFunctionLayer_h
