#ifndef _MultivariatePlot_h
#define _MultivariatePlot_h

/** Kontrolka wywieltajca wykres funkcji dwch i wicej zmiennych.
 */

#include "../UnivariateFunction.h"
#include "IntervalCountLimiter.h"
#include "PlotControl.h"

class MultivariatePlot
    : public PlotControl
{
public:
    enum ToolType
    {
        PAN,
        ZOOM_IN,
        ZOOM_OUT
    };

    MultivariatePlot(
        wxWindow*                   pParent,
        wxWindowID                  id,
        const FunctionBase&         function,
        const VariableNames&        variables,
        PlotStatus*                 pPlotStatus = NULL,
        const wxPoint&              position = wxDefaultPosition,
        const wxSize&               size = wxDefaultSize);

    ~MultivariatePlot();

    bool            KeepAspectRatio() const;
    bool&           rKeepAspectRatio();

    const VariableNames&    Variables() const;
    int             VariableXIndex() const;
    int             VariableYIndex() const;

    void            SetVariables(int variableXIndex, int variableYIndex);

    virtual void    OnAspectChange(bool redraw);
protected:

    virtual void    DrawDisplaced(long x, long y);

    virtual void    Pan(wxCoord x, wxCoord y);

    virtual void    ZoomIn(wxCoord x, wxCoord y);
    virtual void    ZoomRegionIn(wxCoord x,
                                 wxCoord y,
                                 wxCoord width,
                                 wxCoord height);
    virtual void    ZoomOut(wxCoord x, wxCoord y);

    virtual void    UpdateStatusOnMove(long x, long y);

    virtual void    UpdateScale();

private:


    void            EnforceAspectRatio();

    void            FitToWidth(double& rStart, double& rEnd, double newWidth)
                        const;


    /// Indeks (zaczynajc od 1) zmiennej na osi poziomej.
    int                     mVariableXIndex;
    /// Indeks (zaczynajc od 1) zmiennej na osi pionowej.
    int                     mVariableYIndex;
    const VariableNames&    mVariables; ///< Nazwy zmiennych.

    bool                    mKeepAspectRatio;   ///< Czy zachowa proporcje
};

#endif
