#include <precomp.h>

#include "../methods/NewtonIteration.h"

#include "NewtonLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NewtonLayer::NewtonLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// NewtonLayer

NewtonLayer::NewtonLayer(const NewtonLayer& from)
    : XyUnconstrainedLayer(from)
{
}// NewtonLayer

NewtonLayer::~NewtonLayer()
{
}// ~NewtonLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
NewtonLayer::Create(const PlotControl&          plotControl,
                    const Result&               result,
                          CountedPtr<PlotLayer> cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new NewtonLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================


/* virtual */ Region
NewtonLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const NewtonIteration& iteration =
        dynamic_cast<const NewtonIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));
    bounding_box.Extend(iteration.EndPoint()(mVariableXIndex),
                        iteration.EndPoint()(mVariableYIndex));

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
NewtonLayer::DrawIteration(wxDC&       dc,
                                      unsigned    iterationNumber,
                                      bool        highlight) const
{
    const NewtonIteration& iteration =
        dynamic_cast<const NewtonIteration&>
            (mResult.Iteration(iterationNumber));

    PointList   points_to_draw;

    ColumnVector    base_point = iteration.StartingPoint();

    if (highlight || iterationNumber == mResult.FirstIterationNumber())
    {
        if (iterationNumber == mResult.FirstIterationNumber())
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(base_point),
                                        AttributeDatabase::POINT_START,
                                        SQUARE));
        }
        else
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(base_point),
                                        AttributeDatabase::POINT_START,
                                        CIRCLE));
        }
    }


    AttributeDatabase::AttributeType attribute =
        AttributeDatabase::INACTIVE;
    if (highlight)
        if (iteration.StepValid())
            attribute = AttributeDatabase::STEP_IN_DIRECTION;
        else
            attribute = AttributeDatabase::POINT_OUT_OF_BOUNDS;


    DrawLine(dc, base_point, iteration.EndPoint(), attribute);
    points_to_draw.push_back(ColourPointType(PlotToDcPoint(
                                                iteration.EndPoint()),
                                                attribute));

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* NewtonLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            NewtonIteration::ClassId(),
            NewtonLayer::Create);
