/** Kontrola wyboru obiektu.
 *
 * Lista obiektw jest posortowana wg etykiet.
 */

#ifndef _ObjectChoice_h
#define _ObjectChoice_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CountedPtr.h"

// FORWARD REFERENCES
//

template<class T>
class ObjectChoice
    : public wxChoice
{
public:
// LIFECYCLE

    /** Konstruktor
     */
    ObjectChoice()
        : wxChoice()
    {
    }// ObjectChoice

    /** Konstruktor.
     */
    ObjectChoice(wxWindow             *pParent,
                 wxWindowID           id,
                 const wxPoint&       pos = wxDefaultPosition,
                 const wxSize&        size = wxDefaultSize,
                 long                 style = 0,
                 const wxString&      name = wxT(""))
        : wxChoice(pParent, id, pos, size, 0, NULL, style, wxDefaultValidator,
                name)
    {
    }// ObjectChoice

    /** Destruktor.
     */
    ~ObjectChoice()
    {
    }// ~ObjectChoice

// OPERATORS

// OPERATIONS

    /** Tworzenie kontrolki w sytuacji, gdy wywoano konstruktor
     * bezparametrowy.
     */
    bool            Create(wxWindow          *pParent,
                           wxWindowID        id,
                           const wxPoint&    pos,
                           const wxSize&     size,
                           long              style,
                           const wxString&   name)
    {
        return wxChoice::Create(pParent, id, pos, size, 0, NULL,
                wxDefaultValidator, name);
    }

    /** Dodaje obiekt do kontrolki.
     */
    void            Insert(const wxString& label, CountedPtr<T> cpObject, bool sort = false);

    /** Zwraca wskanik na wybrany obiekt.
     */
    CountedPtr<T>   GetSelected() const;

// INQUIRY

protected:
private:
    vector<CountedPtr<T> > mObjects;
};

// IMPLEMENTATION


template<class T>
void
ObjectChoice<T>::Insert(const wxString& label, CountedPtr<T> cpObject, bool sort)
{
    int pos;

    if (sort)
    {// Znajd odpowiednie miejsce do wstawienia tej etykiety.
        unsigned i;
        for (i = 0; i < GetCount(); ++i)
        {
            if (GetString(i) > label) break;    // znaleziono
        }
        pos = wxChoice::Insert(label, i);
    }
    else
    {
        pos = Append(label);
    }

    assert(pos > -1);
    SetSelection(0);

    // Dodaj do wektora obiektw.
    try
    {
        mObjects.insert(mObjects.begin() + pos, cpObject);
    }
    catch(...)
    {
        Delete(pos);
        throw;
    }
}// Insert


template<class T>
CountedPtr<T>
ObjectChoice<T>::GetSelected() const
{
    int pos = GetSelection();

    // W Insert zawsze wybieramy pierwszy, wic jaki musi by zaznaczony.
    assert(pos != wxNOT_FOUND);

    return mObjects[pos];
}// GetSelected

#endif
