#include <precomp.h>

#include "../Logger.h"

#include "ParseErrorText.h"  // class implemented

BEGIN_EVENT_TABLE(ParseErrorText, wxWindow)
    EVT_PAINT(ParseErrorText::OnPaint)
END_EVENT_TABLE()

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ParseErrorText::ParseErrorText(wxWindow*            pParent,
                               wxWindowID           id,
                               const ParseError&    error)
    : wxWindow(pParent, id),
      mBitmap()
{
    // Ustal rozmiar tekstu, potrzebny w tym celu jest kontekst.
    wxBitmap dummy_bitmap(1, 1);
    wxMemoryDC dc;
    wxCoord text_width;
    wxCoord text_height;

    dc.SetFont(GetFont());

    dc.SelectObject(dummy_bitmap);
    dc.GetTextExtent(error.Input(), &text_width, &text_height);

    // Ustaw rozmiar okna i twrz waciw bitmap
    wxSize window_size(text_width  + ARROW_WIDTH, text_height + ARROW_HEIGHT);

    SetMinSize(window_size);
    mBitmap = wxBitmap(window_size.GetWidth(), window_size.GetHeight());

    dc.SelectObject(mBitmap);

    dc.SetBackground(GetBackgroundColour());
    dc.Clear();
    dc.DrawText(error.Input(), 0, 0);

    // Ustal pozycj strzaki
    wxArrayInt widths;
    dc.GetPartialTextExtents(error.Input(), widths);
    widths.Add(widths.Last() + ARROW_WIDTH);    // Pseudo-znak na kocu

    assert (error.Position() <= widths.GetCount());

    LOG(DEBUG, wxT("error pos:  ") << error.Position());
    LOG(DEBUG, wxT("width:      ") << widths[error.Position()]);
    LOG(DEBUG, wxT("text width: ") << text_width);

    wxCoord arrow_middle;
    if (error.Position() == 0)  arrow_middle = widths[0] / 2;
    else
    {
        arrow_middle =
            (widths[error.Position()] + widths[error.Position() - 1]) / 2;
    }

    DrawArrow(dc, text_height, arrow_middle, ARROW_SPREAD, ARROW_HEIGHT);

    dc.SelectObject(wxNullBitmap);
}// ParseErrorText

ParseErrorText::~ParseErrorText()
{
}// ~ParseErrorText


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
ParseErrorText::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);

    dc.DrawBitmap(mBitmap, 0, 0, false);
}// OnPaint

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
void
ParseErrorText::DrawArrow(wxDC&     dc,
                          wxCoord   y,
                          wxCoord   middle,
                          wxCoord   spread,
                          wxCoord   height) const
{
    dc.DrawLine(middle, y, middle, y + height);
    dc.DrawLine(middle - spread, y + spread, middle, y);
    dc.DrawLine(middle + spread, y + spread, middle, y);
}// DrawArrow

const /* static */ wxCoord ParseErrorText::ARROW_HEIGHT = 8;
const /* static */ wxCoord ParseErrorText::ARROW_SPREAD = 3;
const /* static */ wxCoord ParseErrorText::ARROW_WIDTH  =
        ParseErrorText::ARROW_SPREAD * 2 + 1;
