/**  A one line description of the class.
 *
 * A longer description.
 *
 * @see something
 */

#ifndef _PlotAxis_h
#define _PlotAxis_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "ui_util.h"

// FORWARD REFERENCES
//

class PlotAxis
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param vertical Czy o jest pionowa.
     * @param limitToIntegers Ogranicz do liczb cakowitych (uamki nie bd
     * pojawiay si na osi.
     */
    PlotAxis(bool vertical, bool limitToIntegers = false);


    /** Destruktor.
     */
    ~PlotAxis(void);


// OPERATORS
// OPERATIONS

    /** Zwraca bitmap z narysowan osi.
     */
    const wxBitmap&     Draw() const;

    /** Zmienia rozmiar obszaru rysowania osi w pikslach.
     *
     * Szeroko albo wysoko, w zalenoci od orientacji osi.
     */
    void                Resize(wxCoord newLength);

    /** Zadaje nowy zakres wartoci na osi.
     */
    void                Zoom(double start, double end);

    /** Dodaje przesunicie zakresu wartoci osi w pikslach.
     *
     * Wykorzystywane do narysowania osi w trakcie przesuwania obszaru wykresu
     * przez uytkownika.
     */
    void                Displace(wxCoord x);

// ACCESS

    /** Zwraca szeroko obszaru osi w pikslach.
     *
     * W przypadku osi poziomej jest to wysoko, w przypadku pionowej -
     * szeroko.
     */
    // TODO: zastanowi si nad nazw.
    wxCoord             Breadth() const;

    /** Zwraca dugo obszaru osi w pikslach.
     *
     * W przypadku osi pionowej, jest to wysoko.
     */
    wxCoord             Length() const;

    /** Ustawia etykiet.
     */
    void                SetLabel(const wxString& label);

    /** Referencja na flag "tylko cakowite".
     */
    bool&               IntegersOnly();

// INQUIRY

protected:
private:

    /// IntervalCountLimiter biorcy pod uwag szeroko liczb na podziace
    class TextSizeIntervalCountLimiter
        : public IntervalCountLimiter
    {
    public:
        TextSizeIntervalCountLimiter(double     plotStart,
                                     double     plotEnd,
                                     unsigned   pixelLength,
                                     bool       vertical,
                                     bool       limitToIntegers = false);

        virtual unsigned    operator()(double intervalLength) const;
    private:
        double      mPlotStart;
        double      mPlotEnd;
        unsigned    mPixelLength;
        bool        mVertical;
        bool        mLimitToIntegers;
    };

    /** Przerysowuje o na bitmapie.
     */
    void                Redraw();

    /** Rysuje etykiet pod/obok osi.
     */
    void                DrawLabel(wxDC& dc);

    /** Oblicza odlego midzy kolejnymi liczbami na podziace.
     */
    void                UpdateStubInterval();

    bool        mVertical;

    wxString    mLabel;     ///< Etykieta osi.

    double  mPlotStart;     ///< Pocztek osi w jednostkach wykresu.
    double  mPlotEnd;       ///< Koniec osi w jednostkach wykresu.
    wxCoord mPixelLength;   ///< Dugo osi w pikslach.
    /// Przesunicie osi wzgldem normalnego pooenia (w pikslach).
    wxCoord mDisplacement;

    double  mStubInterval;  ///< Odlego midzy oznaczonymi jednostkami.

    wxBitmap    mAxisBitmap;    ///< Bitmapa, na ktrej rysowana jest o.

    wxFont  mAxisFont;          ///< Czcionka dla cyfr na osiach.
    wxFont  mLabelFont;         ///< Czcionka dla etykiet osi.

    bool    mLimitToIntegers;   ///< Tylko liczby cakowite na osi.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _PlotAxis_h
