#include <precomp.h>

#include <wx/dcbuffer.h>
#include <wx/xrc/xmlres.h>

#include "../Logger.h"
#include "AttributeDatabase.h"
#include "PlotControl.h"


// T klas naleaoby tak na dobr sprawd zrefaktorowa. Albo napisa od nowa.
// I poczy z jej pochodnymi.

LOGGER(INFO)

DEFINE_EVENT_TYPE(PLOT_FITVIEW_REQUEST_EVENT);

BEGIN_EVENT_TABLE(PlotControl, wxWindow)
    EVT_ENTER_WINDOW(PlotControl::OnMouseEnter)
    EVT_LEAVE_WINDOW(PlotControl::OnMouseLeave)
    EVT_LEFT_DOWN(PlotControl::OnMouseLeftDown)
    EVT_LEFT_UP(PlotControl::OnMouseLeftUp)
    EVT_RIGHT_UP(PlotControl::OnMouseRightUp)
    EVT_MOTION(PlotControl::OnMouseMove)
    EVT_MOUSEWHEEL(PlotControl::OnMouseWheel)
    EVT_PAINT(PlotControl::OnPaint)
    EVT_SIZE(PlotControl::OnResize)
    EVT_COMMAND(wxID_ANY, PLOT_FITVIEW_REQUEST_EVENT,
            PlotControl::OnFitViewRequest)
END_EVENT_TABLE()

PlotControl::PlotControl(
        wxWindow*           pParent,
        wxWindowID          id,
        const FunctionBase& function,
        PlotStatus*         pPlotStatus,
        const wxPoint&      position,
        const wxSize&       size)
// TODO: poprawi, uzupeni, usun warningi
    : wxWindow(pParent, id, position, size, wxFULL_REPAINT_ON_RESIZE | wxSUNKEN_BORDER),
      mFunction(function),
      mPlotDc(),
      mPlotBitmap(1, 1),
      mDirty(true),
      mpPlotStatus(pPlotStatus),
      mXAxis(false),
      mYAxis(true),
      mTool(PAN),
      mPlotX1(-10),     // bdzie zmienione
      mPlotX2(10),      // w konstruktorach
      mPlotY1(-10),     // klas
      mPlotY2(10),      // pochodnych
      mScaleX(0),
      mScaleY(0),
      mDragStartX(0),
      mDragStartY(0),
      mDragWidth(0),
      mDragHeight(0),
      mWheelRotation(0),
      mMinZoomX(MIN_ZOOMED_WIDTH),
      mMaxZoomX(MAX_ZOOMED_WIDTH),
      mMinZoomY(MIN_ZOOMED_WIDTH),
      mMaxZoomY(MAX_ZOOMED_WIDTH),
      mCursorPan(wxXmlResource::Get()->LoadImage(wxT("CURSOR_PAN"))),
      mCursorZoomIn(wxXmlResource::Get()->LoadImage(wxT("CURSOR_ZOOMIN"))),
      mCursorZoomOut(wxXmlResource::Get()->LoadImage(wxT("CURSOR_ZOOMOUT")))
{
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);  // Sami bdziemy czyci.

    // Potrzebne, aby mona byo ustawi kolor ta itp.
    mPlotDc.SelectObject(mPlotBitmap);
    InitPlotDc();

    // Na wszelki wypadek, aby byy rozsdne wartoci na samym pocztku.
    mXAxis.Zoom(mPlotX1, mPlotX2);
    mYAxis.Zoom(mPlotY1, mPlotY2);

    wxSizeEvent size_event(GetSize());
    AddPendingEvent(size_event);
}// PlotControl

PlotControl::~PlotControl()
{
    // Uwolnij bitmap
    mPlotDc.SelectObject(wxNullBitmap);
}// ~PlotControl


void
PlotControl::SetTool(PlotControl::ToolType tool)
{
    mTool = tool;
}// SetTool


void
PlotControl::OnEraseEvent(wxEraseEvent& event)
{
    // Puste
}// OnEraseEvent

void
PlotControl::OnPaint(wxPaintEvent& event)
{
    wxBufferedPaintDC dc(this);

    // Kiedy mylaem o double bufferingu, ale wyglda dobrze i bez niego.

    if (mDirty)
    {
        Redraw();

        mXAxis.Resize(mPlotBitmap.GetWidth());
        mXAxis.Zoom(mPlotX1, mPlotX2);
        mYAxis.Resize(mPlotBitmap.GetHeight());
        mYAxis.Zoom(mPlotY1, mPlotY2);

        // Te wartoci obecnie nie ulegaj nigdy zmianie.
        mPlotAreaX  = MARGIN_SIZE + mYAxis.Breadth();
        mPlotAreaY  = MARGIN_SIZE;

        mDirty = false;
    }

    dc.SetBackground(AttributeDatabase::GetInstance().
            Brush(AttributeDatabase::PLOT_BACKGROUND));
    dc.Clear();

    LOG(DEBUG, wxT("OnPaint"));

    dc.Blit(MARGIN_SIZE + mYAxis.Breadth(), MARGIN_SIZE, mPlotBitmap.GetWidth(),
            mPlotBitmap.GetHeight(), &mPlotDc, 0, 0, wxCOPY);

    LOG(DEBUG, wxT("RedrawAxes"));
    RedrawAxes(dc);
}// OnPaint

bool
PlotControl::AdjustView(const Region& region)
{
    Region current_viewport(mPlotX1, mPlotY1, mPlotX2, mPlotY2);

    if (region.IsValid() && (!region.IsInside(current_viewport)))
    {
        LOG(DEBUG, wxT("current viewport: ")  << current_viewport.X1()
                                 << wxT(", ") << current_viewport.X2()
                                 << wxT(", ") << current_viewport.Y1()
                                 << wxT(", ") << current_viewport.Y2());
        LOG(DEBUG, wxT("region: ")            << region.X1()
                                 << wxT(", ") << region.X2()
                                 << wxT(", ") << region.Y1()
                                 << wxT(", ") << region.Y2());
        LOG(DEBUG, wxT("plot: ")              << mPlotX1
                                 << wxT(", ") << mPlotX2
                                 << wxT(", ") << mPlotY1
                                 << wxT(", ") << mPlotY2);


        if (region.IsSmaller(current_viewport))
        {   // Odpowiednio przesu widok
            double displacement_x = 0.0;
            double displacement_y = 0.0;

            // TODO: przenie te marginesy w jedno miejsce
            const unsigned MARGIN = 20;

            const double horizontal_margin = MARGIN * mScaleX;
            const double vertical_margin = MARGIN * mScaleY;

            if (region.X1() < mPlotX1)
            {
                displacement_x = region.X1() - mPlotX1 - horizontal_margin;
            }
            else if (region.X2() > mPlotX2)
            {
                displacement_x = region.X2() - mPlotX2 + horizontal_margin;
            }

            if (region.Y1() < mPlotY1)
            {
                displacement_y = region.Y1() - mPlotY1 - vertical_margin;
            }
            else if (region.Y2() > mPlotY2)
            {
                displacement_y = region.Y2() - mPlotY2 + vertical_margin;
            }

            LOG(DEBUG, wxT("dx: ") << displacement_x << wxT("dy: ") << displacement_y);

            mPlotX1 += displacement_x;
            mPlotX2 += displacement_x;
            mPlotY1 += displacement_y;
            mPlotY2 += displacement_y;
        }
        else
        {   // Ustaw cay widok na region + marginesy
            SetView(region);
        }
        return true;
    }
    else    return false;
}// AdjustView

void
PlotControl::SetView(const Region& region)
{
    // Najpierw wstpnie ustal rozmiar i skal...
    mPlotX1 = region.X1();
    mPlotX2 = region.X2();
    mPlotY1 = region.Y1();
    mPlotY2 = region.Y2();

    LOG(DEBUG, wxT("Requested size: ") << mPlotX1 << wxT(", ")
                                       << mPlotX2 << wxT("; ")
                                       << mPlotY1 << wxT(", ")
                                       << mPlotY2);

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);
    LimitZoom(mPlotY1, mPlotY2, mMinZoomY, mMaxZoomY);

    LOG(DEBUG, wxT("After limiting: ") << mPlotX1 << wxT(", ")
                                       << mPlotX2 << wxT("; ")
                                       << mPlotY1 << wxT(", ")
                                       << mPlotY2);
    this->OnAspectChange();

    LOG(DEBUG, wxT("After aspect: ")   << mPlotX1 << wxT(", ")
                                       << mPlotX2 << wxT("; ")
                                       << mPlotY1 << wxT(", ")
                                       << mPlotY2);

    // ...potem wylicz marginesy...
    const unsigned MARGIN = 20;

    const double horizontal_margin = MARGIN * mScaleX;
    const double vertical_margin = MARGIN * mScaleY;

    // ... i zmie zakres ponownie.
    mPlotX1 = region.X1() - horizontal_margin;
    mPlotX2 = region.X2() + horizontal_margin;
    mPlotY1 = region.Y1() - vertical_margin;
    mPlotY2 = region.Y2() + vertical_margin;

    LOG(DEBUG, wxT("With margins: ")   << mPlotX1 << wxT(", ")
                                       << mPlotX2 << wxT("; ")
                                       << mPlotY1 << wxT(", ")
                                       << mPlotY2);
    this->OnAspectChange();

    LOG(DEBUG, wxT("Final: ")          << mPlotX1 << wxT(", ")
                                       << mPlotX2 << wxT("; ")
                                       << mPlotY1 << wxT(", ")
                                       << mPlotY2);
    RefreshPlot();
}// SetView

bool
PlotControl::FitView()
{
    Region  region;

    vector<CountedPtr<PlotLayer> >::const_iterator i;
    for (i = mLayers.begin(); i != mLayers.end(); ++i)
    {
        region += ((*i)->BoundingBox());
    }

    LOG(DEBUG, wxT("region is valid: ") << region.IsValid());
    if (region.IsValid())
    {
        SetView(region);

        return true;
    }

    return false;
}// FitView

void
PlotControl::ExportBitmap()
{
    static wxString save_directory = wxT("");

    LOG(DEBUG, wxT("ExportBitmap"));
    wxFileDialog save_dialog(GetParent(), _("Export plot"), save_directory,
            wxT(""), _("PNG Image (*.png)|*.png"),
            wxSAVE | wxOVERWRITE_PROMPT);

    if (wxID_OK == save_dialog.ShowModal())
    {
        save_directory = save_dialog.GetDirectory();

        wxSize size = GetClientSize();
        wxBitmap mWindowBitmap(size.GetWidth(), size.GetHeight());
	wxMemoryDC dc;
	
        dc.SelectObject(mWindowBitmap);
	
        dc.SetBackground(AttributeDatabase::GetInstance().
	        Brush(AttributeDatabase::PLOT_BACKGROUND));
	dc.Clear();
	dc.Blit(MARGIN_SIZE + mYAxis.Breadth(), MARGIN_SIZE,
                mPlotBitmap.GetWidth(), mPlotBitmap.GetHeight(), &mPlotDc,
		0, 0, wxCOPY);
	RedrawAxes(dc);
	dc.SelectObject(wxNullBitmap);
	
        mWindowBitmap.SaveFile(save_dialog.GetPath(), wxBITMAP_TYPE_PNG);
    }
}// ExportBitmap

void
PlotControl::OnResize(wxSizeEvent& event)
{
    wxSize plot_size = ComputePlotSize();

    LOG(DEBUG, wxT("plot size: ") << plot_size.GetWidth() << wxT("x")
            << plot_size.GetHeight());

    // Wymie bitmap na odp. rozmiarw
    mPlotDc.SelectObject(wxNullBitmap);
    mPlotBitmap = wxBitmap(plot_size.GetWidth(), plot_size.GetHeight());
    LOG(DEBUG, wxT("Backing bitmap depth: ") << mPlotBitmap.GetDepth());
    mPlotDc.SelectObject(mPlotBitmap);

    UpdateScale();

    this->OnAspectChange();

    UpdateScale();

    // Przerysuj wykres.
    RefreshPlot();
}// OnResize

void
PlotControl::OnFitViewRequest(wxCommandEvent& event)
{
    FitView();
}// OnFitViewRequest

void
PlotControl::OnMouseEnter(wxMouseEvent& event)
{
    wxCursor cursor;
    mOldCursor = GetCursor();
    switch (mTool)
    {
        case PAN:
            SetCursor(mCursorPan);
            break;
        case ZOOM_IN:
            SetCursor(mCursorZoomIn);
            break;
        case ZOOM_OUT:
            SetCursor(mCursorZoomOut);
            break;
    }
}// OnMouseEnter

void
PlotControl::OnMouseLeave(wxMouseEvent& event)
{
    // TODO: nie za bardzo dziaa na Macu
    SetCursor(mOldCursor);

    if (mpPlotStatus != NULL)   mpPlotStatus->ClearPosition();
}// OnMouseLeave

void
PlotControl::OnMouseLeftDown(wxMouseEvent& event)
{
#ifdef __WXMSW__
    SetFocus(); // Ustaw focus, eby dziaao kko
#endif

    switch (mTool)
    {
        case PAN:
            CaptureMouse();
            mDragStartX = event.GetX();
            mDragStartY = event.GetY();
            break;
        case ZOOM_IN:
            CaptureMouse();
            StartDragRectangle(event.GetX(), event.GetY());
            break;
        case ZOOM_OUT:
            this->ZoomOut(WindowToPlotAreaX(event.GetX()),
                          WindowToPlotAreaY(event.GetY()));

            UpdateScale();

            RefreshPlot();
            break;
        default:
            LOG(ERROR, wxT("Invalid tool type"));
            abort();
    }
}// OnMouseLeftDown


void
PlotControl::OnMouseLeftUp(wxMouseEvent& event)
{
    switch (mTool)
    {
        case PAN:
            this->Pan(event.GetX(), event.GetY());

            UpdateScale();

            RefreshPlot();
            ReleaseMouse();
            break;
        case ZOOM_IN:
            {

                if ((abs(mDragWidth) > 1) || (abs(mDragHeight) > 1))
                {
                    this->ZoomRegionIn(WindowToPlotAreaX(mDragStartX),
                                       WindowToPlotAreaY(mDragStartY),
                                       mDragWidth, mDragHeight);
                }
                else
                {
                    this->ZoomIn(WindowToPlotAreaX(mDragStartX),
                                 WindowToPlotAreaY(mDragStartY));
                }

                UpdateScale();

                StopDragRectangle();

                RefreshPlot();
            }
            ReleaseMouse();
            break;
        case ZOOM_OUT:
            // Nothing
            break;
    }
}// OnMouseLeftUp

//void
//PlotControl::LimitMaximalZoom(double& rX1, double& rX2) const
//{
//    if (rX2 - rX1 < MIN_ZOOMED_WIDTH)
//    {
//        double middle = (rX2 + rX1) / 2;
//        rX1 = middle - MIN_ZOOMED_WIDTH/2;
//        rX2 = middle + MIN_ZOOMED_WIDTH/2;
//    }
//}// LimitMaximalZoom

void
PlotControl::LimitZoom(double& rA, double& rB, double min, double max) const
{
    LOG(DEBUG, wxT("A: ") << rA << wxT(" B: ") << rB);
    if (rB - rA < min)
    {
        double middle = (rB + rA) / 2;
        rA = middle - min/2;
        rB = middle + min/2;
    }
    else if (rB - rA > max)
    {
        double middle = (rB + rA) / 2;
        rA = middle - max/2;
        rB = middle + max/2;
    }
}// LimitZoom

void
PlotControl::OnMouseRightUp(wxMouseEvent& event)
{
    if (mTool == ZOOM_IN)
    {
        this->ZoomOut(WindowToPlotAreaX(event.GetX()),
                      WindowToPlotAreaY(event.GetY()));

        UpdateScale();

        RefreshPlot();
    }
}// OnMouseRightUp

void
PlotControl::OnMouseMove(wxMouseEvent& event)
{
    long x = event.GetX();
    long y = event.GetY();

    switch(mTool)
    {
        case PAN:
            if (event.LeftIsDown())
            {
                this->DrawDisplaced(x - mDragStartX, y - mDragStartY);
            }
            break;
        case ZOOM_IN:
            if (event.LeftIsDown())
            {
                UpdateDragRectangle(x, y);
            }

            break;
        case ZOOM_OUT:
            // Nothing
            break;
    }

    this->UpdateStatusOnMove(x, y);
}// OnMouseMove

void
PlotControl::OnMouseWheel(wxMouseEvent& event)
{
    mWheelRotation += event.GetWheelRotation();
    LOG(DEBUG, wxT("mWheelRotation: ") << mWheelRotation);

    int steps = mWheelRotation / event.GetWheelDelta();
    mWheelRotation -= event.GetWheelDelta() * steps;
    LOG(DEBUG, wxT("No. steps: ") << steps);
    LOG(DEBUG, wxT("New mWheelRotation: ") << mWheelRotation);


    // Oblicz rodek wykresu (ugh)
    wxSize plot_size = ComputePlotSize();
    wxCoord middle_x = (plot_size.GetWidth() / 2);// + MARGIN_SIZE +
//        mYAxis.Breadth();
    wxCoord middle_y = (plot_size.GetHeight() / 2);// + MARGIN_SIZE;

    if (steps < 0)
    {
        for (int i = 0; i > steps; --i)
        {
            ZoomIn(middle_x, middle_y);
            UpdateScale();
        }
        RefreshPlot();
    }
    else if (steps > 0)
    {
        for (int i = 0; i < steps; ++i)
        {
            ZoomOut(middle_x, middle_y);
            UpdateScale();
        }
        RefreshPlot();
    }

}// OnMouseWheel


void
PlotControl::RefreshPlot()
{
    mDirty = true;
    Refresh();
}// RefreshPlot

//////////////////////////////// ACCESS /////////////////////////////////////

void
PlotControl::AppendLayer(CountedPtr<PlotLayer> cpLayer)
{
    mLayers.push_back(cpLayer);
}// AppendLayer

void
PlotControl::SetFunctionValueExtent(double min, double max) const
{
    mPlotMin    = min;
    mPlotMax    = max;
}// SetFunctionValueExtent


///////////////////////////// PROTECTED/PRIVATE /////////////////////////////

void
PlotControl::StartDragRectangle(wxCoord x, wxCoord y)
{
//    CaptureMouse();
    mDragStartX = x;
    mDragStartY = y;
}// StartDragRectangle

void
PlotControl::UpdateDragRectangle(wxCoord x, wxCoord y)
{
    wxClientDC dc(this);

    dc.SetLogicalFunction(wxINVERT);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);

    if ((mDragWidth != 0) || (mDragHeight != 0))
    {   // Usu narysowan wczeniej ramk
        dc.DrawRectangle(mDragStartX, mDragStartY,
                mDragWidth, mDragHeight);
    }

    // Ogranicz x i y do rozmiaru okna.
    int width;
    int height;
    GetClientSize(&width, &height);

    if (x < 0)              x = 0;
    else if (x > width)     x = width;
    if (y < 0)              y = 0;
    else if (y > height)    y = height;

    mDragWidth = x - mDragStartX;
    mDragHeight = y - mDragStartY;

    dc.DrawRectangle(mDragStartX, mDragStartY,
            mDragWidth, mDragHeight);
}// UpdateDragRectangle

void
PlotControl::StopDragRectangle()
{
    if ((mDragWidth != 0) || (mDragHeight != 0))
    {
        wxClientDC dc(this);

        dc.SetLogicalFunction(wxINVERT);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);

        dc.DrawRectangle(mDragStartX, mDragStartY, mDragWidth, mDragHeight);

        mDragWidth = 0;
        mDragHeight = 0;
    }
//    ReleaseMouse();
}// StopDragRectangle

void
PlotControl::InitPlotDc()
{
    mPlotDc.SetAxisOrientation(true, false);

    mPlotDc.SetBackground(AttributeDatabase::GetInstance().
            Brush(AttributeDatabase::PLOT_BACKGROUND));
}// InitPlotDc

void
PlotControl::Redraw()
{
    mPlotDc.Clear();

    LOG(DEBUG, wxT("Entering redraw"));

    UpdateScale();

    vector<CountedPtr<PlotLayer> >::const_iterator i;
    for (i = mLayers.begin(); i != mLayers.end(); ++i)  (*i)->Draw(mPlotDc);

    // Moga si zmieni skala wartoci funkcji
    UpdateScale();

    // TODO: przenie do jakiego UpdateScale
    if (mpPlotStatus != NULL)
    {
        LOG(DEBUG, wxT("Setting scales in PlotStatus: ") << mScaleValues
                << wxT(", ") << mScaleX << wxT(", ") << mScaleY);
        mpPlotStatus->UpdateFormatStrings(mScaleValues, mScaleX, mScaleY);
    }
    LOG(DEBUG, wxT("Exiting redraw"));
}// Redraw

void
PlotControl::RedrawAxes(wxDC& dc)
{
    const wxBitmap& x_axis_bitmap = mXAxis.Draw();
    const wxBitmap& y_axis_bitmap = mYAxis.Draw();

    dc.DrawBitmap(x_axis_bitmap, mPlotAreaX,
            MARGIN_SIZE + mPlotBitmap.GetHeight());
    dc.DrawBitmap(y_axis_bitmap, MARGIN_SIZE,
            mPlotAreaY);
}// RedrawAxes

wxSize
PlotControl::ComputePlotSize() const
{
    wxSize size = GetClientSize();

    size.SetWidth(size.GetWidth() - mYAxis.Breadth() - 2*MARGIN_SIZE);
    size.SetHeight(size.GetHeight() - mXAxis.Breadth() - 2*MARGIN_SIZE);

    return size;
}// ComputePlotSize

wxCoord
PlotControl::WindowToPlotAreaX(wxCoord x) const
{
    return x - MARGIN_SIZE - mYAxis.Breadth();
}// WindowToPlotAreaX

wxCoord
PlotControl::WindowToPlotAreaY(wxCoord y) const
{
    return y - MARGIN_SIZE;
}// WindowToPlotAreaX


const /* static */ double   PlotControl::ZOOM_RATIO = 0.75;
const /* static */ double   PlotControl::MIN_ZOOMED_WIDTH = 0.001;
const /* static */ double   PlotControl::MAX_ZOOMED_WIDTH = 10e7;
const /* static */ wxCoord  PlotControl::MARGIN_SIZE = 25;
