#include <precomp.h>

#include "../Logger.h"
#include "PlotStatus.h"  // class implemented
#include "ui_util.h"

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PlotStatus::PlotStatus(wxWindow*        pParent,
                       wxWindowID       id,
                       bool             univariate,
                       const wxPoint&   position,
                       const wxSize&    size)
    : wxPanel(pParent, id, position, size),
      mNumberFormat(wxT("%.0f")),
      mUnivariate(univariate)
{
    wxFlexGridSizer* p_main_sizer = new wxFlexGridSizer(2, 2, 0, 0);
    p_main_sizer->AddGrowableCol(1, 2);
    p_main_sizer->SetFlexibleDirection(wxHORIZONTAL);

    // Nazwy zmiennych i tak s pniej ustawiane
    mpVariable1Label = new wxStaticText(this, wxID_ANY, wxT("x:"));
    mpVariable1Value = new wxStaticText(this, wxID_ANY, wxT("-"),
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT | wxST_NO_AUTORESIZE);

    if (!mUnivariate)
    {
        mpVariable2Label = new wxStaticText(this, wxID_ANY, wxT("y:"));
        mpVariable2Value = new wxStaticText(this, wxID_ANY, wxT("-"),
                wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT | wxST_NO_AUTORESIZE);
    }

    mpFunctionLabel = new wxStaticText(this, wxID_ANY, _("value:"));
    mpFunctionValue = new wxStaticText(this, wxID_ANY, wxT("-"),
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT | wxST_NO_AUTORESIZE);

    p_main_sizer->Add(mpVariable1Label, 1, wxALL | wxEXPAND, 2);
    p_main_sizer->Add(mpVariable1Value, 1, wxALL | wxEXPAND, 2);

    if (!mUnivariate)
    {
        p_main_sizer->Add(mpVariable2Label, 1, wxALL | wxEXPAND, 2);
        p_main_sizer->Add(mpVariable2Value, 1, wxALL | wxEXPAND, 2);
    }

    p_main_sizer->Add(mpFunctionLabel, 1, wxALL | wxEXPAND, 2);
    p_main_sizer->Add(mpFunctionValue, 1, wxALL | wxEXPAND, 2);

    SetSizer(p_main_sizer);
    p_main_sizer->SetSizeHints(this);
    Layout();
}// PlotStatus

PlotStatus::~PlotStatus()
{
}// ~PlotStatus


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
PlotStatus::UpdatePosition(double value, double argument1, double argument2)
{
    mpVariable1Value->SetLabel(wxString::Format(mNumberFormat, argument1));
    if (isfinite(value))
    {
        mpFunctionValue->SetLabel(wxString::Format(mNumberFormat, value));
    }
    else    mpFunctionValue->SetLabel(wxT('-'));

    if (!mUnivariate)
    {
        mpVariable2Value->SetLabel(wxString::Format(mNumberFormat, argument2));
    }
}// UpdatePosition


void
PlotStatus::ClearPosition()
{
    mpVariable1Value->SetLabel(wxT("-"));
    if (!mUnivariate)   mpVariable2Value->SetLabel(wxT("-"));
    mpFunctionValue->SetLabel(wxT("-"));
}// ClearPosition

void
PlotStatus::UpdatePositionNoValue(double argument1, double argument2)
{
    mpFunctionValue->SetLabel(wxT("-"));
    mpVariable1Value->SetLabel(wxString::Format(mNumberFormat, argument1));
    if (!mUnivariate)
    {
        mpVariable2Value->SetLabel(wxString::Format(mNumberFormat, argument2));
    }
}// UpdatePositionNoValue

void
PlotStatus::UpdateVariables(const wxString& variable1,
                            const wxString& variable2)
{
    mpVariable1Label->SetLabel(variable1);
    if (!mUnivariate)   mpVariable2Label->SetLabel(variable2);
}// UpdateVariables

void
PlotStatus::UpdateFormatStrings(double functionScale,
                                double variable1Scale,
                                double variable2Scale)
{
    double first_min;

    if (mUnivariate)    first_min = variable1Scale;
    else                first_min = min<double>(variable1Scale, variable2Scale);

    double smallest = min<double>(functionScale, first_min);

    mNumberFormat = NumberFormat(smallest);
}// UpdateFormatStrings

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
