#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Logger.h"

#include "PlotToolbar.h"  // class implemented

BEGIN_EVENT_TABLE(PlotToolbar, wxToolBar)
    EVT_TOOL(ID_TOOL_PAN, PlotToolbar::OnToolSelect)
    EVT_TOOL(ID_TOOL_ZOOMIN, PlotToolbar::OnToolSelect)
    EVT_TOOL(ID_TOOL_ZOOMOUT, PlotToolbar::OnToolSelect)
    EVT_TOOL(ID_TOOL_EXPORT, PlotToolbar::OnExport)
END_EVENT_TABLE()

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PlotToolbar::PlotToolbar(wxWindow*          pParent,
                         wxWindowID         id,
                         PlotControl*       pPlotControl,
                         const wxPoint&     position,
                         const wxSize&      size)
    : wxToolBar(pParent, id, position, size, wxTB_NODIVIDER | wxTB_FLAT),
      mpPlotControl(pPlotControl),
      mLastId(9999)
{
    assert (mpPlotControl != NULL);

    // Piwo dla pierwszej osoby, ktra mi powie, jak mona zainicjowa tablic
    // na licie inicjalizacyjnej. Moim zdaniem si nie da.
    // <marcin@studio4plus.com>
    for (unsigned i = 0; i < ToolbarTool::GROUP_MAX; ++i)   mToolCount[i] = 0;

    SetToolBitmapSize(wxSize(32, 32));
//    SetMargins(4, 4);

    SetUpBasicTools();

    Realize();
}// PlotToolbar


PlotToolbar::~PlotToolbar()
{
}// ~PlotToolbar


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
PlotToolbar::AddCustomTool(const ToolbarTool& tool)
{
    int id = GenerateToolId();

    // Pomi pozycje narzdzi z wczeniejszych grup.

    // pozycja ostatniego narzdzia
    size_t position = TOOL_MAX;   // TODO: zrobi sta
    for (int i = 0; i < tool.Group(); ++i)
    {
        position += mToolCount[i];
        if (mToolCount[i] > 0)  ++position; // dodaj separator
    }

    if (mToolCount[tool.Group()] == 0)
    {   // dodaj separator
        InsertSeparator(position);
        ++position;
    }
    else    position += mToolCount[tool.Group()] + 1;

    // Mamy pozycj nowego narzdzia, zwiksz ilo.
    ++mToolCount[tool.Group()];

    LOG(DEBUG, wxT("Position for new tool: ") << position);

    if (tool.Type() == wxITEM_NORMAL)
    {
        mClickTools[id] = tool.ClickAction();
        Connect(id, wxEVT_COMMAND_TOOL_CLICKED,
                wxCommandEventHandler(PlotToolbar::OnClickTool));

        InsertTool(position, id, tool.Label(), tool.Bitmap(), wxNullBitmap,
                wxITEM_NORMAL, tool.Tip());
    }
    else
    {
        assert (tool.Type() == wxITEM_CHECK);
        mToggleTools[id] = tool.ToggleAction();
        Connect(id, wxEVT_COMMAND_TOOL_CLICKED,
                wxCommandEventHandler(PlotToolbar::OnToggleTool));

        InsertTool(position, id, tool.Label(), tool.Bitmap(), wxNullBitmap,
                wxITEM_CHECK, tool.Tip());
        ToggleTool(id, mToggleTools[id]->IsToggled());
    }
}// AddCustomTool

void
PlotToolbar::OnClickTool(wxCommandEvent& event)
{
    ClickToolContainer::const_iterator tool = mClickTools.find(event.GetId());

    assert (tool != mClickTools.end());

    // Wykonaj akcj
    tool->second->Action();
}// OnClickTool

void
PlotToolbar::OnToggleTool(wxCommandEvent& event)
{
    ToggleToolContainer::const_iterator tool = mToggleTools.find(event.GetId());

    assert (tool != mToggleTools.end());

    // Wykonaj akcj
    bool new_state =
        tool->second->Action(GetToolState(event.GetId()));

    ToggleTool(event.GetId(), new_state);
}// OnToggleTool

void
PlotToolbar::OnToolSelect(wxCommandEvent& event)
{
    switch (event.GetId())
    {
        case ID_TOOL_PAN:     mpPlotControl->SetTool(PlotControl::PAN); break;
        case ID_TOOL_ZOOMIN:  mpPlotControl->SetTool(PlotControl::ZOOM_IN); break;
        case ID_TOOL_ZOOMOUT: mpPlotControl->SetTool(PlotControl::ZOOM_OUT); break;
        default:    assert(true);   // Nie powinno si zdarzy.
    }
}// OnToolSelect

void
PlotToolbar::OnExport(wxCommandEvent& event)
{
    mpPlotControl->ExportBitmap();
}// OnExport

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
PlotToolbar::SetUpBasicTools()
{
    wxBitmap pan_bitmap =
        wxXmlResource::Get()->LoadBitmap(wxT("TOOL_PAN"));
    wxBitmap zoomin_bitmap =
        wxXmlResource::Get()->LoadBitmap(wxT("TOOL_ZOOMIN"));
    wxBitmap zoomout_bitmap =
        wxXmlResource::Get()->LoadBitmap(wxT("TOOL_ZOOMOUT"));
    wxBitmap export_bitmap =
        wxXmlResource::Get()->LoadBitmap(wxT("TOOL_EXPORT"));

    LOG(DEBUG, wxT("Is bitmap ok? ") << zoomin_bitmap.Ok());

    AddTool(ID_TOOL_EXPORT, _("Export"), export_bitmap,
            wxNullBitmap, wxITEM_NORMAL, _("Export plot to file"));
    AddSeparator();
    AddRadioTool(ID_TOOL_PAN, _("Pan"), pan_bitmap, wxNullBitmap,
            _("Pan\nDrag the plot area to pan"));
    AddRadioTool(ID_TOOL_ZOOMIN, _("Zoom In"), zoomin_bitmap, wxNullBitmap,
            _("Zoom In\nClick the plot area or select a rectangle to zoom in"));
    AddRadioTool(ID_TOOL_ZOOMOUT, _("Zoom Out"), zoomout_bitmap, wxNullBitmap,
            _("Zoom Out\nClick the plot area to zoom out"));
}// SetUpBasicTools

/////////////////////////////// PRIVATE    ///////////////////////////////////

int
PlotToolbar::GenerateToolId()
{
    return ++mLastId;
}// GenerateToolId
