/** Pasek narzdzi dla okna wykresu.
 *
 * Zawiera wybr podstawowego narzdzia: przesuwania, powikszania albo
 * zmniejszania.
 * Dodatkowe narzdzia mog by dodawane w miar potrzeby.
 *
 * @see PlotControl, ToolbarTool
 */

#ifndef _PlotToolbar_h
#define _PlotToolbar_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "ClickToolAction.h"
#include "PlotControl.h"
#include "ToggleToolAction.h"
#include "ToolbarTool.h"

// FORWARD REFERENCES
//

class PlotToolbar
    : public wxToolBar
{
    enum {
        ID_TOOL_PAN = 1,
        ID_TOOL_ZOOMIN,
        ID_TOOL_ZOOMOUT,
        ID_TOOL_EXPORT,

        TOOL_MAX
    };

public:
// LIFECYCLE

    /** Konstruktor.
     */
    PlotToolbar(wxWindow*           pParent,
                wxWindowID          id,
                PlotControl*        pPlotControl,
                const wxPoint&      position = wxDefaultPosition,
                const wxSize&       size = wxDefaultSize);


    /** Destruktor.
     */
    ~PlotToolbar(void);


// OPERATORS

// OPERATIONS

    /// Dodaje narzdzie <code>tool</code>
    void    AddCustomTool(const ToolbarTool& tool);

    void    OnClickTool(wxCommandEvent& event);
    void    OnToggleTool(wxCommandEvent& event);

    void    OnToolSelect(wxCommandEvent& event);

    void    OnExport(wxCommandEvent& event);

// ACCESS
// INQUIRY

protected:

    /** Dodaje podstawowe narzdzia do paska (skalowanie i przesuwanie).
     */
    void                SetUpBasicTools();

private:
    int                 GenerateToolId();

    PlotControl*    mpPlotControl;

    typedef std::map<int, CountedPtr<ClickToolAction> >     ClickToolContainer;
    typedef std::map<int, CountedPtr<ToggleToolAction> >    ToggleToolContainer;

    ClickToolContainer  mClickTools;
    ToggleToolContainer mToggleTools;

    int         mLastId;    ///< Id ostatnio dodanego narzdzia.

    /// Liczba narzdzi poszczeglnych grup.
    unsigned    mToolCount[ToolbarTool::GROUP_MAX];

    DECLARE_EVENT_TABLE()
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _PlotToolbar_h
