#include <precomp.h>

#include "../Logger.h"

#include "DoubleValidator.h"
#include "PointControl.h"  // class implemented

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PointControl::PointControl(wxWindow*        pParent,
                           wxWindowID       id,
                           const wxPoint&   pos,
                           const wxSize&    size,
                           long             style,
                           const wxString&  name)
    : wxPanel(pParent, id, pos, size, style | wxTAB_TRAVERSAL, name)
{
    // eby walidacja obejmowaa te wewntrzne okno
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);

    mpMainSizer = new wxStaticBoxSizer(wxHORIZONTAL, this, name);

    mpScrolledPanel = new wxScrolledWindow(this, wxID_ANY, wxDefaultPosition,
            wxDefaultSize, wxTAB_TRAVERSAL);

    mpGridSizer = new wxFlexGridSizer(0, 2, 2, 2);

    // Pierwsza kolumna bierze 1/3 dostpnej szerokoci, druga 2/3
    mpGridSizer->AddGrowableCol(0, 1);
    mpGridSizer->AddGrowableCol(1, 2);

    // Pierwsze lepsze parametry :)
    mpScrolledPanel->SetScrollbars(5, 5, 30, 30);
    mpScrolledPanel->SetSizer(mpGridSizer);
    mpGridSizer->SetSizeHints(mpScrolledPanel);

    mpMainSizer->Add(mpScrolledPanel, 1, wxEXPAND);
    SetSizer(mpMainSizer);
    SetAutoLayout(true);

    SetMinSize(ConvertDialogToPixels(wxSize(-1, mMinHeight)));
}// PointControl


PointControl::~PointControl()
{
}// ~PointControl


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
PointControl::Init(const VariableNames& variables,
                   ColumnVector&        rValues,
                   double               initialValue)
{
    Freeze();

    if (static_cast<size_t>(rValues.nrows()) != variables.GetCount())
    {
        rValues.resize(variables.GetCount());
        for (int i = 1; i <= rValues.nrows(); ++i) rValues(i) = initialValue;
    }

    Clear();

    for (size_t i = 1; i <= variables.GetCount(); ++i)
    {
        AddRow(variables[i-1], rValues(i));
    }

    mpScrolledPanel->TransferDataToWindow();

//    mpMainSizer->Fit(this);
    mpGridSizer->SetVirtualSizeHints(mpScrolledPanel);

//    SetMinSize(ConvertDialogToPixels(wxSize(-1, mMinHeight)));
    mpScrolledPanel->Layout();

    Thaw();
}// Init

//============================= ACCESS     ===================================
void
PointControl::SetCaption(const wxString& caption)
{
    mpMainSizer->GetStaticBox()->SetLabel(caption);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
PointControl::AddRow(const wxString& label, double& rValue)
{
    wxStaticText* p_label = new wxStaticText(mpScrolledPanel, -1, label);
    wxTextCtrl* p_control = new wxTextCtrl(mpScrolledPanel, -1, wxT(""),
            wxDefaultPosition, wxDefaultSize, wxTE_RIGHT);

    LOG(DEBUG, wxT("AddRow: ") << label);

    // TODO: zastanowi si nad zastosowaniem DoubleParameterValidatora.
    p_control->SetValidator(DoubleValidator(rValue));

    mpGridSizer->Add(p_label, 1, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mpGridSizer->Add(p_control, 0, wxEXPAND|wxALL, 2);
}// AddRow

void
PointControl::Clear()
{
    mpGridSizer->Clear(true);
}// Clear

/////////////////////////////// PRIVATE    ///////////////////////////////////

const int PointControl::mMinHeight = 50;
