/**Szablon panelu dla metod optymalizacji w kierunku startujcych
 * z podanego punktu.
 *
 * Uniwersalny panel dla metod posiadajcych 2 parametry:
 * epsilon i punkt startowy.
 *
 * Panele dla poszczeglnych metod s rejestrowane
 * w PointLineSearchPanel.cc.
 */

#ifndef POINTLINESEARCHPANEL_H
#define POINTLINESEARCHPANEL_H

#include "DoubleParameterValidator.h"
#include "MethodPanelInfo.h"
#include "PointControl.h"
#include "ui_util.h"

template <class T>
class PointLineSearchPanel: public wxPanel
{
public:

    PointLineSearchPanel(wxWindow*              parent,
                         int                    id,
                         T*                     pMethod,
                         const wxPoint&         pos=wxDefaultPosition,
                         const wxSize&          size=wxDefaultSize,
                         long                   style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*         CreatePanel(MethodPanelInfo     panelInfo);


private:
    void do_layout();

protected:
    wxStaticText* mpEpsilonLabel;
    wxTextCtrl* mpEpsilonTextCtrl;
};

template <class T>
PointLineSearchPanel<T>::PointLineSearchPanel
                                (wxWindow*              parent,
                                 int                    id,
                                 T*                     pMethod,
                                 const wxPoint&         pos,
                                 const wxSize&          size,
                                 long                   style)
    : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{
    // Tworzenie kontrolek.
    mpEpsilonLabel = new wxStaticText(this, -1, _("Stop condition (\x03B5):"));
    mpEpsilonTextCtrl = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_RIGHT);

    do_layout();

    // Walidator.
    mpEpsilonTextCtrl->SetValidator(DoubleParameterValidator(pMethod->mEpsilon));

}// PointLineSearchPanel

template <class T>
void PointLineSearchPanel<T>::do_layout()
{
    wxBoxSizer* sizer_23 = new wxBoxSizer(wxVERTICAL);
    wxGridSizer* grid_sizer_2 = new wxGridSizer(1, 2, 0, 0);
    grid_sizer_2->Add(mpEpsilonLabel, 0, wxALL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 2);
    grid_sizer_2->Add(mpEpsilonTextCtrl, 0, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
    sizer_23->Add(grid_sizer_2, 0, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(sizer_23);
    sizer_23->Fit(this);
    sizer_23->SetSizeHints(this);
}

template <class T>
/* static */ wxPanel*
PointLineSearchPanel<T>::CreatePanel(MethodPanelInfo   panelInfo)
{
    T* p_method = dynamic_cast<T*>(panelInfo.pMethod());

    if (panelInfo.PanelType() == MethodPanelInfo::STANDALONE)
    {
        // Tworzy panel z panelu metody w kierunku oraz kontrolki
        // ustawiania odcinka pocztkowego.
        PointControl* p_starting_point = new PointControl(panelInfo.pParent(),
                -1, wxDefaultPosition, wxDefaultSize, 0, _("Starting point"));
        p_starting_point->Init((panelInfo.pProblem())->Variables(),
               /*p_method->mStartingPoint*/p_method->mInitialStartingPoint);

        return CreatePanelOfTwo(
                panelInfo.pParent(),
                new PointLineSearchPanel<T>(panelInfo.pParent(), -1, p_method),
                p_starting_point);
    }
    else
    {
        return new PointLineSearchPanel(panelInfo.pParent(), -1, p_method);
    }
}// CreatePanel

#endif // SECTIONLINESEARCHPANEL_H
