#include <precomp.h>

#include "../Logger.h"

#include "ProblemChoice.h"  // class implemented

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ProblemChoice::ProblemChoice()
    : wxChoice()
{
}// ProblemChoice

ProblemChoice::ProblemChoice(wxWindow       *pParent,
                             wxWindowID     id,
                             const wxPoint& pos,
                             const wxSize&  size,
                             long           style,
                             const wxString& name)
    : wxChoice(pParent, id, pos, size, 0, NULL, style, wxDefaultValidator, name)
{
    FillIn();
}// ProblemChoice


ProblemChoice::~ProblemChoice()
{
}// ~ProblemChoice


//============================= OPERATORS ====================================

bool
ProblemChoice::Create(wxWindow          *pParent,
                      wxWindowID        id,
                      const wxPoint&    pos,
                      const wxSize&     size,
                      long              style,
                      const wxString&   name)
{
    bool result;

    result = wxChoice::Create(pParent, id, pos, size, 0, NULL, 0,
            wxDefaultValidator, name);

    if (result) FillIn();

    return result;
}

std::auto_ptr<ProblemBase>
ProblemChoice::CreateSelected() const
{
    LOG(ERROR, wxT("ProblemChoice::CreateSelected is Obsolete!"));
    abort();
//    return GetSelected()->CloneEmpty();
}// CreateSelected

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

ProblemBase*
ProblemChoice::GetSelected() const
{
    ProblemBase* p_selected_problem
        = static_cast<ProblemBase*>(GetClientData(GetSelection()));

    return p_selected_problem;
}// GetSelected

int
ProblemChoice::FindProblemIndex(const ProblemBase* pProblem) const
{
    int i = GetCount() - 1;

    while ((i > 0) && (pProblem != GetClientData(i))) --i;

    return i;
}// FindProblemIndex

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
ProblemChoice::FillIn()
{
    const vector<ProblemBase*> problems = ProblemBase::Problems();
    vector<ProblemBase*>::const_iterator i;

    for (i = problems.begin(); i != problems.end(); ++i)
    {
        // sortowanie
        unsigned index;
        int pos;
        for (index = 0; index < GetCount(); ++index)
        {
            if (GetString(index) > (*i)->Name()) break;    // znaleziono
        }
        pos = wxChoice::Insert((*i)->Name(), index, static_cast<void *>(*i));

        assert(pos > -1);
    }
    SetSelection(0);
}

/////////////////////////////// PRIVATE    ///////////////////////////////////
