/**  Kontrolka wyboru problemu/zadania.
 *
 * @see ProblemBase, wxChoice
 */

#ifndef _ProblemChoice_h
#define _ProblemChoice_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <wx/wx.h>
#include <wx/choice.h>
#include <wx/string.h>

// LOCAL INCLUDES
//
#include "../ProblemBase.h"

// FORWARD REFERENCES
//


class ProblemChoice
    : public wxChoice
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    ProblemChoice(void);


    /** Konstruktor.
     *
     * param pParent Wskanik na okno rodzica. Nie moe by NULL.
     * param id Id okna. -1 oznacza domylne.
     * param pos Pozycja.
     * param size Rozmiar.
     * param style Styl.
     * param name Nazwa.
     *
     * @see wxChoice::wxChoice
     */
    ProblemChoice(wxWindow          *pParent,
                  wxWindowID        id,
                  const wxPoint&    pos = wxDefaultPosition,
                  const wxSize&     size = wxDefaultSize,
                  long              style = 0,
                  const wxString&   name = wxT("ProblemChoice"));


    /** Destruktor.
     */
    ~ProblemChoice(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kontrolk (po uyciu konstruktora bezparametrowego)
     */
    bool Create(wxWindow*         pParent,
                wxWindowID        id,
                const wxPoint&    pos = wxDefaultPosition,
                const wxSize&     size = wxDefaultSize,
                long              style = 0,
                const wxString&   name = wxT("ProblemChoice"));


    /** Tworzy i zwraca obiekt problemu zgodnie z aktualnie wybran opcj.
     */
    std::auto_ptr<ProblemBase>  CreateSelected() const;


// ACCESS

    /** Zwraca wskanik na prototyp aktualnie zaznaczonego problemu.
     */
    ProblemBase*                GetSelected() const;

    /** Zwraca indeks podanego problemu.
     *
     * return Indeks problemu na licie lub -1, jeli nie znaleziono.
     */
    int                         FindProblemIndex
                                    (const ProblemBase* pProblem) const;

// INQUIRY

protected:

    /** Wypenia list problemami.
     */
    void            FillIn();

private:

    // TODO: ukry funkcje zwizane z dodawaniem i usuwaniem elementw wxChoice

    // Prywatny konstruktor kopiujcy i operator przypisania
    DECLARE_NO_COPY_CLASS(ProblemChoice);

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ProblemChoice_h
