#include <precomp.h>

#include "../Logger.h"

#include "ProblemDialog.h"  // class implemented
#include "ProblemPanelFactory.h"

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

BEGIN_EVENT_TABLE(ProblemDialog, wxDialog)
    EVT_CHOICE(ID_PROBLEMCHOICE,    ProblemDialog::OnProblemChoice)
    EVT_BUTTON(wxID_OK,             ProblemDialog::OnOk)
    EVT_BUTTON(wxID_EXIT,           ProblemDialog::OnExit)
    EVT_CLOSE(                      ProblemDialog::OnClose)
END_EVENT_TABLE()

//============================= LIFECYCLE ====================================

ProblemDialog::ProblemDialog(wxWindow*          pParent,
                             wxWindowID         id,
                             ProblemBase*&      rpProblem,
                             bool               exitOnCancel)
    : wxDialog(pParent,
               id,
               _("Define Optimization Problem"),
               wxDefaultPosition,
               wxDefaultSize,
               wxRESIZE_BORDER | wxCAPTION),
      mrpProblem(rpProblem),
      mpTempProblem(NULL),  // Bdzie ustawione w OnProblemChoice
      mExitOnClose(exitOnCancel)
{
    assert(mrpProblem != NULL);

    // Tworz kontrolki
    mpProblemStaticBox  = new wxStaticBox(this, -1, _("Problem"));
    mpProblemChoice     = new ProblemChoice(this, ID_PROBLEMCHOICE);
    mpProblemPanel      = new wxPanel(this, -1);
    mpProblemPanel->SetMinSize(ProblemPanelFactory::GetInstance().MinSize());

    mpOkButton          = new wxButton(this, wxID_OK, _("OK"));

    if (exitOnCancel) mpCancelButton = new wxButton(this, wxID_EXIT, _("Quit"));
    else              mpCancelButton = new wxButton(this, wxID_CANCEL, _("Cancel"));

    // Sizery
    wxSizer* p_staticbox_sizer = new wxStaticBoxSizer(mpProblemStaticBox, wxHORIZONTAL);
    p_staticbox_sizer->Add(mpProblemChoice, 1, wxEXPAND, 0);

    wxSizer* p_button_sizer = new wxBoxSizer(wxHORIZONTAL);
    p_button_sizer->Add(0, 0, 1, 0, 0);
    p_button_sizer->Add(mpOkButton, 0, wxALL, 2);
    p_button_sizer->Add(mpCancelButton, 0, wxALL, 2);

    mpSizer = new wxBoxSizer(wxVERTICAL);
    mpSizer->Add(p_staticbox_sizer, 0, wxALL|wxEXPAND, 2);
    mpSizer->Add(mpProblemPanel, 1, wxALL|wxEXPAND, 2);
    mpSizer->Add(p_button_sizer, 0, wxALL|wxEXPAND, 2);

    SetAutoLayout(true);
    SetSizer(mpSizer);
    mpSizer->Fit(this);
    mpSizer->SetSizeHints(this);

    // Ustaw okienko wyboru oraz panel problemu
    assert(mpProblemChoice->FindProblemIndex(mrpProblem) >= 0);

    mpProblemChoice->SetSelection(
            mpProblemChoice->FindProblemIndex(mrpProblem));

    wxCommandEvent dummy_event;
    OnProblemChoice(dummy_event);

    Layout();
}



ProblemDialog::~ProblemDialog()
{
}// ~ProblemDialog


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
ProblemDialog::OnProblemChoice(wxCommandEvent& event)
{
    ProblemBase* p_new_problem = mpProblemChoice->GetSelected();

    Freeze();
    // Poniewa oba wskazuj na obiekty z listy, to rne obiekty na pewno maj
    // rne wskaniki, a takie same - te same.
    if (mpTempProblem != p_new_problem)
    {
        mpSizer->Detach(mpProblemPanel);

        RemoveChild(mpProblemPanel);
        delete mpProblemPanel;

        // Ustaw problem na podstawie wyboru uytkownika
        mpTempProblem = mpProblemChoice->GetSelected();

        mpProblemPanel = ProblemPanelFactory::GetInstance().
                CreatePanel(mpTempProblem, this);

        // TODO: czy panele nie bd wymagay wasnych walidatorw?
        mpProblemPanel->TransferDataToWindow();

        mpSizer->Insert(1, mpProblemPanel, 1, wxALL|wxEXPAND, 2);
        mpSizer->Layout();

        Layout();
    }
    mpOkButton->MoveAfterInTabOrder(mpProblemPanel);
    mpCancelButton->MoveAfterInTabOrder(mpOkButton);

    Thaw();
}// OnProblemChoice

void
ProblemDialog::OnOk(wxCommandEvent& event)
{
    if (Validate() && TransferDataFromWindow())
    {
        mrpProblem = mpTempProblem;
        EndModal(wxID_OK);
    }
}// OnOk

void
ProblemDialog::OnExit(wxCommandEvent& event)
{
    EndModal(wxID_EXIT);
}// OnExit

void
ProblemDialog::OnClose(wxCloseEvent& event)
{
    if (mExitOnClose)   EndModal(wxID_EXIT);
    else                EndModal(wxID_CANCEL);
}// OnClose

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
