/**  Okno dialogowe wprowadzania zadania optymalizacji.
 *
 * Sluzy do wybrania typu zadania oraz wprowadzenia takich danych, jak funkcja,
 * ograniczenia (zaleznych od problemu).
 *
 * @see wxDialog
 */

#ifndef _ProblemDialog_h
#define _ProblemDialog_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <wx/dialog.h>

// LOCAL INCLUDES
//
#include "../ProblemBase.h"

#include "ProblemChoice.h"

// FORWARD REFERENCES
//


class ProblemDialog
    : public wxDialog
{
public:
// IDs
    enum {
        ID_PROBLEMCHOICE = 1
    };

// LIFECYCLE

    /** Konstruktor domylny.
     */
//    ProblemDialog();

    /** Konstruktor.
     *
     * @param pParent Okno rodzica.
     * @param id Id okna.
     * @param rpProblem Referencja na wskanik na problem. Ten wskanik
     * zostanie ustawiony na prototyp wybranego problemu (zadania
     * optymalizacji). Problem naley sklonowa przed rozpoczciem
     * optymalizacji.
     * @param exitOnCancel Czy wyj gdy zadanie optymalizacji nie zostanie
     * przez uytkownika okrelone ("Wyjd" zamiast "Anuluj"). Powoduje
     * zwrcenie wyniku <code>wxID_EXIT</code> zamiast <code>wxID_CANCEL</code>.
     */
    ProblemDialog(wxWindow*         pParent,
                  wxWindowID        id,
                  ProblemBase*&     rpProblem,
                  bool              exitOnCancel);


    /** Destruktor.
     */
    ~ProblemDialog(void);


// OPERATORS

// OPERATIONS

    // Create

    /** Wywoywane, gdy z listy zostanie wybrany typ zadania optymalizacji.
     *
     * Usuwa stary i tworzy nowy panel z ustawieniami opcji problemu.
     */
    void OnProblemChoice(wxCommandEvent& event);

    /** Wywoywane, gdy zostanie nacinity przycisk Ok.
     */
    void OnOk(wxCommandEvent& event);

    /** Wywoywane, gdy zostanie nacinity przycisk "Wyjd".
     */
    void OnExit(wxCommandEvent& event);

    /** Wywoywany, gdy okno zostanie zamknite.
     */
    void OnClose(wxCloseEvent& event);

// ACCESS
// INQUIRY

protected:
private:
    /// Referencja na edytowany wskanik problemu.
    ProblemBase*&   mrpProblem;

    /** Tymczasowy wskanik na problem (odrzucany w przypadku anulowania
     *  okienka)
     */
    ProblemBase*    mpTempProblem;

    // Waniejsze kontrolki:
    wxButton*       mpOkButton;
    wxButton*       mpCancelButton;
    wxStaticBox*    mpProblemStaticBox;
    ProblemChoice*  mpProblemChoice;
    wxPanel*        mpProblemPanel;
    wxSizer*        mpSizer;

    bool            mExitOnClose;

    DECLARE_EVENT_TABLE()
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ProblemDialog_h
