#include <precomp.h>

#include "../UnconstrainedProblem.h"
#include "ProblemPanelFactory.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ProblemPanelFactory::~ProblemPanelFactory()
{
}// ~ProblemPanelFactory


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void*
ProblemPanelFactory::Register(ProblemIdType id, PanelCreator function)
{
    assert(function != NULL);

    mFactories.insert(pair<ProblemIdType, PanelCreator>(id, function));

    return NULL;
}// Register

wxPanel*
ProblemPanelFactory::CreatePanel(ProblemBase*       pProblem,
                                wxWindow*           pParent) const
{
    //    assert(mFactories.find(pProblem) != mFactories.end());

    map<ProblemIdType, PanelCreator>::const_iterator pos =
            mFactories.find(pProblem->Id());

    // Jeli fabryka nie jest zarejestrowana, to twrz pusty panel
    if (pos != mFactories.end())    return (pos->second)(pProblem, pParent);
    else                            return new wxPanel(pParent, -1);
}// CreatePanel

wxSize
ProblemPanelFactory::MinSize()
{
    // Crude
    wxWindow* p_dummy_window = new wxFrame(NULL, -1, wxT(""));
    map<ProblemIdType, PanelCreator>::iterator i;
    wxSize size(0, 0);

    for (i = mFactories.begin(); i != mFactories.end(); ++i)
    {
        wxPanel *p_panel = i->second(ProblemBase::GetById(i->first),
                                     p_dummy_window);
        size.IncTo(p_panel->GetMinSize());
        p_dummy_window->RemoveChild(p_panel);
        delete p_panel;
    }

    delete p_dummy_window;

    return size;
}// MinSize

//============================= ACCESS     ===================================

/* static */ ProblemPanelFactory&
ProblemPanelFactory::GetInstance()
{
    static ProblemPanelFactory instance;

    return instance;
}// GetInstance

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

ProblemPanelFactory::ProblemPanelFactory()
    : mFactories()
{
}// ProblemPanelFactory

