/**  Fabryka paneli ustawie parametrw problemw.
 */

#ifndef _ProblemPanelFactory_h
#define _ProblemPanelFactory_h

// SYSTEM INCLUDES
//
#include <map>

// PROJECT INCLUDES
//
#include <wx/panel.h>

// LOCAL INCLUDES
//
#include "../ProblemBase.h"

// FORWARD REFERENCES
//


class ProblemPanelFactory
{
public:
    /** Wskanik na funkcj tworzc panel metody.
     *
     * Parametrami s: wskanik na problem, dla ktrego naley utworzy panel
     * oraz okno, ktre bdzie rodzicem panelu.
     */
    typedef wxPanel*(*PanelCreator)(ProblemBase* pProblem, wxWindow* pParent);

// LIFECYCLE

    /** Destruktor.
     */
    ~ProblemPanelFactory(void);


// OPERATORS
// OPERATIONS

    /** Rejestruje funkcj tworzc panel dla danego problemu.
     *
     * @return NULL (warto bez znaczenia, jej zwracanie jest przydatne przy
     * automatycznym rejestrowaniu przez tworzenie statycznej skadowej klasy).
     */
    void*                   Register(ProblemIdType id, PanelCreator function);

    /** Tworzy panel dla danej metody.
     */
    wxPanel*                CreatePanel(ProblemBase*        pProblem,
                                        wxWindow*           pParent) const;

    /** Oblicza minimalny rozmiar, ktry pomieci kady panel.
     */
    wxSize                  MinSize();

// ACCESS

    /** Zwraca referencj na singleton.
     */
    static ProblemPanelFactory& GetInstance();

// INQUIRY

protected:
private:
    /** Konstruktor domylny.
     */
    ProblemPanelFactory(void);

    map<ProblemIdType, PanelCreator>  mFactories;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ProblemPanelFactory_h
