#include <precomp.h>

//#include <wx/intl.h>
//#include <wx/msgdlg.h>

#include "../Logger.h"
#include "../HyperbolicInnerPenalty.h"
#include "../LogarithmicInnerPenalty.h"
#include "../OuterPenalty.h"

#include "ProblemWithPenaltyPanel.h"
#include "ProblemWithPenaltyValidator.h"  // class implemented

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ProblemWithPenaltyValidator::ProblemWithPenaltyValidator(ProblemBase* pProblem)
    : mpProblem(dynamic_cast<ProblemWithPenalty*>(pProblem))
{
    assert(pProblem != NULL);
}// ProblemWithPenaltyValidator

ProblemWithPenaltyValidator::ProblemWithPenaltyValidator(
        const ProblemWithPenaltyValidator& from)
    : mpProblem(from.mpProblem)
{
}// ProblemWithPenaltyValidator

ProblemWithPenaltyValidator::~ProblemWithPenaltyValidator()
{
}// ~ProblemWithPenaltyValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
ProblemWithPenaltyValidator::Clone() const
{
    return new ProblemWithPenaltyValidator(*this);
}// Clone

/* virtual */ bool
ProblemWithPenaltyValidator::Validate(wxWindow* parent)
{
    return GetWindow()->Validate();
}// Validate

/* virtual */ bool
ProblemWithPenaltyValidator::TransferToWindow()
{

    // TODO: to wymaga przerobienia, gdy dojd inne funkcje kary.
    ProblemWithPenaltyPanel* p_panel =
            dynamic_cast<ProblemWithPenaltyPanel*>(GetWindow());

    p_panel->mcpFunction    = mpProblem->mcpFunction;
    p_panel->mcpVariables   = mpProblem->mcpVariables;

    GetWindow()->TransferDataToWindow();

    return true;
}// TransferToWindow

/* virtual */ bool
ProblemWithPenaltyValidator::TransferFromWindow()
{

    ProblemWithPenaltyPanel* p_panel =
            dynamic_cast<ProblemWithPenaltyPanel*>(GetWindow());

    bool result;
    // Bierz dane z podkontrolek

    result = p_panel->TransferDataFromWindow();
    assert(result);

    auto_ptr<PenaltyFunction> ap_penalty_function;

    // Opakuj funkcj
    switch (p_panel->mpTypeChoice->GetSelection())
    {
        case 2:     ap_penalty_function.reset
                        (new LogarithmicInnerPenalty(p_panel->mcpFunction));
                    break;
        case 1:     ap_penalty_function.reset
                        (new HyperbolicInnerPenalty(p_panel->mcpFunction));
                    break;
        case 0:
        default:    ap_penalty_function.reset
                        (new OuterPenalty(p_panel->mcpFunction));
                    break;
    }


    switch (p_panel->mReactionChosen)
    {
        case 0: ap_penalty_function->rOutOfBoundsReaction()
                                = PenaltyFunction::EXCEPTION;
                break;
        case 1: ap_penalty_function->rOutOfBoundsReaction()
                                = PenaltyFunction::MAX_VALUE;
                break;
        default: assert(false);
                 break;

    }

    ap_penalty_function->rConstraints() = p_panel->rConstraints();

    mpProblem->rpPenalty() = ap_penalty_function.get();
    auto_ptr<VariableNames> ap_variables
                            (new VariableNames(*p_panel->mcpVariables));
    mpProblem->SetFunction(auto_ptr<FunctionBase>(ap_penalty_function),
                           ap_variables);
    //mpProblem->auto_ptr<const FunctionBase>(ap_penalty_function);

    LOG(DEBUG,wxT("Epsilon: ") << mpProblem->mEpsilon);

    return result;
}// TransferFromWindow


//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
