#include <precomp.h>

#include <wx/valgen.h>

#include "../Logger.h"
#include "DoubleParameterValidator.h"
#include "ProblemWithPenaltyPanel.h"
#include "PwpAdvancedDialog.h"

LOGGER(DEBUG)

PwpAdvancedDialog::PwpAdvancedDialog(wxWindow*           parent,
                                    int                 id,
                                    ProblemWithPenalty* pProblem,
                                    const wxString&     title,
                                    const wxPoint&      pos,
                                    const wxSize&       size,
                                    long                style)
    : wxDialog(parent,
               id,
               title,
               pos,
               size,
               style),
      mpProblem(pProblem)
{

    mpParent = dynamic_cast<ProblemWithPenaltyPanel*>(GetParent());

    const wxString mpReactionRadio_choices[] = {
        _("Raise error"),
        _("Return maximum value")
    };

    /* Wyczamy, nie ma sensu, aby uytkownik to ustawia.
    mpEpsilonCtrl       = new wxTextCtrl(this, -1, wxT(""),
                            wxDefaultPosition, wxDefaultSize, wxTE_RIGHT);
    */
    mpReuseSpBox        = new wxCheckBox(this, -1, _("Use the same starting point in all iterations"));
    mpReactionRadio     = new wxRadioBox(this, -1,
                                _("Reaction to violating constraints (only inner penalty)"),
                                wxDefaultPosition, wxDefaultSize, 2,
                                mpReactionRadio_choices, 1, wxRA_SPECIFY_ROWS);
    mpOkButton          = new wxButton(this, wxID_OK, _("OK"));
    mpCancelButton      = new wxButton(this, wxID_CANCEL, _("Cancel"));


    set_properties();
    do_layout();

    //validators
//    mpEpsilonCtrl->SetValidator(DoubleParameterValidator(pProblem->mEpsilon));
    mpReactionRadio->SetValidator
        (wxGenericValidator(reinterpret_cast<int*>
                            (&(mpParent->mReactionChosen))));
    mpReuseSpBox->SetValidator
        (wxGenericValidator(reinterpret_cast<bool*>
                            (&(mpProblem->mReuseStartingPoint))));

}

void
PwpAdvancedDialog::set_properties()
{
    mpOkButton->SetDefault();
    mpReactionRadio->SetSelection(mpParent->mReactionChosen);
/*
    switch (mpParent->mpTypeChoice->GetSelection())
    {
        case 2:
        case 1:     mpReactionRadio->Enable(true);
                    break;
        case 0:
        default:    mpReactionRadio->Enable(false);
                    break;
    }
*/
}

void
PwpAdvancedDialog::do_layout()
{

//    wxStaticBoxSizer*   p_epsilon_sizer = new wxStaticBoxSizer
//                        (wxVERTICAL, this, _T("Acceptable approximation"));
    wxBoxSizer*         p_button_sizer  = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer*         p_main_sizer    = new wxBoxSizer(wxVERTICAL);


//    p_epsilon_sizer->Add(mpEpsilonCtrl, 0, wxEXPAND | wxALL, 2);
    p_button_sizer->AddStretchSpacer();
    p_button_sizer->Add(mpOkButton, 0, wxALIGN_RIGHT | wxALL, 2);
    p_button_sizer->Add(mpCancelButton, 0, wxALIGN_RIGHT | wxALL, 2);
//    p_main_sizer->Add(p_epsilon_sizer, 0, wxEXPAND | wxALL, 2);
    p_main_sizer->Add(mpReuseSpBox, 0, wxEXPAND | wxALL, 2);
    p_main_sizer->Add(mpReactionRadio, 0, wxEXPAND | wxALL, 2);
    p_main_sizer->Add(p_button_sizer, 0, wxEXPAND);


    SetAutoLayout(true);
    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);

}
