#include <precomp.h>

#include "../Logger.h"

#include "DoubleValidator.h"
#include "RangeControl.h"  // class implemented

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RangeControl::RangeControl(wxWindow*        pParent,
                           wxWindowID       id,
                           const wxPoint&   pos,
                           const wxSize&    size,
                           long             style,
                           const wxString&  name)
    : wxPanel(pParent, id, pos, size, style | wxTAB_TRAVERSAL, name)
{
    // eby walidacja obejmowaa te wewntrzne okno
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);

    mpMainSizer = new wxStaticBoxSizer(wxHORIZONTAL, this, name);

    mpScrolledPanel = new wxScrolledWindow(this, wxID_ANY, wxDefaultPosition,
            wxDefaultSize, wxTAB_TRAVERSAL);

    mpGridSizer = new wxFlexGridSizer(0, 3, 2, 2);

    // Pierwsza kolumna bierze 1/3 dostpnej szerokoci, druga 2/3
    mpGridSizer->AddGrowableCol(0, 1);
    mpGridSizer->AddGrowableCol(1, 2);
    mpGridSizer->AddGrowableCol(2, 2);

    // Pierwsze lepsze parametry :)
    mpScrolledPanel->SetScrollbars(5, 5, 30, 30);
    mpScrolledPanel->SetSizer(mpGridSizer);
    mpGridSizer->SetSizeHints(mpScrolledPanel);

    mpMainSizer->Add(mpScrolledPanel, 1, wxEXPAND);
    SetSizer(mpMainSizer);
    SetAutoLayout(true);

    SetMinSize(ConvertDialogToPixels(wxSize(-1, mMinHeight)));
}// RangeControl


RangeControl::~RangeControl()
{
}// ~RangeControl


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
RangeControl::Init(const VariableNames& variables,
                   ColumnVector&        rMinBounds,
                   ColumnVector&        rMaxBounds,
                   double               initialMinValue,
                   double               initialMaxValue)
{

    if (static_cast<size_t>(rMinBounds.nrows()) != variables.GetCount())
    {
        rMinBounds.resize(variables.GetCount());
        for (int i = 1; i <= rMinBounds.nrows(); ++i)
            rMinBounds(i) = initialMinValue;
    }

    if (static_cast<size_t>(rMaxBounds.nrows()) != variables.GetCount())
    {
        rMaxBounds.resize(variables.GetCount());
        for (int i = 1; i <= rMaxBounds.nrows(); ++i)
            rMaxBounds(i) = initialMaxValue;
    }

    Clear();

    for (size_t i = 1; i <= variables.GetCount(); ++i)
    {
        AddRow(variables[i-1], rMinBounds(i), rMaxBounds(i));
    }

    mpScrolledPanel->TransferDataToWindow();

//    mpMainSizer->Fit(this);
    mpGridSizer->SetVirtualSizeHints(mpScrolledPanel);

//    SetMinSize(ConvertDialogToPixels(wxSize(-1, mMinHeight)));
    mpScrolledPanel->Layout();
}// Init

//============================= ACCESS     ===================================
void
RangeControl::SetCaption(const wxString& caption)
{
    mpMainSizer->GetStaticBox()->SetLabel(caption);
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
RangeControl::AddRow(const wxString&    label,
                           double&      rMinBound,
                           double&      rMaxBound)
{
    LOG(DEBUG, wxT("Adding row"));

    wxStaticText* p_label = new wxStaticText(mpScrolledPanel, -1, label);
    wxTextCtrl* p_min_control = new wxTextCtrl(mpScrolledPanel, -1, wxT(""),
            wxDefaultPosition, wxDefaultSize, wxTE_RIGHT);
    wxTextCtrl* p_max_control = new wxTextCtrl(mpScrolledPanel, -1, wxT(""),
            wxDefaultPosition, wxDefaultSize, wxTE_RIGHT);

    LOG(DEBUG, wxT("AddRow: ") << label);

    p_min_control->SetValidator(DoubleValidator(rMinBound));
    p_max_control->SetValidator(DoubleValidator(rMaxBound));

    mpGridSizer->Add(p_label, 1, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mpGridSizer->Add(p_min_control, 0, wxEXPAND|wxALL, 2);
    mpGridSizer->Add(p_max_control, 0, wxEXPAND|wxALL, 2);
}// AddRow

void
RangeControl::Clear()
{
    mpGridSizer->Clear(true);
}// Clear

/////////////////////////////// PRIVATE    ///////////////////////////////////

const int RangeControl::mMinHeight = 50;
