#include <precomp.h>

#include "../Logger.h"
#include "ResultLayer.h"  // class implemented

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ResultLayer::ResultLayer(const PlotControl&     plotControl,
                         const Result&          result,
                         CountedPtr<PlotLayer>  cpFunctionLayer)
    : PlotLayer(plotControl), mResult(result), mcpFunctionLayer(cpFunctionLayer)
{
}// ResultLayer

ResultLayer::ResultLayer(const ResultLayer& from)
    : PlotLayer(from),
      mResult(from.mResult),
      mcpFunctionLayer(from.mcpFunctionLayer)
{
}// ResultLayer

ResultLayer::~ResultLayer()
{
}// ~ResultLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
ResultLayer::Highlight(unsigned iterationNumber)
{
    assert ((iterationNumber >= mResult.FirstIterationNumber())
            && (iterationNumber <= mResult.LastIterationNumber()));

    mHighlightedIteration = iterationNumber;
}// Highlight

/* virtual */ Region
ResultLayer::BoundingBox() const
{
    Region bounding_box;

    for (unsigned i = mResult.FirstIterationNumber();
         i <= mResult.LastIterationNumber();
         ++i)
    {
        bounding_box += IterationBoundingBox(i);
        LOG(DEBUG, wxT("New bounding box: ")   << bounding_box.X1() << wxT(", ")
                                               << bounding_box.X2() << wxT("; ")
                                               << bounding_box.Y1() << wxT(", ")
                                               << bounding_box.Y2());
    }

    return bounding_box;
}

//============================= ACCESS     ===================================

unsigned
ResultLayer::HighlightedIterationNumber() const
{
    return mHighlightedIteration;
}// HighlightedIterationNumber

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

ResultLayer::PointPainter::PointPainter(const ResultLayer& layer, wxDC& dc)
    : mLayer(layer), mrDc(dc)
{
}// PointPainter::PointPainter

void
ResultLayer::PointPainter::operator()(
    const ResultLayer::ColourPointType& point) const
{
    mLayer.DrawBigPoint(mrDc, point.mPoint, point.mAttribute, point.mSymbol);
}// PointPainter::()

void
ResultLayer::DrawBigPoint(wxDC&                             dc,
                          wxPoint                           point,
                          AttributeDatabase::AttributeType  attribute,
                          ResultLayer::SymbolType           symbol) const
{
    double DARKEN_FACTOR = 0.65;
    unsigned HALF_SIZE = 4;
    unsigned SMALL_SIZE = 3;

    wxPen   old_pen     = dc.GetPen();
    wxBrush old_brush   = dc.GetBrush();

    const wxColour& colour = AttributeDatabase::GetInstance().Colour(attribute);
    if (colour == *wxWHITE)     dc.SetPen(wxPen(*wxBLACK));
    else
    {
        const wxColour  dark_colour(lround(colour.Red() * DARKEN_FACTOR),
                                    lround(colour.Green() * DARKEN_FACTOR),
                                    lround(colour.Blue() * DARKEN_FACTOR));

        dc.SetPen(wxPen(dark_colour));
    }

    dc.SetBrush(wxBrush(colour));

    switch (symbol)
    {
        case CIRCLE:        dc.DrawCircle(point, HALF_SIZE);
                            break;
        case SMALL_CIRCLE:  dc.DrawCircle(point, SMALL_SIZE);
                            break;
        case SQUARE:        dc.DrawRectangle(point.x - HALF_SIZE,
                                             point.y - HALF_SIZE,
                                             2 * HALF_SIZE,
                                             2 * HALF_SIZE);
                            break;
        case SMALL_SQUARE:  dc.DrawRectangle(point.x - SMALL_SIZE,
                                             point.y - SMALL_SIZE,
                                             2 * SMALL_SIZE,
                                             2 * SMALL_SIZE);

    }

    dc.SetPen(old_pen);
    dc.SetBrush(old_brush);
}
/////////////////////////////// PRIVATE    ///////////////////////////////////
