/** Warstwa wykresu odpowiedzialna za wykrelanie przebiegu optymalizacji.
 *
 * @see PlotControl
 */

#ifndef _ResultLayer_h
#define _ResultLayer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../Result.h"
#include "AttributeDatabase.h"
#include "PlotLayer.h"

// FORWARD REFERENCES
//


class ResultLayer
    : public PlotLayer
{
public:
    typedef std::auto_ptr<ResultLayer>(*Creator)(
            const PlotControl&          plotControl,
            const Result&               result,
            CountedPtr<PlotLayer>       cpFunctionLayer);

    enum SymbolType
    {
        CIRCLE,
        SQUARE,
        SMALL_CIRCLE,
        SMALL_SQUARE
    };

// LIFECYCLE

    /** Konstruktor.
     *
     * @param plotControl Kontrolka, na ktrej ma by rysowana warstwa.
     * @param result Przebieg optymalizacji, ktry ma by rysowany.
     * @param cpFunctionLayer <code>CountedPtr</code> na warstw z wykresem
     * funkcji (pozwala to rysowa pod nim).
     */
    ResultLayer(const PlotControl&      plotControl,
                const Result&           result,
                CountedPtr<PlotLayer>   cpFunctionLayer);

    /** Konstruktor kopiujcy.
     */
    ResultLayer(const ResultLayer& from);

    /** Destruktor.
     */
    ~ResultLayer(void);


// OPERATORS
// OPERATIONS

    /** Zwraca najmniejszy prostoktny obszar obejmujcy dan iteracj.
     *
     * Przy funkcji jednej zmiennej znaczenie maj jedynie skadowe x.
     */
    virtual Region IterationBoundingBox(unsigned iterationNumber) const = 0;

    virtual Region BoundingBox() const;

// ACCESS

    /** Okrela, ktr iteracj wyrni.
     */
    void            Highlight(unsigned iterationNumber);

// INQUIRY

    /** Zwraca numer zaznaczonej iteracji.
     */
    unsigned        HighlightedIterationNumber() const;

protected:
    /// Typ reprezentujcy punkt z atrybutem.
    struct ColourPointType
    {
    public:
        ColourPointType(wxPoint                             point,
                        AttributeDatabase::AttributeType    attribute,
                        SymbolType                          symbol = CIRCLE)
            : mPoint(point), mAttribute(attribute), mSymbol(symbol)
        {
        };

        wxPoint                             mPoint;
        AttributeDatabase::AttributeType    mAttribute;
        SymbolType                          mSymbol;
    };

    /// Lista punktw z atrybutami.
    typedef list<ColourPointType> PointList;

    /// Klasa rysujca punkt (dla algortymw STLowych).
    class PointPainter
    {
    public:
        PointPainter(const ResultLayer& layer, wxDC& dc);

        void operator()(const ColourPointType& point) const;
    private:
        const ResultLayer&  mLayer;
        wxDC&               mrDc;
    };


    void            DrawBigPoint(
                        wxDC&                               dc,
                        wxPoint                             point,
                        AttributeDatabase::AttributeType    attribute,
                        SymbolType                          symbol = CIRCLE)
                    const;



    const Result&               mResult;
    CountedPtr<const PlotLayer> mcpFunctionLayer;

    unsigned        mHighlightedIteration;

private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ResultLayer_h
