#include <precomp.h>

#include "../methods/SecantDivisionIteration.h"

#include "SecantDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SecantDivisionLayer::SecantDivisionLayer
                        (const PlotControl&             plotControl,
                         const Result&                  result,
                               CountedPtr<PlotLayer>    cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// SecantDivisionLayer

SecantDivisionLayer::SecantDivisionLayer
                       (const SecantDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// SecantDivisionLayer


SecantDivisionLayer::~SecantDivisionLayer()
{
}// ~SecantDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
SecantDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);

}// Draw

/* static */ std::auto_ptr<ResultLayer>
SecantDivisionLayer::Create(const PlotControl&     plotControl,
                                 const Result&          result,
                                 CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new SecantDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
SecantDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const SecantDivisionIteration& iteration =
        dynamic_cast<const SecantDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
SecantDivisionLayer::DrawIteration(wxDC&             dc,
                                        unsigned          iterationNumber,
                                        bool              highlight) const
{
    const SecantDivisionIteration& iteration =
        dynamic_cast<const SecantDivisionIteration&>
            (mResult.Iteration(iterationNumber));


    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if (!highlight) return;

    //TODO: Moe jednak zastosowa jako kolorki...
    //AttributeDatabase::AttributeType attr_left, attr_middle, attr_right;
    AttributeDatabase::AttributeType attr = AttributeDatabase::NORMAL;

    DrawBigPoint(dc, iteration.LeftBound(), attr);
    DrawBigPoint(dc, iteration.InsidePoint(), attr);
    DrawBigPoint(dc, iteration.RightBound(), attr);

}// DrawIteration

void
SecantDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const SecantDivisionIteration& iteration =
        dynamic_cast<const SecantDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ void* SecantDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            SecantDivisionIteration::ClassId(),
            SecantDivisionLayer::Create);
