/**Szablon panelu dla metod optymalizacji w kierunku polegajcych
 * na redukcji odcinka.
 *
 * Uniwersalny panel dla metod posiadajcych 3 parametry:
 * epsilon, pocztek zadanego odcinka, koniec zadanego odcinka.
 *
 * Panele dla poszczeglnych metod s rejestrowane
 * w SectionLineSearchPanel.cc.
 */

#ifndef SECTIONLINESEARCHPANEL_H
#define SECTIONLINESEARCHPANEL_H

#include "DoubleParameterValidator.h"
#include "EpsilonControl.h"
#include "InitialSectionControl.h"
#include "InitialSectionValidator.h"
#include "MethodPanelInfo.h"
#include "ui_util.h"

template <class T>
class SectionLineSearchPanel: public wxPanel
{
public:

    SectionLineSearchPanel(wxWindow*                parent,
                           int                      id,
                           T*                       pMethod,
                           bool                     standalone,
                           const wxPoint&           pos=wxDefaultPosition,
                           const wxSize&            size=wxDefaultSize,
                           long                     style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*         CreatePanel(MethodPanelInfo     panelInfo);


private:
    void do_layout();

protected:
    EpsilonControl* mpEpsilonControl;
    InitialSectionControl* mpInitialSectionControl;
    DoubleParameter mLeftBound;
    DoubleParameter mRightBound;
};

template <class T>
SectionLineSearchPanel<T>::SectionLineSearchPanel
                                (wxWindow*                  parent,
                                 int                        id,
                                 T*                         pMethod,
                                 bool                       standalone,
                                 const wxPoint&             pos,
                                 const wxSize&              size,
                                 long                       style)
    : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{

    // Podpanele.
    mpEpsilonControl = new EpsilonControl(this, -1, pMethod->mEpsilon);

    if (standalone)
    {
        mLeftBound  = pMethod->mInitialA;
        mRightBound = pMethod->mInitialB;
        mpInitialSectionControl = new InitialSectionControl(
                this, -1, mLeftBound, mRightBound);

        this->SetValidator(InitialSectionValidator(
                    dynamic_cast<SectionLineSearch*>(pMethod),
                    mLeftBound, mRightBound));
    }
    else
    {
        mpInitialSectionControl = NULL;
    }

    do_layout();

}// SectionLineSearchPanel

template <class T>
void SectionLineSearchPanel<T>::do_layout()
{
    wxBoxSizer* sizer_23 = new wxBoxSizer(wxVERTICAL);
    sizer_23->Add(mpEpsilonControl, 1, wxEXPAND, 0);
    if (mpInitialSectionControl != NULL)
        sizer_23->Add(mpInitialSectionControl, 1, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(sizer_23);
    sizer_23->Fit(this);
    sizer_23->SetSizeHints(this);
}

template <class T>
/* static */ wxPanel*
SectionLineSearchPanel<T>::CreatePanel(MethodPanelInfo   panelInfo)
{
    T* p_method = dynamic_cast<T*>(panelInfo.pMethod());
    bool standalone = panelInfo.PanelType() == MethodPanelInfo::STANDALONE;

    return new SectionLineSearchPanel<T>(panelInfo.pParent(), -1, p_method,
            standalone);

}// CreatePanel

#endif // SECTIONLINESEARCHPANEL_H
