#include <precomp.h>

#include "../Logger.h"
#include "../Symbols.h"

#include "DoubleParameterValidator.h"
#include "StopConditionControl.h"  // class implemented

LOGGER(INFO)

BEGIN_EVENT_TABLE(StopConditionControl, wxPanel)
    EVT_CHOICE(ID_CHOICE_STOPCONDITION,
               StopConditionControl::OnConditionSelected)
END_EVENT_TABLE()

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

StopConditionControl::StopConditionControl(
        wxWindow*          pParent,
        wxWindowID         id,
        const wxPoint&     pos,
        const wxSize&      size,
        long               style)
    : wxPanel(pParent, id, pos, size, wxTAB_TRAVERSAL)
{
    wxSizer* p_sizer_main = new wxStaticBoxSizer(wxVERTICAL, this,
            _("Stop Condition"));
    wxSizer* p_sizer_epsilon = new wxBoxSizer(wxHORIZONTAL);

    // Utwrz kontrolki
    mpConditionChoice = new ObjectChoice<StopConditionBase>(
            this, ID_CHOICE_STOPCONDITION);
    wxStaticText* p_label_epsilon = new wxStaticText(this, -1,
            Symbols::GetInstance()[Symbols::EPSILON] + wxString(wxT(":")));
    mpEpsilonControl = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition,
            wxDefaultSize, wxTE_RIGHT);

    // Powstawiaj do sizerw
    p_sizer_epsilon->Add(p_label_epsilon, 1, wxALL|wxALIGN_CENTER_VERTICAL, 2);
    p_sizer_epsilon->Add(mpEpsilonControl, 2, wxALL|wxALIGN_CENTER_VERTICAL, 2);

    p_sizer_main->Add(mpConditionChoice, 0, wxALL|wxEXPAND, 2);
    p_sizer_main->Add(p_sizer_epsilon, 0, wxEXPAND, 0);

    SetAutoLayout(true);
    SetSizer(p_sizer_main);
    p_sizer_main->Fit(this);
    p_sizer_main->SetSizeHints(this);

    Layout();

    // Walidatory
    mpEpsilonControl->SetValidator(DoubleParameterValidator(mEpsilon));

}// StopConditionControl

StopConditionControl::~StopConditionControl()
{
}// ~StopConditionControl


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void
StopConditionControl::Insert(CountedPtr<StopConditionBase> cpCondition)
{
    // Ze wzgldu na fakt, e kolejno obliczania wyrae jest nieokrelona nie
    // mona uy apCondition->Name() w wywoaniu Insert.
    const wxString& name = cpCondition->Name();

    mpConditionChoice->Insert(name, cpCondition);
}// Insert

void
StopConditionControl::SetList(StopConditions conditions)
{
    CountedPtr<StopConditionBase> cp_condition;
    StopConditions::const_iterator iter;

    for (iter = conditions.begin(); iter != conditions.end(); iter++)
    {
        cp_condition = *iter;
        const wxString& name = cp_condition->Name();
        mpConditionChoice->Insert(name, cp_condition);
        LOG(DEBUG, name);
    }

    // TODO: moe wywoywa SetList z konstruktora
    // zamiast z panelu?
    CountedPtr<StopConditionBase> p_selected =
            mpConditionChoice->GetSelected();
    mEpsilon = p_selected->Epsilon();
}

std::auto_ptr<StopConditionBase>
StopConditionControl::CreateSelected()
{
    // Pobierz warto z okienka "epsilon".
    if (Validate() && TransferDataFromWindow())
    {
        CountedPtr<StopConditionBase> p_selected =
            mpConditionChoice->GetSelected();
        p_selected->rEpsilon() = mEpsilon;
        return p_selected->Clone();
    }
    else
    {
        // TODO: wyjtek w tym miejscu albo zabezpieczenie przed tak sytuacj
        assert(false);
        abort();
    }
}// CreateSelected

int
StopConditionControl::SelectName(const wxString& name)
{
    int position = mpConditionChoice->FindString(name);

    if (position != wxNOT_FOUND) mpConditionChoice->SetSelection(position);

    return position;
}// SelectName

void StopConditionControl::OnConditionSelected(wxCommandEvent& event)
{
    CountedPtr<StopConditionBase> p_selected =
        mpConditionChoice->GetSelected();

    mEpsilon = p_selected->Epsilon();
    TransferDataToWindow();
}// OnConditionSelected

//============================= ACCESS     ===================================

DoubleParameter&
StopConditionControl::rEpsilon()
{
    return mEpsilon;
}// rEpsilon

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
