/** Kontrolka wyboru warunku stopu.
 *
 * Pozwala wybra jeden z utworzonych wczeniej (patrz Insert) warunkw stopu
 * oraz ustawi dla niego warto <em>epsilon</em>.
 */

#ifndef _StopConditionControl_h
#define _StopConditionControl_h

#define ID_CHOICE_STOPCONDITION 1
// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../StopConditionBase.h"

#include "ObjectChoice.h"

// FORWARD REFERENCES
//


class StopConditionControl
    : public wxPanel
{
public:
// LIFECYCLE

    /** Konstruktor.
     */
    StopConditionControl(wxWindow*          pParent,
                         wxWindowID         id,
                         const wxPoint&     pos = wxDefaultPosition,
                         const wxSize&      size = wxDefaultSize,
                         long               style = 0);

    /** Destruktor.
     */
    ~StopConditionControl(void);


// OPERATORS

// OPERATIONS

    /** Dodanie warunku stopu do kontrolki.
     */
    void        Insert(CountedPtr<StopConditionBase> cpCondition);

    /** Dodanie listy warunkw stopu do kontrolki.
     */
    void        SetList(StopConditions conditions);

    /** Tworzy warunek stopu na postawie wyboru uytkownika.
     *
     * Tworzony jest (przez klonowanie) obiekt warunku stopu, a na podstawie
     * zawartoci pola liczbowego ustawiany jest parametr <em>epsilon</em>.
     */
    std::auto_ptr<StopConditionBase>    CreateSelected();

    /** Ustawia zaznaczenie kontrolki na pozycj, ktra odpowiada podanej
     * nazwie.
     *
     * @return Pozycja na licie (liczc od 0) albo <code>wxNOT_FOUND</code>
     */
    int         SelectName(const wxString& name);

    void        OnConditionSelected(wxCommandEvent& event);

// ACCESS

    /** Zwraca referencj na parametr <em>epsilon</em>.
     */
    DoubleParameter& rEpsilon();

// INQUIRY

protected:
private:
    DoubleParameter mEpsilon;
    ObjectChoice<StopConditionBase>*    mpConditionChoice;
    wxTextCtrl*                             mpEpsilonControl;

    DECLARE_EVENT_TABLE();
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _StopConditionControl_h
