#include <precomp.h>

#include "../util.h"
#include "../Symbols.h"
#include "AttributeDatabase.h"
#include "MatrixStyle.h"
#include "MultilineString.h"

#include "TextIterationVisualizer.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TextIterationVisualizer::TextIterationVisualizer()
{
}// TextIterationVisualizer

TextIterationVisualizer::~TextIterationVisualizer()
{
}// ~TextIterationVisualizer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================



//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TextIterationVisualizer::PrintMatrix(const Matrix&      matrix) const
{
    MultilineString multiline;
    MatrixStyle style(matrix);
    wxString string;

    multiline.Append(matrix, style);
    string << multiline.ToString();

    return string;
}// PrintMatrix

wxString
TextIterationVisualizer::PrintMatrix(const Matrix&      matrix,
                                     const wxString&    text) const
{
    MultilineString multiline;
    MatrixStyle style(matrix);
    wxString string;

    multiline.Append(text);
    multiline.Append(matrix, style);
    string << multiline.ToString();

    return string;
}// PrintMatrix

wxString
TextIterationVisualizer::PrintDirectionMatrix(
                            wxTextCtrl*                 pTargetCtrl,
                            const SquareMatrix&         matrix,
                            bool                        highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    return PrintTwoColumnMatrix(pTargetCtrl,
                                matrix,
                                attributes.TextAttribute(
                                    AttributeDatabase::STEP_TRIAL_D1),
                                attributes.TextAttribute(
                                    AttributeDatabase::STEP_TRIAL_D2),
                                highlight);
}// PrintDirectionMatrix

wxString
TextIterationVisualizer::PrintDirectionMatrix(
                            wxTextCtrl*                 pTargetCtrl,
                            const SquareMatrix&         matrix,
                            const wxString&             text,
                            bool                        highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    return PrintTwoColumnMatrix(pTargetCtrl,
                                matrix,
                                attributes.TextAttribute(
                                    AttributeDatabase::STEP_TRIAL_D1),
                                attributes.TextAttribute(
                                    AttributeDatabase::STEP_TRIAL_D2),
                                text,
                                highlight);
}// PrintDirectionMatrix

wxString
TextIterationVisualizer::PrintTwoColumnMatrix(
                            wxTextCtrl*                 pTargetCtrl,
                            const SquareMatrix&         matrix,
                            const wxTextAttr&           firstColumnAttr,
                            const wxTextAttr&           secondColumnAttr,
                            bool                        highlight) const
{
    wxString string;

    if (highlight)
    {
        MultilineString multiline;
        MatrixStyle style(matrix);

        style.SetColumnStyle(1, firstColumnAttr);
        style.SetColumnStyle(2, secondColumnAttr);

        multiline.Append(matrix, style);
        multiline.Print(pTargetCtrl);
    }
    else
    {
        string << PrintMatrix(matrix);
    }

    return string;
}// PrintTwoColumnMatrix

wxString
TextIterationVisualizer::PrintTwoColumnMatrix(
                            wxTextCtrl*                 pTargetCtrl,
                            const SquareMatrix&         matrix,
                            const wxTextAttr&           firstColumnAttr,
                            const wxTextAttr&           secondColumnAttr,
                            const wxString&             text,
                            bool                        highlight) const
{
    wxString string;

    if (highlight)
    {
        MultilineString multiline;
        MatrixStyle style(matrix);

        style.SetColumnStyle(1, firstColumnAttr);
        style.SetColumnStyle(2, secondColumnAttr);

        multiline.Append(text);
        multiline.Append(matrix, style);
        multiline.Print(pTargetCtrl);
    }
    else
    {
        string << PrintMatrix(matrix, text);
    }

    return string;
}// PrintTwoColumnMatrix


void
TextIterationVisualizer::Flush(wxTextCtrl*      pTargetCtrl,
                               wxString&        text,
                               bool             use) const
{
    if (use && (text.Length() > 0))
    {
        pTargetCtrl->AppendText(text);
        text.Clear();
    }
}// Flush

void
TextIterationVisualizer::SetAttribute(wxTextCtrl*       pTargetCtrl,
                                      const wxTextAttr& attribute,
                                      bool              use) const
{
    if (use)    pTargetCtrl->SetDefaultStyle(attribute);
}// SetAttribute

void
TextIterationVisualizer::SetAttributeAndFlush(wxTextCtrl*       pTargetCtrl,
                                              wxString&         text,
                                              const wxTextAttr& attribute,
                                              bool              use) const
{
    if (use)
    {
        Flush(pTargetCtrl, text, true);
        pTargetCtrl->SetDefaultStyle(attribute);
    }
}// SetAttribute

/////////////////////////////// PRIVATE    ///////////////////////////////////
