/** Klasa bazowa dla klas drukujcych iteracje poszczeglnych metod.
 *
 * @see IterationData, TextVisualizer
 */

#ifndef _TextIterationVisualizer_h
#define _TextIterationVisualizer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../FunctionBase.h"
#include "../IterationData.h"

// FORWARD REFERENCES
//


class TextIterationVisualizer
{
public:

    /// Wskanik na metod fabryki obiektu danej klasy.
    typedef std::auto_ptr<const TextIterationVisualizer>(*Creator)();

// LIFECYCLE

    /** Konstruktor domylny.
     */
    TextIterationVisualizer(void);


    /** Destruktor.
     */
    virtual ~TextIterationVisualizer(void);


// OPERATORS

// OPERATIONS

    /** Wypisuje dane iteracji do kontrolki tekstowej.
     *
     * Ostatni wiersz nie zawiera znaku koca linii.
     *
     * @param pTargetCtrl Kontrolka, do ktrej drukowany jest tekst.
     * @param iterationData Obiekt reprezentujcy dane iteracji.
     * @param function Funkcja celu (bez kar itd.).
     * @param highlight Czy jest to zaznaczona iteracja (np. na wykresie) i czy
     * powinna by odpowiednio kolorowana.
     */
    virtual void        Print(wxTextCtrl*           pTargetCtrl,
                              const IterationData&  iterationData,
                              const FunctionBase&   function,
                              bool                  highlight) const = 0;


// ACCESS
// INQUIRY

protected:

    wxString        PrintMatrix(const Matrix&               matrix) const;

    wxString        PrintMatrix(const Matrix&               matrix,
                                const wxString&             text) const;


    wxString        PrintDirectionMatrix(wxTextCtrl*            pTargetCtrl,
                                         const SquareMatrix&    matrix,
                                         bool                   highlight)
                                                                    const;

    wxString        PrintDirectionMatrix(wxTextCtrl*            pTargetCtrl,
                                        const SquareMatrix&     matrix,
                                        const wxString&         text,
                                        bool                    highlight)
                                                                    const;

    wxString        PrintTwoColumnMatrix(wxTextCtrl*          pTargetCtrl,
                                         const SquareMatrix&  matrix,
                                         const wxTextAttr&    firstColumnAttr,
                                         const wxTextAttr&    secondColumnAttr,
                                         bool                 highlight) const;

    wxString        PrintTwoColumnMatrix(wxTextCtrl*          pTargetCtrl,
                                         const SquareMatrix&  matrix,
                                         const wxTextAttr&    firstColumnAttr,
                                         const wxTextAttr&    secondColumnAttr,
                                         const wxString&      text,
                                         bool                 highlight) const;

    void    Flush(wxTextCtrl*       pTargetCtrl,
                  wxString&         text,
                  bool              use = true) const;


    void    SetAttribute(wxTextCtrl*        pTargetCtrl,
                         const wxTextAttr&  attribute,
                         bool               use) const;

    void    SetAttributeAndFlush(wxTextCtrl*        pTargetCtrl,
                                 wxString&          text,
                                 const wxTextAttr&  attribute,
                                 bool               use) const;

private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _TextIterationVisualizer_h
