/** Tekstowy wizualizator wynikw.
 *
 * Wynik przedstawiany jest w postaci tekstowej, kolejno po sobie wydrukowane s
 * informacje dotyczce dziaania w kolejnych iteracjach metody.
 *
 * Przebieg optymalizacji jest poprzedzony danymi zadania optymalizacji oraz
 * wybranej metody.
 *
 * Podawane jest rwnie podsumowanie zawierajce liczb iteracji, liczb
 * wylicze funkcji oraz otrzymane minimum.
 *
 * Kolejne iteracje s drukowane za pomoc obiektu klasy
 * TextIterationVisualizer.
 *
 * @see Visualizer, TextIterationVisualizer
 */

#ifndef _TextVisualizer_h
#define _TextVisualizer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "TextIterationVisualizer.h"
#include "Visualizer.h"

// FORWARD REFERENCES
//


class TextVisualizer
    : public Visualizer
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param pParent Okno rodzica.
     * @param result Referencja na wynik do przedstawienia.
     * @param apIterationVisualizer <code>auto_ptr</code> na wizualizator
     * iteracji. Moe by na <code>NULL</code> jeli wynik nie posiada ani
     * jednej iteracji.
     */
    TextVisualizer(wxWindow*                               pParent,
                   const Result&                           result,
                   auto_ptr<const TextIterationVisualizer> apIterationVisualizer);


    /** Destruktor.
     */
    ~TextVisualizer(void);


// OPERATORS

// OPERATIONS
// ACCESS

    virtual wxString            Name() const;

// INQUIRY

protected:
    /** Drukuje cay tekst wizualizacji.
     */
    void        PrintResult(wxTextCtrl* pTargetCtrl);

    /** Drukuje jedn lini nagwka.
     *
     * Tekst pisany atrybutem nagwka, pusty wiersz odstpu po tekcie.
     */
    void        PrintHeader(wxTextCtrl* pTargetCtrl, const wxString& text);

private:
    /// Wizualizator
    auto_ptr<const TextIterationVisualizer> mapIterationVisualizer;

    /// Kontrolka tekstowa, w ktrej wywietlany jest wynik.
    wxTextCtrl* mpTextCtrl;

    // TODO: legenda?

    // Nie moe by statyczna, gdy musi by inicjowana po zainicjowaiu wx'w
    const wxTextAttr        mNormalAttribute;    ///< Atrybut tekstu zwykego.
    const wxTextAttr        mHeaderAttribute;    ///< Atrybut tekstu nagwkw.

    const static wxString   mName;          ///< Nazwa wizualizatora.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _TextVisualizer_h
