#include <precomp.h>

#include "../methods/AlphaDivisionIteration.h"
#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivAlphaDivision.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivAlphaDivision::TivAlphaDivision()
{
}// TivAlphaDivision

TivAlphaDivision::~TivAlphaDivision()
{
}// ~TivAlphaDivision

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivAlphaDivision::Print(wxTextCtrl*             pTargetCtrl,
                        const IterationData&    iterationData,
                        const FunctionBase&     function,
                        bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const Symbols&           symbols    = Symbols::GetInstance();
    const UnivariateFunction& f =
        dynamic_cast<const UnivariateFunction&>(function);
    const AlphaDivisionIteration& iteration =
        dynamic_cast<const AlphaDivisionIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Current section:\n")
         << PrintSection(iteration.LeftBound(), iteration.RightBound(),
                 iteration.Length())
         << wxT('\n')
         << _("Trial points:")
         << wxT('\n');

    double value_lambda = f.Evaluate(iteration.LeftStep());
    double value_mu     = f.Evaluate(iteration.RightStep());

    wxTextAttr attr_lambda;
    wxTextAttr attr_mu;

    if (iteration.IsMuBetter())
    {
        attr_lambda = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        attr_mu     = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    }
    else
    {
        attr_lambda = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        attr_mu     = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    }

    SetAttributeAndFlush(pTargetCtrl, text, attr_lambda, highlight);
    text << PrintDoubleWithValue(Symbols::GetInstance()[Symbols::LAMBDA],
                iteration.LeftStep(), value_lambda)
         << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl, text, attr_mu, highlight);
    text << PrintDoubleWithValue(Symbols::GetInstance()[Symbols::MU],
            iteration.RightStep(), value_mu)
         << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl, text,
            attributes.TextAttribute(AttributeDatabase::DEFAULT), highlight);

    text << _("New section:");
    if (iteration.IsMuBetter())
    {
        text << PrintCustomSection(symbols[Symbols::LAMBDA],
                                   wxT("b"),
                                   iteration.LeftStep(),
                                   iteration.RightBound(),
                                   true);
    }
    else
    {
        text << PrintCustomSection(wxT("a"),
                                   symbols[Symbols::MU],
                                   iteration.LeftBound(),
                                   iteration.RightStep(),
                                   true);
    }

    Flush(pTargetCtrl, text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivAlphaDivision::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivAlphaDivision);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivAlphaDivision::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            AlphaDivisionIteration::ClassId(),
            TivAlphaDivision::Create);

