#include <precomp.h>

#include "../methods/BolzanoDivisionIteration.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivBolzanoDivision.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivBolzanoDivision::TivBolzanoDivision()
{
}// TivBolzanoDivision

TivBolzanoDivision::~TivBolzanoDivision()
{
}// ~TivBolzanoDivision

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivBolzanoDivision::Print(wxTextCtrl*             pTargetCtrl,
                              const IterationData&    iterationData,
                              const FunctionBase&     function,
                              bool                    highlight) const
{

    assert (pTargetCtrl != NULL);
    const BolzanoDivisionIteration& iteration =
        dynamic_cast<const BolzanoDivisionIteration&>(iterationData);
    wxString text;
    ColumnVector point(1);

    pTargetCtrl->Freeze();

    text << _("Current section:\n");
    text << PrintSection(iteration.LeftBound(),
                         iteration.RightBound(),
                         function,
                         iteration.Length());
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintPoints(pTargetCtrl, iteration, function, highlight);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintNewSection(pTargetCtrl, iteration, function, highlight);


    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivBolzanoDivision::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivBolzanoDivision);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivBolzanoDivision::PrintPoints
                     (wxTextCtrl*                           pTargetCtrl,
                      const BolzanoDivisionIteration&       iteration,
                      const FunctionBase&                   function,
                      bool                                  highlight) const
{

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    //wxTextAttr  attr_left, attr_middle, attr_right;
    wxTextAttr  attr_middle
                = attributes.TextAttribute(AttributeDatabase::NORMAL);
    wxString    text;

    text << _("Middle point:") << wxT("\n");

    //attr_middle = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
/*
    if (iteration.HalfChosen() == BolzanoDivisionIteration::LEFT)
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        attr_right  = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    }
    else
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        attr_right  = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    }
*/

    SetAttributeAndFlush(pTargetCtrl, text, attr_middle, highlight);
    text << PrintDoubleWithValue(wxString(wxT("m")),
                                 iteration.Middle(),
                                 function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintDerivativeInPoint(wxString(wxT("m")),
                                   iteration.Middle(),
                                   function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    return text;
}

wxString
TivBolzanoDivision::PrintNewSection
                     (wxTextCtrl*                       pTargetCtrl,
                      const BolzanoDivisionIteration&   iteration,
                      const FunctionBase&               function,
                      bool                              highlight) const
{
    wxString    text;

    text << _("Section chosen for next iteration") << wxT(":\n");

    switch (iteration.HalfChosen())
    {
        case BolzanoDivisionIteration::LEFT :
                text << _("Left half of current section") << wxT("\n");
                text << PrintCustomSection(wxString(wxT("a")),
                                           wxString(wxT("m")),
                                           iteration.LeftBound(),
                                           iteration.Middle(),
                                           function,
                                           true);
                break;
        case BolzanoDivisionIteration::RIGHT :
                text << _("Right half of current section") << wxT("\n");
                text << PrintCustomSection(wxString(wxT("m")),
                                           wxString(wxT("b")),
                                           iteration.Middle(),
                                           iteration.RightBound(),
                                           function,
                                           true);
                break;
        case BolzanoDivisionIteration::NONE :
                text << _("None, middle point is the minimum.");
                break;
        default: assert(false);
                 break;
    }

    text << wxT("\n");

    Flush(pTargetCtrl, text, highlight);

    return  text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivBolzanoDivision::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            BolzanoDivisionIteration::ClassId(),
            TivBolzanoDivision::Create);

