#include <precomp.h>

#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivConjugateGradientsMethod.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivConjugateGradientsMethod::TivConjugateGradientsMethod()
{
}// TivConjugateGradientsMethod

TivConjugateGradientsMethod::~TivConjugateGradientsMethod()
{
}// TivConjugateGradientsMethod

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ void
TivConjugateGradientsMethod::Print(      wxTextCtrl*       pTargetCtrl,
                                   const IterationData&    iterationData,
                                   const FunctionBase&     function,
                                         bool              highlight) const
{

    assert (pTargetCtrl != NULL);
    Symbols symbols = Symbols::GetInstance();
    const ConjugateGradientsMethodIteration& iteration =
        dynamic_cast<const ConjugateGradientsMethodIteration&>(iterationData);
    wxString text;
    ColumnVector point(1);

    pTargetCtrl->Freeze();

    text << _("Base point") << wxT(": ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.StartingPoint(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintGradient(wxString::Format(wxT("x")),
                          iteration.Gradient());
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    if (!iteration.DirectionReset())
        text << symbols[Symbols::GAMMA]
             << wxString::Format(wxT(" = %10.8g\n"), iteration.Gamma());
    else
        text << _("Direction set to -grad f(x).") << wxT("\n");

    text << _("Direction:\td = ")
         << iteration.Direction();

    text << wxT('\n')
         << _("Step:\t")
         << PrintOptimalStep(iteration.Tau(), iteration.EndPoint(),
                 function);

    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivConjugateGradientsMethod::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivConjugateGradientsMethod);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivConjugateGradientsMethod::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            ConjugateGradientsMethodIteration::ClassId(),
            TivConjugateGradientsMethod::Create);

