#include <precomp.h>

#include "../ConstrainedOptimizationIteration.h"
#include "../Logger.h"
#include "../Symbols.h"
#include "../text-util.h"
#include "AttributeDatabase.h"
#include "TivConstrainedOptimization.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"


LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivConstrainedOptimization::TivConstrainedOptimization()
{
}// TivConstrainedOptimization

TivConstrainedOptimization::~TivConstrainedOptimization()
{
}// ~TivConstrainedOptimization

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ void
TivConstrainedOptimization::Print(      wxTextCtrl*      pTargetCtrl,
                                  const IterationData&   iterationData,
                                  const FunctionBase&    function,
                                        bool             highlight) const
{
    assert (pTargetCtrl != NULL);

    const ConstrainedOptimizationIteration& iteration =
        dynamic_cast<const ConstrainedOptimizationIteration&>(iterationData);

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxTextAttr  attribute = attributes.TextAttribute(AttributeDatabase::NORMAL);

    wxString text;

    pTargetCtrl->Freeze();

    text << _("New approximation:") << wxT('\t');
    text << PrintNewApproximation(pTargetCtrl,
                                  iterationData,
                                  function,
                                  highlight);
    text << wxT('\n');

    if (highlight)
    {
        if (iteration.IsNewApproximationValid())
        {
            attribute = attributes.TextAttribute
                                    (AttributeDatabase::POINT_BEST);
        }
        else
        {
            attribute = attributes.TextAttribute
                                    (AttributeDatabase::POINT_OUT_OF_BOUNDS);
        }
    }
    SetAttributeAndFlush(pTargetCtrl, text, attribute, highlight);

    text << (iteration.IsNewApproximationValid() ?
            _("New approximation respects constraints.") :
            _("New approximation violates constraints."));

    SetAttributeAndFlush(pTargetCtrl, text, AttributeDatabase::GetInstance().
            TextAttribute(AttributeDatabase::NORMAL), highlight);

    text << wxT("\n\n");
    text << wxString::Format(_("Number of function evaluations:\t%d\n"
                               "Number of method iterations:\t%d"),
                             iteration.EvaluationCount(),
                             iteration.IterationCount());

    Flush(pTargetCtrl, text);

    pTargetCtrl->Thaw();
}// Print

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
