#include <precomp.h>

#include "../methods/DFPIteration.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivDFP.h"  // class implemented
#include "ui_util.h"
#include "VisualizerFactory.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivDFP::TivDFP()
{
}// TivDFP

TivDFP::~TivDFP()
{
}// ~TivDFP

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivDFP::Print(wxTextCtrl*             pTargetCtrl,
              const IterationData&    iterationData,
              const FunctionBase&     function,
              bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const DFPIteration& iteration =
        dynamic_cast<const DFPIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Base point:\t")
         << PrintPointWithValueAndGradient(wxT("x"), iteration.StartingPoint(),
                 iteration.Gradient(), function)
         << wxT('\n')
         << PrintMatrix(iteration.D(),
                        _("Approximation of inverted hessian (D): "))
         << wxT("\n\n");

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute(
                             AttributeDatabase::STEP_IN_DIRECTION),
                         highlight && iteration.StepMade());
   text  << _("Direction:\t d = ")
         << iteration.Direction();

    if (iteration.StepMade())
    {
       text << wxT('\n')
            << _("Step:\t")
            << PrintOptimalStep(iteration.Tau(), iteration.EndPoint(),
                    function)
            << wxT('\n');

       SetAttributeAndFlush(pTargetCtrl,
                    text,
                    attributes.TextAttribute(AttributeDatabase::DEFAULT),
                    highlight);
       text << _("Gradient in new point:\t")
            << PrintGradient(wxT("x"), iteration.GradientInEndPoint())
            << wxT("\n\n");

       if (iteration.BackToIdentityMatrix())
       {
           text << _("Approximation matrix reset:")
                << wxT('\n')
                << PrintMatrix(iteration.NewD(), _("New D: "));
       }
       else
       {
           text << _("Next approximation:")
                << wxT('\n')
                << _("Point difference: \t p = ")
                << iteration.PointDifference()
                << wxT('\n')
                << _("Gradient difference: \t q = ")
                << iteration.GradientDifference()
                << wxT('\n');
           if (iteration.NewD().ncols() != 0)
           {
                text << PrintMatrix(iteration.NewD(), _("New D: "))
                     << wxT("\n\n")
                     << _("New direction:\t d = ")
                     << iteration.NewDirection();
           }
       }

    }// StepMade if

    pTargetCtrl->AppendText(text);

    SetAttribute(pTargetCtrl,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight);
    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivDFP::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivDFP);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivDFP::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            DFPIteration::ClassId(),
            TivDFP::Create);

