#include <precomp.h>

#include "../methods/DerivativeIteration.h"
#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivDerivative.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivDerivative::TivDerivative()
{
}// TivDerivative

TivDerivative::~TivDerivative()
{
}// TivDerivative

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivDerivative::Print(      wxTextCtrl*       pTargetCtrl,
                     const IterationData&    iterationData,
                     const FunctionBase&     function,
                           bool              highlight) const
{

    assert (pTargetCtrl != NULL);
    const DerivativeIteration& iteration =
        dynamic_cast<const DerivativeIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Iteration initial point") << wxT(":\n");
    text << PrintDoubleWithValue(wxString::Format(wxT("x")),
                                 iteration.StartingPoint(),
                                 function);
    text << wxT("\n");
    text << PrintDerivativeInPoint(wxString(_("x")),
                                   iteration.StartingPoint(),
                                   function);
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);


    text << PrintTrialPoints(pTargetCtrl, iteration, function, highlight);
    text << wxT("\n");


    text << _("New point") << wxT(":\n");
    text << PrintDoubleWithValue(wxString::Format(wxT("x")),
                                 iteration.TrialPoints().back(),
                                 function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);



    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivDerivative::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivDerivative);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivDerivative::PrintTrialPoints
                     (wxTextCtrl*                           pTargetCtrl,
                      const DerivativeIteration&   iteration,
                      const FunctionBase&                   function,
                      bool                                  highlight) const
{

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    Symbols symbols = Symbols::GetInstance();
    wxTextAttr  attr_rejected, attr_accepted, attr_normal;
    wxString    text;
    double      alpha = iteration.FirstAlpha();
    double      gamma = iteration.Gamma();
    size_t      i;


    attr_rejected = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    attr_accepted = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    attr_normal   = attributes.TextAttribute(AttributeDatabase::NORMAL);

    text << _("Trial points:") << wxT("\n");

    const std::vector<double>&  trial_points = iteration.TrialPoints();

    for (i = 0; i < (trial_points.size() - 1); i++)
    {
        SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);
        text << symbols[Symbols::ALPHA] << wxT(" = ");
        text << wxString::Format(wxT("%10.8g"), alpha) << wxT(" -> ");
        SetAttributeAndFlush(pTargetCtrl, text, attr_rejected, highlight);
        text << PrintDoubleWithValue(wxString::Format(wxT("t%d"), i),
                                     trial_points[i],
                                     function);
        text << wxT('\t') << _("- rejected") << wxT("\n");

        alpha *= gamma;
    }

    SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);
    text << symbols[Symbols::ALPHA] << wxT(" = ");
        text << wxString::Format(wxT("%10.8g"), alpha) << wxT(" -> ");
    SetAttributeAndFlush(pTargetCtrl, text, attr_accepted, highlight);
    text << PrintDoubleWithValue(wxString::Format(wxT("t%d"), i),
                                 trial_points[i],
                                 function);
    text << wxT('\t') << _("- accepted") << wxT("\n");

    SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);

    //Flush(pTargetCtrl, text, highlight);

    return text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivDerivative::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            DerivativeIteration::ClassId(),
            TivDerivative::Create);

