#include <precomp.h>

#include "../methods/FibonacciDivisionIteration.h"
#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivFibonacciDivision.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivFibonacciDivision::TivFibonacciDivision()
{
}// TivFibonacciDivision

TivFibonacciDivision::~TivFibonacciDivision()
{
}// ~TivFibonacciDivision

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivFibonacciDivision::Print(      wxTextCtrl*       pTargetCtrl,
                            const IterationData&    iterationData,
                            const FunctionBase&     function,
                                  bool              highlight) const
{

    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const Symbols&           symbols    = Symbols::GetInstance();
    const UnivariateFunction& f =
        dynamic_cast<const UnivariateFunction&>(function);
    const FibonacciDivisionIteration& iteration =
        dynamic_cast<const FibonacciDivisionIteration&>(iterationData);
    wxString text, string_lambda, string_mu;

    pTargetCtrl->Freeze();

    text << _("Current section:\n")
         << PrintSection(iteration.LeftBound(), iteration.RightBound(),
                 iteration.Length())
         << wxT("\n\n")
         << _("Trial points:")
         << wxT('\n');

    double lambda           = iteration.Lambda();
    double mu               = iteration.Mu();
    double value_in_lambda  = f.Evaluate(lambda);
    double value_in_mu      = f.Evaluate(mu);

    wxTextAttr attr_lambda;
    wxTextAttr attr_mu;

    if (value_in_lambda > value_in_mu)
    {
        attr_lambda = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        attr_mu     = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    }
    else
    {
        attr_lambda = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        attr_mu     = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    }

    double temp_coefficient = iteration.LambdaCoefficient();
    SetAttributeAndFlush(pTargetCtrl, text, attr_lambda, highlight);

    if (!iteration.Last())
    {
        text << wxT('|')
             << symbols[Symbols::LAMBDA]
             << wxT(" - a|/|b - a| = ")
             << wxString::Format(wxT("%12.7g"), temp_coefficient)
             << wxT('\n');
    }

    text << PrintDoubleWithValue(Symbols::GetInstance()[Symbols::LAMBDA],
                                 lambda,
                                 value_in_lambda)
         << wxT('\n');

    temp_coefficient = iteration.MuCoefficient();
    SetAttributeAndFlush(pTargetCtrl, text, attr_mu, highlight);

    if (!iteration.Last())
    {
        text << wxT('|')
             << symbols[Symbols::MU]
             << wxT(" - a|/|b - a| = ")
             << wxString::Format(wxT("%12.7g"), temp_coefficient)
             << wxT('\n');
    }

    text << PrintDoubleWithValue(Symbols::GetInstance()[Symbols::MU],
                                 mu,
                                 value_in_mu)
         << wxT("\n\n");

    SetAttributeAndFlush(pTargetCtrl, text,
            attributes.TextAttribute(AttributeDatabase::DEFAULT), highlight);

    text << _("New section:");
    switch (iteration.NextRightBound())
    {
        case FibonacciDivisionIteration::B :
            text << PrintCustomSection(symbols[Symbols::LAMBDA],
                                       wxT("b"),
                                       iteration.Lambda(),
                                       iteration.RightBound(),
                                       true);
            break;
        case FibonacciDivisionIteration::MU :
            text << PrintCustomSection(wxT("a"),
                                       symbols[Symbols::MU],
                                       iteration.LeftBound(),
                                       iteration.Mu(),
                                       true);
            break;
        case FibonacciDivisionIteration::LAMBDA :
            text << PrintCustomSection(wxT("a"),
                                       symbols[Symbols::LAMBDA],
                                       iteration.LeftBound(),
                                       iteration.Lambda(),
                                       true);
            break;
        default:
            assert(false);
            break;

    }

    Flush(pTargetCtrl, text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivFibonacciDivision::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivFibonacciDivision);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivFibonacciDivision::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            FibonacciDivisionIteration::ClassId(),
            TivFibonacciDivision::Create);

