#include <precomp.h>

#include "../methods/HookeJeevesIteration.h"
#include "../Logger.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivHookeJeeves.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivHookeJeeves::TivHookeJeeves()
{
}// TivHookeJeeves

TivHookeJeeves::~TivHookeJeeves()
{
}// ~TivHookeJeeves


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
TivHookeJeeves::Print(wxTextCtrl*             pTargetCtrl,
                      const IterationData&    iterationData,
                      const FunctionBase&     function,
                      bool                    highlight) const
{
    assert (pTargetCtrl != NULL);

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    const HookeJeevesIteration& iteration =
        dynamic_cast<const HookeJeevesIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Base point: ")
         << PrintPointWithValue(iteration.OldBasePoint(), function)
         << wxT('\n')
         << PrintPointWithValue(wxT("x"), iteration.StartingPoint(),function)
         << wxT("\n\n");

    // TODO: to by mona byo przenie do PrintTrialSteps
    Flush(pTargetCtrl, text);

    PrintTrialSteps(pTargetCtrl, iteration, function, highlight);

    if (iteration.IsStepCancelled())
    {
        SetAttribute(pTargetCtrl,
                attributes.TextAttribute(AttributeDatabase::DEFAULT),
                highlight);
        text  = _("No better value.");
        text << wxT('\n');

        SetAttributeAndFlush(pTargetCtrl,
                text,
                attributes.TextAttribute(AttributeDatabase::POINT_BASE),
                highlight);
        text << _("Moving back to base point: ");
        text << PrintPointWithValue(wxT("x"),  iteration.EndPoint(), function)
             << wxT('\n');

        SetAttributeAndFlush(pTargetCtrl,
                text,
                attributes.TextAttribute(AttributeDatabase::DEFAULT),
                highlight);

        text << PrintTauContraction(iteration);
    }
    else
    {
        SetAttribute(pTargetCtrl,
                attributes.TextAttribute(AttributeDatabase::POINT_BASE),
                highlight);
        text  = _("New base point: ");
        text << PrintPoint(iteration.BasePoint()) << wxT("\n");

        StepVector(pTargetCtrl, text, iteration, highlight);

        SetAttributeAndFlush(pTargetCtrl,
                text,
                attributes.TextAttribute(AttributeDatabase::STEP_IN_DIRECTION),
                highlight);
        text << _("Step to new point: ");
        text << PrintPointWithValue(wxT('x'), iteration.EndPoint(),function);

        SetAttributeAndFlush(pTargetCtrl,
                text,
                attributes.TextAttribute(AttributeDatabase::DEFAULT),
                highlight);
    }

    Flush(pTargetCtrl, text);
    pTargetCtrl->Thaw();
}// Print

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
TivHookeJeeves::PrintTrialSteps(
        wxTextCtrl*                 pTargetCtrl,
        const HookeJeevesIteration& iteration,
        const FunctionBase&         function,
        bool                        highlight) const
{
    wxString text;
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    HookeJeevesIteration::TrialPointContainer::const_iterator i;
    const HookeJeevesIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    text  = _("Trial steps:");
    text << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl,
            text,
            attributes.TextAttribute(AttributeDatabase::STEP_TRIAL_D1),
            highlight);
    unsigned step_no = 1;
    for (i = trial_steps.begin(); i != trial_steps.end(); ++i, ++step_no)
    {
        text << PrintTrialStep(*i, function);

        // Dwa kroki na kierunek, chyba e pierwszy by udany, kolejne 1-2 kroki
        // s w drugim kierunku.
        if (i->mBetter || (step_no == 2))
        {
            SetAttributeAndFlush(pTargetCtrl,
                    text,
                    attributes.TextAttribute(AttributeDatabase::STEP_TRIAL_D2),
                    highlight);
        }
    }

    Flush(pTargetCtrl, text);
}// PrintTrialSteps

/////////////////////////////// PRIVATE    ///////////////////////////////////
