#include <precomp.h>

#include "../methods/HookeJeevesDiscreteIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivHookeJeevesDiscrete.h"  // class implemented
#include "VisualizerFactory.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivHookeJeevesDiscrete::TivHookeJeevesDiscrete()
{
}// TivHookeJeevesDiscrete

TivHookeJeevesDiscrete::~TivHookeJeevesDiscrete()
{
}// ~TivHookeJeevesDiscrete


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivHookeJeevesDiscrete::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivHookeJeevesDiscrete);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/* virtual */ wxString
TivHookeJeevesDiscrete::PrintTrialStep(
        const HookeJeevesIteration::TrialPoint& step,
        const FunctionBase&                     function) const
{
    wxString text;

    text  = wxT("  ");
    text << PrintPointWithValue(step.mPoint, function)
        << wxT('\t');
    if (step.mBetter)   text << _("- accepted");
    else                text << _("- rejected");
    text << wxT('\n');

    return text;
}// PrintTrialStep

/* virtual */ void
TivHookeJeevesDiscrete::StepVector(wxTextCtrl*                  pTargetCtrl,
                                   wxString&                    rText,
                                   const HookeJeevesIteration&  iteration,
                                   bool                         highlight) const
{
    SetAttributeAndFlush(pTargetCtrl,
                         rText,
                         AttributeDatabase::GetInstance().TextAttribute(
                             AttributeDatabase::DEFAULT),
                         highlight);
    rText << _("Step vector: ")
          << PrintPoint(iteration.BasePoint())
          << wxT(" - ")
          << PrintPoint(iteration.OldBasePoint())
          << wxT(" =\n\t= ")
          << PrintPoint(iteration.StepVector())
          << wxT('\n');
}// StepVector

/* virtual */ wxString
TivHookeJeevesDiscrete::PrintTauContraction(
        const HookeJeevesIteration& iteration) const
{
    return EmbedString(_("%s contraction:"),
                Symbols::GetInstance()[Symbols::TAU])
            + wxString::Format(wxT(" %g -> %g"),
                iteration.OldTau(), iteration.NewTau());
}// PrintTauContraction

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivHookeJeevesDiscrete::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            HookeJeevesDiscreteIteration::ClassId(),
            TivHookeJeevesDiscrete::Create);
