#include <precomp.h>

#include "../methods/NewtonIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivNewton.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivNewton::TivNewton()
{
}// TivNewton

TivNewton::~TivNewton()
{
}// ~TivNewton

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivNewton::Print(      wxTextCtrl*       pTargetCtrl,
                 const IterationData&    iterationData,
                 const FunctionBase&     function,
                       bool              highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const NewtonIteration& iteration =
        dynamic_cast<const NewtonIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Base point") << wxT(": ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.StartingPoint(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintGradient(wxString::Format(wxT("x")),
                          iteration.Gradient());
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintMatrix(iteration.Hessian(), wxString(_("H[f(x)]   = ")));
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute
                            (iteration.StepValid() ?
                             AttributeDatabase::STEP_IN_DIRECTION :
                             AttributeDatabase::POINT_OUT_OF_BOUNDS),
                         highlight);

    text << _("New point") << wxT(":  ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.EndPoint(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    pTargetCtrl->AppendText(text);

    SetAttribute(pTargetCtrl,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight);
    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivNewton::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivNewton);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivNewton::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            NewtonIteration::ClassId(),
            TivNewton::Create);

