#include <precomp.h>

#include "../methods/NewtonModifiedIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivNewtonModified.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivNewtonModified::TivNewtonModified()
{
}// TivNewtonModified

TivNewtonModified::~TivNewtonModified()
{
}// ~TivNewtonModified

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivNewtonModified::Print(      wxTextCtrl*       pTargetCtrl,
                 const IterationData&    iterationData,
                 const FunctionBase&     function,
                       bool              highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const NewtonModifiedIteration& iteration =
        dynamic_cast<const NewtonModifiedIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Base point") << wxT(": ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.StartingPoint(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintGradient(wxString::Format(wxT("x")),
                          iteration.Gradient());
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintMatrix(iteration.Hessian(), wxString(_("H[f(x)]   = ")));
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute
                                    (AttributeDatabase::STEP_IN_DIRECTION),
                         highlight);

    text << _("Direction:\td = ")
         << iteration.Direction();

    text << wxT('\n')
         << _("Step:\t")
         << PrintOptimalStep(iteration.Tau(), iteration.EndPoint(),
                 function);

    /*
    text << _("New point") << wxT(":  ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.EndPoint(),
                                function);*/
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    pTargetCtrl->AppendText(text);

    SetAttribute(pTargetCtrl,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight);
    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivNewtonModified::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivNewtonModified);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivNewtonModified::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            NewtonModifiedIteration::ClassId(),
            TivNewtonModified::Create);

