#include <precomp.h>

#include "../methods/NewtonRaphsonIteration.h"
#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivNewtonRaphson.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivNewtonRaphson::TivNewtonRaphson()
{
}// TivNewtonRaphson

TivNewtonRaphson::~TivNewtonRaphson()
{
}// TivNewtonRaphson

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivNewtonRaphson::Print(      wxTextCtrl*       pTargetCtrl,
                       const IterationData&    iterationData,
                       const FunctionBase&     function,
                             bool              highlight) const
{
    assert (pTargetCtrl != NULL);

    const NewtonRaphsonIteration& iteration =
        dynamic_cast<const NewtonRaphsonIteration&>(iterationData);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString    text;
    wxTextAttr  attr_end;

    attr_end = attributes.TextAttribute(AttributeDatabase::POINT_BEST);


    pTargetCtrl->Freeze();

    text << _("Iteration initial point") << wxT(":\n");
    text << PrintDoubleWithValue(wxString::Format(wxT("x")),
                                 iteration.StartingPoint(),
                                 function);
    text << wxT("\n");
    text << PrintDerivativeInPoint(wxString(_("x")),
                                   iteration.StartingPoint(),
                                   function);
    text << wxT("\n");
    text << PrintSecondDerivativeInPoint(wxString(_("x")),
                                         iteration.StartingPoint(),
                                         function);
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);


    text << _("New point") << wxT(":\n");
    SetAttributeAndFlush(pTargetCtrl, text, attr_end, highlight);
    text << PrintDoubleWithValue(wxString::Format(wxT("x")),
                                 iteration.EndPoint(),
                                 function);
    text << wxT("\n");
    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute(AttributeDatabase::NORMAL),
                         highlight);


    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivNewtonRaphson::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivNewtonRaphson);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivNewtonRaphson::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            NewtonRaphsonIteration::ClassId(),
            TivNewtonRaphson::Create);

