#include <precomp.h>

#include "../methods/PowellVariant2Iteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivPowellVariant2.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivPowellVariant2::TivPowellVariant2()
{
}// TivPowellVariant2

TivPowellVariant2::~TivPowellVariant2()
{
}// ~TivPowellVariant2

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* static */ std::auto_ptr<const TextIterationVisualizer>
TivPowellVariant2::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivPowellVariant2);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/* virtual */ wxString
TivPowellVariant2::PrintNewBase(wxTextCtrl*                     pTargetCtrl,
                                const PowellIteration&          iterationData,
                                bool                            highlight)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const PowellVariant2Iteration& iteration =
        dynamic_cast<const PowellVariant2Iteration&>(iterationData);
    wxString text;

    text << _("Base modification condition check:")
         << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl,
                 text,
                 attributes.TextAttribute(AttributeDatabase::STEP_IN_DIRECTION),
                 highlight && iteration.BaseChanged());
    text << wxString::Format(wxT("p = %d \t"), iteration.P());
    SetAttributeAndFlush(pTargetCtrl,
                 text,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight && iteration.BaseChanged());

    text << Symbols::GetInstance()[Symbols::TAU] << wxT("_p")
         << wxString::Format(wxT(" = %g \t delta = %g"),
                             iteration.TauMax(), iteration.Delta())
         << wxT('\n')
         << wxString::Format(wxT("|| x - x_s || = %g"),
                 iterationData.TrialDistance().norm_Frobenius())
         << wxT("\n\n")
         << wxString::Format(wxT("new delta = %g "), iteration.NewDelta());

    if (iteration.BaseChanged())
    {
       text << Symbols::GetInstance()[Symbols::GREATER_EQUAL]
            << wxString::Format(wxT(" %g\n\n"), 0.8);
       Flush(pTargetCtrl, text, highlight);

       wxTextAttr attr1 = attributes.TextAttribute(AttributeDatabase::DEFAULT);
       wxTextAttr attr2 = attributes.TextAttribute(AttributeDatabase::DEFAULT);

       unsigned p = iteration.P();
       if (p == 1)  attr1 = attributes.TextAttribute(
                                AttributeDatabase::STEP_IN_DIRECTION);
       else         attr2 = attributes.TextAttribute(
                                AttributeDatabase::STEP_IN_DIRECTION);

       text << PrintTwoColumnMatrix(pTargetCtrl, iterationData.NewD(),
                    attr1, attr2, _("New base (D): "), highlight);
    }
    else
    {
       text << wxString::Format(wxT("< %g\n"), 0.8)
            << _("Base left unchanged.");
    }

    return text;
}// PrintNewBase

/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivPowellVariant2::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            PowellVariant2Iteration::ClassId(),
            TivPowellVariant2::Create);

