#include <precomp.h>

#include "../methods/QuadraticApproximationIteration.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivQuadraticApproximation.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivQuadraticApproximation::TivQuadraticApproximation()
{
}// TivQuadraticApproximation

TivQuadraticApproximation::~TivQuadraticApproximation()
{
}// ~TivQuadraticApproximation

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivQuadraticApproximation::Print(wxTextCtrl*             pTargetCtrl,
                                 const IterationData&    iterationData,
                                 const FunctionBase&     function,
                                 bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const QuadraticApproximationIteration& iteration =
        dynamic_cast<const QuadraticApproximationIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Current section:\n")
         << PrintSection(iteration.LeftBound(), iteration.RightBound(),
                 function, iteration.Length())
         << wxT('\n');

    wxTextAttr attr_m = attributes.TextAttribute(AttributeDatabase::DEFAULT);
    wxTextAttr attr_x = attributes.TextAttribute(AttributeDatabase::DEFAULT);

    if (highlight && iteration.Succeeded())
    {
        if (iteration.XIsBetter())
        {
            attr_m = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
            attr_x = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        }
        else
        {
            attr_m = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
            attr_x = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        }
    }
    else if (highlight && iteration.XOutOfBounds())
    {
        attr_x = attributes.TextAttribute(
                AttributeDatabase::POINT_OUT_OF_BOUNDS);
    }

    SetAttributeAndFlush(pTargetCtrl, text, attr_m, highlight);
    text << _("Trial point:\t")
         << PrintDoubleWithValue(wxT("m"), iteration.TrialPoint(), function)
         << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl, text, attr_x, highlight);
    text << _("Approximation minimizer:\t")
         << PrintDoubleWithValue(wxT("x"), iteration.ApproximationStationaryPoint(), function);

    SetAttributeAndFlush(pTargetCtrl, text, attributes.TextAttribute(
                AttributeDatabase::DEFAULT), highlight);
    if (iteration.NextSection() == QuadraticApproximationIteration::AM)
    {
        text << wxT("\n\n")
             << _("Next section:\t")
             << PrintCustomSection(wxT("a"), wxT("m"),
                     iteration.LeftBound(), iteration.TrialPoint(), true);
    }
    else if (iteration.NextSection() == QuadraticApproximationIteration::XB)
    {
        text << wxT("\n\n")
             << _("Next section:\t")
             << PrintCustomSection(wxT("x"), wxT("b"),
                     iteration.ApproximationStationaryPoint(),
                     iteration.RightBound(),
                     true);
    }
    else if (iteration.NextSection() == QuadraticApproximationIteration::MB)
    {
        text << wxT("\n\n")
             << _("Next section:\t")
             << PrintCustomSection(wxT("m"), wxT("b"),
                     iteration.TrialPoint(), iteration.RightBound(), true);
    }
    else if (iteration.NextSection() == QuadraticApproximationIteration::AX)
    {
        text << wxT("\n\n")
             << _("Next section:\t")
             << PrintCustomSection(wxT("a"), wxT("x"),
                     iteration.LeftBound(),
                     iteration.ApproximationStationaryPoint(), true);
    }
    else if (iteration.XOutOfBounds())
    {
        text << wxT("\n\n")
             << _("Approximation minimizer out of bounds, optimization \
failed!");
    }

    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivQuadraticApproximation::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivQuadraticApproximation);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivQuadraticApproximation::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            QuadraticApproximationIteration::ClassId(),
            TivQuadraticApproximation::Create);

