#include <precomp.h>

#include "../methods/RosenbrockIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "MatrixStyle.h"
#include "MultilineString.h"
#include "TivRosenbrock.h"  // class implemented
#include "ui_util.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivRosenbrock::TivRosenbrock()
{
}// TivRosenbrock

TivRosenbrock::~TivRosenbrock()
{
}// ~TivRosenbrock

//============================= OPERATORS ====================================
/* virtual */ void
TivRosenbrock::Print(wxTextCtrl*             pTargetCtrl,
                     const IterationData&    iterationData,
                     const FunctionBase&     function,
                     bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const RosenbrockIteration& iteration =
        dynamic_cast<const RosenbrockIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << PrintStartingParameters(pTargetCtrl, iteration, function,
            highlight);
    text << PrintTrialSteps(pTargetCtrl, iteration, function, highlight);
    text << wxT('\n');
    text << PrintFinalParameters(pTargetCtrl, iteration, function);
    if (iteration.BaseChanged())
    {
        text << wxT("\n\n")
             << _("Base rotation:")
             << wxT('\n');
        pTargetCtrl->AppendText(text);
        PrintNewBase(pTargetCtrl, iteration, function, highlight);
    }
    else
    {
        pTargetCtrl->AppendText(text);
    }

    pTargetCtrl->Thaw();
}// Print

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
wxString
TivRosenbrock::PrintStartingParameters(
                        wxTextCtrl*                         pTargetCtrl,
                        const RosenbrockIteration&          iterationData,
                        const FunctionBase&                 function,
                        bool                                highlight) const
{
    wxString text;

    text << _("Base point: ")
         << PrintPointWithValue(wxT("x_min"), iterationData.StartingPoint(),
                function)
         << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintDirectionMatrix(pTargetCtrl, iterationData.D(),
            _("Base (D): "), highlight);

    return text;
}// PrintStartingParameters

void
TivRosenbrock::PrintNewBase(
                        wxTextCtrl*                         pTargetCtrl,
                        const RosenbrockIteration&          iterationData,
                        const FunctionBase&                 function,
                        bool                                highlight) const
{
    static const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    static const wxTextAttr& attr_d1 = attributes.TextAttribute(
            AttributeDatabase::STEP_TRIAL_D1);
    static const wxTextAttr& attr_d2 = attributes.TextAttribute(
            AttributeDatabase::STEP_TRIAL_D2);
    MultilineString multiline;
    MatrixStyle style_d(iterationData.D());
    MatrixStyle style_a(iterationData.A());

    if (highlight)
    {
        style_d.SetColumnStyle(1, attr_d1);
        style_d.SetColumnStyle(2, attr_d2);
    }
    multiline.Append(_("Old D: "));
    multiline.Append(iterationData.D(), style_d);
    multiline.Append(wxT("  A: "));
    multiline.Append(iterationData.A(), style_a);
    multiline.Append(_("  New D: "));
    multiline.Append(iterationData.NewD(), style_d);
    multiline.Print(pTargetCtrl);

}// PrintNewBase

/////////////////////////////// PRIVATE    ///////////////////////////////////
