#include <precomp.h>

#include "../methods/RosenbrockDiscreteIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivRosenbrockDiscrete.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivRosenbrockDiscrete::TivRosenbrockDiscrete()
{
}// TivRosenbrockDiscrete

TivRosenbrockDiscrete::~TivRosenbrockDiscrete()
{
}// ~TivRosenbrockDiscrete
//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivRosenbrockDiscrete::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivRosenbrockDiscrete);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/* virtual */ wxString
TivRosenbrockDiscrete::PrintTrialSteps(
        wxTextCtrl*                         pTargetCtrl,
        const RosenbrockIteration&          iterationData,
        const FunctionBase&                 function,
        bool                                highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const wxTextAttr& attr_d1 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D1);
    const wxTextAttr& attr_d2 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D2);
    const wxTextAttr& attr_default = attributes.TextAttribute(
        AttributeDatabase::DEFAULT);

    const RosenbrockDiscreteIteration& iteration =
        dynamic_cast<const RosenbrockDiscreteIteration&>(iterationData);
    wxString text;

    const wxString  TRIAL_ACCEPTED(_("- accepted"));
    const wxString  TRIAL_REJECTED(_("- rejected"));
    const wxString  TAU(Symbols::GetInstance()[Symbols::TAU]);

    RosenbrockDiscreteIteration::TrialPointContainer::const_iterator i;
    const RosenbrockDiscreteIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    text << wxT("\n\n")
         << _("Trial steps:")
         << wxT('\n');

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        text << wxT("  ");

        if ((i - trial_steps.begin()) % 2 == 0)
        {
            SetAttributeAndFlush(pTargetCtrl, text, attr_d1, highlight);
        }
        else
        {
            SetAttributeAndFlush(pTargetCtrl, text, attr_d2, highlight);
        }

        text << PrintPointWithValue(i->mPoint, function)
             << wxT('\t');
        if (i->mBetter)     text << TRIAL_ACCEPTED;
        else                text << TRIAL_REJECTED;
        text << wxT('\t')
             << TAU
             << wxString::Format(wxT(" : %g -> %g"), i->mTau, i->mNewTau)
             << wxT('\n');
    }// for

    SetAttributeAndFlush(pTargetCtrl, text, attr_default, highlight);
    return text;
}// PrintTrialSteps

/* virtual */ wxString
TivRosenbrockDiscrete::PrintFinalParameters(
                        wxTextCtrl*                         pTargetCtrl,
                        const RosenbrockIteration&          iterationData,
                        const FunctionBase&                 function) const
{
    const RosenbrockDiscreteIteration& iteration =
        dynamic_cast<const RosenbrockDiscreteIteration&>(iterationData);
    wxString text;

    text << _("Sum of successful step lengths (")
         << Symbols::GetInstance()[Symbols::LAMBDA] << wxT("): ")
         << PrintPoint(iterationData.Lambda())
         << wxT('\n')
         << _("Final point: ")
         << PrintPointWithValue(wxT("x_min"), iteration.EndPoint(), function)
         << wxT('\n')
         << _("Minimal step length in directions (")
         << Symbols::GetInstance()[Symbols::TAU] << wxT(" min): ")
         << wxString::Format(wxT("%g"), iteration.TauMin());

    return text;
}// PrintFinalParameters

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivRosenbrockDiscrete::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            RosenbrockDiscreteIteration::ClassId(),
            TivRosenbrockDiscrete::Create);
