#include <precomp.h>

#include "../methods/RosenbrockOptimalIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivRosenbrockOptimal.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivRosenbrockOptimal::TivRosenbrockOptimal()
{
}// TivRosenbrockOptimal

TivRosenbrockOptimal::~TivRosenbrockOptimal()
{
}// ~TivRosenbrockOptimal
//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivRosenbrockOptimal::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivRosenbrockOptimal);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/* virtual */ wxString
TivRosenbrockOptimal::PrintTrialSteps(
        wxTextCtrl*                         pTargetCtrl,
        const RosenbrockIteration&          iterationData,
        const FunctionBase&                 function,
        bool                                highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const wxTextAttr& attr_d1 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D1);
    const wxTextAttr& attr_d2 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D2);
    const wxTextAttr& attr_default = attributes.TextAttribute(
        AttributeDatabase::DEFAULT);

    const RosenbrockOptimalIteration& iteration =
        dynamic_cast<const RosenbrockOptimalIteration&>(iterationData);
    wxString text;

    const wxString  TAU(Symbols::GetInstance()[Symbols::TAU]);

    RosenbrockOptimalIteration::TrialPointContainer::const_iterator i;
    const RosenbrockOptimalIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    text << wxT("\n\n")
         << _("Trial steps:")
         << wxT('\n');

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        text << wxT("  ");

        if (highlight)
        {
            if (i == trial_steps.begin())
            {
                SetAttributeAndFlush(pTargetCtrl, text, attr_d1, highlight);
            }
            else
            {
                SetAttributeAndFlush(pTargetCtrl, text, attr_d2, highlight);
            }
        }
        text << PrintPointWithValue(i->mPoint, function)
             << wxT('\t')
             << TAU
             << wxString::Format(wxT(" : %g"), i->mTau)
             << wxT('\n');
    }// for

    SetAttributeAndFlush(pTargetCtrl, text, attr_default, highlight);
    return text;
}// PrintTrialSteps

/* virtual */ wxString
TivRosenbrockOptimal::PrintFinalParameters(
                        wxTextCtrl*                         pTargetCtrl,
                        const RosenbrockIteration&          iterationData,
                        const FunctionBase&                 function) const
{
    wxString text;

    text << _("Sum of successful step lengths (")
         << Symbols::GetInstance()[Symbols::LAMBDA] << wxT("): ")
         << PrintPoint(iterationData.Lambda())
         << wxT('\n')
         << _("Final point: ")
         << PrintPointWithValue(wxT("x_min"), iterationData.EndPoint(),
                 function);

    return text;
}// PrintFinalParameters

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivRosenbrockOptimal::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            RosenbrockOptimalIteration::ClassId(),
            TivRosenbrockOptimal::Create);
