#include <precomp.h>

#include "../methods/SecantDivisionIteration.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivSecantDivision.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivSecantDivision::TivSecantDivision()
{
}// TivSecantDivision

TivSecantDivision::~TivSecantDivision()
{
}// ~TivSecantDivision

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivSecantDivision::Print(wxTextCtrl*             pTargetCtrl,
                              const IterationData&    iterationData,
                              const FunctionBase&     function,
                              bool                    highlight) const
{

    assert (pTargetCtrl != NULL);
    const SecantDivisionIteration& iteration =
        dynamic_cast<const SecantDivisionIteration&>(iterationData);
    wxString text;
    ColumnVector point(1);

    pTargetCtrl->Freeze();

    text << _("Current section:\n");
    text << PrintSection(iteration.LeftBound(),
                         iteration.RightBound(),
                         function,
                         iteration.Length());
    text << wxT("\n\n");

    Flush(pTargetCtrl, text, highlight);
    text << PrintDerivativeInPoint(wxString(wxT("a")),
                                   iteration.LeftBound(),
                                   function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintDerivativeInPoint(wxString(wxT("b")),
                                   iteration.RightBound(),
                                   function);
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintPoints(pTargetCtrl, iteration, function, highlight);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintNewSection(pTargetCtrl, iteration, function, highlight);


    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivSecantDivision::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivSecantDivision);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivSecantDivision::PrintPoints
                     (wxTextCtrl*                           pTargetCtrl,
                      const SecantDivisionIteration&        iteration,
                      const FunctionBase&                   function,
                      bool                                  highlight) const
{

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    //wxTextAttr  attr_left, attr_inside, attr_right;
    wxTextAttr  attr_inside
                = attributes.TextAttribute(AttributeDatabase::NORMAL);
    wxString    text;

    text << _("Inside point:") << wxT("\n");

    //attr_inside = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
/*
    if (iteration.PartChosen() == SecantDivisionIteration::LEFT)
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        attr_right  = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    }
    else
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        attr_right  = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    }
*/

    SetAttributeAndFlush(pTargetCtrl, text, attr_inside, highlight);
    text << PrintDoubleWithValue(wxString(_("x")),
                                 iteration.InsidePoint(),
                                 function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintDerivativeInPoint(wxString(_("x")),
                                   iteration.InsidePoint(),
                                   function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    return text;
}

wxString
TivSecantDivision::PrintNewSection
                     (wxTextCtrl*                       pTargetCtrl,
                      const SecantDivisionIteration&    iteration,
                      const FunctionBase&               function,
                      bool                              highlight) const
{
    wxString    text;

    text << _("Section chosen for next iteration") << wxT(":\n");

    switch (iteration.PartChosen())
    {
        case SecantDivisionIteration::LEFT :
                text << _("Left part of current section") << wxT("\n");
                text << PrintCustomSection(wxString(wxT("a")),
                                           wxString(wxT("x")),
                                           iteration.LeftBound(),
                                           iteration.InsidePoint(),
                                           function,
                                           true);
                break;
        case SecantDivisionIteration::RIGHT :
                text << _("Right part of current section") << wxT("\n");
                text << PrintCustomSection(wxString(wxT("x")),
                                           wxString(wxT("b")),
                                           iteration.InsidePoint(),
                                           iteration.RightBound(),
                                           function,
                                           true);
                break;
        case SecantDivisionIteration::NONE :
                text << _("None, middle point is the minimum.");
                break;
        default: assert(false);
                 break;
    }

    text << wxT("\n");

    Flush(pTargetCtrl, text, highlight);

    return  text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivSecantDivision::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            SecantDivisionIteration::ClassId(),
            TivSecantDivision::Create);

