#include <precomp.h>

#include "../methods/SteepestDescentIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivSteepestDescent.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivSteepestDescent::TivSteepestDescent()
{
}// TivSteepestDescent

TivSteepestDescent::~TivSteepestDescent()
{
}// ~TivSteepestDescent

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivSteepestDescent::Print(wxTextCtrl*             pTargetCtrl,
                          const IterationData&    iterationData,
                          const FunctionBase&     function,
                          bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const SteepestDescentIteration& iteration =
        dynamic_cast<const SteepestDescentIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << _("Base point:\t")
         << PrintPointWithValueAndGradient(wxT("x"), iteration.StartingPoint(),
                 iteration.Gradient(), function)
         << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute(
                             AttributeDatabase::STEP_IN_DIRECTION),
                         highlight && iteration.StepMade());
    text << _("Direction:\td = ")
         << iteration.Direction();

    if (iteration.StepMade())
    {
        text << wxT('\n')
             << _("Step:\t")
             << PrintOptimalStep(iteration.Tau(), iteration.EndPoint(),
                     function);
    }


    pTargetCtrl->AppendText(text);

    SetAttribute(pTargetCtrl,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight);
    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivSteepestDescent::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivSteepestDescent);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivSteepestDescent::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            SteepestDescentIteration::ClassId(),
            TivSteepestDescent::Create);

