#include <precomp.h>

#include "ToolbarTool.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ToolbarTool::ToolbarTool(std::auto_ptr<ClickToolAction> apAction,
                         GroupType                      group,
                         const wxBitmap&                bitmap,
                         const wxString&                label,
                         const wxString&                tip)
    : mButtonType(wxITEM_NORMAL),
      mcpClickAction(apAction),
      mcpToggleAction(),            // puste
      mGroup(group),
      mBitmap(bitmap),
      mLabel(label),
      mTip(tip)
{
}// ToolbarTool

ToolbarTool::ToolbarTool(std::auto_ptr<ToggleToolAction>    apAction,
                         GroupType                          group,
                         const wxBitmap&                    bitmap,
                         const wxString&                    label,
                         const wxString&                    tip)
    : mButtonType(wxITEM_CHECK),
      mcpClickAction(),             // puste
      mcpToggleAction(apAction),
      mGroup(group),
      mBitmap(bitmap),
      mLabel(label),
      mTip(tip)
{
}// ToolbarTool

ToolbarTool::ToolbarTool(const ToolbarTool& from)
    : mButtonType(from.mButtonType),
      mcpClickAction(from.mcpClickAction),
      mcpToggleAction(from.mcpToggleAction),
      mGroup(from.mGroup),
      mBitmap(from.mBitmap),
      mLabel(from.mLabel),
      mTip(from.mTip)
{
}

ToolbarTool::~ToolbarTool()
{
}// ~ToolbarTool


//============================= OPERATORS ====================================

ToolbarTool&
ToolbarTool::operator=(const ToolbarTool& from)
{
    mButtonType     = from.mButtonType;
    mGroup          = from.mGroup;
    mcpClickAction  = from.mcpClickAction;
    mcpToggleAction = from.mcpToggleAction;
    mBitmap         = from.mBitmap;
    mLabel          = mLabel;
    mTip            = mTip;
}// =

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

CountedPtr<ClickToolAction>
ToolbarTool::ClickAction() const
{
    assert (mButtonType == wxITEM_NORMAL);

    return mcpClickAction;
}// ClickAction

CountedPtr<ToggleToolAction>
ToolbarTool::ToggleAction() const
{
    assert (mButtonType == wxITEM_CHECK);

    return mcpToggleAction;
}// ToggleAction

const wxBitmap&
ToolbarTool::Bitmap() const
{
    return mBitmap;
}// Bitmap

ToolbarTool::GroupType
ToolbarTool::Group() const
{
    return mGroup;
}// Group

const wxString&
ToolbarTool::Label() const
{
    return mLabel;
}// Label

const wxString&
ToolbarTool::Tip() const
{
    return mTip;
}// ShortDescription

wxItemKind
ToolbarTool::Type() const
{
    return mButtonType;
}// Type

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
