/** Klasa reprezentujca narzdzie umieszczane na pasku.
 *
 * To nie jest kontrolka GUI. Obiekty tej klasy okrelaj wygld przycisku oraz
 * wywoywan akcj.
 *
 * @see PlotToolbar
 */

#ifndef _ToolbarTool_h
#define _ToolbarTool_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CountedPtr.h"

#include "ClickToolAction.h"
#include "ToggleToolAction.h"

// FORWARD REFERENCES
//

class ToolbarTool
{
public:

    /// Grupy narzdzi.
    enum GroupType {
        SET_VIEW        = 0,///< Ustawia widok
        LAYER_SPECIFIC,     ///< Specyficzne dla warstwy

        GROUP_MAX
    };

// LIFECYCLE

    /** Konstruktor.
     */
    ToolbarTool(std::auto_ptr<ClickToolAction>  apAction,
                GroupType                       group,
                const wxBitmap&                 bitmap,
                const wxString&                 label,
                const wxString&                 tip);

    /** Konstruktor.
     */
    ToolbarTool(std::auto_ptr<ToggleToolAction> apAction,
                GroupType                       group,
                const wxBitmap&                 bitmap,
                const wxString&                 label,
                const wxString&                 tip);

    /** Konstruktor kopiujcy.
     */
    ToolbarTool(const ToolbarTool& from);

    /** Destruktor.
     */
    ~ToolbarTool(void);


// OPERATORS

    /** Operator przypisania.
     */
    ToolbarTool& operator=(const ToolbarTool& from);

// OPERATIONS
// ACCESS

    /** Akcja narzdzia zwyczajnie klikanego (tylko, gdy Type zwraca
     * <code>wxITEM_NORMAL</code>).
     */
    CountedPtr<ClickToolAction>     ClickAction() const;

    /** Akcja narzdzia on/off (tylko, gdy Type zwraca
     * <code>wxITEM_CHECK</code>).
     */
    CountedPtr<ToggleToolAction>    ToggleAction() const;

    /** Bitmapa przycisku.
     */
    const wxBitmap&     Bitmap() const;

    /** Nazwa narzdzia/napis na przycisku.
     */
    const wxString&     Label() const;

    /** Grupa, do ktrej naley narzdzie.
     */
    GroupType           Group() const;

    /** Krtki opis narzdzia (pojawia si w tooltipie).
     */
    const wxString&     Tip() const;

    /** Typ narzdzia: <code>wxITEM_NORMAL</code> lub <code>wxITEM_CHECK</code>.
     */
    wxItemKind          Type() const;

// INQUIRY

protected:
private:
    wxItemKind  mButtonType;    ///< Typ przycisku (wxITEM_NORMAL albo CHECK).

    /** Akcja zwizana z tym narzdziem. Zdefiniowane tylko gdy
     * <code>mItemType == wxITEM_NORMAL</code>.
     */
    CountedPtr<ClickToolAction>     mcpClickAction;

    /** Akcja zwizana z tym narzdziem. Zdefiniowane tylko gdy
     * <code>mItemType == wxITEM_CHECK</code>.
     */
    CountedPtr<ToggleToolAction>    mcpToggleAction;

    GroupType   mGroup;

    wxBitmap    mBitmap;
    wxString    mLabel;
    wxString    mTip;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ToolbarTool_h
