#include <precomp.h>

#include "../methods/UniformDivisionIteration.h"

#include "UniformDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UniformDivisionLayer::UniformDivisionLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// UniformDivisionLayer

UniformDivisionLayer::UniformDivisionLayer(const UniformDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// UniformDivisionLayer

UniformDivisionLayer::~UniformDivisionLayer()
{
}// ~UniformDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
UniformDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    DrawIteration(dc, mHighlightedIteration);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
UniformDivisionLayer::Create(const PlotControl&     plotControl,
                             const Result&          result,
                             CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new UniformDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
UniformDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const UniformDivisionIteration& iteration =
        dynamic_cast<const UniformDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    // TODO: mona to zrobi jako bardziej inteligentnie.
    double MARGIN = 0.05;

    return Region(iteration.TrialPoint() - MARGIN, 0,
                  iteration.TrialPoint() + MARGIN, 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
UniformDivisionLayer::DrawIteration(wxDC&             dc,
                                    unsigned          iterationNumber) const
{
    const UniformDivisionIteration& iteration =
        dynamic_cast<const UniformDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    // Punkt.
    DrawBigPoint(dc, iteration.TrialPoint(),
            AttributeDatabase::NORMAL);

}// DrawIteration

void
UniformDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const UniformDivisionIteration& iteration =
        dynamic_cast<const UniformDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* UniformDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            UniformDivisionIteration::ClassId(),
            UniformDivisionLayer::Create);
