/** Warstwa wykresu rysujca przebieg optymalizacji z ograniczeniami w wielu
 * wymiarach.
 *
 * @see PlotControl
 */

#ifndef _UnivariateConstrainedLayer_h
#define _UnivariateConstrainedLayer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "PointControl.h"
#include "UnivariateConstraintsLayer.h"
#include "UnivariateFunctionLayer.h"
#include "UnivariateResultLayer.h"

// FORWARD REFERENCES
//


class UnivariateConstrainedLayer
    : public UnivariateResultLayer
{
public:
// LIFECYCLE

    /** Konstruktor
     */
    UnivariateConstrainedLayer(
            const PlotControl&                          plotControl,
            const Result&                               result,
            CountedPtr<PlotLayer>                       cpFunctionLayer,
            std::auto_ptr<UnivariateConstraintsLayer>   apConstraintsLayer,
            std::auto_ptr<UnivariateFunctionLayer>      apPenaltyLayer);

    /** Konstruktor kopiujcy.
     */
    UnivariateConstrainedLayer(const UnivariateConstrainedLayer& from);

    /** Destruktor.
     */
    ~UnivariateConstrainedLayer(void);

// OPERATORS
// OPERATIONS

    virtual void    Draw(wxDC& dc) const;

    virtual vector<ToolbarTool> CreateTools(PlotControl* pPlotControl);

    /** Metoda fabryki.
     */
    static std::auto_ptr<ResultLayer>   Create(
                                const PlotControl&      plotControl,
                                const Result&           result,
                                CountedPtr<PlotLayer>   cpFunctionLayer);

// ACCESS

    virtual Region  IterationBoundingBox(unsigned iterationNumber) const;

// INQUIRY

protected:
private:

    void            DrawIteration(wxDC&     dc,
                                  unsigned  iterationNumber,
                                  bool      highlight) const;


    // Nie moe to by auto_ptr, bo warstwy s przechowywane w kontenerze.
    CountedPtr<UnivariateConstraintsLayer>  mcpConstraintsLayer;
    CountedPtr<UnivariateFunctionLayer>     mcpPenaltyLayer;

    const static void*  mRegistrationDummy;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _UnivariateConstrainedLayer_h
