#include <precomp.h>

#include "../Logger.h"
#include "../UnivariateFunction.h"

#include "ContourPlotter.h" // Potrzebne stae
#include "PlotControl.h"
#include "UnivariateConstraintsLayer.h"  // class implemented

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UnivariateConstraintsLayer::UnivariateConstraintsLayer
                                   (const PlotControl&           plotControl,
                                    const Constraint::Container& constraints)
    : ConstraintsLayer(plotControl, constraints)//,
      //mVariableXIndex(1),
      //mVariableYIndex(2)
{
}// UnivariateConstraintsLayer

UnivariateConstraintsLayer::UnivariateConstraintsLayer(const UnivariateConstraintsLayer& from)
    : ConstraintsLayer(from)//,
      //mVariableXIndex(from.mVariableXIndex),
      //mVariableYIndex(from.mVariableYIndex)
{
}// UnivariateConstraintsLayer

UnivariateConstraintsLayer::~UnivariateConstraintsLayer()
{
}// ~UnivariateConstraintsLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
UnivariateConstraintsLayer::Draw(wxDC& dc) const
{
    wxCoord pixel_width;
    wxCoord pixel_height;

    dc.GetSize(&pixel_width, &pixel_height);

    // FIXME: wywali rysowanie conreciem i zrobi szukanie miejsc zerowych
    // przez przegld zupeny

//    // Alokacja pamici:
//
//    // Odlego midzy kolejnymi prbkami (w pikslach).
//    const int SAMPLE_BOX_SIZE = 5;
//    // Siatka musi mie o 1 wicej krawd ni komrek.
//    int column_count =
//        static_cast<int>(ceil(
//                    static_cast<double>(pixel_width) / SAMPLE_BOX_SIZE)) + 1;
//    int row_count    =
//        static_cast<int>(ceil(
//                    static_cast<double>(pixel_height) / SAMPLE_BOX_SIZE)) + 1;
//
//    double columns[column_count];
//    double rows[row_count];
//    double levels[1] = { 0.0 };
//    double** samples;
//
//    // A nie da si lepiej?
//    samples = new double*[column_count];
//    for (int i = 0; i < column_count; ++i) samples[i] = new double[row_count];
//
//    // TODO: uoglni
//    ColumnVector point(2);
//    point = 0;

    // Na potrzeby kolorowania.
    unsigned equality_constraint_index = 0;
    unsigned inequality_constraint_index = 0;

    LOG(DEBUG, wxT("Constraint count: ") << static_cast<int>(mConstraints.size()));
    Constraint::Container::const_iterator constraint_iterator;
    for (constraint_iterator = mConstraints.begin();
         constraint_iterator != mConstraints.end();
         ++constraint_iterator)
    {
        const UnivariateFunction& function =
            dynamic_cast<const UnivariateFunction&>
                (constraint_iterator->Function());

        // TO jest tutaj, poniewa GetConstraintPen przesuwa do przodu
        // odpowiedni indeks.
        const wxPen& normal =
            GetConstraintPen(*constraint_iterator,
                             equality_constraint_index,     // in out
                             inequality_constraint_index);  // in out

        double previous_value =
            function.EvaluateNoThrow(mTargetPlot.DcToPlotX(0));
        for (wxCoord x = 1; x < pixel_width; ++x)
        {
            double current_value = function.Evaluate(mTargetPlot.DcToPlotX(x));
            if ((previous_value < 0) && (current_value > 0)
                    || (previous_value > 0) && (current_value < 0)
                    || (current_value == 0))
            {
                dc.SetPen(normal);
                dc.DrawLine(x, 0, x, pixel_height);

                if (constraint_iterator->Type() != Constraint::EQUAL)
                {
                    wxPen dashed = normal;
                    dashed.SetStyle(ContourPlotter::DASH_STYLE);
                    dc.SetPen(dashed);

                    // Ustal, z ktrej strony narysowa linie przerywan.
                    int x_dashed;
                    if (((constraint_iterator->Type() ==
                            Constraint::GREATER_EQUAL_THAN)
                                && (previous_value > current_value))
                        || ((constraint_iterator->Type() ==
                            Constraint::LESS_EQUAL_THAN)
                                && (previous_value < current_value)))
                    {
                        x_dashed = x + ContourPlotter::OUTLINE_DISTANCE;
                    }
                    else
                    {
                        x_dashed = x - ContourPlotter::OUTLINE_DISTANCE;
                    }

                    dc.DrawLine(x_dashed, 0, x_dashed, pixel_height);
                }
            }
            previous_value = current_value;
        }

    } // for

    // Legenda
    DrawLegend(dc);
}// Draw

//============================= ACCESS     ===================================

//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

