#include <precomp.h>

#include "../Logger.h"

#include "AttributeDatabase.h"
#include "PlotControl.h"
#include "UnivariateFunctionLayer.h"  // class implemented

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UnivariateFunctionLayer::UnivariateFunctionLayer(
        const PlotControl&          targetPlot,
        const UnivariateFunction&   function)
    : PlotLayer(targetPlot), mFunction(function)
{
}// UnivariateFunctionLayer

UnivariateFunctionLayer::UnivariateFunctionLayer(
        const UnivariateFunctionLayer& from)
    : PlotLayer(from),
      mFunction(from.mFunction)
{
}// UnivariateFunctionLayer
UnivariateFunctionLayer::~UnivariateFunctionLayer()
{
}// ~UnivariateFunctionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
UnivariateFunctionLayer::Draw(wxDC& dc) const
{
    wxCoord pixel_width;
    wxCoord pixel_height;

    dc.GetSize(&pixel_width, &pixel_height);

    assert (pixel_width > 1);

    double plot_x1 = mTargetPlot.X1();
    double plot_x2 = mTargetPlot.X2();

    assert (plot_x1 < plot_x2);

    double current_value;
    double previous_value;

    previous_value = mFunction.EvaluateNoThrow(plot_x1);

    dc.SetPen(AttributeDatabase::GetInstance().
            Pen(AttributeDatabase::PLOT_FUNCTION));

    wxCoord current_y;// = 0;  // Inicjowane, aby pozby si warninga
    wxCoord previous_y = mTargetPlot.PlotToDcY(previous_value);
    for (long x = 1; x < pixel_width; ++x)
    {
        current_value = mFunction.EvaluateNoThrow(mTargetPlot.DcToPlotX(x));

        current_y = mTargetPlot.PlotToDcY(current_value);
        if (isfinite(current_value) && isfinite(previous_value))
        {
            dc.DrawLine(x-1, lround(previous_y), x, lround(current_y));
        }
        previous_value = current_value;
        previous_y = current_y;
    }

}// Draw


//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
