#include <precomp.h>

#include "../ProblemBase.h"

#include "PlotControl.h"
#include "UnivariateResultLayer.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UnivariateResultLayer::UnivariateResultLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : ResultLayer(plotControl, result, cpFunctionLayer)
{
}// UnivariateResultLayer

UnivariateResultLayer::UnivariateResultLayer(const UnivariateResultLayer& from)
    : ResultLayer(from)
{
}// UnivariateResultLayer

UnivariateResultLayer::~UnivariateResultLayer()
{
}// ~UnivariateResultLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxPoint
UnivariateResultLayer::PlotToDcPoint(double point) const
{
    ColumnVector point_vector(1);
    point_vector(1) = point;

    return wxPoint(mTargetPlot.PlotToDcX(point),
               mTargetPlot.PlotToDcY(
                   mResult.Problem().BaseFunction().EvaluateNoThrow(point_vector)));
}// PlotToDcPoint

void
UnivariateResultLayer::DrawFinalResult(wxDC& dc) const
{
    if ( ! mResult.IsAborted() )
    {
        DrawBigPoint(dc, mResult.FinalResult()(1),
                AttributeDatabase::GetInstance().
                AttributeDatabase::POINT_RESULT,
                SQUARE);
    }
}// DrawFinalResult

void
UnivariateResultLayer::DrawBigPoint(
        wxDC&                               dc,
        double                              point,
        AttributeDatabase::AttributeType    attribute,
        ResultLayer::SymbolType             symbol) const
{
    ResultLayer::DrawBigPoint(dc, PlotToDcPoint(point), attribute, symbol);
}// DrawBigPoint

void
UnivariateResultLayer::DrawVerticalLine(
        wxDC&                               dc,
        double                              point,
        AttributeDatabase::AttributeType    attribute) const
{
    wxPen   old_pen(dc.GetPen());
    dc.SetPen(AttributeDatabase::GetInstance().Pen(attribute));

    wxCoord x = mTargetPlot.PlotToDcX(point);
    wxSize  size(dc.GetSize());

    // wxy rysuj bez ostatniego punktu
    dc.DrawLine(x, 0, x, size.GetHeight() + 1);

    dc.SetPen(old_pen);
}// DrawVerticalLine

void
UnivariateResultLayer::DrawRangeBackground(
        wxDC&                               dc,
        wxCoord                             left,
        wxCoord                             right,
        AttributeDatabase::AttributeType    attribute) const
{
    wxPen   old_pen(dc.GetPen());
    wxBrush old_brush(dc.GetBrush());

    dc.SetPen(AttributeDatabase::GetInstance().Pen(attribute));
    dc.SetBrush(AttributeDatabase::GetInstance().Brush(attribute));

    
    wxSize size(dc.GetSize());

    // Nie rysuj, gdy nie ma czego.
    if (left > right)   return;
    if ((right < 0) || (left > size.GetWidth()))    return;

    // Gdy left i right s rwne, rysujemy kresk.
    dc.DrawRectangle(left, 0, max(right - left, 1) , size.GetHeight());

    dc.SetPen(old_pen);
    dc.SetBrush(old_brush);
}// DrawRangeBackground

void
UnivariateResultLayer::GreyOutBackground(
        wxDC&                               dc,
        AttributeDatabase::AttributeType    attribute) const
{
    wxBrush old_brush(dc.GetBackground());

    dc.SetBackground(AttributeDatabase::GetInstance().Brush(attribute));

    dc.Clear();

    dc.SetBackground(old_brush);
}// GreyOutBackground

AttributeDatabase::AttributeType
UnivariateResultLayer::BackgroundAttribute(unsigned iterationNumber) const
{
    switch (mHighlightedIteration - iterationNumber)
    {
        case 0:  return AttributeDatabase::PLOT_BACKGROUND;
        case 1:  return AttributeDatabase::PLOT_OUTSIDE_BG3;
        case 2:  return AttributeDatabase::PLOT_OUTSIDE_BG2;
        default: return AttributeDatabase::PLOT_OUTSIDE_BG;
    }
}// BackgroundAttribute


/////////////////////////////// PRIVATE    ///////////////////////////////////
