/**Szablon panelu dla metod z optymalizacj w kierunku.
 *
 * Uniwersalny panel dla metod posiadajcych 3 parametry:
 * metoda optymalizacji w kierunku, warunek stopu, punkt startowy.
 *
 * Panele dla poszczeglnych metod s rejestrowane
 * w MethodWithLineSearchPanel.cc.
 */

#ifndef VARIANTEDMETHODWITHLINESEARCHPANEL_H
#define VARIANTEDMETHODWITHLINESEARCHPANEL_H

#include <wx/valgen.h>

#include "DoubleParameterValidator.h"
#include "LineSearchControl.h"
#include "LineSearchValidator.h"
#include "MethodPanelInfo.h"
#include "MethodPanelFactory.h"
#include "MethodWithLineSearchPanel.h"
#include "PointControl.h"
#include "StopConditionControl.h"
#include "StopConditionValidator.h"

template <class T>
class VariantedMethodWithLineSearchPanel
    : public MethodWithLineSearchPanel<T>
{
public:

    /** Konstruktor.
     */
    VariantedMethodWithLineSearchPanel
                               (wxWindow*             parent,
                                int                   id,
                                T*                    pMethod,
                                const ProblemBase&    problem,
                                const wxPoint&        pos=wxDefaultPosition,
                                const wxSize&         size=wxDefaultSize,
                                long                  style=0);

    /** Tworzy panel tej klasy.
     *
     * Metoda wywoywana przez MethodPanelFactory.
     */
    static wxPanel*     CreatePanel(MethodPanelInfo     panelInfo);

private:

    void do_layout();

protected:

    //LineSearchControl*      mpLineSearch;
    //StopConditionControl*   mpStopCondition;
    //PointControl*           mpStartingPoint;
    wxBoxSizer*             mpModificationSizer;
    wxCheckBox*             mpModificationCheckBox;
};

template <class T>
VariantedMethodWithLineSearchPanel<T>::VariantedMethodWithLineSearchPanel
                                           (wxWindow*             parent,
                                            int                   id,
                                            T*                    pMethod,
                                            const ProblemBase&    problem,
                                            const wxPoint&        pos,
                                            const wxSize&         size,
                                            long                  style)
    : MethodWithLineSearchPanel<T>
                        (parent, id, pMethod, problem, pos, size, style)
{
    // Tworzenie kontrolek.

    mpModificationSizer = new wxBoxSizer
                        (wxVERTICAL/*, this, _T("")*/);

    mpModificationCheckBox = new wxCheckBox(this,
                                            -1,
                                            _("Use modification"),
                                            wxDefaultPosition,
                                            wxDefaultSize,
                                            0,
                                            wxDefaultValidator
                                            /*TODO: validator*/
                                            );

    do_layout();

    // Walidatory.
    mpModificationCheckBox->SetValidator
                            (wxGenericValidator(&(pMethod->mModification)));

}// MethodWithLineSearchPanel


template <class T>
/* static */ wxPanel*
VariantedMethodWithLineSearchPanel<T>::CreatePanel(MethodPanelInfo panelInfo)
{
    return new VariantedMethodWithLineSearchPanel(
                panelInfo.pParent(),
                -1,
                dynamic_cast<T*>(panelInfo.pMethod()),
                *panelInfo.pProblem());

} // CreatePanel

template <class T>
void
VariantedMethodWithLineSearchPanel<T>::do_layout()
{

    MethodWithLineSearchPanel<T>::do_layout();

    mpModificationSizer->Add(mpModificationCheckBox, 0, wxEXPAND, 0);

    MethodWithLineSearchPanel<T>::mpMainSizer->Insert
                            (0, mpModificationSizer, 0, wxEXPAND, 0);

    wxPanel::SetAutoLayout(true);
    SetSizer(MethodWithLineSearchPanel<T>::mpMainSizer);
    MethodWithLineSearchPanel<T>::mpMainSizer->Fit(this);
    MethodWithLineSearchPanel<T>::mpMainSizer->SetSizeHints(this);

}

#endif // VARIANTEDMETHODWITHLINESEARCHPANEL_H
