#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Logger.h"
#include "../ProblemBase.h"

#include "VisualizationFrame.h"  // class implemented
#include "Visualizer.h"
#include "VisualizerFactory.h"

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

VisualizationFrame::VisualizationFrame(
        wxWindow*                   pParent,
        CountedPtr<const Result>    cpResult)
    : wxFrame(pParent, wxID_ANY, wxT("")), mcpResult(cpResult)
{
    // Ikona
    wxIconBundle icons;
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM16")));
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM32")));
    icons.AddIcon(wxXmlResource::Get()->LoadIcon(wxT("ICON_EDUOPTIM48")));
    SetIcons(icons);

    wxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);

    mpVisualizerNotebook = new wxNotebook(this, wxID_ANY);

    VisualizerFactory::VisualizerContainer visualizers =
        VisualizerFactory::CreateVisualizers(mpVisualizerNotebook, *mcpResult);

    assert(visualizers.size() > 0);

    VisualizerFactory::VisualizerContainer::iterator i;
    for (i = visualizers.begin(); i != visualizers.end(); ++i)
    {
        mpVisualizerNotebook->AddPage(*i, (*i)->Name(), true);
    }

    mpVisualizerNotebook->SetSelection(0);

    p_main_sizer->Add(mpVisualizerNotebook, 1, wxEXPAND | wxALL, 2);

    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);

    SetTitle(GenerateTitle());

    Layout();
}// VisualizationFrame

VisualizationFrame::~VisualizationFrame()
{
}// ~VisualizationFrame


//============================= OPERATORS ====================================


//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

wxString
VisualizationFrame::GenerateTitle()
{
    wxString title = wxString::Format(wxT("%u: "),
            ++VisualizationFrame::mWindowNumber);

    title << mcpResult->Problem().UsedMethod().Name();

    return title;
}// GenerateTitle

/* static */ unsigned VisualizationFrame::mWindowNumber = 0;
