/** Fabryka odpowiedzialna za tworzenie obiektw wizualizacji dla danej iteracji
 *
 * Wszystkie metody klasy s statyczne.
 *
 * Dodawanie wizualizatorw wymaga niestety grzebania w tej klasie...
 *
 * @see Visualizer
 */

#ifndef _VisualizerFactory_h
#define _VisualizerFactory_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CountedPtr.h"
#include "../Result.h"

#include "ResultLayer.h"
#include "TextIterationVisualizer.h"
#include "TextVisualizer.h"
#include "Visualizer.h"
#include "FullPlotVisualizer.h"


// FORWARD REFERENCES
//


class VisualizerFactory
{
public:

    /// Kontener wskanikw na tworzone wizualizacje.
    typedef vector<Visualizer*> VisualizerContainer;

// LIFECYCLE

// OPERATORS

// OPERATIONS

    /** Tworzy list wizualizatorw na podstawie danego obiektu wyniku.
     *
     * Tworzony jest najwyej jeden wizualizator danego typu (tekstowy, wykres
     * itd.), ich kolejno na licie jest taka sama we wszystkich wywoaniach.
     *
     * Wynik musi posiada przynajmniej jedn iteracj. Wizualizatory (pochodne
     * klasy Visualizer) s tworzone na podstawie identyfikatora klasy iteracji
     * (pochodne IterationData). W razie poraki poszukiwanie prowadzone jest
     * dla kolejnych klas nadrzdnych, co pozwala znale bardziej oglne
     * wizualizatory.
     *
     * Tworzonym wizualizatorom s przekazywane utworzone wizualizatory
     * iteracji.
     */
    static VisualizerContainer CreateVisualizers(wxWindow*      pParent,
                                                 const Result&  result);


    /** Rejestruje funkcj tworzc tekstowy wizualizator iteracji.
     *
     * @param id Identyfikator klasy
     * @param function Wskanik na funkcj tworzc odpowiedni wizualizator.
     */
    static void*        RegisterTextVisualizer
                            (IterationData::IterationDataIdType     id,
                             const TextIterationVisualizer::Creator function);

    /** Rejestruje funkcj tworzc wizualizator graficzny przebiegu
     * optymalizacji.
     */
    static void*        RegisterFullPlotVisualizer
                            (IterationData::IterationDataIdType     id,
                             const ResultLayer::Creator             creator);

    /** Rejestruje (no, powiedzmy, raczej ustawia flag) wizualizator z wykresem
     * iteracja-warto.
     */
    static void*        RegisterIterationValueVisualizer
                            (IterationData::IterationDataIdType);

// ACCESS
// INQUIRY

protected:

    /// Klasa przechowujca kreatory (fabryki) rnych typw wizualizacji.
    class VisualizerCreators
    {
    public:
        /// Konstruktor
        VisualizerCreators();

        /// Inicjuje niezainicjowane pola wartociami z <code>creators</code>.
        void Merge(const VisualizerCreators& creators);

        /// Sprawdza, czy adna z fabryk nie jest ustawiona (wszystkie NULL).
        bool IsEmpty() const;
        /// Sprawdza, czy wszystkie fabryki zostay ustawione.
        bool IsComplete() const;

    // DATA

        /// Funkcja tworzca tekstowy wizualizator iteracji.
        TextIterationVisualizer::Creator    mTextCreator;
        ResultLayer::Creator                mFullPlotCreator;
        bool                                mIterationValueFlag;
        // TODO: pozostae
    }; // VisualizerCreators

    /** Odwzorowanie: ID klasy iteracji -> zestaw kreatorw wizualizatorw dla
     *  tej klasy.
     */
    typedef map<IterationData::IterationDataIdType,
                VisualizerCreators>                 CreatorMapType;

    static CreatorMapType&         CreatorMap();
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _VisualizerFactory_h
