/** Warstwa wykresu rysujca przebieg optymalizacji z ograniczeniami w wielu
 * wymiarach.
 *
 * @see PlotControl
 */

#ifndef _XyConstrainedLayer_h
#define _XyConstrainedLayer_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "MultivariateFunctionLayer.h"
#include "PointControl.h"
#include "XyConstraintsLayer.h"
#include "XyResultLayer.h"

// FORWARD REFERENCES
//


class XyConstrainedLayer
    : public XyResultLayer
{
public:
// LIFECYCLE

    /** Konstruktor
     */
    XyConstrainedLayer(
            const PlotControl&                          plotControl,
            const Result&                               result,
            CountedPtr<PlotLayer>                       cpFunctionLayer,
            std::auto_ptr<XyConstraintsLayer>           apConstraintsLayer,
            std::auto_ptr<MultivariateFunctionLayer>    apPenaltyLayer);

    /** Konstruktor kopiujcy.
     */
    XyConstrainedLayer(const XyConstrainedLayer& from);

    /** Destruktor.
     */
    ~XyConstrainedLayer(void);

// OPERATORS
// OPERATIONS

    virtual void    Draw(wxDC& dc) const;

    virtual vector<ToolbarTool> CreateTools(PlotControl* pPlotControl);

    /** Metoda fabryki.
     */
    static std::auto_ptr<ResultLayer>   Create(
                                const PlotControl&      plotControl,
                                const Result&           result,
                                CountedPtr<PlotLayer>   cpFunctionLayer);

// ACCESS

    virtual Region  IterationBoundingBox(unsigned iterationNumber) const;

// INQUIRY

protected:
private:

    void            DrawIteration(wxDC&     dc,
                                  unsigned  iterationNumber,
                                  bool      highlight) const;

    void            DrawFirstIteration(wxDC&     dc,
                                       unsigned  iterationNumber,
                                       bool      highlight) const;


    // Nie moe to by auto_ptr, bo warstwy s przechowywane w kontenerze.
    CountedPtr<XyConstraintsLayer>          mcpConstraintsLayer;
    CountedPtr<MultivariateFunctionLayer>   mcpPenaltyLayer;

    const static void*  mRegistrationDummy;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _XyConstrainedLayer_h
