#include <precomp.h>

#include "../Logger.h"

#include "PlotControl.h"
#include "XyConstraintsLayer.h"  // class implemented
#include "conrec.h"

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

XyConstraintsLayer::XyConstraintsLayer(const PlotControl&           plotControl,
                                       const Constraint::Container& constraints,
                                       unsigned                     variableCount)
    : ConstraintsLayer(plotControl, constraints),
      mVariableXIndex(1),
      mVariableYIndex(2),
      mVariableCount(variableCount) // Tak na dobr spraw to powinien by adapter funkcji...
{
}// XyConstraintsLayer

XyConstraintsLayer::XyConstraintsLayer(const XyConstraintsLayer& from)
    : ConstraintsLayer(from),
      mVariableXIndex(from.mVariableXIndex),
      mVariableYIndex(from.mVariableYIndex),
      mVariableCount(from.mVariableCount)
{
}// XyConstraintsLayer

XyConstraintsLayer::~XyConstraintsLayer()
{
}// ~XyConstraintsLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
XyConstraintsLayer::Draw(wxDC& dc) const
{
    wxCoord pixel_width;
    wxCoord pixel_height;

    dc.GetSize(&pixel_width, &pixel_height);

    // Alokacja pamici:

    // Odlego midzy kolejnymi prbkami (w pikslach).
    const int SAMPLE_BOX_SIZE = 7;
    // Siatka musi mie o 1 wicej krawd ni komrek.
    int column_count =
        static_cast<int>(ceil(
                    static_cast<double>(pixel_width) / SAMPLE_BOX_SIZE)) + 1;
    int row_count    =
        static_cast<int>(ceil(
                    static_cast<double>(pixel_height) / SAMPLE_BOX_SIZE)) + 1;

    double columns[column_count];
    double rows[row_count];
    double levels[1] = { 0.0 };
    double** samples;

    // A nie da si lepiej?
    samples = new double*[column_count];
    for (int i = 0; i < column_count; ++i) samples[i] = new double[row_count];

    // TODO: uoglni
    ColumnVector point(2);
    point = 0;

    // Na potrzeby kolorowania.
    unsigned equality_constraint_index = 0;
    unsigned inequality_constraint_index = 0;

    LOG(DEBUG, wxT("Constraint count: ") << static_cast<int>(mConstraints.size()));
    Constraint::Container::const_iterator constraint_iterator;
    for (constraint_iterator = mConstraints.begin();
         constraint_iterator != mConstraints.end();
         ++constraint_iterator)
    {
        const FunctionBase& function = constraint_iterator->Function();
        for (int i = 0; i < column_count; ++i)
        {
            columns[i] = i * SAMPLE_BOX_SIZE;
            point(mVariableXIndex) = mTargetPlot.DcToPlotX(i * SAMPLE_BOX_SIZE);
            for (int j = 0; j < row_count; ++j)
            {
                rows[j] = j * SAMPLE_BOX_SIZE;
                point(mVariableYIndex) =
                    mTargetPlot.DcToPlotY(j * SAMPLE_BOX_SIZE);

                samples[i][j] = function.EvaluateNoThrow(point);
            }
        }

        dc.SetPen(GetConstraintPen(*constraint_iterator,
                                   equality_constraint_index,       // in out
                                   inequality_constraint_index));   // in out

        ContourPlotter contour(1);
        // TODO: poprawi przesunicie w pionie
        ::conrec(samples,
                 0, column_count - 1,
                 0, row_count - 1,
                 static_cast<double *>(columns),
                 static_cast<double *>(rows),
                 1,
                 static_cast<double *>(levels),
                 contour);

        contour.Collapse();

        if (constraint_iterator->Type() == Constraint::EQUAL)
        {
            contour.Draw(dc, false);
        }
        else
        {
            contour.Draw(dc,
                         function,
                         mVariableXIndex,
                         mVariableYIndex,
                         mVariableCount,
                         mTargetPlot,
                         constraint_iterator->Type() == Constraint::GREATER_EQUAL_THAN);
        }
    }// for


    // Zwolnij pami
    for (int i = 0; i < column_count; ++i) delete[] samples[i];
    delete[] samples;

    // Legenda
    DrawLegend(dc);
}// Draw

//============================= ACCESS     ===================================

//============================= INQUIRY    ===================================

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

