#include <precomp.h>

#include "AttributeDatabase.h"
#include "PlotControl.h"
#include "XyResultLayer.h"  // class implemented



/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

XyResultLayer::XyResultLayer(const PlotControl&     plotControl,
                             const Result&          result,
                             CountedPtr<PlotLayer>  cpFunctionLayer)
    : ResultLayer(plotControl, result, cpFunctionLayer),
      mVariableXIndex(1),   // Na razie mog by tylko dwie zmienne
      mVariableYIndex(2)
{
}// XyResultLayer

XyResultLayer::XyResultLayer(const XyResultLayer& from)
    : ResultLayer(from),
      mVariableXIndex(from.mVariableXIndex),
      mVariableYIndex(from.mVariableYIndex)
{
}// XyResultLayer

XyResultLayer::~XyResultLayer()
{
}// ~XyResultLayer


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
XyResultLayer::DrawFinalResult(wxDC& dc) const
{
    if ( ! mResult.IsAborted() )
    {
        DrawBigPoint(dc, mResult.FinalResult(),
                AttributeDatabase::GetInstance().
                AttributeDatabase::POINT_RESULT,
                SQUARE);
    }
}// DrawFinalResult

wxPoint
XyResultLayer::PlotToDcPoint(const ColumnVector& point) const
{
    return wxPoint(mTargetPlot.PlotToDcX(point(mVariableXIndex)),
                   mTargetPlot.PlotToDcY(point(mVariableYIndex)));
}// PlotToDcPoint

void
XyResultLayer::DrawBigPoint(wxDC&                            dc,
                            const ColumnVector&              point,
                            AttributeDatabase::AttributeType attribute,
                            ResultLayer::SymbolType          symbol) const
{
    ResultLayer::DrawBigPoint(dc, PlotToDcPoint(point), attribute, symbol);
}// DrawBigPoint

void
XyResultLayer::DrawLine(wxDC&                               dc,
                        const ColumnVector&                 point1,
                        const ColumnVector&                 point2,
                        AttributeDatabase::AttributeType    attribute,
                        int                                 style) const
{
    wxPen   old_pen = dc.GetPen();

    wxPen   pen = AttributeDatabase::GetInstance().Pen(attribute);

    if (style == wxSOLID)   pen.SetWidth(SOLID_WIDTH);
    else                    pen.SetWidth(FANCY_WIDTH);
    pen.SetStyle(style);

    dc.SetPen(pen);
    dc.DrawLine(mTargetPlot.PlotToDcX(point1(mVariableXIndex)),
                mTargetPlot.PlotToDcY(point1(mVariableYIndex)),
                mTargetPlot.PlotToDcX(point2(mVariableXIndex)),
                mTargetPlot.PlotToDcY(point2(mVariableYIndex)));

    dc.SetPen(old_pen);
}// DrawLine

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ int XyResultLayer::SOLID_WIDTH = 2;
const /* static */ int XyResultLayer::FANCY_WIDTH = 1;
