#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "SubsequentIterationsAction.h"
#include "XyUnconstrainedLayer.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

XyUnconstrainedLayer::XyUnconstrainedLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyResultLayer(plotControl, result, cpFunctionLayer),
      mDrawSubsequentIterations(true)
{
}// XyUnconstrainedLayer

XyUnconstrainedLayer::XyUnconstrainedLayer(const XyUnconstrainedLayer& from)
    : XyResultLayer(from),
      mDrawSubsequentIterations(from.mDrawSubsequentIterations)
{
}// XyUnconstrainedLayer

XyUnconstrainedLayer::~XyUnconstrainedLayer()
{
}// ~XyUnconstrainedLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
XyUnconstrainedLayer::Draw(wxDC& dc) const
{
    unsigned i;

    // Wszystko rysujemy na wykresie funkcji.
    mcpFunctionLayer->Draw(dc);

    for (i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }

    if (mDrawSubsequentIterations)
    {
        for (i++;
             i <= mResult.LastIterationNumber();
             ++i)
        {
            DrawIteration(dc, i, false);
        }
    }

    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* virtual */ vector<ToolbarTool>
XyUnconstrainedLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(XyResultLayer::CreateTools(pPlotControl));

    ToolbarTool subsequent_tool(
            auto_ptr<ToggleToolAction>
                (new SubsequentIterationsAction(pPlotControl, this)),
            ToolbarTool::LAYER_SPECIFIC,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_SUBSEQUENT")),
            _("Subsequent iterations"),
            _("Show/Hide iterations after highlighted"));

    tools.push_back(subsequent_tool);

    return tools;
}// CreateTools

void
XyUnconstrainedLayer::SetDrawSubsequentIterations(bool toggle)
{
    mDrawSubsequentIterations = toggle;
}// SetDrawSubsequentIterations
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
bool
XyUnconstrainedLayer::DrawsSubsequentIterations() const
{
    return mDrawSubsequentIterations;
}// DrawsSubsequentIterations

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////
