#include <precomp.h>

#include "../Logger.h"
#include "../Symbols.h"

#include "ui_util.h"

LOGGER(INFO);

wxPanel*
CreatePanelOfTwo(wxWindow* pParent, wxWindow*  pTopWindow, wxWindow* pBottomWindow)
{
    wxPanel* p_panel = new wxPanel(pParent, -1);

    wxBoxSizer* p_sizer = new wxBoxSizer(wxVERTICAL);

    // TODO: czy to konieczne?
    pTopWindow->Reparent(p_panel);
    pBottomWindow->Reparent(p_panel);

    p_sizer->Add(pTopWindow, 1, wxEXPAND, 0);
    p_sizer->Add(pBottomWindow, 1, wxEXPAND, 0);

    p_panel->SetAutoLayout(true);
    p_panel->SetSizer(p_sizer);
    p_sizer->Fit(p_panel);
    p_sizer->SetSizeHints(p_panel);

    return p_panel;
}// CreatePanelOfTwo

wxString
NumberFormat(double interval_length)
{
    // Ustal liczb cyfr po przecinku
    // Np.: dla przedziau 0,01 mamy log10(0,01) = -2, co daje nam przyrost co
    // 0,01, wic potrzebujemy 2 cyfry. Zaokrglanie w d jest potrzebne, gdy
    // mamy przedzia o dugoci nie bdcej wielokrotnoci 10.
    int precision = max<int>(0,
            static_cast<int>(-floor(log10(interval_length))));

    assert (precision >= 0);
    LOG(DEBUG, wxT("Creating format string for numbers. Precision: ") <<
                (precision));
    wxString number_format = wxString::Format(wxT("%%.%df"), precision);
    LOG(DEBUG, wxT("Number format: ") << number_format);

    return number_format;
}// NumberFormat


double
FindBestInterval(double                         initialSpan,
                 long long                      initialIntervalLength,
                 const IntervalCountLimiter&    intervalCountLimiter)
{
    const double MIN = 0.000000001;
    double span;
//    double plot_width = mPlotEnd - mPlotStart;

    assert (initialSpan > 0);
    assert (initialIntervalLength > 0);

    // Znajd wielokrotno 10 wiksz od szerokoci caego przedziau.
    for (span = MIN; (span < initialSpan) && (span < DBL_MAX/100); span *= 10)
        ;

    // Znajd podzia na tyle przedziaw, eby nie przekroczy maksimum.
    int dividers[] = { 1, 2, 5 };
    int divider_magnitude = 1;
    int divider_index = 1;  // Zaczynamy od 2

    double interval_length = initialIntervalLength;
    double next_interval_length = span;
    while (floor(initialSpan/next_interval_length) <
            intervalCountLimiter(next_interval_length))
    {
        interval_length = next_interval_length;

        // Dziel kolejno przez 2, 5, 10, 20, 50, 100...
        next_interval_length =
            span/(dividers[divider_index] * divider_magnitude);

        ++divider_index;

        if (divider_index == 3)
        {
            divider_index = 0;
            divider_magnitude *= 10;
        }

        LOG(DEBUG, wxT("Next interval length: ") << next_interval_length);
        LOG(DEBUG, wxT("Condition: ") << floor(initialSpan/next_interval_length));
    }

    return interval_length;
}// FindBestInterval

