#include <precomp.h>

#include <newmat/newmatrc.h>        // Nie ma w precomp.h, uywany tylko tu

#include "Logger.h"
#include "OptimizationError.h"
#include "Symbols.h"
#include "util.h"

LOGGER(WARNING)


const ColumnVector NullPoint(0);
const int MATRIX_PRECISION = 4; // TODO: konfigurowalne ?


int
Sgn(double d)
{
    if (d == 0) return 0;
    else
    {
        if (d < 0) return -1;
        else return 1;
    }
}

int
CompareIntegers(int *a, int *b)
{
    return *a - *b;
}

double
QuadraticForm(const SymmetricMatrix& matrix, const ColumnVector& point)
{
    assert(matrix.nrows() == point.nrows());

    Matrix      temp_matrix;
    RowVector   temp_vector;

    temp_vector = point.as_row() * matrix;

    temp_matrix = temp_vector * point;

    return temp_matrix.as_scalar();
}

void
RethrowNewmatError(const BaseException&     error,
                   std::auto_ptr<Result>    ap_result)
{
    throw OptimizationError(_("Internal computation error:\n")
            + wxString::FromAscii(error.what()), ap_result);
}// RethrowNewmatError

/*
 * Operatory wysyania macierzy do wxTextOutputStream
 */
wxTextOutputStream&
operator<<(wxTextOutputStream& os, const BaseMatrix& X)
{
   GeneralMatrix* gm = ((BaseMatrix&)X).Evaluate();
   operator<<(os, *gm);
   gm->tDelete();

   return os;
}

wxTextOutputStream&
operator<<(wxTextOutputStream& os, const GeneralMatrix& X)
{
   MatrixRow mr((GeneralMatrix*)&X, LoadOnEntry);
   int w = 10;
   int nr = X.Nrows();

   for (int i=1; i<=nr; i++)
   {
      int skip = mr.skip;
      int storage = mr.storage;
      Real* store = mr.data;
      skip *= w;

      os << wxT("[ ");
      while (skip-- > 0) os << wxT(" ");
      while (storage-- > 0)
      {
          if (*store >= 0) os << wxT(" ");
          os << *(store++) << wxT(" ");
      }
      os << wxT("]\n");
      mr.Next();
   }
   return os;
}

wxTextOutputStream&
operator<<(wxTextOutputStream& os, const std::vector<int>& vect)
{

    std::vector<int>::const_iterator iter;

    for (iter = vect.begin(); iter != vect.end(); iter++)
        os << *iter << wxT(" ");

    return os;
}

wxString&
operator<<(wxString& string, const ColumnVector& point)
{
    wxString format = MatrixNumberFormat(point);

    string << wxT("[ ");

    // Dodaj spacj po kadej liczbie.
    format << wxT(' ');

    for (int i = 1; i <= point.Nrows(); ++i)
    {
        string << wxString::Format(format, point(i));
    }

    string << wxT("]T");

    return string;
}// <<

wxString&
operator<<(wxString& string, const std::vector<int>& vect)
{
    std::vector<int>::const_iterator iter;

    for (iter = vect.begin(); iter != vect.end(); iter++)
        string << wxString::Format(wxT("%d "), *iter);

    return string;
}

// A pono mona uy wxLocale::GetInfo...
// TODO: zmieni przy okazji
wxChar
DiscoverDecimalSeparator()
{
    // W duchu prawdziwych odkrywcw...
    wxString text(wxString::Format(wxT("%3.1g"), 0.5));

    LOG(DEBUG, text);

    if ((text[1] == ',') || (text[1] == '.'))   return text[1];
    else
    {
        LOG(ERROR, wxT("Decimal separator doesn't seem to be neither period nor comma.\nFalling back to period."));
        return '.';
    }
}// DiscoverDecimalSeparator

int
DigitCount(double number, bool maxAsInfinity)
{
    if (number == 0.0)  return 1;
    else
    {
        number = abs(number);

        double logarithm = floor(log10(number));
        if (logarithm < 0 || (number == DBL_MAX && maxAsInfinity))
            return 1;
        else
            return static_cast<int>(logarithm + 1);
    }
}// DigitCount

int
DigitCount(const GeneralMatrix& matrix, bool maxAsInfinity)
{
    // Ustal najwiksz ilo cyfr przed przecinkiem (po jest staa).
    int digits = 0;
    MatrixRow mr((GeneralMatrix*)&matrix, LoadOnEntry);

    for (int i = 1; i <= matrix.Nrows(); ++i)
    {
        int storage = mr.storage;
        Real* store = mr.data;

        while (storage-- > 0)   digits = max<int>(digits,
                                         DigitCount(*store++, maxAsInfinity));

        mr.Next();
    }

    return digits;
}// DigitCount

wxString
MatrixNumberFormat(const GeneralMatrix& matrix,
                   bool                 maxAsInfinity)
{
    return wxString::Format(wxT("%% %d.%df"),
                DigitCount(matrix, maxAsInfinity) + 2 + MATRIX_PRECISION,
                MATRIX_PRECISION);

}// MatrixNumberFormat

wxString
PaddedInfinitySymbol(const GeneralMatrix& matrix)
{
    wxString string;

    string =  Symbols::GetInstance()[Symbols::INFINITY];
    string.Pad(DigitCount(matrix, true) + 2 + MATRIX_PRECISION - 1,
            wxT(' '), false);

    return string;
}// PaddedInfinitySymbol

size_t
RandomIndex(size_t  leftIndex, size_t  rightIndex)
{
    assert (leftIndex < rightIndex);

    return rand() % (rightIndex - leftIndex) + leftIndex;
}

double
RandomDouble(double leftBound, double rightBound)
{
    assert (leftBound < rightBound);

	return ((double)rand() / RAND_MAX) * (rightBound - leftBound) + leftBound;
}

double
RandomCauchy(double x_0, double lambda)
{
    return lambda * tan(M_PI * (RandomDouble(0.0, 1.0) - 0.5)) + x_0;
}
