#ifndef _util_h
#define _util_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "DoubleParameter.h"
#include "Result.h"

// FORWARD REFERENCES
//


// Staa oznaczajca brak punktu.
extern const ColumnVector NullPoint;

/// Funkcja signum
int                     Sgn(double d);

/// Funkcja porwnujca do sortowania wxArrayInt.
int CompareIntegers(int *a, int *b);

/** Oblicz form kwadratow.
 *
 * Funkcja oblicza form kwadratow macierzy A wg wzoru xT * A * x.
 *
 * @param matrix macierz.
 * @param point wektor.
 *
 * @return Forma kwadratowa macierzy.
 */
double                  QuadraticForm(const SymmetricMatrix&    matrix,
                                      const ColumnVector&       point);

/// Wyrzuca wyjtek z odp. komunikatem.
void                    RethrowNewmatError(const BaseException&    error,
                                           std::auto_ptr<Result>   ap_result);

/*
 * Operatory wysyania do strumienia macierzy dla wxTextOutputStream
 */
wxTextOutputStream&     operator<<(wxTextOutputStream&  os,
                                   const BaseMatrix&    X);

wxTextOutputStream&     operator<<(wxTextOutputStream&  os,
                                   const GeneralMatrix& X);

wxTextOutputStream&     operator<<(wxTextOutputStream& os,
                                   const std::vector<int>& vect);

// Wypisywanie wektora kolumnowego w postaci transponowanej do stringa.
wxString&               operator<<(wxString& string, const ColumnVector& point);

// Wpisywanie vectora liczb cakowitych do wxStringa.
wxString&     operator<<(wxString& string, const std::vector<int>& vect);

/// Sprawdza, jaki jest separator czci uamkowej (',' czy '.')
wxChar                  DiscoverDecimalSeparator();

/// Ustala liczb cyfr przed przecinkiem.
int                     DigitCount(double               number,
                                   bool                 maxAsInfinity = false);

/// Ustala najwiksz ilo cyfr przed przecinkiem.
int                     DigitCount(const GeneralMatrix& matrix,
                                   bool                 maxAsInfinity = false);

/// Generuje format dla wszystkich liczb w macierzy.
wxString                MatrixNumberFormat(
                            const GeneralMatrix& matrix,
                            bool                 maxAsInfinity = false);

// Zwraca symbol nieskoczonoci z formatowaniem uwzgldniajcym najdusz
// liczb w caej macierzy.
wxString                PaddedInfinitySymbol(
                            const GeneralMatrix& matrix);

/// dynamic_cast dla wskanikw w auto_ptrach.
template<typename T_to, typename T_from>
inline std::auto_ptr<T_to> aptr_dynamic_cast(std::auto_ptr<T_from> aPointer)
{
    auto_ptr<T_to> apNew(dynamic_cast<T_to*>(aPointer.release()));

    return apNew;
}

size_t                  RandomIndex(size_t  leftIndex, size_t  rightIndex);

double                  RandomDouble(double leftBound, double rightBound);

double                  RandomCauchy(double x_0, double lambda);

#endif  // _util_h
