#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct AlphaDivisionData
    {
    };

    typedef test_group<AlphaDivisionData> AlphaDivisionTestType;
    typedef AlphaDivisionTestType::object AlphaDivisionTest;
    AlphaDivisionTestType alpha_division_group("alpha division");


    // Zoty podzia samodzielny.
    template<>
    template<>
    void AlphaDivisionTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Alpha division standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(x - 7)^4"), *ap_problem);

            AlphaDivisionTester method;
            method.SetEpsilon(0.1);
            // method.SetAlpha(0.01);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << 7;
            LOG(INFO, wxT("Golden Division (standalone) result: ") << endl
                    << ap_result->FinalResult());
            ensure("Golden Division (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(), proper_result, 0.1));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // AlphaDivision samodzielna.
    template<>
    template<>
    void AlphaDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Alpha division in multidimensional problems"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(5 - x)^4"), *ap_problem);

            AlphaDivisionTester method;
            method.SetEpsilon(0.0001);
            //method.SetAlpha(0.8);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << 5;
            LOG(INFO, wxT("AlphaDivision (standalone) result: ") << endl
                    << ap_result->FinalResult());
            ensure("AlphaDivision (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(), proper_result, 0.1));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Alfa podzia dla metod wielu zmiennych
    template<>
    template<>
    void AlphaDivisionTest::test<3>()
    {
        try
        {

            LOG(INFO,wxT("Alpha division in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));

            AlphaDivisionTester tester;

            tester.SetEpsilon(0.1);

            result = tester.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("AlphaDivision result: ") << endl << result);
            ensure("AlphaDivision correct result", abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

}
