#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct BolzanoDivisionData
    {
    };

    typedef test_group<BolzanoDivisionData> BolzanoDivisionTestType;
    typedef BolzanoDivisionTestType::object BolzanoDivisionTest;
    BolzanoDivisionTestType bolzano_division_group("bolzano division");


    // Bolzano dla metod wielu zmiennych
    template<>
    template<>
    void BolzanoDivisionTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Bolzano division in multidimensional problems"));

            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));

            BolzanoDivisionTester tester;

            tester.SetEpsilon(0.1);

            // TODO: usun
            ensure("Differentiable", ap_problem->Function().IsDifferentiable());

            result = tester.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("BolzanoDivision result: ") << endl << result);
            ensure("BolzanoDivision correct result", abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Bolzano samodzielna.
    template<>
    template<>
    void BolzanoDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Bolzano division standalona"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(x-7)^2 + x"), *ap_problem);

            BolzanoDivisionTester tester;
            tester.SetEpsilon(0.01);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << 6.5;
            LOG(INFO, wxT("Bolzano (standalone) result: ") <<
                    ap_result->FinalResult());
            ensure("Bolzano (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(),
                                 proper_result, 0.01));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
