#include <src/precomp.h>

#include <tut.h>

#include <src/NMCircumferenceCondition.h>
#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/HyperbolicInnerPenalty.h>
#include <src/LogarithmicInnerPenalty.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/OuterPenalty.h>
#include <src/ParseError.h>
#include <src/NMCircumferenceCondition.h>
#include <src/ProblemWithPenalty.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/ValueCondition.h>

//#include <src/methods/Newton.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;

namespace tut
{

    struct ConstrainedOptimizationTestData
    {
    };

    typedef test_group<ConstrainedOptimizationTestData> ConstrainedOptimizationTestType;
    typedef ConstrainedOptimizationTestType::object ConstrainedOptimizationTest;
    ConstrainedOptimizationTestType constrained_optimization
                                            ("constrained optimization");

///////////////////////////// Outer penalties /////////////////////////////////


    template<>
    template<>
    void ConstrainedOptimizationTest::test<1>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<OuterPenalty> ap_penalty_function
                (new OuterPenalty(CountedPtr<const FunctionBase>(ap_function)));

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);
//
            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x - y - 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();
/// dodawanie funkcji kary do problemu i wzajemnie
//

            // TODO: zrobi SetProblem wirtualne i dla funkcji kary
            // zaimplementowa wersj, ktra ustawi go funkcji wewntrznej.
            // Wtedy bdzie mona wywali ponisz linijk.
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
            correct_result << 0.0 << -2.0;

/// przygotowywanie warunku stopu
//
            CustomValueCondition* pCondition = new CustomValueCondition;
            pCondition->rEpsilon() = 0.00001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<NewtonTester> ap_method(new NewtonTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(pCondition);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO, wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.5));

            LOG(INFO, wxT("test<1> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<2>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<OuterPenalty> ap_penalty_function
                (new OuterPenalty(CountedPtr<const FunctionBase>(ap_function)));

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::EQUAL);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint);

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();
/// dodawanie funkcji kary do problemu i wzajemnie
//

            // TODO: zrobi SetProblem wirtualne i dla funkcji kary
            // zaimplementowa wersj, ktra ustawi go funkcji wewntrznej.
            // Wtedy bdzie mona wywali ponisz linijk.
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
            correct_result = -1.0;

/// przygotowywanie warunku stopu
//
            CustomValueCondition* pCondition = new CustomValueCondition;
            pCondition->rEpsilon() = 0.00001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<NewtonTester> ap_method(new NewtonTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(pCondition);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO, wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.5));

            LOG(INFO, wxT("test<2> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>

///////////////////////////// Inner penalties /////////////////////////////////
/*
    template<>
    template<>
    void ConstrainedOptimizationTest::test<3>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<HyperbolicInnerPenalty> ap_penalty_function
                (new HyperbolicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x - y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
//            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
//            correct_result << 0.0 << 0.0;

/// przygotowywanie warunku stopu
//
            CustomValueCondition* pCondition = new CustomValueCondition;
            pCondition->rEpsilon() = 0.00001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<NewtonTester> ap_method(new NewtonTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(pCondition);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            bool failed = false;

            try
            {
                auto_ptr<const Result>
                    ap_result(ap_problem->StartSolve().release());
            }
            catch (const Error& error)
            {
                if (error.What().Contains
                            (wxString(wxT("Hessian is singular"))))
                failed = true;
                LOG(ERROR, error.What());
            }

            if(failed)
            {
                LOG(INFO,wxT("Newton didn't make it, as expected 2"));
            }

            ensure(failed);

/// sprawdzanie wyniku


//            LOG(INFO,wxT("Result:\n") << *ap_result);
//            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
//            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

//            ensure("Correct result",
//                    EqualVectors(ap_result->FinalResult(),correct_result,0.5));

            LOG(INFO,wxT("test<3> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>
*/

    template<>
    template<>
    void ConstrainedOptimizationTest::test<4>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<HyperbolicInnerPenalty> ap_penalty_function
                (new HyperbolicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x - y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
            correct_result << -1.0 << -1.0;

/// przygotowywanie warunku stopu
//
            ValueCondition* pCondition = new ValueCondition;
            pCondition->rEpsilon() = 0.0001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<SteepestDescentTester>
                                        ap_method(new SteepestDescentTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(pCondition);

            ap_method->SetLineSearchMethod
                    (dynamic_cast<LineSearch *>(Method::GetById
                    (DichotomousDivision::ClassId())->Clone().release()));

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n")
                                    << correct_result.as_row());
            LOG(DEBUG, wxT("FinalResult:\n")
                                    << ap_result->FinalResult().as_row());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.25));

            LOG(INFO,wxT("test<4> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<4>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<5>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<LogarithmicInnerPenalty> ap_penalty_function
                (new LogarithmicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x - y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
            correct_result << -1.0 << -1.0;

/// przygotowywanie warunku stopu
//
            ValueCondition* pCondition = new ValueCondition;
            pCondition->rEpsilon() = 0.0001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<SteepestDescentTester>
                                        ap_method(new SteepestDescentTester);
            //auto_ptr<PolakRibiereTester>
            //                            ap_method(new PolakRibiereTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(pCondition);

            auto_ptr<LineSearch> ap_line_search
                                            (new DichotomousDivisionTester());
            dynamic_cast<DichotomousDivisionTester&>(*ap_line_search).
                        SetEpsilon(0.000001);

            ap_method->SetLineSearchMethod(ap_line_search.release());

            //ap_method->SetLineSearchMethod
            //    (dynamic_cast<LineSearch*>(Method::GetById
            //        (DichotomousDivision::ClassId())->Clone().release()));
            //ap_method->SetLineSearchMethod
            //        (dynamic_cast<LineSearch *>(Method::GetById
            //        (Derivative::ClassId())->Clone().release()));

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.25));

            LOG(INFO,wxT("test<5> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<5>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<6>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<LogarithmicInnerPenalty> ap_penalty_function
                (new LogarithmicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x - y + 2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << -2.0 << -10.0;
            correct_result << -1.0 << -1.0;

/// przygotowywanie warunku stopu
//
            NMCircumferenceCondition* p_condition =
                                new NMCircumferenceCondition();
            p_condition->rEpsilon() = 0.001;

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<NelderMeadTester> ap_method(new NelderMeadTester);

            ap_method->SetStartingPoint(starting_point);
            ap_method->SetStopCondition(p_condition);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.25));

            LOG(INFO,wxT("test<6> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<6>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<7>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<LogarithmicInnerPenalty> ap_penalty_function
                (new LogarithmicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x-5"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x-2"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);

////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector correct_result(dimensions);
            correct_result << 2.0;

/// przygotowywanie warunku stopu
//

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<DichotomousDivisionTester> ap_method(new DichotomousDivisionTester);

            ap_method->SetA(1);
            ap_method->SetB(6);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),
                                 correct_result,
                                 0.25));

            LOG(INFO,wxT("test<7> finished."));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<7>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<8>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<HyperbolicInnerPenalty> ap_penalty_function
                (new HyperbolicInnerPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x-10"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x-4"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);

////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector correct_result(dimensions);
            correct_result << 4.0;

/// przygotowywanie warunku stopu
//

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<DichotomousDivisionTester> ap_method(new DichotomousDivisionTester);

            ap_method->SetA(1);
            ap_method->SetB(17);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),
                                 correct_result,
                                 0.25));

            LOG(INFO,wxT("test<8> finished."));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<8>


    template<>
    template<>
    void ConstrainedOptimizationTest::test<9>()
    {
        try
        {
            LOG(INFO,wxT("Function:\nx^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2"), *ap_variables, true));
            FunctionBase& r_function = *ap_function;

/// tworzenie funkcji kary
//
            auto_ptr<OuterPenalty> ap_penalty_function
                (new OuterPenalty(CountedPtr<const FunctionBase>
                                                (ap_function)));
            //ap_penalty_function->SetProblem(ap_problem.get());

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x-10"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function.release();
            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("x-4"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::GREATER_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);
            //ap_penalty_function->rOutOfBoundsReaction()
            //             = PenaltyFunction::EXCEPTION;

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);

////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector correct_result(dimensions);
            correct_result << 4.0;

/// przygotowywanie warunku stopu
//

/// tworzenie metody i dodawanie jej parametrw
//
            auto_ptr<DerivativeTester> ap_method(new DerivativeTester);

            ap_method->SetStartingPoint(-6);

            ap_problem->SetMethod(auto_ptr<Method>(ap_method));

/// AKCJA
//
            auto_ptr<const Result>
                ap_result(ap_problem->StartSolve().release());

/// sprawdzanie wyniku
//
            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),
                                 correct_result,
                                 0.25));

            LOG(INFO,wxT("test<8> finished."));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<9>

}
