#include <src/precomp.h>

#include <tut.h>

#include <src/Constraint.h>
#include <src/LinearParser.h>
#include <src/Logger.h>
#include <src/UnconstrainedProblem.h>

LOGGER(WARNING);

namespace tut
{
    struct ConstraintData
    {
    };

    typedef test_group<ConstraintData> ConstraintTestType;
    typedef ConstraintTestType::object ConstraintTest;
    ConstraintTestType constraint_group("constraints");

    // Ograniczenie rwnociowe
    template<>
    template<>
    void ConstraintTest::test<1>()
    {
        LinearParser parser;

        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase> ap_function_dummy(
                parser.CreateFunction(wxT("0"), *ap_variables, true));
        std::auto_ptr<ProblemBase> ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());


        ColumnVector x(2);

        auto_ptr<FunctionBase>
            ap_function(parser.CreateFunction(wxT("x + y"),
                        *ap_variables, true));

        ap_function->SetProblem(ap_problem.get());

        ap_problem->SetFunction(ap_function_dummy, ap_variables);

        Constraint constraint(auto_ptr<const FunctionBase>(ap_function),
                Constraint::EQUAL);

        x(1) =  1.0;
        x(2) = -1.0;

        ensure("x is valid", constraint.IsValid(x));

        x(1) = 0.0;
        x(2) = 10.0;

        ensure("x is invalid", !constraint.IsValid(x));

        ensure("value for x is proper", constraint.Value(x) == 10.0);
    }

    // Ograniczenie nierwnociowe
    template<>
    template<>
    void ConstraintTest::test<2>()
    {
        LinearParser parser;

        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase> ap_function_dummy(
                parser.CreateFunction(wxT("0"), *ap_variables, true));
        std::auto_ptr<ProblemBase> ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        ColumnVector x(2);

        auto_ptr<FunctionBase>
            ap_function(parser.CreateFunction(_T("x + y - 10"), *ap_variables,
                        true));

        ap_problem->SetFunction(ap_function_dummy, ap_variables);

        ap_function->SetProblem(ap_problem.get());

        Constraint constraint(auto_ptr<const FunctionBase>(ap_function),
                Constraint::LESS_EQUAL_THAN);


        x(1) = 1.0;
        x(2) = 5.0;

        ensure("x is valid", constraint.IsValid(x));

        x(1) = 2.0;
        x(2) = 10.0;

        LOG(INFO, constraint.Value(x));

        ensure("x is invalid", !constraint.IsValid(x));

        ensure("value for x is proper", constraint.Value(x) == 2.0);
    }
}
