#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/UnconstrainedProblem.h>

#include <src/methods/DichotomousDivision.h>
#include <src/methods/DFP.h>
#include <src/methods/UniformDivision.h>

#include <src/Logger.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;
/*
class DFPTester
    : public DFP
{
public:
    DFPTester::DFPTester()
    {
    }// DFPTester

    void SetStartingPoint(ColumnVector point)
    {
        mStartingPoint = point;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition *pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{
    struct DFPData
    {
    };

    typedef test_group<DFPData> DFPTestType;
    typedef DFPTestType::object DFPTest;
    DFPTestType dfp_group("dfp");

    // Test 1.
    template<>
    template<>
    void DFPTest::test<1>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);

            DFPTester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            SquaredCustomValueCondition* p_condition =
                new SquaredCustomValueCondition();
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("DFP test 1 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.2));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Funkcja Powella.
    template<>
    template<>
    void DFPTest::test<2>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            DFPTester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            CustomValueCondition* p_condition = new CustomValueCondition();
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
//          LOG(INFO, wxT("DFP test 2 result: ") << endl <<
//                    ap_result->FinalResult());
//          ColumnVector proper_result(4);
//          proper_result << 0 << 0 << 0 << 0;
//          ensure("Proper result", EqualVectors(
//              proper_result, ap_result->FinalResult(), 0.2));

        }
        catch (const Error& error)
        {
            ensure("Caught exception", true);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Problem 2.23.
    template<>
    template<>
    void DFPTest::test<3>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("a^10 + b^10 + c^10 + d^10 + e^10"), *ap_problem);

            DFPTester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 10 << 10 << 10 << 10 << 10;
            method.SetStartingPoint(starting_point);

            CustomValueCondition* p_condition = new CustomValueCondition();
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("DFP test 3 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(5);
            proper_result << 0 << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Funkcja Powella - warunek z odlegoci punktw.
    template<>
    template<>
    void DFPTest::test<4>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            DFPTester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition();
            p_condition->rEpsilon() = 0.1;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("DFP test 4 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(4);
            proper_result << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.5));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

}

