#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{
    struct DichotomousDivisionData
    {
    };

    typedef test_group<DichotomousDivisionData> DichotomousDivisionTestType;
    typedef DichotomousDivisionTestType::object DichotomousDivisionTest;
    DichotomousDivisionTestType
                            dichotomous_division_group("dichotomous division");


    // DichotomousDivision dla metod wielu zmiennych
    template<>
    template<>
    void DichotomousDivisionTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Dichotomous division in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5)^2")));
            DichotomousDivisionTester method;

            method.SetEpsilon(0.1);
            result = method.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("DichotomousDivision result: ") << endl << result);
            ensure("DichotomousDivision correct result", abs(result - 5) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // DichotomousDivision samodzielna.
    template<>
    template<>
    void DichotomousDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Dichotomous division standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(x - 7)^4"), *ap_problem);

            DichotomousDivisionTester method;
            method.SetEpsilon(0.1);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << 7;
            LOG(INFO, wxT("DichotomousDivision (standalone) result: ") << endl
                    << ap_result->FinalResult());
            ensure("DichotomousDivision (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(), proper_result, 0.1));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}

