#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{
    struct FibonacciDivisionData
    {
    };

    typedef test_group<FibonacciDivisionData> FibonacciDivisionTestType;
    typedef FibonacciDivisionTestType::object FibonacciDivisionTest;
    FibonacciDivisionTestType fibonacci_division_group("fibonacci division");


    // Fibonacci sequence generation
    template<>
    template<>
    void FibonacciDivisionTest::test<1>()
    {

        LOG(INFO,wxT("Fibonacci sequence generation"));

        FibonacciDivisionTester fibonacci_tester;

        vector<unsigned int>             correct_result;
        vector<unsigned int>             result;

        correct_result.push_back(1);
        correct_result.push_back(1);
        correct_result.push_back(2);
        correct_result.push_back(3);
        correct_result.push_back(5);
        correct_result.push_back(8);
        correct_result.push_back(13);
        correct_result.push_back(21);
        correct_result.push_back(34);
        correct_result.push_back(55);
        correct_result.push_back(89);
        correct_result.push_back(144);
        correct_result.push_back(233);
        correct_result.push_back(377);
        correct_result.push_back(610);
        correct_result.push_back(987);
        correct_result.push_back(1597);
        correct_result.push_back(2584);
        correct_result.push_back(4181);
        correct_result.push_back(6765);
        correct_result.push_back(10946);

        result = fibonacci_tester.GenerateFibonacciSequence(60, 150, 0.01);

        vector<unsigned int>::const_iterator iter;

        assert(correct_result.size() == result.size());

        LOG(DEBUG,wxT("no.  correct  calculated"));

        for (unsigned int i = 0; i < result.size(); ++i)
        {
            LOG(DEBUG,wxString::Format(wxT("%3d  %7d  %10d"),
                                       i,
                                       correct_result[i],
                                       result[i]));
        }

        ensure("Correct fibonacci number generation",correct_result == result);

        LOG(INFO,wxT("-------------------------------------------------"));

    }

    // FibonacciDivision dla metod wielu zmiennych
    template<>
    template<>
    void FibonacciDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Fibonacci division in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));
            FibonacciDivisionTester method;

            method.SetEpsilon(0.1);
            result = method.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("FibonacciDivision result: ") << endl << result);
            ensure("FibonacciDivision correct result", abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Fibonacci Division standalone
    template<>
    template<>
    void FibonacciDivisionTest::test<3>()
    {
        try
        {

            LOG(INFO,wxT("Fibonacci division standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(100 - x)^2"), *ap_problem);

            FibonacciDivisionTester tester;

            double accuracy = 0.01;
            tester.SetEpsilon(accuracy);
            tester.SetA(60);
            tester.SetB(150);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));

            ColumnVector    correct_result(1);
            correct_result = 100;

            LOG(DEBUG,wxT("Accuracy : ") << accuracy);
            LOG(DEBUG,wxT("Result   : ") << ap_result->FinalResult());

            ensure("Fibonacci division correct result",
                        fabs(correct_result(1) - (ap_result->FinalResult())(1)) <= accuracy);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Fibonacci Division standalone
    template<>
    template<>
    void FibonacciDivisionTest::test<4>()
    {
        try
        {

            LOG(INFO,wxT("Fibonacci division standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(100 - x)^2"), *ap_problem);

            FibonacciDivisionTester tester;

            double accuracy = 35;
            tester.SetEpsilon(accuracy);
            tester.SetA(60);
            tester.SetB(150);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));

            ColumnVector    correct_result(1);
            correct_result = 100;

            LOG(DEBUG,wxT("Accuracy : ") << accuracy);
            LOG(DEBUG,wxT("Result   : ") << ap_result->FinalResult());

            ensure("Fibonacci division correct result",
                        fabs(correct_result(1) - (ap_result->FinalResult())(1)) <= accuracy);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
