#include <src/precomp.h>

#include <tut.h>

#include <src/NonlinearParser.h>
#include <src/SectionLineSearch.h>

#include "tests_util.h"

LOGGER(INFO);

using namespace std;

class FindInitialSectionTester
    : SectionLineSearch
{
public:

    void Test(const UnivariateFunction&  function,
              double&                    rLeft,
              double&                    rRight,
              double                     initialSectionLength) const
    {
        FindInitialSection(function, rLeft, rRight, initialSectionLength);
    }

private:
    virtual auto_ptr<Result>    Solve(const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                                const throw(OptimizationError)
    {
        return auto_ptr<Result>();
    }

    virtual auto_ptr<Method>    Clone() const
    {
        return auto_ptr<Method>(NULL);
    }

    virtual const wxString&     Name() const
    {
        return mName;
    }
    virtual wxString&           rName()
    {
        return mName;
    }

    virtual MethodIdType        Id() const
    {
        return 0;
    }
private:
    static wxString mName;
};

wxString FindInitialSectionTester::mName(wxT("FindInitialSectionTester"));

namespace tut
{

    struct FindInitialSectionData
    {
        FindInitialSectionTester tester;
    };

    typedef test_group<FindInitialSectionData> FindInitialSectionTestType;
    typedef FindInitialSectionTestType::object FindInitialSectionTest;
    FindInitialSectionTestType find_initial_section_group("FindInitialSection");


    // Unimodalna, start nad minimum
    template<>
    template<>
    void FindInitialSectionTest::test<1>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(wxT("x^2 - x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -10", fabs(left + 10) < 0.00001);
        ensure("Right bound equals 10", fabs(right - 10) < 0.00001);
    }

    // Unimodalna, minimum po stronie dodatniej.
    template<>
    template<>
    void FindInitialSectionTest::test<2>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 - 12 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals 0", fabs(left) < 0.00001);
        ensure("Right bound equals 10", fabs(right - 10) < 0.00001);
    }

    // Unimodalna, minimum po stronie ujemnej.
    template<>
    template<>
    void FindInitialSectionTest::test<3>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 + 12 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -40", fabs(left + 40) < 0.00001);
        ensure("Right bound equals 10", fabs(right - 10) < 0.00001);
    }

    // Unimodalna, minimum po stronie ujemnej, wicej ni jeden krok
    template<>
    template<>
    void FindInitialSectionTest::test<4>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 + 40 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -40", fabs(left + 40) < 0.00001);
        ensure("Right bound equals 10", fabs(right - 10) < 0.00001);
    }

    template<>
    template<>
    void FindInitialSectionTest::test<5>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^4 - 100*x^2 - x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals 0", fabs(left) < 0.00001);
        ensure("Right bound equals 10", fabs(right - 10) < 0.00001);
    }

// Testy dla drugiej metody
#if 0
    // Unimodalna, start nad minimum (przesunie si w stron dodatniej)
    template<>
    template<>
    void FindInitialSectionTest::test<1>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(wxT("x^2 - x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -5", fabs(left + 5) < 0.00001);
        ensure("Right bound equals 25", fabs(right - 25) < 0.00001);
    }

    // Unimodalna, minimum po stronie dodatniej.
    template<>
    template<>
    void FindInitialSectionTest::test<2>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 - 12 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -5", fabs(left + 5) < 0.00001);
        ensure("Right bound equals 25", fabs(right - 25) < 0.00001);
    }

    // Unimodalna, minimum po stronie ujemnej.
    template<>
    template<>
    void FindInitialSectionTest::test<3>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 + 12 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -25", fabs(left + 25) < 0.00001);
        ensure("Right bound equals 5", fabs(right - 5) < 0.00001);
    }

    // Unimodalna, minimum po stronie ujemnej, wicej ni jeden skok.
    template<>
    template<>
    void FindInitialSectionTest::test<4>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^2 + 60 * x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -65", fabs(left + 65) < 0.00001);
        ensure("Right bound equals 5", fabs(right - 5) < 0.00001);
    }

    template<>
    template<>
    void FindInitialSectionTest::test<5>()
    {
        auto_ptr<const ProblemBase> ap_problem(ProblemWithFunction(
                    wxT("x^4 - 100*x^2 - x")));
        const UnivariateFunction& function = dynamic_cast<const UnivariateFunction&>
            (ap_problem->Function());

        double left;
        double right;

        tester.Test(function, left, right, 10);

        LOG(DEBUG, wxT("left: ") << left << wxT(", right: ") << right);
        ensure("Left bound equals -5", fabs(left + 5) < 0.00001);
        ensure("Right bound equals 25", fabs(right - 25) < 0.00001);
    }
#endif
}
