#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/ValueCondition.h>
#include <src/UnconstrainedProblem.h>

#include <src/methods/FletcherReeves.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class FletcherReevesTester
    : public FletcherReeves
{
public:

    FletcherReevesTester()
        : FletcherReeves()
    {
    }

    void
    SetModification(bool modification)
    {
        mModification = modification;
    }

    void
    SetStartingPoint(const ColumnVector& startingPoint)
    {
        mStartingPoint = startingPoint;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{

    struct FletcherReevesData
    {
    };

    typedef test_group<FletcherReevesData> FletcherReevesTestType;
    typedef FletcherReevesTestType::object FletcherReevesTest;
    FletcherReevesTestType fletcher_reeves_group("fletcher_reeves");

    template<>
    template<>
    void FletcherReevesTest::test<1>()
    {

        try
        {
            LOG(INFO,wxT("test<1>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            FletcherReevesTester method;

            method.SetLineSearchMethod
                    (dynamic_cast<LineSearch *>(Method::GetById
                    (DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition();
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.001));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>


    template<>
    template<>
    void FletcherReevesTest::test<2>()
    {

        try
        {
            LOG(INFO,wxT("test<2>\nTest run with x^6 + 2*y^4 + z^2"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("x^6 + 2*y^4 + z^2"),
                                 *ap_problem);


            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                    (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);

            FletcherReevesTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition();
            pCondition->rEpsilon() = 0.0001;
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 8.0;
            correct_result = 0.0;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.1));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const OptimizationError& error)
        {
            LOG(ERROR, wxT("Optimization error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>

    template<>
    template<>
    void FletcherReevesTest::test<3>()
    {

        try
        {
            LOG(INFO,wxT("test<3>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x\nModification used"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            FletcherReevesTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition();
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            method.SetModification(true);
            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.001));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>


    template<>
    template<>
    void FletcherReevesTest::test<4>()
    {

        try
        {
            LOG(INFO,wxT("test<4>\nTest run with x^6 + 2*y^4 + z^2\nModification used"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("x^6 + 2*y^4 + z^2"),
                                 *ap_problem);

            FletcherReevesTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition();
            pCondition->rEpsilon() = 0.0001;
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 8.0;
            correct_result = 0.0;

            method.SetModification(true);
            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.1));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const OptimizationError& error)
        {
            LOG(ERROR, wxT("Optimization error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<4>

    // Test 2 startujcy z minimum.
    template<>
    template<>
    void FletcherReevesTest::test<5>()
    {

        try
        {
            LOG(INFO,wxT("test<5>\nTest run with x^6 + 2*y^4 + z^2"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("x^6 + 2*y^4 + z^2"),
                                 *ap_problem);


            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                    (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);

            FletcherReevesTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition();
            pCondition->rEpsilon() = 0.0001;
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result = 0.0;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.1));

            LOG(INFO,wxT("test<5> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const OptimizationError& error)
        {
            LOG(ERROR, wxT("Optimization error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<5>

    // Test 3 z warunkiem na warto funkcji.
    template<>
    template<>
    void FletcherReevesTest::test<6>()
    {

        try
        {
            LOG(INFO,wxT("test<6>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x\nModification used"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            FletcherReevesTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            ValueCondition* pCondition = new ValueCondition();
            method.SetStopCondition(pCondition);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            method.SetModification(true);
            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.001));

            LOG(INFO,wxT("test<6> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<6>

}

