#include <src/precomp.h>

#include <tut.h>

#include <src/Constraint.h>
#include <src/LinearFunction.h>
#include <src/LinearParser.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/OuterPenalty.h>
#include <src/UnconstrainedProblem.h>
#include <src/DirectionalUnivariateAdapter.h>
#include "tests_util.h"

LOGGER(INFO);

using namespace std;

namespace tut
{
    struct FunctionData
    {
    };

    typedef test_group<FunctionData> FunctionTestType;
    typedef FunctionTestType::object FunctionTest;
    FunctionTestType function_group("functions");

    // TODO: Zrobi wicej testw.

/*
    // Zewntrzna funkcja kary.
    template<>
    template<>
    void FunctionTest::test<2>()
    {
        LinearParser l_parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        auto_ptr<FunctionBase> ap_function(
                l_parser.CreateFunction(wxT("0*x + 0*y + 0.0"), *ap_variables,
                    true));

        ap_function->SetProblem(ap_problem.get());

        auto_ptr<OuterPenalty> ap_penalty(new
                OuterPenalty(auto_ptr<const FunctionBase>(ap_function)));

        ap_function = l_parser.CreateFunction(wxT("x + y"), *ap_variables,
                false);
        ap_function->SetProblem(ap_problem.get());

        Constraint constraint1 =
            Constraint(auto_ptr<const FunctionBase>(ap_function),
                       Constraint::LESS_EQUAL_THAN);

        ap_penalty->AddConstraint(constraint1);
        ap_problem->SetFunction(auto_ptr<FunctionBase>(ap_penalty),
                ap_variables);

        // TODO: sprawdzi wspczynniki

        ColumnVector x(2);

        ensure("2 variables", x.nrows() == 2);

        double result1, result2;

        x(1) = 1.0;
        x(2) = 1.0;

        result1 = ap_problem->Function().Evaluate(x);

        x(1) = -1.0;
        x(2) = -2.0;

        result2 = ap_problem->Function().Evaluate(x);

        LOG(DEBUG,wxT("Result in [ 1.0, 1.0]: ") << result1);
        LOG(DEBUG,wxT("Result in [-1.0, 2.0]: ") << result2);

        ensure("Evaluate returns 4.0", result1 == 4.0);
        ensure("Evaluate returns 0.0", result2 == 0.0);

    }
*/
    // Funkcja rniczkowalna, pochodne jednej zmiennej
    template<>
    template<>
    void FunctionTest::test<3>()
    {
        try
        {
            NonlinearParser parser;
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            auto_ptr<FunctionBase>  ap_function(
                    parser.CreateFunction(wxT("x + 2*y"), *ap_variables, true));
            std::auto_ptr<ProblemBase> ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            ap_problem->SetFunction(ap_function, ap_variables);
            const FunctionBase& r_function = ap_problem->Function();

            ensure("Is differentiable", r_function.IsDifferentiable());


            ColumnVector x(2);
            double derivative;

            x << 0.0 << 1.0;

            derivative = r_function.PartialDerivative(x,
                    ap_problem->Variables().Index(wxT("x")));

            ensure("Value of partial derivative on x in 0 equal to 1.0",
                     (derivative > 0.99999) && (derivative < 1.00001));

            derivative = r_function.PartialDerivative(x,
                    ap_problem->Variables().Index(wxT("y")));
            ensure("Value of partial derivative on y in 1 equal to 2.0",
                     (derivative > 1.99999) && (derivative < 2.00001));

        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxString::FromAscii(error.what()));
        }
    }

    // Funkcja rniczkowalna, gradient
    template<>
    template<>
    void FunctionTest::test<4>()
    {
        NonlinearParser parser;
        VariableNames variables;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase>  ap_function(
                parser.CreateFunction(wxT("x + 2*y"), *ap_variables, true));
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        ap_problem->SetFunction(ap_function, ap_variables);
        ColumnVector x(2);
        ColumnVector proper_g(2);

        const FunctionBase& function = ap_problem->Function();

        ensure("Is differentiable", function.IsDifferentiable());

        try
        {
            proper_g << 1.0 << 2.0;
            x << 0.0 << 1.0;

            ColumnVector g = function.Gradient(x);
            ensure("Value of gradient is [1, 2]", EqualMatrices(g, proper_g));
        }
        catch (const Exception& error)
        {
            LOG(DEBUG, wxString::FromAscii(error.what()));
            ensure("No errors", false);
        }
    }

    // Adapter funkcji jednej zmiennej
    template<>
    template<>
    void FunctionTest::test<5>()
    {
        LinearParser parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        double value;

        ColumnVector x(2);
        ColumnVector d(2);

        x << 0.0 << 0.0;
        d << 1.0 << 1.0;

        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        auto_ptr<FunctionBase>
            ap_function(parser.CreateFunction(wxT("x + y"), *ap_variables, true));

        ap_problem->SetFunction(ap_function, ap_variables);

        const FunctionBase& r_function = ap_problem->Function();

        d = d/d.norm_Frobenius();
        auto_ptr<const DirectionalUnivariateAdapter>
            ap_univariate(new DirectionalUnivariateAdapter(r_function, x, d));

        ColumnVector point(1);
        point << 2.0;

        value = ap_univariate->Evaluate(point);
        LOG(DEBUG, wxT("DirectionalUnivariateAdapter: Value in 2.0: ")
                                                            << value);
        ensure("Correct result of evaluation",
                (value >= 2.82840) && (value <= 2.82844));

        // TODO: usun...
        {
            const UnivariateFunction* p_univariate = ap_univariate.get();
            p_univariate->Evaluate(2.0);
        }
        // TODO: ...dotd

        ensure("Both Evaluate()s return the same value",
                value == ap_univariate->Evaluate(2.0));

        value = ap_univariate->PartialDerivative(point, 0);
        LOG(DEBUG, wxT("DirectionalUnivariateAdapter: Derivative in 2.0: ")
                                                            << value);
        ensure("Correct derivative", (value >= 1.41420) && (value <= 1.41422));
    }

    // TODO: test na pochodne mieszane, poprawno wyniku + odwracalno
    // kolejnoci rniczkowania

    // Hesjan
    template<>
    template<>
    void FunctionTest::test<6>()
    {
        NonlinearParser parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("8*x^2 + 4*x*y + 5*y^2"),
                    *ap_variables, true));
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        ap_problem->SetFunction(ap_function, ap_variables);
        const FunctionBase& r_function = ap_problem->Function();

        ColumnVector x(2);
        x << 10.0 << 10.0;
        SquareMatrix proper_hessian(x.nrows());
        proper_hessian << 16 << 4 << 4 << 10;

        LOG(DEBUG,_T("F(x,y)  = 8*x^2 + 4*x*y + 5*y^2"));
        LOG(DEBUG,_T("(x0,y0) = \n") << x);
        LOG(DEBUG,_T("Oczekiwany hesjan =\n") << proper_hessian)

        SquareMatrix hessian = r_function.Hessian(x);

        LOG(DEBUG,_T("Otrzymany hesjan  =\n") << hessian);

        ensure("Value of hessian is:\n[16, 4 ]\n[ 4, 10]",
               EqualMatrices(hessian, proper_hessian, 0.001));

    }

    // Bdy oblicze
    template<>
    template<>
    void FunctionTest::test<7>()
    {
        NonlinearParser parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase> ap_function;
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        bool gotException;

        try
        {
            // Dzielenie przez zero:
            ap_function =
                parser.CreateFunction(wxT("4.5/0"), *ap_variables, true);

            ap_problem->SetFunction(ap_function, ap_variables);
            const FunctionBase& function1 = ap_problem->Function();

            gotException = false;

            try
            {
                function1.Evaluate(ColumnVector(0));
            }
            catch (const EvaluationError& error)
            {
                gotException = true;
            }
            ensure("Expected error occured, check 1", gotException);

            // Obliczenie wartoci poza dziedzin
            ap_variables.reset(new VariableNames);
            ap_function =
                parser.CreateFunction(wxT("sqrt(x)"), *ap_variables, true);

            ap_problem->SetFunction(ap_function, ap_variables);
            const FunctionBase& function2 = ap_problem->Function();

            ColumnVector x(1);

            x << 4;

            ensure("Proper value", function2.Evaluate(x) == 2);

            x << -2;

            gotException = false;
            try
            {
                function2.Evaluate(x);
            }
            catch (const EvaluationError& error)
            {
                gotException = true;
            }

            ensure("Expected error occured, check 2", gotException);
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("No exceptions", false);
        }
    }

    // Funkcja bez zmiennych
    template<>
    template<>
    void FunctionTest::test<8>()
    {
        NonlinearParser parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<FunctionBase> ap_function;
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        ap_function = parser.CreateFunction(wxT("5/2"), *ap_variables, true);
        ap_problem->SetFunction(ap_function, ap_variables);

        ensure("Proper value",
                ap_problem->Function().Evaluate(ColumnVector(0)) == 2.5);
    }
}
