#include <src/precomp.h>

#include <tut.h>

#include <src/FunctionTree.h>
#include <src/VariableNames.h>

namespace tut
{
    struct FunctionTreeData
    {
    };

    typedef test_group<FunctionTreeData> FunctionTreeTestType;
    typedef FunctionTreeTestType::object FunctionTreeTest;
    FunctionTreeTestType function_tree_group("function tree");

    double add(double a, double b)
    {
        return a + b;
    }

    double negate(double a)
    {
        return -a;
    }

    // Tworzenie wartoci i zmiennej oraz obliczanie.
    template<>
    template<>
    void FunctionTreeTest::test<1>()
    {
        FunctionTree value(5.0);
        FunctionTree variable(wxT("a"));

        VariableNames variables;
        variables.Add(wxT("a"));

        ColumnVector point(1);

        ensure(value.Evaluate(point, variables) == 5.0);

        point(1) = 3.0;
        ensure(variable.Evaluate(point, variables) == 3.0);

        point(1) = -6.3;
        ensure(variable.Evaluate(point, variables) == -6.3);
    }

    // Pojedyncze operatory
    template<>
    template<>
    void FunctionTreeTest::test<2>()
    {
        FunctionTree addition(add,
                std::auto_ptr<const FunctionTree>(new FunctionTree(5.0)),
                std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("a"))));
        FunctionTree negation(negate,
                std::auto_ptr<const FunctionTree>(new FunctionTree(3.5)));

        VariableNames variables;
        variables.Add(wxT("a"));

        ColumnVector point(1);
        point(1) = 3.0;

        ensure(negation.Evaluate(point, variables) == -3.5);
        ensure(addition.Evaluate(point, variables) == 8.0);
    }

    // Bardziej zoone drzewo, wiele zmiennych.
    template<>
    template<>
    void FunctionTreeTest::test<3>()
    {
        // 5 + sin(-a + b) + c
        FunctionTree function(add,
          std::auto_ptr<const FunctionTree>(new FunctionTree(5.0)),
          std::auto_ptr<const FunctionTree>(new FunctionTree(
            add,
            std::auto_ptr<const FunctionTree>(new FunctionTree(
              sin,
              std::auto_ptr<const FunctionTree>(new FunctionTree(
                add,
                std::auto_ptr<const FunctionTree>(new FunctionTree(
                  negate,
                  std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("a"))))),
                std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("b"))))))),
            std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("c"))))));


        VariableNames variables;
        variables.AddIfNotPresent(wxT("a"));
        variables.AddIfNotPresent(wxT("b"));
        variables.AddIfNotPresent(wxT("c"));

        ColumnVector point(3);

        point << 0.0 << 0.0 << 0.0;

        ensure(function.Evaluate(point, variables) == 5.0);

        point(1) = 1.0;

        double result = function.Evaluate(point, variables);
        ensure((result > 4.158) && (result < 4.159));

        point(2) = -2.14;
        result = function.Evaluate(point, variables);
        ensure((result > 4.99) && (result < 5.0));

        point(3) = -4.0;
        result = function.Evaluate(point, variables);
        ensure((result > 0.99) && (result < 1.0));
    }

    // Test ContainsUnknown [variables]
    template<>
    template<>
    void FunctionTreeTest::test<4>()
    {
        // 5 + sin(-a + b) + c
        FunctionTree function(add,
          std::auto_ptr<const FunctionTree>(new FunctionTree(5.0)),
          std::auto_ptr<const FunctionTree>(new FunctionTree(
            add,
            std::auto_ptr<const FunctionTree>(new FunctionTree(
              sin,
              std::auto_ptr<const FunctionTree>(new FunctionTree(
                add,
                std::auto_ptr<const FunctionTree>(new FunctionTree(
                  negate,
                  std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("a"))))),
                std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("b"))))))),
            std::auto_ptr<const FunctionTree>(new FunctionTree(wxT("c"))))));

        VariableNames variables;
        variables.AddIfNotPresent(wxT("a"));
        variables.AddIfNotPresent(wxT("b"));

        ensure(function.ContainsUnknown(variables) == true);

        variables.AddIfNotPresent(wxT("c"));
        ensure(function.ContainsUnknown(variables) == false);
    }
}
