#include <src/precomp.h>

#include <tut.h>
#include <wx/mstream.h>
#include <wx/fileconf.h>

#include <src/History.h>
#include <src/Logger.h>

LOGGER(INFO);

using namespace std;

namespace tut
{
    struct HistoryData
    {
    };

    typedef test_group<HistoryData> HistoryTestType;
    typedef HistoryTestType::object HistoryTest;
    HistoryTestType history_group("history");


    // Dodanie tekstu (2 razy)
    template<>
    template<>
    void HistoryTest::test<1>()
    {
        History history;
        wxArrayString proper_array;

        history.InsertUserString(wxT("abcd"));

        proper_array.Add(wxT("abcd"));

        LOG(DEBUG, static_cast<int>(history.Strings().GetCount()));

        ensure("First string added", history.Strings() == proper_array);

        history.InsertUserString(wxT("abcd"));

        ensure("First string added", history.Strings() == proper_array);
    }

    // Zapenienie historii
    template<>
    template<>
    void HistoryTest::test<2>()
    {
        History history;
        wxArrayString proper_array;

        history.InsertUserString(wxT("test9"));
        history.InsertUserString(wxT("test8"));
        history.InsertUserString(wxT("test7"));
        history.InsertUserString(wxT("test6"));
        history.InsertUserString(wxT("test5"));
        history.InsertUserString(wxT("test4"));
        history.InsertUserString(wxT("test3"));
        history.InsertUserString(wxT("test2"));
        history.InsertUserString(wxT("test1"));
        history.InsertUserString(wxT("test0"));

        proper_array.Add(wxT("test0"));
        proper_array.Add(wxT("test1"));
        proper_array.Add(wxT("test2"));
        proper_array.Add(wxT("test3"));
        proper_array.Add(wxT("test4"));
        proper_array.Add(wxT("test5"));
        proper_array.Add(wxT("test6"));
        proper_array.Add(wxT("test7"));
        proper_array.Add(wxT("test8"));
        proper_array.Add(wxT("test9"));

        ensure("First 10 strings", history.Strings() == proper_array);

        history.InsertUserString(wxT("test"));

        proper_array.RemoveAt(9);

        proper_array.Insert(wxT("test"), 0);

        ensure("After overfilling", history.Strings() == proper_array);
    }

    template<>
    template<>
    void HistoryTest::test<3>()
    {
        History history;
        wxArrayString proper_array;

        history.InsertUserString(wxT("test9"));
        history.InsertUserString(wxT("test8"));
        history.InsertUserString(wxT("test7"));
        history.InsertUserString(wxT("test6"));
        history.InsertUserString(wxT("test5"));
        history.InsertUserString(wxT("test4"));
        history.InsertUserString(wxT("test3"));
        history.InsertUserString(wxT("test2"));
        history.InsertUserString(wxT("test1"));
        history.InsertUserString(wxT("test0"));
        history.InsertPermanentString(wxT("permanent"));

        proper_array.Add(wxT("test0"));
        proper_array.Add(wxT("test1"));
        proper_array.Add(wxT("test2"));
        proper_array.Add(wxT("test3"));
        proper_array.Add(wxT("test4"));
        proper_array.Add(wxT("test5"));
        proper_array.Add(wxT("test6"));
        proper_array.Add(wxT("test7"));
        proper_array.Add(wxT("test8"));
        proper_array.Add(wxT("test9"));
        proper_array.Add(wxT("permanent"));


        ensure("First 10 strings", history.Strings() == proper_array);

        history.InsertUserString(wxT("permanent"));

        proper_array.RemoveAt(10);
        proper_array.Insert(wxT("permanent"), 0);

        ensure("After using permanent", history.Strings() == proper_array);

        history.InsertUserString(wxT("test"));

        proper_array.RemoveAt(10);
        proper_array.Insert(wxT("test"), 0);

        ensure("After overfilling", history.Strings() == proper_array);
    }

    // Zapis i odczyt
    template<>
    template<>
    void HistoryTest::test<4>()
    {
        wxFileConfig* p_config = new wxFileConfig();
        wxMemoryOutputStream memory_output;
        History history;

        history.InsertUserString(wxT("test8"));
        history.InsertUserString(wxT("test7"));
        history.InsertUserString(wxT("test6"));
        history.InsertUserString(wxT("test5"));
        history.InsertUserString(wxT("test4"));
        history.InsertUserString(wxT("test3"));
        history.InsertUserString(wxT("test2"));
        history.InsertUserString(wxT("test1"));
        history.InsertUserString(wxT("test0"));
        history.InsertPermanentString(wxT("permanent"));

        p_config->SetPath(wxT("/"));
        history.Save(p_config);
        p_config->SetPath(wxT("/"));
        p_config->Save(memory_output);

        delete p_config;

        History history2;

        wxMemoryInputStream memory_input(memory_output);


        p_config = new wxFileConfig(memory_input);

        p_config->SetPath(wxT("/"));
        history2.Load(p_config);

        LOG(DEBUG, static_cast<int>(history2.Strings().GetCount()));
        wxArrayString   proper_history = history.Strings();
        proper_history.RemoveAt(9); // Usu 'permanent', on nie jest zapisywany

        LOG(DEBUG, static_cast<int>(history2.Strings().GetCount()));
        ensure("Loading successful", proper_history == history2.Strings());

        delete p_config;
    }

}
