#include <src/precomp.h>

#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/CustomValueCondition.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/Marquardt.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class MarquardtTester
    : public Marquardt
{
public:

    MarquardtTester()
        : Marquardt()
    {
    }

    void
    SetStartingPoint(const ColumnVector& startingPoint)
    {
        mStartingPoint = startingPoint;
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{

    struct MarquardtData
    {
    };

    typedef test_group<MarquardtData> MarquardtTestType;
    typedef MarquardtTestType::object MarquardtTest;
    MarquardtTestType marquardt_group("marquardt's");

    template<>
    template<>
    void MarquardtTest::test<1>()
    {

        try
        {
            LOG(INFO,_T("test<1>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(_T("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);


            MarquardtTester markwardt_tester;


            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            markwardt_tester.SetStartingPoint(starting_point);

            CustomValueCondition* pCondition = new CustomValueCondition;
            markwardt_tester.SetStopCondition(pCondition);

            auto_ptr<const Result> ap_result
                (markwardt_tester.StartOptimize(*ap_problem).release());

            LOG(INFO,  _T("Result:\n") << *ap_result);
            LOG(DEBUG, _T("CorrectResult:\n") << correct_result);
            LOG(DEBUG, _T("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.001));

            LOG(INFO,_T("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, _T("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>

    // Test 1 z warunkiem na warto funkcji.
    template<>
    template<>
    void MarquardtTest::test<2>()
    {

        try
        {
            LOG(INFO,_T("test<1>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(_T("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);


            MarquardtTester markwardt_tester;


            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            markwardt_tester.SetStartingPoint(starting_point);

            ValueCondition* pCondition = new ValueCondition;
            pCondition->rEpsilon() = 0.000001;
            markwardt_tester.SetStopCondition(pCondition);

            auto_ptr<const Result> ap_result
                (markwardt_tester.StartOptimize(*ap_problem).release());

            LOG(INFO,  _T("Result:\n") << *ap_result);
            LOG(DEBUG, _T("CorrectResult:\n") << correct_result);
            LOG(DEBUG, _T("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.001));

            LOG(INFO,_T("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, _T("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>
}

