#include <src/precomp.h>

#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/Logger.h>
#include <src/LinearParser.h>
#include <src/NMBestWorstCondition.h>
#include <src/NMCircumferenceCondition.h>
#include <src/NMPreviousWorstCondition.h>
#include <src/NMWorstReplacementCondition.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/NelderMead.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class NelderMeadTester
    : public NelderMead
{
public:

    NelderMeadTester()
        : NelderMead()
    {
    }


    const vector<PointWithValue>
    TestInitialSimplexGeneration(const  ColumnVector&   centralPoint,
                                        double          sideLength,
                                 const  FunctionBase*   function)
    {

        NelderMead::Simplex simplex(centralPoint,
                                    DoubleParameter(sideLength, 0, true),
                                    function);

        return simplex.rPoints();
    }


    ColumnVector
    TestCentroidFinding(vector<ColumnVector>& initialSimplex,
                        const ProblemBase& problem)
    {

        const FunctionBase& function    = problem.Function();
        ColumnVector        result_point;

        NelderMead::Simplex simplex(initialSimplex,&function);

        return simplex.Centroid(0, simplex.Size() - 2).mPoint;

    }


    vector<ColumnVector>
    TestShrinking(vector<ColumnVector> initialSimplex,
                  const ProblemBase& problem,
                  double shrinkageCoefficient)
    {
        const FunctionBase& function    = problem.Function();
        vector<ColumnVector> result_vector;
        size_t  i;


        NelderMead::Simplex simplex(initialSimplex,&function);

        simplex.ShrinkTowardBest(shrinkageCoefficient);


        for (i = 0; i < simplex.rPoints().size(); ++i)
        {
            LOG(DEBUG, wxT("P") << static_cast<int>(i) << wxT(" = ")
                    << simplex.rPoints()[i].mPoint.as_row());
            LOG(DEBUG,wxT("Value = ") << simplex.rPoints()[i].mValue);
            result_vector.push_back(simplex.rPoints()[i].mPoint);
        }


        return result_vector;
    }

    void
    SetReflectionCoefficient(double d)
    {
        mReflectionCoefficient = d;
    }

    void
    SetExpansionCoefficient(double d)
    {
        mExpansionCoefficient = d;
    }

    void
    SetContractionCoefficient(double d)
    {
        mContractionCoefficient = d;
    }

    void
    SetShrinkageCoefficient(double d)
    {
        mShrinkageCoefficient = d;
    }

    void SetStartingPoint(ColumnVector& point)
    {
        mStartingPoint  = point;
    }

    void SetSideLength(double length)
    {
        mSideLength  = length;
    }

    void SetStopCondition(const NMStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }
};
*/
namespace tut
{

    struct NelderMeadData
    {
    };

    typedef test_group<NelderMeadData> NelderMeadTestType;
    typedef NelderMeadTestType::object NelderMeadTest;
    NelderMeadTestType nelder_mead_group("nelder-mead");

    template<>
    template<>
    void NelderMeadTest::test<1>()
    {
        try
        {

            LOG(INFO,_T("test<1>\nTesting simplex generation fromula."));
            LOG(INFO,_T("This test has no correctness check."));

            LinearParser    parser;
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            auto_ptr<FunctionBase> ap_function_dummy(
                parser.CreateFunction(wxT("0"), *ap_variables, true));
            std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
            ap_function_dummy->SetProblem(ap_problem.get());

            unsigned int            dimensions = 4;

            NelderMeadTester        nelder_mead_tester;

            vector<PointWithValue>  result;
            ColumnVector            base_vertex(dimensions);

            base_vertex = 0.0;
            result      = nelder_mead_tester.TestInitialSimplexGeneration
                                  (base_vertex, 2.0, ap_function_dummy.get());


            assert(result.size() == (dimensions + 1));

            for (size_t i = 0; i < result.size(); ++i)
            {
                LOG(DEBUG,wxT("Point no.") << static_cast<int>(i)
                        << endl << result[i].mPoint);
            }


            vector<PointWithValue>::const_iterator outer_iter;
            vector<PointWithValue>::const_iterator inner_iter;
            ColumnVector side(dimensions);

            for (outer_iter =  result.begin();
                 outer_iter != result.end();
                 ++outer_iter)
                 for (inner_iter =  outer_iter, ++inner_iter;
                      inner_iter != result.end();
                      ++inner_iter)
                 {
                     side = outer_iter->mPoint - inner_iter->mPoint;
                     LOG(INFO,wxT("Simplex' side length: ") <<
                             side.norm_Frobenius());
                 }

            LOG(INFO,_T("test<1> finished."));
            LOG(INFO,_T("-----------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>

    template<>
    template<>
    void NelderMeadTest::test<2>()
    {
        try
        {
            LOG(INFO,
                _T("test<2>\nTesting for correct centroid calculation."));


            auto_ptr<ProblemBase> ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->
                                                                    Clone());
            SetNonlinearFunction(wxT("x^2 + y^2"), *ap_problem);

            NelderMeadTester nelder_mead_tester;

            unsigned int dimensions = ap_problem->VariableCount();


            ColumnVector initial_point1(dimensions);
            ColumnVector initial_point2(dimensions);
            ColumnVector initial_point3(dimensions);

            initial_point3(1) =  3.0;
            initial_point3(2) = -3.0;
            initial_point2(1) =  2.0;
            initial_point2(2) =  2.0;
            initial_point1(1) =  0.0;
            initial_point1(2) =  0.0;

            vector<ColumnVector> initial_simplex1;

            initial_simplex1.push_back(initial_point1);
            initial_simplex1.push_back(initial_point2);
            initial_simplex1.push_back(initial_point3);



            ColumnVector initial_point4(dimensions);
            ColumnVector initial_point5(dimensions);
            ColumnVector initial_point6(dimensions);

            initial_point4(1) =  1.0;
            initial_point4(2) =  1.0;
            initial_point5(1) =  0.0;
            initial_point5(2) =  0.0;
            initial_point6(1) =  0.0;
            initial_point6(2) =  0.0;

            vector<ColumnVector> initial_simplex2;

            initial_simplex2.push_back(initial_point4);
            initial_simplex2.push_back(initial_point5);
            initial_simplex2.push_back(initial_point6);



            vector<ColumnVector> result, correct_result;


            ColumnVector correct_result_point1(dimensions);
            ColumnVector correct_result_point2(dimensions);

            correct_result_point1 = 1.0;
            correct_result_point2 = 0.0;

            correct_result.push_back(correct_result_point1);
            correct_result.push_back(correct_result_point2);


            result.push_back(nelder_mead_tester.TestCentroidFinding
                                            (initial_simplex1, *ap_problem));
            result.push_back(nelder_mead_tester.TestCentroidFinding
                                            (initial_simplex2, *ap_problem));

            vector<ColumnVector>::const_iterator    iter;

            for (iter = result.begin(); iter != result.end(); ++iter)
            {
                LOG(INFO,_T("Wyliczony punkt ciezkosci:\n") << *iter);
            }

            for (iter  = correct_result.begin();
                 iter != correct_result.end();
                 ++iter)
            {
                LOG(INFO,_T("Prawidlowy punkt ciezkosci:\n") << *iter);
            }

            ensure("Correct centroid found", result == correct_result);

            LOG(INFO,_T("test<2> finished."));
            LOG(INFO,_T("-----------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }// test<2>

    template<>
    template<>
    void NelderMeadTest::test<3>()
    {
        try
        {
            LOG(INFO,_T("test<3>\nShrinking test."));

            auto_ptr<ProblemBase> ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->
                                                                    Clone());
            SetNonlinearFunction(wxT("x^2 + y^2"), *ap_problem);

            NelderMeadTester nelder_mead_tester;

            unsigned int dimensions = ap_problem->VariableCount();


            vector<ColumnVector> result, correct_result;

            ColumnVector initial_point1(dimensions);
            ColumnVector initial_point2(dimensions);
            ColumnVector initial_point3(dimensions);

            initial_point1(1) =  2.0;
            initial_point1(2) =  2.0;
            initial_point2(1) = -2.0;
            initial_point2(2) = -2.0;
            initial_point3(1) =  1.0;
            initial_point3(2) =  1.0;

            vector<ColumnVector> initial_vector;

            initial_vector.push_back(initial_point1);
            initial_vector.push_back(initial_point2);
            initial_vector.push_back(initial_point3);



            ColumnVector correct_result_point1(dimensions),
                         correct_result_point2(dimensions),
                         correct_result_point3(dimensions);

            correct_result_point1 =  1.0;
            correct_result_point2 = -1.25;
            correct_result_point3 =  1.75;

            correct_result.push_back(correct_result_point1);
            correct_result.push_back(correct_result_point2);
            correct_result.push_back(correct_result_point3);



            result = nelder_mead_tester.TestShrinking(initial_vector,
                                                      *ap_problem,
                                                      0.25);

            LOG(DEBUG,wxT("result point 1: ") << result[0].as_row());
            LOG(DEBUG,wxT("result point 2: ") << result[1].as_row());
            LOG(DEBUG,wxT("result point 3: ") << result[2].as_row());
            LOG(DEBUG,wxT("correct result point 1: ")
                            << correct_result[0].as_row());
            LOG(DEBUG,wxT("correct result point 2: ")
                            << correct_result[1].as_row());
            LOG(DEBUG,wxT("correct result point 3: ")
                            << correct_result[2].as_row());


            ensure("Shrinking done properly", result == correct_result);

            LOG(INFO,_T("test<3> finished."));
            LOG(INFO,_T("-----------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }// test<3>

    // Prbny przebieg z funkcj x^2 + y^2.
    template<>
    template<>
    void NelderMeadTest::test<4>()
    {
        try
        {
            LOG(INFO,_T("test<4>\nTest run for x^2 + y^2."));

            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("x^2 + y^2"), *ap_problem);

            NelderMeadTester nelder_mead_tester;

            unsigned int dimensions = ap_problem->VariableCount();


            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 5.0;
            correct_result = 0.0;

            nelder_mead_tester.SetStartingPoint(starting_point);

            NMCircumferenceCondition* p_condition =
                                new NMCircumferenceCondition();
            p_condition->rEpsilon() = 0.001;
            nelder_mead_tester.SetStopCondition(p_condition);


            auto_ptr<const Result> ap_result(nelder_mead_tester.StartOptimize(*ap_problem).release());


            LOG(DEBUG,wxT("Result       : \n") << *ap_result);
            LOG(INFO, wxT("CorrectResult: \n") << correct_result);
            LOG(INFO, wxT("FinalResult  : \n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.1));

            LOG(INFO,_T("test<4> finished."));
            LOG(INFO,_T("-----------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<4>

    // Prbny przebieg z bananoid z minimum w punkcie (2;1).
    template<>
    template<>
    void NelderMeadTest::test<5>()
    {
        LOG(INFO,_T("test<5>\nTest run for banana-like function."));

        auto_ptr<ProblemBase> ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);

        NelderMeadTester nelder_mead_tester;

        unsigned int dimensions = ap_problem->VariableCount();

        NMWorstReplacementCondition* p_condition =
            new NMWorstReplacementCondition
                (CountedPtr<StandardStopCondition>(new DistanceCondition));
        p_condition->rEpsilon() = 0.0001;
        nelder_mead_tester.SetStopCondition(p_condition);


        ColumnVector starting_point(dimensions);
        ColumnVector correct_result(dimensions);
        starting_point = 10.0;
        correct_result << 2 << 1;

        nelder_mead_tester.SetStartingPoint(starting_point);

        auto_ptr<const Result> ap_result(nelder_mead_tester.StartOptimize(*ap_problem).release());

        LOG(DEBUG,wxT("Result       : \n") << *ap_result);
        LOG(INFO, wxT("CorrectResult: \n") << correct_result);
        LOG(INFO, wxT("FinalResult  : \n") << ap_result->FinalResult());

        ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),correct_result,0.1));

        LOG(INFO,_T("test<5> finished."));
        LOG(INFO,_T("-----------------------------------------------"));
    }

    // Prbny przebieg dla funkcji jednej zmiennej x^2.
    template<>
    template<>
    void NelderMeadTest::test<6>()
    {
        LOG(INFO,_T("test<5>\nTest run for x^2."));

        auto_ptr<ProblemBase> ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        SetNonlinearFunction(wxT("(x - 2)^2"), *ap_problem);

        NelderMeadTester nelder_mead_tester;

        unsigned int dimensions = ap_problem->VariableCount();


        ColumnVector starting_point(dimensions);
        ColumnVector correct_result(dimensions);
        starting_point = 10.0;
        correct_result << 2.0;

        NMBestWorstCondition* p_condition =
            new NMBestWorstCondition
                (CountedPtr<StandardStopCondition>(new ValueCondition));
        p_condition->rEpsilon() = 0.0001;
        nelder_mead_tester.SetStopCondition(p_condition);

        nelder_mead_tester.SetStartingPoint(starting_point);

        auto_ptr<const Result> ap_result(nelder_mead_tester.StartOptimize(*ap_problem).release());

        LOG(DEBUG,wxT("Result       : \n") << *ap_result);
        LOG(INFO, wxT("CorrectResult: \n") << correct_result);
        LOG(INFO, wxT("FinalResult  : \n") << ap_result->FinalResult());

        ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),correct_result,0.1));

        LOG(INFO,_T("test<6> finished."));
        LOG(INFO,_T("-----------------------------------------------"));
    }

    template<>
    template<>
    void NelderMeadTest::test<7>()
    {
        LOG(INFO,_T("test<7>"));

        auto_ptr<ProblemBase> ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        SetNonlinearFunction(wxT("3*x^4 + 4*x^3 + 6*y^2"), *ap_problem);

        NelderMeadTester nelder_mead_tester;

        unsigned int dimensions = ap_problem->VariableCount();


        ColumnVector starting_point(dimensions);
        ColumnVector correct_result(dimensions);
        starting_point = 10.0;
        correct_result << -1 << 0;

        NMPreviousWorstCondition* p_condition =
            new NMPreviousWorstCondition
                (CountedPtr<StandardStopCondition>(new SlopeCondition));
        p_condition->rEpsilon() = 0.0001;
        nelder_mead_tester.SetStopCondition(p_condition);

        nelder_mead_tester.SetStartingPoint(starting_point);

        auto_ptr<const Result> ap_result(nelder_mead_tester.StartOptimize(*ap_problem).release());

        LOG(DEBUG,wxT("Result       : \n") << *ap_result);
        LOG(INFO, wxT("CorrectResult: \n") << correct_result);
        LOG(INFO, wxT("FinalResult  : \n") << ap_result->FinalResult());

        ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),correct_result,0.1));

        LOG(INFO,_T("test<5> finished."));
        LOG(INFO,_T("-----------------------------------------------"));
    }

}
