#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/NewtonModified.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class NewtonModifiedTester
    : public NewtonModified
{
public:

    NewtonModifiedTester()
        : NewtonModified()
    {
    }

    void
    SetStartingPoint(const ColumnVector& startingPoint)
    {
        mStartingPoint = startingPoint;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{

    struct NewtonModifiedData
    {
    };

    typedef test_group<NewtonModifiedData> NewtonModifiedTestType;
    typedef NewtonModifiedTestType::object NewtonModifiedTest;
    NewtonModifiedTestType newton_modified_group("newton_modified");

    template<>
    template<>
    void NewtonModifiedTest::test<1>()
    {

        try
        {
            LOG(INFO,wxT("test<1>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            NewtonModifiedTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            method.SetStartingPoint(starting_point);

            method.SetStopCondition(new CustomValueCondition);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.0001));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>

    // Test 1 z warunkiem na odlego punktw.
    template<>
    template<>
    void NewtonModifiedTest::test<2>()
    {

        try
        {
            LOG(INFO,wxT("test<2>\nTest run with 4*x^2 + 3*y^2 - 4*x*y + x."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            NewtonModifiedTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 0.0;
            correct_result << -0.1875 << -0.125;

            method.SetStartingPoint(starting_point);

            method.SetStopCondition(new DistanceCondition);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.0001));

            LOG(INFO,wxT("test<2> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>

}

