#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct NewtonRaphsonData
    {
    };

    typedef test_group<NewtonRaphsonData> NewtonRaphsonTestType;
    typedef NewtonRaphsonTestType::object NewtonRaphsonTest;
    NewtonRaphsonTestType newton_raphson_group("newton-raphson");


    // Newtona-Raphsona samodzielna.
    template<>
    template<>
    void NewtonRaphsonTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Newton-Raphson standalona"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(100 - x)^2"), *ap_problem);

            NewtonRaphsonTester  tester;

            double              accuracy            = 0.01;
            ColumnVector        correct_result(1);

            correct_result  = 100;

            tester.SetEpsilon(accuracy);
            tester.SetStartingPoint(80);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));

            LOG(DEBUG,wxT("Accuracy : ") << accuracy);
            LOG(INFO,wxT("Correct result : ") << correct_result(1));
            LOG(INFO,wxT("Result : ") << ap_result->FinalResult()(1));

            ensure("Newton-Raphson correct result",
                        EqualVectors(correct_result,ap_result->FinalResult(),
                        accuracy));


            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
