#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/Newton.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class NewtonTester
    : public Newton
{
public:

    NewtonTester()
        : Newton()
    {
    }

    void
    SetStartingPoint(const ColumnVector& startingPoint)
    {
        mStartingPoint = startingPoint;
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{

    struct NewtonData
    {
    };

    typedef test_group<NewtonData> NewtonTestType;
    typedef NewtonTestType::object NewtonTest;
    NewtonTestType newton_group("newton");

    template<>
    template<>
    void NewtonTest::test<1>()
    {
        try
        {
            LOG(INFO,_T("test<1>\nTest run with 8*x^2 + 4*x*y + 5*y^2."));

            NewtonTester newton_tester;

            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("8*x^2 + 4*x*y + 5*y^2"), *ap_problem);
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 10.0;
            correct_result = 0.0;

            newton_tester.SetStartingPoint(starting_point);

            CustomValueCondition* pCondition = new CustomValueCondition;
            pCondition->rEpsilon() = 0.00001;
            newton_tester.SetStopCondition(pCondition);

            auto_ptr<const Result> ap_result
                        (newton_tester.StartOptimize(*ap_problem).release());

            LOG(INFO, wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.00001));

            LOG(INFO,_T("test<1> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>

    template<>
    template<>
    void NewtonTest::test<2>()
    {
        try
        {
            LOG(INFO,_T("test<1>\nTest run with 3*x^4 + 4*x^3 + 6*y^2."));

            NewtonTester newton_tester;

            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("3*x^4 + 4*x^3 + 6*y^2"), *ap_problem);
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = -10.0;
            correct_result << -1 << 0;

            newton_tester.SetStartingPoint(starting_point);

            CustomValueCondition* pCondition = new CustomValueCondition;
            pCondition->rEpsilon() = 0.00001;
            newton_tester.SetStopCondition(pCondition);

            auto_ptr<const Result> ap_result
                        (newton_tester.StartOptimize(*ap_problem).release());

            LOG(INFO,wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.00001));

            LOG(INFO,_T("test<1> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>

    // Test 2 z warunkiem na warto funkcji.
    template<>
    template<>
    void NewtonTest::test<3>()
    {
        try
        {
            LOG(INFO,_T("test<3>\nTest run with 3*x^4 + 4*x^3 + 6*y^2."));

            NewtonTester newton_tester;

            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("3*x^4 + 4*x^3 + 6*y^2"), *ap_problem);
            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = -10.0;
            correct_result << -1 << 0;

            newton_tester.SetStartingPoint(starting_point);

            ValueCondition* pCondition = new ValueCondition;
            pCondition->rEpsilon() = 0.00001;
            newton_tester.SetStopCondition(pCondition);

            auto_ptr<const Result> ap_result
                        (newton_tester.StartOptimize(*ap_problem).release());

            LOG(INFO,wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                    EqualVectors(ap_result->FinalResult(),correct_result,0.00001));

            LOG(INFO,_T("test<3> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>
}
