#include <src/precomp.h>

#include <tut.h>

#include <src/Observer.h>
#include <src/Observable.h>
#include <src/Logger.h>

LOGGER(INFO)

class ObserverTester
    : public Observer
{
public:
    ObserverTester()
        : mCount(0)
    {
    }

    virtual void Update()
    {
        ++mCount;
        LOG(DEBUG, wxT("Update!"));
    }

    unsigned Count()
    {
        return mCount;
    }

private:
    unsigned    mCount;
};

class ObservableTester
    : public Observable
{
public:
    ObservableTester()
    {
    }

    void TriggerNotify() const
    {
        NotifyObservers();
    }
};

namespace tut
{
    struct ObserverData
    {
    };

    typedef test_group<ObserverData> ObserverTestType;
    typedef ObserverTestType::object ObserverTest;
    ObserverTestType observer_group("observer");

    // Test obserwatorw
    template<>
    template<>
    void ObserverTest::test<1>()
    {
        ObserverTester observer1;
        ObserverTester observer2;
        const ObservableTester observable;

        ensure("Observer1's initial state", observer1.Count() == 0);
        ensure("Observer2's initial state", observer2.Count() == 0);

        observable.AttachObserver(&observer1);

        observable.TriggerNotify();

        ensure("Observer1 notified 1-st time", observer1.Count() == 1);
        ensure("Observer2 not notified", observer2.Count() == 0);


        observable.AttachObserver(&observer2);

        observable.TriggerNotify();

        ensure("Observer1 notified 2-nd time", observer1.Count() == 2);
        ensure("Observer2 notified 1-st time", observer2.Count() == 1);
    }

    // Test usuwania obserwatora
    template<>
    template<>
    void ObserverTest::test<2>()
    {
        ObserverTester* p_observer1;
        ObserverTester* p_observer2;
        ObservableTester observable;

        p_observer1 = new ObserverTester();
        p_observer2 = new ObserverTester();

        observable.AttachObserver(p_observer1);
        observable.AttachObserver(p_observer2);

        observable.TriggerNotify();

        ensure(p_observer1->Count() == 1);
        ensure(p_observer2->Count() == 1);

        delete p_observer1;

        observable.TriggerNotify();

        ensure(p_observer2->Count() == 2);

        delete p_observer2;

        observable.TriggerNotify();

        // Jak si nie wyoy, to znaczy, e dziaa ok ;)
    }
}// namespace tut
