#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/HyperbolicInnerPenalty.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/OuterPenalty.h>
#include <src/ParseError.h>
#include <src/ProblemWithPenalty.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/ValueCondition.h>

#include <src/methods/Newton.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;

namespace tut
{

    struct PenaltyFunctionTestData
    {
    };

    typedef test_group<PenaltyFunctionTestData> PenaltyFunctionTestType;
    typedef PenaltyFunctionTestType::object PenaltyFunctionTest;
    PenaltyFunctionTestType penalty_function_test_group("penalty function");


    // Zewntrzna funkcja kary.
    template<>
    template<>
    void PenaltyFunctionTest::test<1>()
    {
        LinearParser l_parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        auto_ptr<FunctionBase> ap_function(
                l_parser.CreateFunction(wxT("0*x + 0*y + 0.0"), *ap_variables,
                    true));

        ap_function->SetProblem(ap_problem.get());

        auto_ptr<OuterPenalty> ap_penalty(new
                OuterPenalty(auto_ptr<const FunctionBase>(ap_function)));

        ap_function = l_parser.CreateFunction(wxT("x + y"), *ap_variables,
                false);
        ap_function->SetProblem(ap_problem.get());

        Constraint constraint1 =
            Constraint(auto_ptr<const FunctionBase>(ap_function),
                       Constraint::LESS_EQUAL_THAN);

        ap_penalty->AddConstraint(constraint1);
        ap_problem->SetFunction(auto_ptr<FunctionBase>(ap_penalty),
                ap_variables);

        // TODO: sprawdzi wspczynniki

        ColumnVector x(2);

        ensure("2 variables", x.nrows() == 2);

        double result1, result2;

        x(1) = 1.0;
        x(2) = 1.0;

        result1 = ap_problem->Function().Evaluate(x);

        x(1) = -1.0;
        x(2) = -2.0;

        result2 = ap_problem->Function().Evaluate(x);

        LOG(DEBUG,wxT("Result in [ 1.0, 1.0]: ") << result1);
        LOG(DEBUG,wxT("Result in [-1.0,-2.0]: ") << result2);

        ensure("Evaluate returns 4.0", result1 == 4.0);
        ensure("Evaluate returns 0.0", result2 == 0.0);

    }

    // Hiperboliczna wewntrzna funkcja kary.
    template<>
    template<>
    void PenaltyFunctionTest::test<2>()
    {
        LinearParser l_parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        std::auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
        auto_ptr<FunctionBase> ap_function(
                l_parser.CreateFunction(wxT("0*x + 0*y + 0.0"), *ap_variables,
                    true));

        ap_function->SetProblem(ap_problem.get());

        auto_ptr<HyperbolicInnerPenalty> ap_penalty(new HyperbolicInnerPenalty
                                (auto_ptr<const FunctionBase>(ap_function)));

        ap_function = l_parser.CreateFunction(wxT("x + y"), *ap_variables,
                false);
        ap_function->SetProblem(ap_problem.get());

        Constraint constraint1 =
            Constraint(auto_ptr<const FunctionBase>(ap_function),
                       Constraint::LESS_EQUAL_THAN);

        ap_penalty->AddConstraint(constraint1);
        ap_problem->SetFunction(auto_ptr<FunctionBase>(ap_penalty),
                ap_variables);

        // TODO: sprawdzi wspczynniki

        ColumnVector x(2);

        ensure("2 variables", x.nrows() == 2);

        double result1, result2, result3;

        x(1) = -1.0;
        x(2) = -1.0;

        result1 = ap_problem->Function().Evaluate(x);

        x(1) = 0.0;
        x(2) = 0.0;

        result2 = ap_problem->Function().Evaluate(x);

        x(1) = 1.0;
        x(2) = 1.0;

        result3 = ap_problem->Function().Evaluate(x);

        LOG(DEBUG,wxT("Result in [-1.0,-1.0]: ") << result1);
        LOG(DEBUG,wxT("Result in [ 0.0, 0.0]: ") << result2);
        LOG(DEBUG,wxT("Result in [ 1.0, 1.0]: ") << result3);

        ensure("Evaluate returns 0.5", result1 == 0.5);
        ensure("Evaluate returns DBL_MAX", result2 == DBL_MAX);
        ensure("Evaluate returns DBL_MAX again", result3 == DBL_MAX);

    }

}
