#include <src/precomp.h>

#include <tut.h>

#include <src/CustomValueCondition.h>
#include <src/DistanceCondition.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/PolakRibiere.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(WARNING);

using namespace std;
/*
class PolakRibiereTester
    : public PolakRibiere
{
public:

    PolakRibiereTester()
        : PolakRibiere()
    {
    }

    void
    SetStartingPoint(const ColumnVector& startingPoint)
    {
        mStartingPoint = startingPoint;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{

    struct PolakRibiereData
    {
    };

    typedef test_group<PolakRibiereData> PolakRibiereTestType;
    typedef PolakRibiereTestType::object PolakRibiereTest;
    PolakRibiereTestType polak_ribiera_group("polak_ribiere");

    template<>
    template<>
    void PolakRibiereTest::test<1>()
    {

        try
        {
            LOG(INFO,wxT("test<1>\nTest run with 5*x^2 + y^2 - 4*x*y - 2*x +3."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("5*x^2 + y^2 - 4*x*y - 2*x + 3"),
                                 *ap_problem);


            PolakRibiereTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            CustomValueCondition* pCondition = new CustomValueCondition;
            method.SetStopCondition(pCondition);


            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point = 5.0;
            correct_result << 1.0 << 2.0;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.0001));

            LOG(INFO,wxT("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>

    // Test 1 z warunkiem na kwadrat gradientu, startuje z minimum.
    template<>
    template<>
    void PolakRibiereTest::test<2>()
    {

        try
        {
            LOG(INFO,wxT("test<2>\nTest run with 5*x^2 + y^2 - 4*x*y - 2*x +3."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("5*x^2 + y^2 - 4*x*y - 2*x + 3"),
                                 *ap_problem);


            PolakRibiereTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            SquaredCustomValueCondition* pCondition =
                new SquaredCustomValueCondition;
            method.SetStopCondition(pCondition);


            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << 1.0 << 2.0;
            correct_result << 1.0 << 2.0;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.0001));

            LOG(INFO,wxT("test<2> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<2>

    // Test 1 z warunkiem na odlego punktw, startuje z minimum.
    template<>
    template<>
    void PolakRibiereTest::test<3>()
    {

        try
        {
            LOG(INFO,wxT("test<3>\nTest run with 5*x^2 + y^2 - 4*x*y - 2*x +3."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("5*x^2 + y^2 - 4*x*y - 2*x + 3"),
                                 *ap_problem);


            PolakRibiereTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            DistanceCondition* pCondition = new DistanceCondition;
            method.SetStopCondition(pCondition);


            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector starting_point(dimensions);
            ColumnVector correct_result(dimensions);
            starting_point << 1.0 << 2.0;
            correct_result << 1.0 << 2.0;

            method.SetStartingPoint(starting_point);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  wxT("Result:\n") << *ap_result);
            LOG(DEBUG, wxT("CorrectResult:\n") << correct_result);
            LOG(DEBUG, wxT("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result,0.0001));

            LOG(INFO,wxT("test<3> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>

}

