#include <src/precomp.h>

#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/ValueCondition.h>
#include <src/UnconstrainedProblem.h>

#include <src/methods/DichotomousDivision.h>
#include <src/methods/PowellVariant1.h>
#include <src/methods/PowellVariant2.h>
#include <src/methods/UniformDivision.h>

#include <src/Logger.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;
/*
class PowellVariant1Tester
    : public PowellVariant1
{
public:
    PowellVariant1Tester::PowellVariant1Tester()
    {
    }// PowellVariant1Tester

    void SetStartingPoint(ColumnVector point)
    {
        mStartingPoint = point;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};

class PowellVariant2Tester
    : public PowellVariant2
{
public:
    PowellVariant2Tester::PowellVariant2Tester()
    {
    }// PowellVariant2Tester

    void SetStartingPoint(ColumnVector point)
    {
        mStartingPoint = point;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{
    struct PowellData
    {
    };

    typedef test_group<PowellData> PowellTestType;
    typedef PowellTestType::object PowellTest;
    PowellTestType powell_group("powell");

    // Wariant 1 - test 1.
    template<>
    template<>
    void PowellTest::test<1>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 1 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.02));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Wariant 1 - agodniejsza bananoida.
    template<>
    template<>
    void PowellTest::test<2>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("10*(y - x^2)^2 + (1 - x)^2"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);
            method.SetLineSearchMethod(ap_linesearch.release());

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 2 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.01));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 1 - funkcja Powella.
    template<>
    template<>
    void PowellTest::test<3>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 3 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(4);
            proper_result << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 1 - problem 2.23.
    template<>
    template<>
    void PowellTest::test<4>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("a^10 + b^10 + c^10 + d^10 + e^10"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 10 << 10 << 10 << 10 << 10;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 4 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(5);
            proper_result << 0 << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 1 - agodniejsza bananoida, warunek na warto funkcji.
    template<>
    template<>
    void PowellTest::test<5>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("10*(y - x^2)^2 + (1 - x)^2"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            ValueCondition* p_condition = new ValueCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);
            method.SetLineSearchMethod(ap_linesearch.release());

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 5 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.01));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 1 - funkcja Powella, warunek na nachylenie.
    template<>
    template<>
    void PowellTest::test<6>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            PowellVariant1Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            SlopeCondition* p_condition = new SlopeCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 6 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(4);
            proper_result << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Wariant 2 - test 1.
    template<>
    template<>
    void PowellTest::test<7>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            DistanceCondition* pCondition = new DistanceCondition;
            method.SetStopCondition(pCondition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 7 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.02));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }


    // Wariant 2 - agodniejsza bananoida.
    template<>
    template<>
    void PowellTest::test<8>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("10*(y - x^2)^2 + (1 - x)^2"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            DistanceCondition* pCondition = new DistanceCondition;
            method.SetStopCondition(pCondition);

            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);
            method.SetLineSearchMethod(ap_linesearch.release());

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 8 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.05));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }



    // Wariant 2 - funkcja Powella.
    template<>
    template<>
    void PowellTest::test<9>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            DistanceCondition* pCondition = new DistanceCondition;
            pCondition->rEpsilon() = 0.0001;
            method.SetStopCondition(pCondition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 9 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(4);
            proper_result << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.15));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 2 - problem 2.23.
    template<>
    template<>
    void PowellTest::test<10>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("a^10 + b^10 + c^10 + d^10 + e^10"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 10 << 10 << 10 << 10 << 10;
            method.SetStartingPoint(starting_point);

            DistanceCondition* pCondition = new DistanceCondition;
            pCondition->rEpsilon() = 0.0001;
            method.SetStopCondition(pCondition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 10 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(5);
            proper_result << 0 << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 2 - agodniejsza bananoida, warunek na nachylenie.
    template<>
    template<>
    void PowellTest::test<11>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("10*(y - x^2)^2 + (1 - x)^2"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            SlopeCondition* p_condition = new SlopeCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<LineSearch> ap_linesearch(dynamic_cast<LineSearch*>
                (new DichotomousDivisionTester));
            dynamic_cast<DichotomousDivisionTester*>
                (ap_linesearch.get())->SetEpsilon(0.0001);
            method.SetLineSearchMethod(ap_linesearch.release());

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 11 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.01));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Wariant 2 - funkcja Powella, warunek na warto funkcji.
    template<>
    template<>
    void PowellTest::test<12>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(a + 10*b)^2 + 5*(c - d)^2 + (b - 2*c)^4 + 10*(a - d)^4"), *ap_problem);

            PowellVariant2Tester method;

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point << 1 << 2 << 3 << 4;
            method.SetStartingPoint(starting_point);

            ValueCondition* p_condition = new ValueCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Powell test 12 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(4);
            proper_result << 0 << 0 << 0 << 0;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}

