#include <src/precomp.h>

//#include <memory>
//#include <newmat/newmat.h>
#include <tut.h>

#include <src/AugmentedForm.h>
#include <src/HyperbolicInnerPenalty.h>
#include <src/LinearParser.h>
#include <src/LinearProblem.h>
#include <src/Logger.h>
#include <src/ProblemWithPenalty.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;

namespace tut
{
    struct ProblemData
    {
    };

    typedef test_group<ProblemData> ProblemTestType;
    typedef ProblemTestType::object ProblemTest;
    ProblemTestType problem_group("problems");

    // Tworzenie i obliczanie wartoci funkcji
    template<>
    template<>
    void ProblemTest::test<1>()
    {
        RowVector properCoefficients(2);

        auto_ptr<ProblemBase> ap_linear_problem;
        ap_linear_problem = ProblemBase::GetById(LinearProblem::ClassId())->Clone();

        LinearProblem* p_linear_problem = dynamic_cast<LinearProblem*>
            (ap_linear_problem.get());

        SetLinearFunction(wxT("x + 2*y - 1"), *p_linear_problem);

        properCoefficients(1) = 1.0;
        properCoefficients(2) = 2.0;

        RowVector coefficients = p_linear_problem->Coefficients();

        ensure(properCoefficients == coefficients);
        ensure(-1 == p_linear_problem->A0());

        ColumnVector x(p_linear_problem->Variables().GetCount());
        ensure(x.Nrows() == 2);

        x(1) = 2.5;
        x(2) = -1.0;

        // TODO: czy ten test nie jest teraz bez sensu?
        ensure(p_linear_problem->Function().Evaluate(x) == -0.5);
    }

    // Liniowy problem z ograniczeniami, augmented form
    template<>
    template<>
    void ProblemTest::test<2>()
    {
        LinearParser parser;
        auto_ptr<ProblemBase> ap_linear_problem;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        Constraint constraint;
        auto_ptr<FunctionBase> ap_constraint_function;
        Constraint::Container::const_iterator iter;

        ap_linear_problem = ProblemBase::GetById(LinearProblem::ClassId())->Clone();
        LinearProblem* p_linear_problem = dynamic_cast<LinearProblem*>
            (ap_linear_problem.get());

        auto_ptr<FunctionBase> ap_function(
            parser.CreateFunction(wxT("x + 2*y +5*z"), *ap_variables, true));


        ap_constraint_function = auto_ptr<FunctionBase>(
                parser.CreateFunction(wxT("x + 2*z - 3"),
                *ap_variables,
                false));
        ap_constraint_function->SetProblem(p_linear_problem);
        constraint = Constraint(
                auto_ptr<const FunctionBase>(ap_constraint_function),
                Constraint::LESS_EQUAL_THAN);
        p_linear_problem->AddConstraint(constraint);

        ap_constraint_function = auto_ptr<FunctionBase>(
                parser.CreateFunction(wxT("y + z - 1"),
                *ap_variables,
                false));
        ap_constraint_function->SetProblem(p_linear_problem);
        constraint = Constraint(
                auto_ptr<const FunctionBase>(ap_constraint_function),
                Constraint::GREATER_EQUAL_THAN);
        p_linear_problem->AddConstraint(constraint);

        ap_constraint_function = auto_ptr<FunctionBase>(
                parser.CreateFunction(wxT("x + y - 7"),
                *ap_variables,
                false));
        ap_constraint_function->SetProblem(p_linear_problem);
        constraint = Constraint(
                auto_ptr<const FunctionBase>(ap_constraint_function),
                Constraint::EQUAL);
        p_linear_problem->AddConstraint(constraint);

        p_linear_problem->SetFunction(ap_function, ap_variables);

        ensure(p_linear_problem->ConstraintCount() == 3);

        ColumnVector x(3);
        ColumnVector y(3);
        x << 2.0 << 5.0 << 0.0;
        y << 4.0 << 0.0 << 0.0;

        for (iter  = p_linear_problem->Constraints().begin();
             iter != p_linear_problem->Constraints().end();
             iter++)
        {
            ensure("x valid", iter->IsValid(x));
            ensure("y not valid", !iter->IsValid(y));
        }

        Matrix A, proper_A(3, 5);
        ColumnVector b, proper_b(3);
        ColumnVector c, proper_c(5);

        proper_A << 1 << 0 << 2 << 1 << 0
                 << 0 << 1 << 1 << 0 << -1
                 << 1 << 1 << 0 << 0 << 0;
        proper_b << 3 << 1 << 7;
        proper_c << 1 << 2 << 5 << 0 << 0;
        AugmentedForm proper_form(proper_A, proper_b, proper_c, 3);

        LOG(DEBUG, wxT("Proper form: ") << endl << proper_form);
        AugmentedForm result_form(*p_linear_problem);
        LOG(DEBUG, wxT("Result form: ") << endl << result_form);
        ensure("Proper augmented form", result_form == proper_form);

    }

    // Liniowy problem z ograniczeniami, augmented form
    template<>
    template<>
    void ProblemTest::test<3>()
    {
        LinearParser parser;
        auto_ptr<ProblemBase> ap_linear_problem;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        Constraint constraint;
        auto_ptr<FunctionBase> ap_constraint_function;
        Constraint::Container::const_iterator iter;

        ap_linear_problem = ProblemBase::GetById(LinearProblem::ClassId())->Clone();
        LinearProblem* p_linear_problem = dynamic_cast<LinearProblem*>
            (ap_linear_problem.get());

        auto_ptr<FunctionBase> ap_function(
            parser.CreateFunction(wxT("2*x + y"), *ap_variables, true));

        ap_constraint_function = parser.CreateFunction(wxT("x + y - 3"),
                    *ap_variables,
                    false);
        ap_constraint_function->SetProblem(p_linear_problem);
        constraint = Constraint(
                auto_ptr<const FunctionBase>(ap_constraint_function),
                Constraint::GREATER_EQUAL_THAN);
        p_linear_problem->AddConstraint(constraint);

        ap_constraint_function = parser.CreateFunction(wxT("x + 2*y - 4"),
                    *ap_variables,
                    false);
        ap_constraint_function->SetProblem(p_linear_problem);
        constraint = Constraint(
                auto_ptr<const FunctionBase>(ap_constraint_function),
                Constraint::GREATER_EQUAL_THAN);
        p_linear_problem->AddConstraint(constraint);

        p_linear_problem->SetFunction(ap_function, ap_variables);

        LOG(DEBUG, wxT("Constraint count: ") << p_linear_problem->ConstraintCount());
        ensure(p_linear_problem->ConstraintCount() == 2);

        Matrix A, proper_A(2, 4);
        ColumnVector b, proper_b(2);
        ColumnVector c, proper_c(4);

        proper_A << 1 << 1 << -1 << 0
                 << 1 << 2 << 0 << -1;

        proper_b << 3 << 4;
        proper_c << 2 << 1 << 0 << 0;

        AugmentedForm proper_form(proper_A, proper_b, proper_c, 2);
        LOG(DEBUG, wxT("Proper form: ") << endl << proper_form);
        AugmentedForm result_form(*p_linear_problem);
        LOG(DEBUG, wxT("Result form: ") << endl << result_form);
        ensure("Proper augmented form", result_form == proper_form);
    }

    template<>
    template<>
    void ProblemTest::test<4>()
    {
        try
        {
            LOG(INFO,_T("Function:\nx^2 + y^2"));

            LinearParser    linear_parser;
            NonlinearParser nonlinear_parser;
            auto_ptr<VariableNames>    ap_variables(new VariableNames);

/// tworzenie problemu
//
            auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(ProblemWithPenalty::ClassId())->Clone());

/// tworzenie funkcji celu
//
            std::auto_ptr<FunctionBase>     ap_function
                            (nonlinear_parser.CreateFunction
                                    (wxT("x^2 + y^2"), *ap_variables, true));

/// tworzenie funkcji kary
//
            FunctionBase& r_function = *ap_function;
            auto_ptr<HyperbolicInnerPenalty> ap_penalty_function
                (new HyperbolicInnerPenalty(ap_function));

/// tworzenie ogranicze
//
            auto_ptr<FunctionBase> ap_constraint_function
                (linear_parser.CreateFunction(wxT("x + y"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint1(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

            ap_constraint_function =
                (linear_parser.CreateFunction(wxT("- x + y"),
                                              *ap_variables, false));
            ap_constraint_function->SetProblem(ap_problem.get());

            Constraint constraint2(auto_ptr<const FunctionBase>
                    (ap_constraint_function), Constraint::LESS_EQUAL_THAN);

/// dodawanie ogranicze do funkcji kary
//
            ap_penalty_function->AddConstraint(constraint1);
            ap_penalty_function->AddConstraint(constraint2);

/// dodawanie gotowej funkcji kary do problemu
//
            (dynamic_cast<ProblemWithPenalty*>(ap_problem.get()))->rpPenalty() =
                                                    ap_penalty_function.get();

/// dodawanie funkcji kary do problemu i wzajemnie
//
            r_function.SetProblem(ap_problem.get());
            ap_problem->SetFunction
                (auto_ptr<FunctionBase>(ap_penalty_function), ap_variables);


////////////////////////
/// przygotowywanie punktw
//
            unsigned int dimensions = ap_problem->VariableCount();
            ColumnVector x(dimensions);
            x << 0.0 << -2.0;

/// sprawdzanie wyniku
//
            LOG(DEBUG,wxT("Value with Penalty   : ")
                    << ap_problem->Function().Evaluate(x));
            LOG(DEBUG,wxT("Value without Penalty: ")
                    << ap_problem->BaseFunction().Evaluate(x));

            ensure("BaseFunction returns a function without penalties",
                   (ap_problem->Function().Evaluate(x) == 5 &&
                    ap_problem->BaseFunction().Evaluate(x) == 4));

            LOG(INFO,_T("test<4> finished."));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>
}
