#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct QuadraticApproximationData
    {
    };

    typedef test_group<QuadraticApproximationData>
                                        QuadraticApproximationTestType;
    typedef QuadraticApproximationTestType::object QuadraticApproximationTest;
    QuadraticApproximationTestType
                    quadratic_approximation_group("quadratic approximation");


    // Aproksymacja kwadratowa samodzielna.
    template<>
    template<>
    void QuadraticApproximationTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Quadratic Approximation standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("x^2 + x + 1"), *ap_problem);

            QuadraticApproximationTester method;
            method.SetEpsilon(0.01);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << -0.5;
            LOG(INFO, wxT("QuadraticApproximation (standalone) result: ") <<
                    ap_result->FinalResult() << endl);
            ensure("Quadratic Approximation (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(), proper_result, 0.1));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Aproksymacja kwadratowa dla metod wielu zmiennych
    template<>
    template<>
    void QuadraticApproximationTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Quadratic approximation in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));

            QuadraticApproximationTester tester;

            tester.SetEpsilon(0.1);

            result = tester.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("QuadraticApproximation result: ") << endl << result);
            ensure("QuadraticApproximation correct result", abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Aproksymacja kwadratowa samodzielna - funkcja kwadratowa z maksimum jako
    // jedynym punktem stacjonarnym.
    template<>
    template<>
    void QuadraticApproximationTest::test<3>()
    {
        try
        {

            LOG(INFO,wxT("Quadratic Approximation standalone - unbounded function"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("-(x^2)"), *ap_problem);

            QuadraticApproximationTester method;
            method.SetEpsilon(0.01);

            auto_ptr<const Result> ap_result
                        (method.StartOptimize(*ap_problem).release());

            ColumnVector proper_result(1);
            proper_result << -0.5;
            LOG(INFO, wxT("QuadraticApproximation (standalone) result: ") <<
                    ap_result->FinalResult() << endl);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            ensure("Caught exception", true);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

}
