#include <src/precomp.h>

#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/SlopeCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>

#include <src/methods/DichotomousDivision.h>
#include <src/methods/RosenbrockDiscrete.h>
#include <src/methods/RosenbrockOptimal.h>
#include <src/methods/UniformDivision.h>

#include <src/Logger.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;
/*
class RosenbrockDiscreteTester
    : public RosenbrockDiscrete
{
public:
    RosenbrockDiscreteTester::RosenbrockDiscreteTester()
    {
    }// RosenbrockDiscreteTester

    void SetTau(ColumnVector tau)
    {
        mTau = tau;
    }

    void SetDelta(double delta)
    {
        mDelta = delta;
    }

    void SetAlpha(double alpha)
    {
        mAlpha = alpha;
    }

    void SetBeta(double beta)
    {
        mBeta = beta;
    }

    void SetStartingPoint(ColumnVector point)
    {
        mStartingPoint = point;
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }
};

class RosenbrockOptimalTester
    : public RosenbrockOptimal
{
public:
    RosenbrockOptimalTester::RosenbrockOptimalTester()
    {
    }// RosenbrockOptimalTester

    void SetStartingPoint(ColumnVector point)
    {
        mStartingPoint = point;
    }

    void SetLineSearchMethod(const LineSearch* pLineSearch)
    {
        mcpLineSearch.Reset(pLineSearch);
    }

    void SetStopCondition(const StandardStopCondition* pCondition)
    {
        mcpStopCondition.Reset(pCondition);
    }

};
*/
namespace tut
{
    struct RosenbrockData
    {
    };

    typedef test_group<RosenbrockData> RosenbrockTestType;
    typedef RosenbrockTestType::object RosenbrockTest;
    RosenbrockTestType rosenbrock_group("rosenbrock");

    // Test 1.
    template<>
    template<>
    void RosenbrockTest::test<1>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);
            ColumnVector tau(ap_problem->Variables().GetCount());
            tau = 0.1;

            RosenbrockDiscreteTester method;
            method.SetTau(tau);

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 1 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Bananoida.
    template<>
    template<>
    void RosenbrockTest::test<2>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("100*(y - x^2)^2 + (1 - x)^2"), *ap_problem);
            ColumnVector tau(ap_problem->Variables().GetCount());
            tau = 0.01;

            RosenbrockDiscreteTester method;
            method.SetTau(tau);
            method.SetDelta(0.00001);

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            p_condition->rEpsilon() = 0.00001;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());

            LOG(INFO, wxT("Rosenbrock test 2 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.01));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Test 1 z warunkiem na nachylenie.
    template<>
    template<>
    void RosenbrockTest::test<3>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);
            ColumnVector tau(ap_problem->Variables().GetCount());
            tau = 0.1;

            RosenbrockDiscreteTester method;
            method.SetTau(tau);

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            SlopeCondition* p_condition = new SlopeCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 3 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Test 1 z warunkiem na warto funkcji.
    template<>
    template<>
    void RosenbrockTest::test<4>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);
            ColumnVector tau(ap_problem->Variables().GetCount());
            tau = 0.1;

            RosenbrockDiscreteTester method;
            method.SetTau(tau);

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            ValueCondition* p_condition = new ValueCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 4 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.1));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Bananoida z warunkiem na nachylenie.
    template<>
    template<>
    void RosenbrockTest::test<5>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("100*(y - x^2)^2 + (1 - x)^2"), *ap_problem);
            ColumnVector tau(ap_problem->Variables().GetCount());
            tau = 0.01;

            RosenbrockDiscreteTester method;
            method.SetTau(tau);
            method.SetDelta(0.00001);

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -5;
            starting_point(2) = 15;
            method.SetStartingPoint(starting_point);

            SlopeCondition* p_condition = new SlopeCondition;
            p_condition->rEpsilon() = 0.00001;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());

            LOG(INFO, wxT("Rosenbrock test 5 result: ") << endl <<
                      ap_result->FinalResult());
            ColumnVector proper_result(2);
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.01));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Test 1 z krokiem optymalnym.
    template<>
    template<>
    void RosenbrockTest::test<6>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            ColumnVector proper_result(2);

            SetNonlinearFunction(wxT("(x - 2)^4 + (x - 2*y)^2"), *ap_problem);

            RosenbrockOptimalTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = 0;
            starting_point(2) = 3;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 6 result: ") << endl <<
                      ap_result->FinalResult());
            proper_result << 2 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.2));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Bananoida z krokiem optymalnym.
    template<>
    template<>
    void RosenbrockTest::test<7>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            ColumnVector proper_result(2);

            SetNonlinearFunction(wxT("100*(y - x^2)^2 + (1 - x)^2"),
                    *ap_problem);

            RosenbrockOptimalTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -6;
            starting_point(2) = 2;
            method.SetStartingPoint(starting_point);

            DistanceCondition* p_condition = new DistanceCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);


            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 7 result: ") << endl <<
                      ap_result->FinalResult());
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.2));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Bananoida z krokiem optymalnym, warunek na nachylenie.
    template<>
    template<>
    void RosenbrockTest::test<8>()
    {

        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            ColumnVector proper_result(2);

            SetNonlinearFunction(wxT("100*(y - x^2)^2 + (1 - x)^2"),
                    *ap_problem);

            RosenbrockOptimalTester method;

            method.SetLineSearchMethod(dynamic_cast<LineSearch *>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone().release()));

            ColumnVector starting_point(ap_problem->Variables().GetCount());
            starting_point(1) = -6;
            starting_point(2) = 2;
            method.SetStartingPoint(starting_point);

            SlopeCondition* p_condition = new SlopeCondition;
            p_condition->rEpsilon() = 0.0001;
            method.SetStopCondition(p_condition);

            auto_ptr<const Result>
                ap_result(method.StartOptimize(*ap_problem).release());
            LOG(INFO, wxT("Rosenbrock test 8 result: ") << endl <<
                      ap_result->FinalResult());
            proper_result << 1 << 1;
            ensure("Proper result", EqualVectors(proper_result, ap_result->FinalResult(), 0.2));
        }

        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

}

