#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct SecantDivisionData
    {
    };

    typedef test_group<SecantDivisionData> SecantDivisionTestType;
    typedef SecantDivisionTestType::object SecantDivisionTest;
    SecantDivisionTestType secant_division_group("secant division");


    // Metoda sicznych dla wielu zmiennych
    template<>
    template<>
    void SecantDivisionTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Secant division in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));

            SecantDivisionTester tester;

            tester.SetEpsilon(0.1);

            result = tester.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("SecantDivision result: ") << endl << result);
            ensure("SecantDivision correct result", abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Metoda siecznych samodzielna.
    template<>
    template<>
    void SecantDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Secant division standalona"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(100 - x)^2"), *ap_problem);

            SecantDivisionTester    tester;

            double              accuracy            = 0.01;
            ColumnVector        correct_result(1);

            correct_result  = 100;

            tester.SetEpsilon(accuracy);
            tester.SetA(60);
            tester.SetB(150);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));


            LOG(DEBUG,wxT("Accuracy : ") << accuracy);
            LOG(INFO,wxT("Correct result : ") << correct_result(1));
            LOG(INFO,wxT("Result : ") << ap_result->FinalResult()(1));

            ensure("Secant division correct result",
                        EqualVectors(correct_result,ap_result->FinalResult(),
                        accuracy));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
