#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{

    struct SignOfDerivativeData
    {
    };

    typedef test_group<SignOfDerivativeData> SignOfDerivativeTestType;
    typedef SignOfDerivativeTestType::object SignOfDerivativeTest;
    SignOfDerivativeTestType sign_of_derivative_group("sign of derivative");


    // Metoda znaku pochodnej dla metod wielu zmiennych
    template<>
    template<>
    void SignOfDerivativeTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Sign of derivative method\n\tin multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5.26)^2")));

            SignOfDerivativeTester tester;

            tester.SetEpsilon(0.1);

            result = tester.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                    (ap_problem->Function()));

            LOG(INFO, wxT("Sign of derivative method result: ") << endl << result);
            ensure("Sign of derivative method correct result",
                                                    abs(result - 5.26) < 0.1);

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }


    // Metoda znaku pochodnej samodzielna.
    template<>
    template<>
    void SignOfDerivativeTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Sign of derivative standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(4 - x)^4"), *ap_problem);

            SignOfDerivativeTester    tester;

            double              accuracy            = 0.01;
            ColumnVector        correct_result(1);

            correct_result  = 4;

            tester.SetEpsilon(accuracy);
            tester.SetAlpha(0.25);
            tester.SetGamma(0.5);
            tester.SetStartingPoint(20.0);

            auto_ptr<const Result> ap_result(tester.StartOptimize(*ap_problem));


            LOG(INFO,wxT("Accuracy : ") << accuracy);
            LOG(INFO,wxT("Correct result : ") << correct_result(1));
            LOG(INFO,wxT("Result : ") << ap_result->FinalResult()(1));

            ensure("Sign of derivative method correct result",
                        EqualVectors(correct_result,ap_result->FinalResult(),
                        accuracy));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
