#include <src/precomp.h>

//#include <memory>
#include <tut.h>

#include <src/Error.h>
#include <src/LinearFunction.h>
#include <src/LinearProblem.h>
#include <src/ParseError.h>
#include <src/methods/Simplex.h>
#include <src/Logger.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;

namespace tut
{
    struct SimplexData
    {
    };

    typedef test_group<SimplexData> SimplexTestType;
    typedef SimplexTestType::object SimplexTest;
    SimplexTestType simplex_group("simplex");

    template<>
    template<>
    void SimplexTest::test<1>()
    {
        try
        {
            LinearParser parser;
            auto_ptr<ProblemBase> ap_problem(
                    ProblemBase::GetById(LinearProblem::ClassId())->Clone());
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            Constraint constraint;

            LinearProblem* p_problem = dynamic_cast<LinearProblem*>(ap_problem.get());

            auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("2*x + 3*y"), *ap_variables, true));

            auto_ptr<FunctionBase> ap_constraint_function;

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("0.5*x + y - 1"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("0.67*x - y + 2"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_problem->SetFunction(ap_function, ap_variables);

            Simplex *method
                = dynamic_cast<Simplex*>(Method::GetById(Simplex::ClassId()));
            auto_ptr<const Result> ap_result;

            ap_result  = auto_ptr<const Result>(
                        (method->StartOptimize(*ap_problem).release()));

            ColumnVector proper_result(2);
            proper_result << 0.0 << 1.0;

            ensure("Simplex test 1 correct result", EqualVectors
                    (proper_result, ap_result->FinalResult()));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }

    }


    template<>
    template<>
    void SimplexTest::test<2>()
    {
        try
        {
            LinearParser parser;
            auto_ptr<ProblemBase> ap_problem(
                        ProblemBase::GetById(LinearProblem::ClassId())->Clone());
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            Constraint constraint;

            LinearProblem* p_problem = dynamic_cast<LinearProblem*>(ap_problem.get());

            auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("2*x + y"), *ap_variables, true));

            auto_ptr<FunctionBase> ap_constraint_function;

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("x + y - 3"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("x + 2*y - 4"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_problem->SetFunction(ap_function, ap_variables);

            Simplex *method
                = dynamic_cast<Simplex*>(Method::GetById(Simplex::ClassId()));
            auto_ptr<const Result> ap_result;

            ap_result  = auto_ptr<const Result>(
                    (method->StartOptimize(*ap_problem).release()));

            ColumnVector proper_result(2);
            proper_result << 0.0 << 3.0;

            ensure("Simplex test 2 correct result",
                    EqualVectors(proper_result, ap_result->FinalResult()));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }

    }

    template<>
    template<>
    void SimplexTest::test<3>()
    {
        bool with_exception = false;
        try
        {
            LinearParser parser;
            auto_ptr<ProblemBase> ap_problem(
                        ProblemBase::GetById(LinearProblem::ClassId())->Clone());
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            Constraint constraint;

            LinearProblem* p_problem = dynamic_cast<LinearProblem*>(ap_problem.get());

            auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("2*x + y"), *ap_variables, true));

            auto_ptr<FunctionBase> ap_constraint_function;

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("x + y - 3"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::LESS_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("x + y - 4"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_problem->SetFunction(ap_function, ap_variables);

            Simplex *method
                = dynamic_cast<Simplex*>(Method::GetById(Simplex::ClassId()));
            auto_ptr<const Result> ap_result;

            ap_result  = auto_ptr<const Result>(
                    (method->StartOptimize(*ap_problem).release()));

        }
        catch (const Error& error)
        {
            LOG(INFO, error.What());
            with_exception = true;
        }
        ensure("Simplex test 3 caught exception", with_exception);
    }

    template<>
    template<>
    void SimplexTest::test<4>()
    {
        bool with_exception = false;
        try
        {
            LinearParser parser;
            auto_ptr<ProblemBase> ap_problem(
                        ProblemBase::GetById(LinearProblem::ClassId())->Clone());
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            Constraint constraint;

            LinearProblem* p_problem = dynamic_cast<LinearProblem*>(ap_problem.get());

            auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("-2*x - 3*y"), *ap_variables, true));

            auto_ptr<FunctionBase> ap_constraint_function;

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("0.5*x + y - 1"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("0.67*x - y + 2"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_problem->SetFunction(ap_function, ap_variables);

            Simplex *method
                = dynamic_cast<Simplex*>(Method::GetById(Simplex::ClassId()));
            auto_ptr<const Result> ap_result;

            ap_result  = auto_ptr<const Result>(
                    (method->StartOptimize(*ap_problem).release()));
        }
        catch (const Error& error)
        {
            LOG(INFO, error.What());
            with_exception = true;
        }
        ensure("Simplex test 4 caught exception", with_exception);
    }

    template<>
    template<>
    void SimplexTest::test<5>()
    {
        try
        {
            LinearParser parser;
            auto_ptr<ProblemBase> ap_problem(
                        ProblemBase::GetById(LinearProblem::ClassId())->Clone());
            auto_ptr<VariableNames> ap_variables(new VariableNames);
            Constraint constraint;

            LinearProblem* p_problem = dynamic_cast<LinearProblem*>(ap_problem.get());

            auto_ptr<FunctionBase> ap_function(
                parser.CreateFunction(wxT("-x - 2*y"), *ap_variables, true));

            auto_ptr<FunctionBase> ap_constraint_function;

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("-x - y + 10"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_constraint_function = auto_ptr<FunctionBase>(parser.CreateFunction
                        (wxT("2*x - y + 4"),
                        *ap_variables,
                        false));
            ap_constraint_function->SetProblem(p_problem);
            constraint = Constraint(
                        auto_ptr<const FunctionBase>(ap_constraint_function),
                        Constraint::GREATER_EQUAL_THAN);
            p_problem->AddConstraint(constraint);

            ap_problem->SetFunction(ap_function, ap_variables);

            Simplex *method
                = dynamic_cast<Simplex*>(Method::GetById(Simplex::ClassId()));
            auto_ptr<const Result> ap_result;

            ap_result  = auto_ptr<const Result>(
                    (method->StartOptimize(*ap_problem).release()));

            ColumnVector proper_result(2);
            proper_result << 2.0 << 8.0;

            ensure("Simplex test 5 correct result",
                    EqualVectors(proper_result, ap_result->FinalResult()));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
    }

}
