#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/Logger.h>
#include <src/Tokenizer.h>

LOGGER(INFO)

namespace tut
{
    struct TokenizerData
    {
    };

    typedef test_group<TokenizerData> TokenizerTestType;
    typedef TokenizerTestType::object TokenizerTest;
    TokenizerTestType tokenizer_group("tokenizer");

    template<>
    template<>
    void TokenizerTest::test<1>()
    {
        VariableNames variables;
        Tokenizer::TokenType token;
        wxString input(wxT("10 + a -5 * sin(bc_d + 5)"));

        vector<Tokenizer::TokenType> properTokens;
        vector<size_t> properPositions;

        properPositions.push_back(0);
        properTokens.push_back(Tokenizer::NUMBER);
        properPositions.push_back(3);
        properTokens.push_back(Tokenizer::PLUS);
        properPositions.push_back(5);
        properTokens.push_back(Tokenizer::VARIABLE);
        properPositions.push_back(7);
        properTokens.push_back(Tokenizer::MINUS);
        properPositions.push_back(8);
        properTokens.push_back(Tokenizer::NUMBER);
        properPositions.push_back(10);
        properTokens.push_back(Tokenizer::MULTIPLY);
        properPositions.push_back(12);
        properTokens.push_back(Tokenizer::FUNCTION);
        properPositions.push_back(15);
        properTokens.push_back(Tokenizer::OPEN_PAREN);
        properPositions.push_back(16);
        properTokens.push_back(Tokenizer::VARIABLE);
        properPositions.push_back(21);
        properTokens.push_back(Tokenizer::PLUS);
        properPositions.push_back(23);
        properTokens.push_back(Tokenizer::NUMBER);
        properPositions.push_back(24);
        properTokens.push_back(Tokenizer::CLOSE_PAREN);
        properPositions.push_back(25);
        properTokens.push_back(Tokenizer::END);

        Tokenizer::FunctionMap function_map;
        function_map[wxT("sin")] = (Tokenizer::FunctionInfo){ sin, true };

        Tokenizer tokenizer(input, variables, true, function_map);

        for (size_t i = 0; i < properTokens.size(); ++i)
        {
            LOG(DEBUG, static_cast<unsigned>(tokenizer.Position()));
            ensure("Expected position",
                    tokenizer.Position() == properPositions[i]);

            token = tokenizer.Peek();
            ensure("Expected token", token == properTokens[i]);

            if (token != Tokenizer::END)    tokenizer.Consume();
        }

        ensure(variables.Index(wxT("a")) != wxNOT_FOUND);
        ensure(variables.Index(wxT("bc_d")) != wxNOT_FOUND);
        ensure(variables.GetCount() == 2);
        ensure(tokenizer.Position() == input.Length());
    }

    template<>
    template<>
    void TokenizerTest::test<2>()
    {
        VariableNames variables;
        bool gotException = false;

        variables.AddIfNotPresent(wxT("a"));
        variables.AddIfNotPresent(wxT("b"));

        Tokenizer tokenizer(wxT("a + b + c"), variables, false);

        try
        {
            for (int i = 0; i < 5; ++i)
            {
                tokenizer.Peek();
                tokenizer.Consume();
            }
        }
        catch (const Error& error)
        {
            gotException = true;
        }

        ensure("Expected exception", gotException);
    }

    template<>
    template<>
    void TokenizerTest::test<3>()
    {
        VariableNames variables;
        bool gotException = false;

        Tokenizer tokenizer(wxT("a + 3"), variables, true);

        try
        {
            tokenizer.Expect(Tokenizer::VARIABLE);
        }
        catch (const Error& error)
        {
            ensure("Unexpected exception", false);
        }

        ensure("Check PLUS", tokenizer.Peek() == Tokenizer::PLUS);

        tokenizer.Consume();

        try
        {
            tokenizer.Expect(Tokenizer::OPEN_PAREN);
        }
        catch (const Error& error)
        {
            gotException = true;
        }
        ensure("Expected exception", gotException);
    }

    template<>
    template<>
    void TokenizerTest::test<4>()
    {
        VariableNames   variables;

        Tokenizer::FunctionMap function_map;
        function_map[wxT("sin")] = (Tokenizer::FunctionInfo){ sin, true };
        function_map[wxT("abs")] = (Tokenizer::FunctionInfo){ abs, false };

        Tokenizer tokenizer(wxT("x + sin(3.14 + x)"), variables, true, function_map);

        while (tokenizer.Peek() != Tokenizer::END)  tokenizer.Consume();

        ensure("Is differentible", tokenizer.AllDifferentiable() == true);

        Tokenizer tokenizer2(wxT("x + abs(-x)"), variables, true, function_map);

        while (tokenizer2.Peek() != Tokenizer::END)  tokenizer2.Consume();

        ensure("Is not differentible", tokenizer2.AllDifferentiable() == false);
    }
}
