#include <src/precomp.h>

#include <tut.h>

#include <src/Error.h>
#include <src/UnconstrainedProblem.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;


namespace tut
{
    struct UniformDivisionData
    {
    };

    typedef test_group<UniformDivisionData> UniformDivisionTestType;
    typedef UniformDivisionTestType::object UniformDivisionTest;
    UniformDivisionTestType uniform_division_group("uniform division");

    // UniformDivision dla metod wielu zmiennych
    template<>
    template<>
    void UniformDivisionTest::test<1>()
    {
        try
        {

            LOG(INFO,wxT("Uniform division in multidimensional problems"));

            NonlinearParser parser;
            double result;

            auto_ptr<const ProblemBase>
                ap_problem(GetUnivariateFunction(wxT("(x-5)^2")));

            UniformDivisionTester method;

            method.SetEpsilon(1);
            result = method.FindOptimalStep(
                    dynamic_cast<const UnivariateFunction &>
                                        (ap_problem->Function()));

            LOG(INFO, wxT("UniformDivision result: ") << endl << result);
            ensure("UniformDivision correct result", result == 5);

            LOG(INFO,wxT("-------------------------------------------------"));
        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // UniformDivision samodzielna.
    template<>
    template<>
    void UniformDivisionTest::test<2>()
    {
        try
        {

            LOG(INFO,wxT("Uniform division standalone"));

            auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById
                    (UnconstrainedProblem::ClassId())->Clone());
            SetNonlinearFunction(wxT("(x - 7)^4"), *ap_problem);

            UniformDivisionTester method;
            method.SetEpsilon(0.1);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            ColumnVector proper_result(1);
            proper_result << 7;
            LOG(INFO, wxT("UniformDivision (standalone) result: ") << endl
                    << ap_result->FinalResult());
            ensure("UniformDivision (standalone) correct result",
                    EqualVectors(ap_result->FinalResult(), proper_result));

            LOG(INFO,wxT("-------------------------------------------------"));

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }
}
