#ifndef _tests_util_h
#define _tests_util_h

#include <newmat/newmat.h>
#include <wx/string.h>

#include <src/ProblemBase.h>
#include <src/LinearParser.h>
#include <src/NonlinearParser.h>
#include <src/VariableNames.h>

#include <src/methods/AlphaDivision.h>
#include <src/methods/BolzanoDivision.h>
#include <src/methods/Derivative.h>
#include <src/methods/DFP.h>
#include <src/methods/DichotomousDivision.h>
#include <src/methods/EvolutionaryMethod.h>
#include <src/methods/FletcherReeves.h>
#include <src/methods/HookeJeevesDiscrete.h>
#include <src/methods/HookeJeevesOptimal.h>
#include <src/methods/Marquardt.h>
#include <src/methods/NelderMead.h>
#include <src/methods/Newton.h>
#include <src/methods/NewtonModified.h>
#include <src/methods/NewtonRaphson.h>
#include <src/methods/PolakRibiere.h>
#include <src/methods/PowellVariant1.h>
#include <src/methods/PowellVariant2.h>
#include <src/methods/QuadraticApproximation.h>
#include <src/methods/RosenbrockDiscrete.h>
#include <src/methods/RosenbrockOptimal.h>
#include <src/methods/SecantDivision.h>
#include <src/methods/SignOfDerivative.h>
#include <src/methods/SteepestDescent.h>
#include <src/methods/UniformDivision.h>
#include <src/methods/FibonacciDivision.h>


void SetNonlinearFunction(const wxString&  input, ProblemBase& rProblem);

void SetLinearFunction(const wxString& input, ProblemBase& rProblem);

bool EqualVectors(const  ColumnVector& v1,
                  const  ColumnVector& v2,
                  double delta = 0.001);

bool EqualMatrices(const  Matrix& m1,
                   const  Matrix& m2,
                   double delta = 0.001);

auto_ptr<const ProblemBase> ProblemWithFunction(const wxString& input);

// TODO: to powinno znikn zastpione przez powysze
auto_ptr<const ProblemBase> GetUnivariateFunction(const wxString& input);


///////////////////////// LINEAR METHODS //////////////////////////////////////

///////////////////////// AlphaDivisionTester /////////////////////////////////

class AlphaDivisionTester
    : public AlphaDivision
{
public:
    AlphaDivisionTester(void);

    void SetEpsilon(double epsilon);
    void SetAlpha(double alpha);
};

///////////////////////// BolzanoDivisionTester ///////////////////////////////

class BolzanoDivisionTester
    : public BolzanoDivision
{
public:
    BolzanoDivisionTester(void);

    void SetEpsilon(double epsilon);
    void    SetA(double a);
    void    SetB(double b);
};

///////////////////////// DerivativeTester //////////////////////////////////

class DerivativeTester
    : public Derivative
{
    public:

    DerivativeTester();

    void SetEpsilon(double epsilon);
    void SetAlpha(double alpha);
    void SetGamma(double gamma);
    void SetStartingPoint(double startingPoint);
};

///////////////////////// DichotomousDivisionTester ///////////////////////////

class DichotomousDivisionTester
    : public DichotomousDivision
{
public:
    DichotomousDivisionTester(void);

    void SetEpsilon(double epsilon);
    void    SetA(double a);
    void    SetB(double b);
};

///////////////////////// FibonacciDivisionTester /////////////////////////////

class FibonacciDivisionTester
    : public FibonacciDivision
{
    public:

    FibonacciDivisionTester();

    std::vector<unsigned int> GenerateFibonacciSequence(double a,
                                                        double b,
                                                        double epsilon) const;

    void    SetEpsilon(double epsilon);
    void    SetA(double a);
    void    SetB(double b);
};

///////////////////////// NewtonRaphsonTester //////////////////////////////////

class NewtonRaphsonTester
    : public NewtonRaphson
{
    public:

    NewtonRaphsonTester();

    void SetEpsilon(double epsilon);
    void SetStartingPoint(double startingPoint);
};

///////////////////////// QuadraticApproximationTester ////////////////////////

class QuadraticApproximationTester
    : public QuadraticApproximation
{
public:
    QuadraticApproximationTester(void);

    void SetEpsilon(double epsilon);
};

///////////////////////// SecantDivisionTester ////////////////////////////////

class SecantDivisionTester
    : public SecantDivision
{
public:

    SecantDivisionTester(void);

    void SetEpsilon(double epsilon);
    void    SetA(double a);
    void    SetB(double b);
};

///////////////////////// SignOfDerivativeTester //////////////////////////////////

class SignOfDerivativeTester
    : public SignOfDerivative
{
    public:

    SignOfDerivativeTester();

    void SetEpsilon(double epsilon);
    void SetAlpha(double alpha);
    void SetGamma(double gamma);
    void SetStartingPoint(double startingPoint);
};

///////////////////////// UniformDivisionTester ///////////////////////////////

class UniformDivisionTester
    : public UniformDivision
{
public:
    UniformDivisionTester(void);

    void SetEpsilon(double epsilon);
};


///////////////////////// MULTIDIMENSIONAL METHODS ////////////////////////////

///////////////////////// DFPTester ///////////////////////////////////////////

class DFPTester
    : public DFP
{
public:

    DFPTester();

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition *pCondition);

};

///////////////////////// EvolutionaryMethodTester ////////////////////////////

class EvolutionaryMethodTester
    : public EvolutionaryMethod
{
public:

    EvolutionaryMethodTester();

    void SetRangeMinBounds(const ColumnVector& rangeMinBounds);

    void SetRangeMaxBounds(const ColumnVector& rangeMaxBounds);

    void SetCrossoverRate(double crossoverRate);

    void SetSurvivalRate(double survivalRate);

    void SetRadioactivity(double radioactivity);

    void SetNumberOfChromosomes(unsigned numberOfChromosomes);

    void SetSelectionMethod(const SelectionMethod* pSelectionMethod);

    void SetMutationMethod(const MutationMethod* pMutationMethod);

    void SetCrossoverMethod(const CrossoverMethod* pCrossoverMethod);

    void SetStopCondition(const EMStopCondition* pCondition);

};

///////////////////////// FletcherReevesTester ////////////////////////////////

class FletcherReevesTester
    : public FletcherReeves
{
public:

    FletcherReevesTester();

    void SetModification(bool modification);

    void SetStartingPoint(const ColumnVector& startingPoint);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// HookeJeevesDiscreteTester ///////////////////////////

class HookeJeevesDiscreteTester
    : public HookeJeevesDiscrete
{
public:

    HookeJeevesDiscreteTester();

    const ColumnVector Test1();

    void SetEpsilon(double epsilon);

    void SetStartingPoint(ColumnVector point);

};

///////////////////////// HookeJeevesOptimalTester ////////////////////////////

class HookeJeevesOptimalTester
    : public HookeJeevesOptimal
{
public:

    HookeJeevesOptimalTester();

    void SetStopCondition(const StandardStopCondition *pCondition);

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(auto_ptr<const Method> apLineSearch);

};

///////////////////////// MarquardtTester ////////////////////////////////////

class MarquardtTester
    : public Marquardt
{
public:

    MarquardtTester();

    void SetStartingPoint(const ColumnVector& startingPoint);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// NelderMeadTester ////////////////////////////////////

class NelderMeadTester
    : public NelderMead
{
public:

    NelderMeadTester();

    const vector<PointWithValue>
    TestInitialSimplexGeneration(const  ColumnVector&   centralPoint,
                                        double          sideLength,
                                 const  FunctionBase*   function);

    ColumnVector
    TestCentroidFinding(vector<ColumnVector>& initialSimplex,
                        const ProblemBase& problem);

    vector<ColumnVector>
    TestShrinking(vector<ColumnVector> initialSimplex,
                  const ProblemBase& problem,
                  double shrinkageCoefficient);

    void SetReflectionCoefficient(double d);

    void SetExpansionCoefficient(double d);

    void SetContractionCoefficient(double d);

    void SetShrinkageCoefficient(double d);

    void SetStartingPoint(ColumnVector& point);

    void SetSideLength(double length);

    void SetStopCondition(const NMStopCondition* pCondition);

};

///////////////////////// NewtonTester ////////////////////////////////////////

class NewtonTester
    : public Newton
{
public:

    NewtonTester();

    void SetStartingPoint(const ColumnVector& startingPoint);

    void SetStopCondition(const StandardStopCondition* pCondition);
};

///////////////////////// NewtonModifiedTester ////////////////////////////////

class NewtonModifiedTester
    : public NewtonModified
{
public:

    NewtonModifiedTester();

    void SetStartingPoint(const ColumnVector& startingPoint);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// PolakRibiereTester //////////////////////////////////

class PolakRibiereTester
    : public PolakRibiere
{
public:

    PolakRibiereTester();

    void SetStartingPoint(const ColumnVector& startingPoint);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// PowellVariant1Tester ////////////////////////////////

class PowellVariant1Tester
    : public PowellVariant1
{
public:

    PowellVariant1Tester();

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// PowellVariant2Tester ////////////////////////////////

class PowellVariant2Tester
    : public PowellVariant2
{
public:

    PowellVariant2Tester();

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// SteepestDescentTester ///////////////////////////////

class SteepestDescentTester
    : public SteepestDescent
{
public:

    SteepestDescentTester();

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition *pCondition);

};

///////////////////////// RosenbrockDiscreteTester ////////////////////////////

class RosenbrockDiscreteTester
    : public RosenbrockDiscrete
{
public:

    RosenbrockDiscreteTester();

    void SetTau(ColumnVector tau);

    void SetDelta(double delta);

    void SetAlpha(double alpha);

    void SetBeta(double beta);

    void SetStartingPoint(ColumnVector point);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

///////////////////////// RosenbrockOptimalTester /////////////////////////////

class RosenbrockOptimalTester
    : public RosenbrockOptimal
{
public:

    RosenbrockOptimalTester();

    void SetStartingPoint(ColumnVector point);

    void SetLineSearchMethod(const LineSearch* pLineSearch);

    void SetStopCondition(const StandardStopCondition* pCondition);

};

#endif
